
-- Esborrat del paquet (especificaci i cos)
-- -----------------------------------------
-- DROP PACKAGE BODY pk_utils;
-- DROP PACKAGE pk_utils;


-- ******************************************************************************************************************************
-- ******************************************************************************************************************************
-- *																*
-- *						FASE D'ESPECIFICACI								*
-- *																*
-- ******************************************************************************************************************************
-- ******************************************************************************************************************************


-- --------------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------------
-- Nom_paquet: 		pk_utils
-- Fase: 
-- Descripci:		paquet amb un procediment cridat per tots els altres procediments i tres funcions
-- 			molt tils per la crrega de dades inicial
-- 
-- Procediments:	p_registrarLog, 
-- Funcions:		f_ num_aleatori, f_ data_aleatoria, f_opcio_aleatoria
-- --------------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE pk_utils IS



	PROCEDURE p_registrarLog (
	  data		IN logProcediments.data%TYPE,
	  nom_proc	IN logProcediments.nom_procediment%TYPE,
	  pEntrada	IN logProcediments.parametres_entrada%TYPE,
	  pSortida	IN logProcediments.sortida%TYPE,
	  correcte	IN logProcediments.correcte%TYPE,
	  confirmar	IN logProcediments.confirmar%TYPE
  	);


	FUNCTION f_num_aleatori RETURN NUMBER;


	FUNCTION f_data_aleatoria RETURN VARCHAR2;


	FUNCTION f_opcio_aleatoria(i_votacio IN NUMBER, cod_pais CHAR) RETURN NUMBER;


END; --fi especificaci paquet pk_utils
/

















-- ******************************************************************************************************************************
-- ******************************************************************************************************************************
-- *																*
-- *						FASE D'IMPLEMENTACI								*
-- *																*
-- ******************************************************************************************************************************
-- ******************************************************************************************************************************




-- --------------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------------
-- Nom_paquet: 		pk_utils
-- Fase: 
-- Descripci:		paquet amb un procediment cridat per tots els altres procediments i tres funcions
-- 			molt tils per la crrega de dades inicial
-- 
-- Procediments:	p_registrarLog, 
-- Funcions:		f_ num_aleatori, f_ data_aleatoria, f_opcio_aleatoria
-- --------------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE BODY pk_utils IS


	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom_procediment:	p_registrarLog
	-- 
	-- Descripci:		insereix un registre a la taula logProcediments i confirma la transacci o b 
	--			la desf (si hi ha excepcions o el parmetre confirmar = 0). En cas de desfer
	--			una transacci tamb mostra un missatge descriptiu.
	-- Parmetres entrada:	data 		TIMESTAMP(0)
	--			nom_proc	VARCHAR(30 CHAR)	NOT NULL
	--			pEntrada	VARCHAR(650 CHAR)	NOT NULL
	--			pSortida	VARCHAR(200 CHAR)	NOT NULL
	--			correcte	NUMBER(1)		NOT NULL CHECK(correcte IN(0,1))
	--			confirmar	NUMBER(1)		NOT NULL CHECK(confirmar IN(0,1))
	-- Parmetres sortida:	--
	-- Excepcions:		camp_obligatori	-> quan un parmetre d'entrada s buit o NULL
	--			camp_incorrecte	-> quan els parmetres correcte i/o confirmar no sn 0 o 1
	--			OTHERS		-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_registrarLog (
	  data		IN logProcediments.data%TYPE,
	  nom_proc	IN logProcediments.nom_procediment%TYPE,
	  pEntrada	IN logProcediments.parametres_entrada%TYPE,
	  pSortida	IN logProcediments.sortida%TYPE,
	  correcte	IN logProcediments.correcte%TYPE,
	  confirmar	IN logProcediments.confirmar%TYPE
	  ) IS

	  	camp_obligatori		EXCEPTION;
		camp_incorrecte		EXCEPTION;

		BEGIN

		  IF (  data IS NULL OR 
			nom_proc IS NULL OR 
			pSortida IS NULL OR 
			correcte IS NULL OR 
			confirmar IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (	correcte < 0 OR
			correcte > 1 OR
			confirmar < 0 OR
			confirmar > 1 OR
			correcte <> confirmar) THEN
				RAISE camp_incorrecte;
		  END IF;

		  IF (confirmar = 0) THEN
			ROLLBACK;

		  END IF;

		  -- Tant si la transacci s'ha desfet (confirmar = 0), com si no, es registrar el procediment a la taula de log
		  INSERT INTO logProcediments (id, data, nom_procediment, parametres_entrada, sortida, correcte, confirmar)
		  VALUES (-1, data, nom_proc, pEntrada, pSortida, correcte, confirmar);
		  COMMIT;


		EXCEPTION

		WHEN camp_obligatori THEN

		   ROLLBACK;
		   DBMS_OUTPUT.PUT_LINE('Error al registrar el log, algun camp s buit o NULL.');
		   DBMS_OUTPUT.PUT_LINE('S''ha desfet la transacci');

		WHEN camp_incorrecte THEN

		   ROLLBACK;
		   DBMS_OUTPUT.PUT_LINE('Error al registrar el log, els arguments correcte i/o confirmar no tenen un valor vlid.');
		   DBMS_OUTPUT.PUT_LINE('S''ha desfet la transacci');
	
		WHEN OTHERS THEN

		   ROLLBACK;
		   DBMS_OUTPUT.PUT_LINE('Error al registrar el log. Error SQL: '||SQLCODE||'-'||SQLERRM);
		   DBMS_OUTPUT.PUT_LINE('S''ha desfet la transacci');
	
		END;








	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom_funci:		f_num_aleatori
	-- 
	-- Descripci:		calcula un nombre aleatori d'un mxim de 9 xifres que no sigui a la taula ciutadans i el retorna.
	--			
	-- Parmetres entrada:	--

	-- Retorn:		un nombre aleatori d'un mxim de 9 xifres
	-- 
	-- Excepcions:		OTHERS	-> qualsevol esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	FUNCTION f_num_aleatori RETURN NUMBER IS

		num 	NUMBER(9) := 0;
		trobat	NUMBER(1) := 0;
		err	VARCHAR2(200 CHAR);

		BEGIN

		  -- es genera un valor aleatori d'un mxim de 9 dgits i es mira si ja existeixi a la taula ciutadans (si existeix, trobat = 1)
		  num := DBMS_RANDOM.value * 1000000000;
		  SELECT COUNT(*) INTO trobat FROM ciutadans WHERE num_aleatori = num;

		  -- mentre el valor existeixi a la taula ciutadans ( trobat = 1 ), s'aniran generant nous aleatoris 
		  WHILE ( trobat = 1 ) LOOP
			num := DBMS_RANDOM.value * 1000000000;
		  	SELECT COUNT(*) INTO trobat FROM ciutadans WHERE num_aleatori = num;
		  END LOOP;

		  RETURN num;

		EXCEPTION

		WHEN OTHERS THEN
	
		   err := 'Error: '||SQLCODE||'-'||SQLERRM;
		   DBMS_OUTPUT.PUT_LINE(err);
		   RETURN num;

		END;










	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom_funci:		f_data_aleatoria
	-- 
	-- Descripci:		Genera una cadena aleatria que representa una data en format dd/mm/yyyy 
	-- 			dintre del rang 01/01/1901 - 31/12/1992 (excloent els 29 de febrer)
	--			
	-- Parmetres entrada:	--

	-- Retorn:		una cadena aleatria que representa una data en format dd/mm/yyyy
	-- 
	-- Excepcions:		OTHERS	-> qualsevol esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	FUNCTION f_data_aleatoria RETURN VARCHAR2 IS

		data_x	VARCHAR2(12 CHAR);
		dia_x	NUMBER(3) := 0;
		mes_x	NUMBER(3) := 0;
		any_x	NUMBER(5) := 0;
		trobat	NUMBER(1) := 0;
		n_dies	NUMBER(2) := 0;
		err	VARCHAR2(200 CHAR);

		BEGIN

		   -- obtenim un nmero aleatori de dos dgits dins el rang {1, 12} que representaran un mes
		   trobat := 0;
		   WHILE (trobat = 0) LOOP

			mes_x := DBMS_RANDOM.value * 100;
			IF (mes_x > 0 AND mes_x < 13) THEN trobat := 1; END IF;

		   END LOOP;

		   -- en funci del mes obtingut determinem els dies que t		
		   n_dies := 31;
		   IF (mes_x = 2) THEN n_dies := 28;
		   ELSIF (mes_x = 4) THEN n_dies := 30;
		   ELSIF (mes_x = 6) THEN n_dies := 30;
		   ELSIF (mes_x = 9) THEN n_dies := 30;
		   ELSIF (mes_x = 11) THEN n_dies := 30;
		   END IF;

		   -- obtenim un nmero aleatori de dos dgits dins el rang {1, n_dies} que representaran un dia del mes
		   trobat := 0;
		   WHILE (trobat = 0) LOOP

			dia_x := DBMS_RANDOM.value * 100;
			IF (dia_x > 0 AND dia_x <= n_dies ) THEN trobat := 1; END IF;

		   END LOOP;

		   -- obtenim un nmero alatori de quatre dgits dins el rang {1901, 1992} que representaran un any
		   trobat := 0;
		   WHILE (trobat = 0) LOOP

			any_x := DBMS_RANDOM.value * 10000;
			IF ( any_x > 1900 AND any_x <= 1992 ) THEN trobat := 1; END IF;

		   END LOOP;

		   -- formatem la cadena de retorn amb els valors aleatoris obtinguts
		   data_x := dia_x||'/'||mes_x||'/'||any_x;


		   RETURN data_X;


		EXCEPTION

		WHEN OTHERS THEN
	
		   err := 'Error: '||SQLCODE||'-'||SQLERRM;
		   DBMS_OUTPUT.PUT_LINE(err);
		   RETURN data_x;

		END;




	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom_funci:		f_opcio_aleatoria
	-- 
	-- Descripci:		Donat el codi d'una votaci i d'un codi ISO d'un pas que particip a la votaci, la funci genera un nombre 
	-- 			aleatori que representa una opci de vot vlida dins la votaci i vlida pels electors del pas (electors de
	-- 			pasos diferents que participin a una mateixa votaci tindran un codi de pas diferent que determinar quines
	-- 			opcions de vot tenen disponibles. Aix servir, per exemple, perqu un ciutad alemany no pugui votar ERC).
	-- 
	-- 			Notes: 
	-- 			1) Es considera que les opcions de vot que pot escollir un elector seran opcions que pertanyin al mateix
	-- 			pas que l'elector o b determinades opcions que pertanyen a tots els pasos. Aquestes darreres sn les
	-- 			opcions de vot 'SI' i 'NO', en el cas que la votaci fos una consulta, i 'BLANC' i 'NUL' en el cas d'una
	-- 			votaci normal. Totes quatre estan representades pel codi ISO 'ZQ' que no est assignat a cap pas i que 
	-- 			l'he fet servir com a codi de 'pas neutre'.
	-- 			
	-- 			2) Com que, escollides aleatriament, les opcions de vot 'BLANC' i 'NUL' tenen la mateixa probabilitat de
	-- 			ser escollides que les altres opcions i podrien guanyar fcilment les eleccions, per fer-ho ms realista s'ha
	-- 			moderat aquesta aleatorietat fent que aproximadament el 90% de les vegades que el nmero aleatori representi
	-- 			una d'aquestes opcions, es descarti el nmero i se'n generi un altre.
	-- 
	--			
	-- Parmetres entrada:	i_votacio	NUMBER		(hauria de repesentar l'identificador d'una votaci amb opcions de vot associades)
	-- 			cod_pais	CHAR		(hauria de representar el codi ISO del pas d'un elector)
	-- 
	-- Retorn:		Si i_votaci referencia una votaci amb opcions de vot associades i cod_pais referencia el codi ISO d'un pas, 
	-- 			retornar un nmero natural que representa el codi d'una opci de vot vlida per la votaci i pels electors 
	-- 			d'aquell pas (incloent opcions de vot 'BLANC' i 'NUL'). En cas d'error, retorna zero i l'excepci.
	-- 
	-- Excepcions:		votacio_inexistent	-> quan i_votacio no referencia cap votaci.
	-- 			codi_pais_inexistent	-> quan cod_pais no referencia el codi ISO de cap pas del sistema
	-- 			no_opcions		-> quan la votaci existeix per encara no t opcions de vot associades
	-- 			pais_votacio_inexistent	-> quan la votaci existeix i el codi ISO referencia un pas del sistema
	-- 						   per aquest pas no participa a la votaci (la votaci no t cap cens
	-- 						   d'aquest pas).
	-- 			OTHERS			-> qualsevol esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	FUNCTION f_opcio_aleatoria(i_votacio IN NUMBER, cod_pais CHAR) RETURN NUMBER IS

		aleatori	NUMBER(4) := 0;
		trobat		NUMBER(1) := 0;
		moderador	NUMBER := 0;
		opcions		NUMBER := 0;
		opc_min		NUMBER(2) := 0;
		opc_max		NUMBER(2) := 0;
		pais		VARCHAR2(30 CHAR);
		err		VARCHAR2(200 CHAR);
		votacio_inexistent	EXCEPTION;
		codi_pais_inexistent	EXCEPTION;
		no_opcions		EXCEPTION;
		pais_votacio_inexistent	EXCEPTION;

		BEGIN

		-- comprovem que l'identificador de votaci existeixi
		trobat := 0;
		SELECT COUNT(*) INTO trobat FROM votacions WHERE id = i_votacio;
		IF ( trobat = 0 ) THEN
			RAISE votacio_inexistent;
		END IF;

		-- comprovem que el codi de pas de l'elector existeixi
		trobat := 0;
		SELECT COUNT(*) INTO trobat FROM paisos WHERE codi_pais = cod_pais;
		IF ( trobat = 0 ) THEN
			RAISE codi_pais_inexistent;
		END IF;


		-- comprovem que la votacio tingui opcions associades
		SELECT COUNT(*) INTO opcions FROM opcionsVotacio WHERE id_votacio = i_votacio;
		IF ( opcions = 0) THEN
			RAISE no_opcions;
		END IF;


		-- comprovem que el codi del pais estigui associat a la votaci
		SELECT nom_pais INTO pais FROM paisos WHERE codi_pais = cod_pais;
		trobat := 0;
		SELECT COUNT(*) INTO trobat FROM votacionsPaisos WHERE id_pais = pais AND id_votacio = i_votacio;
		IF ( trobat = 0 ) THEN
			RAISE pais_Votacio_inexistent;
		END IF;


		-- determinem l'identificador d'opci mxim i mnim de la votaci en qesti
		SELECT MIN(id_opcio) INTO opc_min FROM opcionsVotacio WHERE id_votacio = i_votacio;
		SELECT MAX(id_opcio) INTO opc_max FROM opcionsVotacio WHERE id_votacio = i_votacio;


		trobat := 0;
		moderador := 0;
		WHILE ( trobat = 0 ) LOOP

			moderador := moderador+1;

			-- es genera un aleatori d'un mxim de dues xifres (per ara considerem que no hi haur ms 
			-- de 99 opcions de vot diferent en una votaci)
			aleatori := DBMS_RANDOM.value * 100;

			-- si l'aleatori est dintre el rang d'opcions possibles per la votaci, llavors...
			IF (aleatori >= opc_min AND aleatori <= opc_max ) THEN

				-- comprovem si representa una opci que correspon realment a la votaci (en cas afirmatiu, posar trobat a 1)
				SELECT COUNT(*) INTO trobat FROM opcionsVotacio WHERE id_votacio = i_votacio AND id_opcio = aleatori;

				IF ( trobat = 1 ) THEN

					-- comprovem si representa una opci del pas de l'elector (si no ho s posar trobat a 0)
					SELECT COUNT(*) INTO trobat FROM opcions WHERE id = aleatori AND codi_pais = cod_pais;

					IF ( trobat = 0 ) THEN

						-- Aqu dins, l'aleatori no representa cap opci del pas de l'elector. Tenim dos casos:
						-- 1) representa una opci que no pot escollir l'elector (no s del seu pas)
						-- 2) representa una opci que poden escollir els electors de tots els pasos. Es tracta de
						--    les opcions de vot 'SI', 'NO', 'BLANC' o  'NUL' i que estan representades
						--    per un codi de pas neutre 'ZQ'.

						-- comprovem si representa una opci del 'pas neutre' ZQ.
						-- En cas afirmatiu trobat valdr 1
						SELECT COUNT(*) INTO trobat FROM opcions WHERE id = aleatori AND codi_pais = 'ZQ';

						-- Com que aleatriament les opcions de vot del pas neutre ZQ tenen la mateixa probabilitat
						-- que la resta d'opcions de vot, podrien arribar a guanyar les eleccions fcilment opcions de
						-- de vot en BLANC o opcions de vot NUL. Per fer-ho ms realista, moderarem la quantitat d'aquests
						-- vots. Considerarem vlids un 10% (de promig) dels nmeros aleatoris que representin aquestes
						-- opcions de vot. Es consideraran vlides aquelles opcions de vot del pas neutre que s'obtinguin
						-- en les iteracions del bucle acabades en 1 {1, 11, 21, ..., n10+1}
						IF ( trobat = 1 ) THEN

							IF ( MOD(moderador, 10) <> 1) THEN
								trobat := 0;
							END IF;

						END IF;

					END IF;

				END IF;

			-- l'aleatori no est dintre el rang d'opcions possibles per la votaci
			ELSE
				trobat := 0;
			END IF;

		END LOOP;

		RETURN aleatori;


		EXCEPTION

		WHEN votacio_inexistent THEN
	
		   err := 'Error: l''identificador de votacio no correspon a cap votacio del sistema.';
		   DBMS_OUTPUT.PUT_LINE(err);
		   RETURN aleatori;

		WHEN codi_pais_inexistent THEN
	
		   err := 'Error: el codi del pais de l''elector no correspon a cap pais del sistema.';
		   DBMS_OUTPUT.PUT_LINE(err);
		   RETURN aleatori;

		WHEN no_opcions THEN
	
		   err := 'Error: la votacio no te opcions associades.';
		   DBMS_OUTPUT.PUT_LINE(err);
		   RETURN aleatori;

		WHEN pais_votacio_inexistent THEN
	
		   err := 'Error: el pais representat pel seu codi no participa a la votacio.';
		   DBMS_OUTPUT.PUT_LINE(err);
		   RETURN aleatori;

		WHEN OTHERS THEN
	
		   err := 'Error: '||SQLCODE||'-'||SQLERRM;
		   DBMS_OUTPUT.PUT_LINE(err);
		   RETURN aleatori;

		END;











END; --fi implementaci paquet pk_utils
/


