

-- Esborrat del paquet 
-- --------------------
-- DROP PACKAGE BODY pk_territoris;
-- DROP PACKAGE pk_territoris;


-- ******************************************************************************************************************************
-- ******************************************************************************************************************************
-- *																*
-- *						FASE D'ESPECIFICACI								*
-- *																*
-- ******************************************************************************************************************************
-- ******************************************************************************************************************************



-- -----------------------------------------------------------------------------------------------------------------------------
-- -----------------------------------------------------------------------------------------------------------------------------
-- Nom_paquet: 		pk_territoris
-- 
-- Descripci:		especificaci de diferents procediments relacionats amb els territoris i els seus censos
-- 
-- Procediments:	p_altaCodisISO, p_altaPais, p_altaRegio, p_altaLocalitat, p_altaCensGlobal, p_altaCensEstatal
-- 			p_altaCensRegional, p_altaCensLocal, p_baixaCensGlobal, p_baixaCensEstatl, p_baixaCensRegional,
-- 			p_baixaCensLocal
-- Funcions:		--
-- ------------------------------------------------------------------------------------------------------------------------------
-- ------------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE pk_territoris IS



	PROCEDURE p_altaCodisISO (
	  codi			IN	codisISO.codi_pais%TYPE,
	  pais			IN	codisISO.nom_pais%TYPE,
	  RSP			OUT	VARCHAR2
	  );




	PROCEDURE p_altaPais (
	  pais			IN	paisos.nom_pais%TYPE,
	  codi			IN	paisos.codi_pais%TYPE,
	  RSP			OUT	VARCHAR2
	  );



	PROCEDURE p_altaRegio (
	  id_regio		IN	regions.id%TYPE,
	  pais			IN	regions.id_pais%TYPE,
	  regio			IN	regions.nom_regio%TYPE,
	  RSP			OUT	VARCHAR2
	  );



	PROCEDURE p_altaLocalitat (
	  id			IN	localitats.id%TYPE,
	  regio			IN	localitats.id_regio%TYPE,
	  localitat		IN	localitats.nom_localitat%TYPE,
	  RSP			OUT	VARCHAR2
	  );



	PROCEDURE p_altaCensGlobal (
	  n_cens		IN	censGlobal.nom_cens%TYPE,
	  RSP			OUT	VARCHAR2
	  );



	PROCEDURE p_altaCensEstatal (
	  nom_cens		IN	censosEstatals.nom_cens%TYPE,
	  id_cens_global	IN	censosEstatals.id_cens_global%TYPE,
	  cod_pais		IN	censosEstatals.codi_pais%TYPE,
	  RSP			OUT	VARCHAR2
	  );



	PROCEDURE p_altaCensRegional (
	  nom_cens		IN	censosRegionals.nom_cens%TYPE,
	  id_cens_estat		IN	censosRegionals.id_cens_estatal%TYPE,
	  id_regio		IN	censosRegionals.id_regio%TYPE,
	  nom_regio		IN	censosRegionals.nom_regio%TYPE,
	  RSP			OUT	VARCHAR2
	  );


	PROCEDURE p_altaCensLocal (
	  nom_cens		IN	censosLocals.nom_cens%TYPE,
	  id_cens_regio		IN	censosLocals.id_cens_regional%TYPE,
	  id_localitat		IN	censosLocals.id_localitat%TYPE,
	  nom_localitat		IN	censosLocals.nom_localitat%TYPE,
	  num_censats		IN	censosLocals.ciutadans_censats%TYPE,
	  RSP			OUT	VARCHAR2
	  );



	PROCEDURE p_baixaCensGlobal (
	  nom_cens_g	IN	censGlobal.nom_cens%TYPE,
	  RSP		OUT	VARCHAR2
	  );


	PROCEDURE p_baixaCensEstatal (
	  nom_cens_e	IN	censosEstatals.nom_cens%TYPE,
	  RSP		OUT	VARCHAR2
	  );


	PROCEDURE p_baixaCensRegional (
	  nom_cens_r	IN	censosRegionals.nom_cens%TYPE,
	  RSP		OUT	VARCHAR2
	  );


	PROCEDURE p_baixaCensLocal (
	  nom_cens_l	IN	censosLocals.nom_cens%TYPE,
	  RSP		OUT	VARCHAR2
	  );




END; --fi especificaci paquet pk_territoris
/









-- ******************************************************************************************************************************
-- ******************************************************************************************************************************
-- *																*
-- *						FASE D'IMPLEMENTACI								*
-- *																*
-- ******************************************************************************************************************************
-- ******************************************************************************************************************************




-- -----------------------------------------------------------------------------------------------------------------------------
-- -----------------------------------------------------------------------------------------------------------------------------
-- Nom_paquet: 		pk_territoris
-- 
-- Descripci:		implementaci de diferents procediments relacionats amb els territoris i els seus censos
-- 
-- Procediments:	p_altaCodisISO, p_altaPais, p_altaRegio, p_altaLocalitat, p_altaCensGlobal, p_altaCensEstatal
-- 			p_altaCensRegional, p_altaCensLocal, p_baixaCensGlobal, p_baixaCensEstatl, p_baixaCensRegional,
-- 			p_baixaCensLocal
-- Funcions:		--
-- ------------------------------------------------------------------------------------------------------------------------------
-- ------------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE BODY pk_territoris IS



	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaCodisISO
	-- 			
	-- Descripci:		insereix un registre a la taula codisISO i crida el procediment p_registrarLog del paquet pk_utils
	-- 			
	-- Parmetres entrada:	codi	CHAR(2)		PK
	-- 			pais	VARCHAR2(30)	NOT NULL	UNIQUE
	-- 			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori	-> quan un parmetre d'entrada s buit o NULL
	-- 			camp_incorrecte -> quan el codi ISO no t exactament 2 carcters
	-- 			codi_existent	-> codi referencia un codi que ja existeix a la taula codisISO
	-- 			pais_existent	-> quan pais referencia el nom d'un pas que ja existeix a la taula codisISO (nom pas s UNIQUE)
	--			OTHERS		-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaCodisISO (
	  codi		IN	codisISO.codi_pais%TYPE,
	  pais		IN	codisISO.nom_pais%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_altaCodisISO';
	  	parametres_entrada 	VARCHAR2(40 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
	  	camp_obligatori 	EXCEPTION;
		camp_incorrecte 	EXCEPTION;
		codi_existent		EXCEPTION;
		pais_existent		EXCEPTION;


		BEGIN

		  parametres_entrada := codi||', '||pais;

		  IF (  codi IS NULL OR 
			pais IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (length(codi) <> 2) THEN
				RAISE camp_incorrecte;
		  END IF;



		  -- comprovem que el codi del pas no existeixi ja a la taula codisISO
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM codisISO WHERE codi_pais = codi;
		  IF ( trobat = 1 ) THEN
			RAISE codi_existent;
		  END IF;

		  -- comprovem que el nom del pas no existeixi ja a la taula codisISO (el nom del pas s clau alternativa)
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM codisISO WHERE nom_pais = pais;
		  IF ( trobat = 1 ) THEN
			RAISE pais_existent;
		  END IF;


		  INSERT INTO codisISO(codi_pais, nom_pais) VALUES(codi, pais);

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN camp_incorrecte THEN

		   RSP := 'ERROR: el codi ha de tenir 2 caracters.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN codi_existent THEN

		   RSP := 'ERROR: el codi del pais ja existeix a la taula codisISO.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN pais_existent THEN

		   RSP := 'ERROR: el nom del pais ja existeix a la taula codisISO. El nom ha de ser unic.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;






	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaPais
	-- 			
	-- Descripci:		insereix un registre a la taula paisos i crida el procediment p_registrarLog del paquet pk_utils
	-- 			
	-- Parmetres entrada:	nom_pais	VARCHAR2(30 CHAR)	PK
	-- 			codi_pais	CHAR(2)			NOT NULL	UNIQUE	REFERENCES codisISO(codi_pais)
	-- 			
	-- Parmetres sortida:	RSP		VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 							al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada s buit o NULL
	-- 			camp_incorrecte 	-> quan el codi ISO no t exactament 2 carcters
	-- 			codiISOException	-> quan codi no referencia cap codi ISO
	-- 			pais_existent		-> quan pais referencia el nom d'un pas que ja existeix a la taula paisos
	-- 			codi_existent		-> codi referencia un codi que ja existeix a la taula paisos  (codi_pais s UNIQUE)
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaPais (
	  pais		IN	paisos.nom_pais%TYPE,
	  codi		IN	paisos.codi_pais%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_altaPais';
	  	parametres_entrada 	VARCHAR2(40 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
	  	camp_obligatori 	EXCEPTION;
		camp_incorrecte 	EXCEPTION;
		codiISO_inexistent	EXCEPTION;
		pais_existent		EXCEPTION;
		codi_existent		EXCEPTION;


		BEGIN

		  parametres_entrada := pais||', '||codi;

		  IF (  pais IS NULL OR 
			codi IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (length(codi) <> 2) THEN
				RAISE camp_incorrecte;
		  END IF;

		  -- comprovem que 'codi' referenci un codiISO
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM codisISO WHERE codi_pais = codi;
		  IF ( trobat = 0 ) THEN
			RAISE codiISO_inexistent;
		  END IF;

		  -- comprovem que el nom del pas no existeixi ja a la taula paisos
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM paisos WHERE nom_pais = pais;
		  IF ( trobat = 1 ) THEN
			RAISE pais_existent;
		  END IF;


		  -- comprovem que el codi del pas no existeixi ja a la taula paisos  (el codi del pas s clau alternativa)
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM paisos WHERE codi_pais = codi;
		  IF ( trobat = 1 ) THEN
			RAISE codi_existent;
		  END IF;


		  -- donem d'alta el pas
		  INSERT INTO paisos(nom_pais, codi_pais) VALUES(pais, codi);

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN camp_incorrecte THEN

		   RSP := 'ERROR: el codi ha de tenir 2 carcters.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN codiISO_inexistent THEN

		   RSP := 'ERROR: l''identificador de codi ISO no correspon a cap codi ISO del sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN pais_existent THEN

		   RSP := 'ERROR: el nom del pais ja existeix a la taula paisos.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	

		WHEN codi_existent THEN

		   RSP := 'ERROR: el codi del pais ja existeix a la taula paisos. El codi del pais ha de ser unic.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;








	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaRegio
	-- 			
	-- Descripci:		insereix un registre a la taula Regions i crida el procediment p_registrarLog del paquet pk_utils
	-- 			
	-- Parmetres entrada:	id_regio	NUMBER		PK		AUTO INCREMENT
	-- 			pais		VARCHAR2(30)	NOT NULL	REFERENCES paisos(nom_pais)
	-- 			regio		VARCHAR2(40)	NOT NULL	UNIQUE(pais, regio)
	-- 
	-- Parmetres sortida:	RSP		VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 							al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori	-> quan un parmetre d'entrada s buit o NULL
	-- 			camp_incorrecte -> quan el parmetre d'entrada id s >= 0
	-- 			pais_inexistent	-> quan pais no refeencia cap pas
	--			OTHERS		-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaRegio (
	  id_regio	IN	regions.id%TYPE,
	  pais		IN	regions.id_pais%TYPE,
	  regio		IN	regions.nom_regio%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_altaRegio';
	  	parametres_entrada 	VARCHAR2(80 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
	  	camp_obligatori 	EXCEPTION;
		camp_incorrecte 	EXCEPTION;
		pais_inexistent		EXCEPTION;


		BEGIN

		  parametres_entrada := id_regio||', '||pais||', '||regio;

		  IF (  id_regio IS NULL OR
			pais IS NULL OR 
			regio IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF ( id_regio >=0 ) THEN
				RAISE camp_incorrecte;
		  END IF;


		  -- comprovem pais referenci un pais
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM paisos WHERE nom_pais = pais;
		  IF ( trobat = 0 ) THEN
			RAISE pais_inexistent;
		  END IF;


		  INSERT INTO regions(id, id_pais, nom_regio) VALUES(id_regio, pais, regio);

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN camp_incorrecte THEN

		   RSP := 'ERROR: el primer parametre no es correcte.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN pais_inexistent THEN

		   RSP := 'ERROR: l''identificador del pais no correspon a cap pais del sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;










	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaLocalitat
	-- 			
	-- Descripci:		insereix un registre a la taula Localitats i crida el procediment p_registrarLog del paquet pk_utils
	-- 			
	-- Parmetres entrada:	id		NUMBER		PK		AUTO INCREMENT
	-- 			regio		VARCHAR2(40)	NOT NULL	REFERENCES regions(id)
	-- 			localitat	VARCHAR2(40)	NOT NULL	UNIQUE(regio, localitat)
	-- 
	-- Parmetres sortida:	RSP		VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 							al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada s buit o NULL
	-- 			camp_incorrecte 	-> quan el parmetre d'entrada id s >= 0
	-- 			regio_inexistent	-> quan regio no referencia cap regi del sistema
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaLocalitat (
	  id		IN	localitats.id%TYPE,
	  regio		IN	localitats.id_regio%TYPE,
	  localitat	IN	localitats.nom_localitat%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_altaLocalitat';
	  	parametres_entrada 	VARCHAR2(100 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
	  	camp_obligatori 	EXCEPTION;
		camp_incorrecte 	EXCEPTION;
		regio_inexistent	EXCEPTION;


		BEGIN

		  parametres_entrada := id||', '||regio||', '||localitat;

		  IF (  id IS NULL OR
			regio IS NULL OR 
			localitat IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF ( id >=0 ) THEN
				RAISE camp_incorrecte;
		  END IF;


		  -- comprovem 	que regio referenci una regio existent
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM regions WHERE id = regio;
		  IF ( trobat = 0 ) THEN
			RAISE regio_inexistent;
		  END IF;


		   INSERT INTO localitats(id, id_regio, nom_localitat) VALUES(id, regio, localitat);

		   RSP := 'OK';
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN camp_incorrecte THEN

		   RSP := 'ERROR: el primer parametre no es correcte.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN regio_inexistent THEN

		   RSP := 'ERROR: l''identificador de regio no correspon a cap regio del sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;









	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaCensGlobal
	-- 			
	-- Descripci:		insereix un registre a la taula censGlobal i crida el procediment p_registrarLog del paquet pk_utils
	-- 			
	-- Parmetres entrada:	n_cens		VARCHAR2(30)	PK
	-- 
	-- Parmetres sortida:	RSP		VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 							al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori	-> quan un parmetre d'entrada s buit o NULL
	-- 			camp_incorrecte -> quan el nom del cens supera els 30 carcters
	-- 			cens_existent	-> quan el nom del cens ja existeix a la taula censGlobal
	--			OTHERS		-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaCensGlobal (
	  n_cens	IN	censGlobal.nom_cens%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_altaCensGlobal';
	  	parametres_entrada 	VARCHAR2(60 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
	  	camp_obligatori 	EXCEPTION;
		camp_incorrecte 	EXCEPTION;
		cens_existent		EXCEPTION;


		BEGIN

		  parametres_entrada := n_cens;

		  IF ( n_cens IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;


		  IF ( length(n_cens) > 30 ) THEN
				RAISE camp_incorrecte;
		  END IF;


		  -- comprovem que el nom del cens no existeixi ja a la taula censGlobal
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM censGlobal WHERE nom_cens = n_cens;
		  IF ( trobat = 1 ) THEN
			RAISE cens_existent;
		  END IF;


		  INSERT INTO censGlobal(nom_cens) VALUES(n_cens);

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: El nom del cens es buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN camp_incorrecte THEN

		   RSP := 'ERROR: El nom del cens es massa llarg (maxim 30 caracters).';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN cens_existent THEN

		   RSP := 'ERROR: El nom del cens ja existeix a la taula censGlobal.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;






	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaCensEstatal
	-- 			
	-- Descripci:		insereix un registre a la taula censosEstatals i crida el procediment p_registrarLog del paquet pk_utils
	-- 			
	-- Parmetres entrada:	nom_cens	VARCHAR2(30)	PK
	-- 			id_cens_global	VARCHAR2(30)	NOT NULL	REFERENCES censGlobal(nom_cens)
	-- 			cod_pais	CHAR(2)		NOT NULL	REFERENCES paisos(codi_pais)
	-- 
	-- Parmetres sortida:	RSP		VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 							al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada s buit o NULL
	-- 			camp_incorrecte 	-> quan el codi del pas no t exactament 2 carcters
	-- 			cens_global_inexistent	-> quan id_cens_global no referencia el cens global del sistema
	-- 			pais_inexistent		-> quan cod_pais no referencia el codi de cap pas del sistema
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaCensEstatal (
	  nom_cens		IN	censosEstatals.nom_cens%TYPE,
	  id_cens_global	IN	censosEstatals.id_cens_global%TYPE,
	  cod_pais		IN	censosEstatals.codi_pais%TYPE,
	  RSP			OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_altaCensEstatal';
	  	parametres_entrada 	VARCHAR2(70 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
	  	camp_obligatori 	EXCEPTION;
		camp_incorrecte 	EXCEPTION;
		cens_global_inexistent	EXCEPTION;
		pais_inexistent		EXCEPTION;


		BEGIN


		  parametres_entrada := nom_cens||', '||id_cens_global||', '||cod_pais;

		  IF (  nom_cens IS NULL OR
			id_cens_global IS NULL OR 
			cod_pais IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF ( length(cod_pais) <> 2 ) THEN
				RAISE camp_incorrecte;
		  END IF;



		  -- comprovem que el cens global existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM censGlobal WHERE nom_cens = id_cens_global;
		  IF ( trobat = 0 ) THEN
			RAISE cens_global_inexistent;
		  END IF;
		  

		  -- comprovem el codi_pais referenci un pais
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM paisos WHERE codi_pais = cod_pais;
		  IF ( trobat = 0 ) THEN
			RAISE pais_inexistent;
		  END IF;

		  -- afegim el cens estatal
		  INSERT INTO censosEstatals(nom_cens, id_cens_global, codi_pais) VALUES(nom_cens, id_cens_global, cod_pais);

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN camp_incorrecte THEN

		   RSP := 'ERROR: el codi del pais ha de tenir 2 caracters.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN cens_global_inexistent THEN

		   RSP := 'ERROR: l''identificador de cens global no es correspon amb el cens global del sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN pais_inexistent THEN

		   RSP := 'ERROR: l''identificador del codi del pais no correspon a cap pais del sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;








	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaCensRegional
	-- 			
	-- Descripci:		insereix un registre a la taula censosRegionals i crida el procediment p_registrarLog del paquet pk_utils
	-- 			
	-- Parmetres entrada:	nom_cens	VARCHAR2(40)	PK
	-- 			id_cens_estat	VARCHAR2(30)	NOT NULL	REFERENCES censosEstatals(nom_cens)
	-- 			id_regio	NUMBER		NOT NULL	REFERENCES regions(id)
	-- 			nom_regio	VARCHAR2(40)
	-- 
	-- Parmetres sortida:	RSP		VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 							al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada s buit o NULL
	-- 			camp_incorrecte 	-> quan id_regio t un valor negatiu o zero
	-- 			cens_estatal_inexistent	-> quan l'id_cens_estat no referencia cap cens estatal existent
	-- 			regio_inexistent	-> quan id_regio no referencia cap regio existent
	-- 			n_regio_incoherent	-> quan nom_regio i id_regio sn mtuament incoherents
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaCensRegional (
	  nom_cens		IN	censosRegionals.nom_cens%TYPE,
	  id_cens_estat		IN	censosRegionals.id_cens_estatal%TYPE,
	  id_regio		IN	censosRegionals.id_regio%TYPE,
	  nom_regio		IN	censosRegionals.nom_regio%TYPE,
	  RSP			OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_altaCensRegional';
	  	parametres_entrada 	VARCHAR2(130 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		nom_reg			VARCHAR2(40 CHAR);
	  	camp_obligatori 	EXCEPTION;
		camp_incorrecte 	EXCEPTION;
		cens_estatal_inexistent	EXCEPTION;
		regio_inexistent	EXCEPTION;
		n_regio_incoherent	EXCEPTION;


		BEGIN


		  parametres_entrada := nom_cens||', '||id_cens_estat||', '||id_regio||', '||nom_regio;

		  IF (  nom_cens IS NULL OR
			id_cens_estat IS NULL OR 
			id_regio IS NULL OR 
			nom_regio IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF ( id_regio <= 0 ) THEN
				RAISE camp_incorrecte;
		  END IF;

		  -- comprovem que el cens estatal existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM censosEstatals WHERE nom_cens = id_cens_estat;
		  IF ( trobat = 0 ) THEN
			RAISE cens_estatal_inexistent;
		  END IF;

		  -- comprovem 	que id_regio referenci una regio existent
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM regions WHERE id = id_regio;
		  IF ( trobat = 0 ) THEN
			RAISE regio_inexistent;
		  END IF;

		  -- comprovem que el nom de la regi sigui coherent amb el cens regional
		  SELECT nom_regio INTO nom_reg FROM regions WHERE id = id_regio;
		  IF ( nom_regio <> nom_reg ) THEN
			RAISE n_regio_incoherent;
		  END IF;

		  -- afegim el cens regional
		  INSERT INTO censosRegionals(nom_cens, id_cens_estatal, id_regio, nom_regio) VALUES(nom_cens, id_cens_estat, id_regio, nom_regio);

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN camp_incorrecte THEN

		   RSP := 'ERROR: l''identificador de regio no es un enter positiu.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN cens_estatal_inexistent THEN

		   RSP := 'ERROR: l''identificador de cens estatal no correspon a cap cens estatal del sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN regio_inexistent THEN

		   RSP := 'ERROR: l''identificador de regio no correspon a cap regio del sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN n_regio_incoherent THEN

		   RSP := 'ERROR: el nom de regio no es coherent amb el cens regional.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;










	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaCensLocal
	-- 			
	-- Descripci:		insereix un registre a la taula censosLocals i crida el procediment p_registrarLog del paquet pk_utils
	-- 			
	-- Parmetres entrada:	nom_cens	VARCHAR2(40)	PK
	-- 			id_cens_regio	VARCHAR2(40)	NOT NULL	REFERENCES censosRegionals(nom_cens)
	-- 			id_localitat	NUMBER		NOT NULL	REFERENCES localitats(id)
	-- 			nom_localitat	VARCHAR2(40)	NOT NULL
	-- 			num_censats	NUMBER		NOT NULL	DEFAULT 0	CHECK(num_censats >=0)
	-- 			
	-- Parmetres sortida:	RSP		VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 							al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada s buit o NULL
	-- 			camp_incorrecte 	-> quan id_localitat t un valor negatiu o zero i/o quan num_censats no s zero
	-- 			cens_regio_inexistent	-> quan l'id_cens_regio no referencia cap cens regional existent
	-- 			localitat_inexistent	-> quan id_localitat no referencia cap localitat existent
	-- 			n_localitat_incoherent	-> quan nom_localitat i id_localitat sn mtuament incoherents
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaCensLocal (
	  nom_cens		IN	censosLocals.nom_cens%TYPE,
	  id_cens_regio		IN	censosLocals.id_cens_regional%TYPE,
	  id_localitat		IN	censosLocals.id_localitat%TYPE,
	  nom_localitat		IN	censosLocals.nom_localitat%TYPE,
	  num_censats		IN	censosLocals.ciutadans_censats%TYPE,
	  RSP			OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_altaCensLocal';
	  	parametres_entrada 	VARCHAR2(150 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		nom_local		VARCHAR2(40 CHAR);
	  	camp_obligatori 	EXCEPTION;
		camp_incorrecte 	EXCEPTION;
		cens_regio_inexistent	EXCEPTION;
		localitat_inexistent	EXCEPTION;
		n_localitat_incoherent	EXCEPTION;


		BEGIN


		  parametres_entrada := nom_cens||', '||id_cens_regio||', '||id_localitat||', '||nom_localitat||', '||num_censats;

		  IF (  nom_cens IS NULL OR 
			id_cens_regio IS NULL OR 
			id_localitat IS NULL OR 
			nom_localitat IS NULL OR
			num_censats IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF ( id_localitat <= 0 OR
			num_censats < 0 OR
			num_censats > 0 ) THEN
				RAISE camp_incorrecte;
		  END IF;

		  -- comprovem que el cens regional existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM censosRegionals WHERE nom_cens = id_cens_regio;
		  IF ( trobat = 0 ) THEN
			RAISE cens_regio_inexistent;
		  END IF;

		  -- comprovem 	que la localitat existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM localitats WHERE id = id_localitat;
		  IF ( trobat = 0 ) THEN
			RAISE localitat_inexistent;
		  END IF;

		  -- comprovem que el nom de la localitat sigui coherent amb el cens
		  SELECT nom_localitat INTO nom_local FROM localitats WHERE id = id_localitat;
		  IF ( nom_localitat <> nom_local ) THEN
			RAISE n_localitat_incoherent;
		  END IF;

		  -- afegim el cens local
		  INSERT INTO censosLocals(nom_cens, id_cens_regional, id_localitat, nom_localitat, ciutadans_censats) 
		  VALUES(nom_cens, id_cens_regio, id_localitat, nom_localitat, num_censats);

		   RSP := 'OK';
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN camp_incorrecte THEN

		   RSP := 'ERROR: l''id de localitat ha de ser un enter positiu i el nombre de ciutadans censats zero.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		WHEN cens_regio_inexistent THEN

		   RSP := 'ERROR: l''identificador de cens regional no correspon a cap cens regional del sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN localitat_inexistent THEN

		   RSP := 'ERROR: l''identificador de localitat no correspon a cap localitat del sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN n_localitat_incoherent THEN

		   RSP := 'ERROR: el nom de localitat no es coherent amb el cens local.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;


 






	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_baixaCensGlobal
	-- 			
	-- Descripci:		esborra un registre (l'nic) de la taula censGlobal (noms en el cas que no s'hi hagi celebrat cap votaci*) i
	-- 			crida el procediment p_registrarLog del paquet pk_utils.
	-- 
	-- 			*Nota: En el cas que no es controls, l'esborrat d'un cens global provacaria l'esborrat en cascada de gaireb 
	-- 			totes les dades relacionades amb les votacions del cens; la qual cosa es considera (considero) que no t cap
	-- 			sentit. Per tant, per no perdre informaci de votacions que ja s'hagin celebrat, noms es permetr donar de
	-- 			baixa un cens global si no s'hi ha celebrat cap votaci (o si no n'hi ha cap associada al cens que s'estigui 
	-- 			celebrant en aquest moment).
	-- 			
	-- Parmetres entrada:	nom_cens_g	VARCHAR2(30)	
	-- 
	-- Parmetres sortida:	RSP		VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 							al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada s buit o NULL
	-- 			cens_inexistent		-> quan nom_cens_g no referencia cap cens
	-- 			votacions_obertes	-> quan el cens t alguna votaci associada en estat 'oberta'
	-- 			votacions_finalitzades	-> quan el cens t alguna votaci associada en estat 'finalitzada'
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_baixaCensGlobal (
	  nom_cens_g	IN	censGlobal.nom_cens%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_baixaCensGlobal';
	  	parametres_entrada 	VARCHAR2(35 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		i_estat			NUMBER := 0;
		n_votacions_estat	NUMBER := 0;
	  	camp_obligatori 	EXCEPTION;
		cens_inexistent		EXCEPTION;
		votacions_obertes	EXCEPTION;
		votacions_finalitzades	EXCEPTION;


		BEGIN

		  parametres_entrada := nom_cens_g;

		  IF ( nom_cens_g IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;

		  -- comprovem que el cens existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM censGlobal WHERE nom_cens = nom_cens_g;
		  IF ( trobat = 0 ) THEN
			RAISE cens_inexistent;
		  END IF;

		  -- comprovem que hi hagi cap votaci en estat 'oberta' o 'finalitzada' que tingui associat el cens
		  n_votacions_estat := 0;
		  SELECT id INTO i_estat FROM estatsVotacio WHERE nom_estat = 'oberta';
		  SELECT COUNT(*) INTO n_votacions_estat FROM votacions WHERE id_cens_global = nom_cens_g AND id_estat = i_estat;
		  IF (n_votacions_estat > 0) THEN
			RAISE votacions_obertes;
		  END IF;
		  
		  n_votacions_estat := 0;
		  SELECT id INTO i_estat FROM estatsVotacio WHERE nom_estat = 'finalitzada';
		  SELECT COUNT(*) INTO n_votacions_estat FROM votacions WHERE id_cens_global = nom_cens_g AND id_estat = i_estat;
		  IF (n_votacions_estat > 0) THEN
			RAISE votacions_finalitzades;
		  END IF;
		  

		  DELETE FROM censGlobal WHERE nom_cens = nom_cens_g;

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: El nom del cens es buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);



		WHEN cens_inexistent THEN

		   RSP := 'ERROR: no es pot eliminar un cens que no existeix.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN votacions_obertes THEN

		   RSP := 'ERROR: no es pot eliminar un cens que tingui alguna votacio celebrant-se';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN votacions_finalitzades THEN

		   RSP := 'ERROR: no es pot eliminar un cens on ja s''hi hagin celebrat votacions.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;








	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_baixaCensEstatal
	-- 			
	-- Descripci:		esborra un registre de la taula censosEstatals (noms en el cas que no s'hi hagi celebrat cap votaci*) i
	-- 			crida el procediment p_registrarLog del paquet pk_utils.
	-- 
	-- 			*Nota: En el cas que no es controls, l'esborrat d'un cens estatal provacaria l'esborrat en cascada de gaireb 
	-- 			totes les dades relacionades amb les votacions del cens; la qual cosa es considera (considero) que no t cap
	-- 			sentit. Per tant, per no perdre informaci de votacions que ja s'hagin celebrat, noms es permetr donar de
	-- 			baixa un cens estatal si no s'hi ha celebrat cap votaci (o si no n'hi ha cap associada al cens que s'estigui 
	-- 			celebrant en aquest moment).
	-- 			
	-- Parmetres entrada:	nom_cens_e	VARCHAR2(30 CHAR)	
	-- 
	-- Parmetres sortida:	RSP		VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 							al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada s buit o NULL
	-- 			cens_inexistent		-> quan nom_cens_e no referencia cap cens
	-- 			votacions_obert_final	-> quan el cens t alguna votaci associada en estat 'oberta' i/o 'finalitzada'
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_baixaCensEstatal (
	  nom_cens_e	IN	censosEstatals.nom_cens%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_baixaCensEstatal';
	  	parametres_entrada 	VARCHAR2(35 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		i_estat_oberta		NUMBER := 0;
		i_estat_final		NUMBER := 0;
		i_estat_votacio		NUMBER := 0;
		n_censos_associats	NUMBER := 0;
	  	camp_obligatori 	EXCEPTION;
		cens_inexistent		EXCEPTION;
		votacions_obert_final	EXCEPTION;


		BEGIN

		  parametres_entrada := nom_cens_e;

		  IF ( nom_cens_e IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;

		  -- comprovem que el cens existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM censosEstatals WHERE nom_cens = nom_cens_e;
		  IF ( trobat = 0 ) THEN
			RAISE cens_inexistent;
		  END IF;

		  -- comprovem que hi hagi cap votaci en estat 'oberta' o 'finalitzada' que tingui associat el cens
		  trobat := 0;
		  n_censos_associats := 0;
		  SELECT COUNT(*) INTO n_censos_associats FROM censosAssociats WHERE cens_estatal = nom_cens_e;
		  IF ( n_censos_associats > 0 ) THEN


			SELECT id INTO i_estat_oberta FROM estatsVotacio WHERE nom_estat = 'oberta';
			SELECT id INTO i_estat_final FROM estatsVotacio WHERE nom_estat = 'finalitzada';

			-- recorrem les votacions que tenen el cens associat
			FOR votacio IN (SELECT DISTINCT id_votacio FROM censosAssociats WHERE cens_estatal = nom_cens_e) LOOP
				
				trobat := trobat;
				IF ( trobat = 0 ) THEN

					SELECT id_estat INTO i_estat_votacio FROM votacions WHERE id = votacio.id_votacio;
					IF ( i_estat_votacio = i_estat_oberta OR i_estat_votacio = i_estat_final ) THEN
						trobat := 1;
					END IF;

				END IF;
				
			END LOOP;

		  END IF;


		  IF ( trobat = 1 ) THEN
			RAISE votacions_obert_final;
		  END IF;

		  
		  -- Si el cens no ha estat associat a cap votaci i/o no s'ha associat a cap votaci que ja hagi comenat o que
		  -- hagi finalitzat, es podr esborrar

		  DELETE FROM censosEstatals WHERE nom_cens = nom_cens_e;


		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: El nom del cens es buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN cens_inexistent THEN

		   RSP := 'ERROR: no es pot eliminar un cens que no existeix.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN votacions_obert_final THEN

		   RSP := 'ERROR: no es pot eliminar un cens que tingui alguna votacio celebrant-se o finalitzada';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;









	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_baixaCensRegional
	-- 			
	-- Descripci:		esborra un registre de la taula censosRegionals (noms en el cas que no s'hi hagi celebrat cap votaci*) i
	-- 			crida el procediment p_registrarLog del paquet pk_utils.
	-- 
	-- 			*Nota: En el cas que no es controls, l'esborrat d'un cens regional provacaria l'esborrat en cascada de gaireb 
	-- 			totes les dades relacionades amb les votacions del cens; la qual cosa es considera (considero) que no t cap
	-- 			sentit. Per tant, per no perdre informaci de votacions que ja s'hagin celebrat, noms es permetr donar de
	-- 			baixa un cens regional si no s'hi ha celebrat cap votaci (o si no n'hi ha cap associada al cens que s'estigui 
	-- 			celebrant en aquest moment).
	-- 			
	-- Parmetres entrada:	nom_cens_r	VARCHAR2(40 CHAR)	
	-- 
	-- Parmetres sortida:	RSP		VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 							al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada s buit o NULL
	-- 			cens_inexistent		-> quan nom_cens_r no referencia cap cens
	-- 			votacions_obert_final	-> quan el cens t alguna votaci associada en estat 'oberta' i/o 'finalitzada'
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_baixaCensRegional (
	  nom_cens_r	IN	censosRegionals.nom_cens%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_baixaCensRegional';
	  	parametres_entrada 	VARCHAR2(35 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		i_estat_oberta		NUMBER := 0;
		i_estat_final		NUMBER := 0;
		i_estat_votacio		NUMBER := 0;
		n_censos_associats	NUMBER := 0;
	  	camp_obligatori 	EXCEPTION;
		cens_inexistent		EXCEPTION;
		votacions_obert_final	EXCEPTION;


		BEGIN

		  parametres_entrada := nom_cens_r;

		  IF ( nom_cens_r IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;

		  -- comprovem que el cens existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM censosRegionals WHERE nom_cens = nom_cens_r;
		  IF ( trobat = 0 ) THEN
			RAISE cens_inexistent;
		  END IF;

		  -- comprovem que hi hagi cap votaci en estat 'oberta' o 'finalitzada' que tingui associat el cens
		  trobat := 0;
		  n_censos_associats := 0;
		  SELECT COUNT(*) INTO n_censos_associats FROM censosAssociats WHERE cens_regional = nom_cens_r;
		  IF ( n_censos_associats > 0 ) THEN


			SELECT id INTO i_estat_oberta FROM estatsVotacio WHERE nom_estat = 'oberta';
			SELECT id INTO i_estat_final FROM estatsVotacio WHERE nom_estat = 'finalitzada';

			-- recorrem les votacions que tenen el cens associat
			FOR votacio IN (SELECT DISTINCT id_votacio FROM censosAssociats WHERE cens_regional = nom_cens_r) LOOP
				
				trobat := trobat;
				IF ( trobat = 0 ) THEN

					SELECT id_estat INTO i_estat_votacio FROM votacions WHERE id = votacio.id_votacio;
					IF ( i_estat_votacio = i_estat_oberta OR i_estat_votacio = i_estat_final ) THEN
						trobat := 1;
					END IF;

				END IF;
				
			END LOOP;

		  END IF;


		  IF ( trobat = 1 ) THEN
			RAISE votacions_obert_final;
		  END IF;

		  
		  -- Si el cens no ha estat associat a cap votaci i/o no s'ha associat a cap votaci que ja hagi comenat o que
		  -- hagi finalitzat, es podr esborrar

		  DELETE FROM censosRegionals WHERE nom_cens = nom_cens_r;


		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: El nom del cens es buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN cens_inexistent THEN

		   RSP := 'ERROR: no es pot eliminar un cens que no existeix.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN votacions_obert_final THEN

		   RSP := 'ERROR: no es pot eliminar un cens que tingui alguna votacio celebrant-se o finalitzada';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;









	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_baixaCensLocal
	-- 			
	-- Descripci:		esborra un registre de la taula censosLocals (noms en el cas que no s'hi hagi celebrat cap votaci*) i
	-- 			crida el procediment p_registrarLog del paquet pk_utils.
	-- 
	-- 			*Nota: En el cas que no es controls, l'esborrat d'un cens local provacaria l'esborrat en cascada de fora
	-- 			dades relacionades amb el cens (ciutadans, vots emesos, etc.); la qual cosa es considera (considero) que no t
	-- 			cap sentit per a votacions celebrades o celebrant-se. Per no perdre informaci d'aquestes votacions, noms es 
	-- 			permetr donar de baixa un cens local si no s'hi ha celebrat cap votaci (o si no n'hi ha cap associada al cens 
	-- 			que s'estigui celebrant en aquest moment).
	-- 			
	-- Parmetres entrada:	nom_cens_l	VARCHAR2(40 CHAR)	
	-- 
	-- Parmetres sortida:	RSP		VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 							al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada s buit o NULL
	-- 			cens_inexistent		-> quan nom_cens_r no referencia cap cens
	-- 			votacions_obert_final	-> quan el cens t alguna votaci associada en estat 'oberta' i/o 'finalitzada'
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_baixaCensLocal (
	  nom_cens_l	IN	censosLocals.nom_cens%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_baixaCensLocal';
	  	parametres_entrada 	VARCHAR2(35 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		i_estat_oberta		NUMBER := 0;
		i_estat_final		NUMBER := 0;
		i_estat_votacio		NUMBER := 0;
		n_censos_associats	NUMBER := 0;
	  	camp_obligatori 	EXCEPTION;
		cens_inexistent		EXCEPTION;
		votacions_obert_final	EXCEPTION;


		BEGIN

		  parametres_entrada := nom_cens_l;

		  IF ( nom_cens_l IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;

		  -- comprovem que el cens existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM censosLocals WHERE nom_cens = nom_cens_l;
		  IF ( trobat = 0 ) THEN
			RAISE cens_inexistent;
		  END IF;

		  -- comprovem que hi hagi cap votaci en estat 'oberta' o 'finalitzada' que tingui associat el cens
		  trobat := 0;
		  n_censos_associats := 0;
		  SELECT COUNT(*) INTO n_censos_associats FROM censosAssociats WHERE id_cens_local = nom_cens_l;
		  IF ( n_censos_associats > 0 ) THEN


			SELECT id INTO i_estat_oberta FROM estatsVotacio WHERE nom_estat = 'oberta';
			SELECT id INTO i_estat_final FROM estatsVotacio WHERE nom_estat = 'finalitzada';

			-- recorrem les votacions que tenen el cens associat
			FOR votacio IN (SELECT DISTINCT id_votacio FROM censosAssociats WHERE id_cens_local = nom_cens_l) LOOP
				
				trobat := trobat;
				IF ( trobat = 0 ) THEN

					SELECT id_estat INTO i_estat_votacio FROM votacions WHERE id = votacio.id_votacio;
					IF ( i_estat_votacio = i_estat_oberta OR i_estat_votacio = i_estat_final ) THEN
						trobat := 1;
					END IF;

				END IF;
				
			END LOOP;

		  END IF;


		  IF ( trobat = 1 ) THEN
			RAISE votacions_obert_final;
		  END IF;

		  
		  -- Si el cens no ha estat associat a cap votaci i/o no s'ha associat a cap votaci que ja hagi comenat o que
		  -- hagi finalitzat, es podr esborrar

		  DELETE FROM censosLocals WHERE nom_cens = nom_cens_l;


		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: El nom del cens es buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN cens_inexistent THEN

		   RSP := 'ERROR: no es pot eliminar un cens que no existeix.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN votacions_obert_final THEN

		   RSP := 'ERROR: no es pot eliminar un cens que tingui alguna votacio celebrant-se o finalitzada';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;










END; --fi implementaci paquet pk_territoris
/

