

-- Esborrat del paquet (especificaci i cos)
-- -----------------------------------------
-- DROP PACKAGE BODY pk_ciutadans;
-- DROP PACKAGE pk_ciutadans;



-- ******************************************************************************************************************************
-- ******************************************************************************************************************************
-- *																*
-- *						FASE D'ESPECIFICACI								*
-- *																*
-- ******************************************************************************************************************************
-- ******************************************************************************************************************************



-- ----------------------------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------------
-- Nom_paquet: 		pk_ciutadans
-- 
-- Descripci:		especificaci dels procediments d'ABM realcionats directament amb els ciutadans
-- 			
-- Procediments:	p_altaCiutada, p_baixaCiutada, p_modificaCiutada_dataNaix, p_modificaCiutada_censLocal
-- 
-- Funcions:		--
-- ----------------------------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE pk_ciutadans IS




	PROCEDURE p_altaCiutada (
	  n_aleatori		IN	ciutadans.num_aleatori%TYPE,
	  cens_local		IN	ciutadans.id_cens_local%TYPE,
	  id_local		IN	ciutadans.id_localitat%TYPE,
	  data_naix		IN	ciutadans.data_naixement%TYPE,
	  pais			IN	ciutadans.nom_pais%TYPE,
	  baixa			IN	ciutadans.baixa_logica%TYPE,
	  participacions	IN	ciutadans.num_participacions%TYPE,
	  RSP			OUT	VARCHAR2
	  );



	PROCEDURE p_baixaCiutada (
	  n_aleatori		IN	ciutadans.num_aleatori%TYPE,
	  RSP			OUT	VARCHAR2
	  );


	PROCEDURE p_modificaCiutada_dataNaix (
	  n_aleatori		IN	ciutadans.num_aleatori%TYPE,
	  nova_data_naix	IN	ciutadans.data_naixement%TYPE,
	  RSP			OUT	VARCHAR2
	  );


	PROCEDURE p_modificaCiutada_censLocal (
	  n_aleatori		IN	ciutadans.num_aleatori%TYPE,
	  nou_cens_local	IN	censosLocals.nom_cens%TYPE,
	  RSP			OUT	VARCHAR2
	  );


END; --fi especificaci paquet pk_ciutadans
/










-- ******************************************************************************************************************************
-- ******************************************************************************************************************************
-- *																*
-- *						FASE D'IMPLEMENTACI								*
-- *																*
-- ******************************************************************************************************************************
-- ******************************************************************************************************************************


-- ----------------------------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------------
-- Nom_paquet: 		pk_ciutadans
-- 
-- Descripci:		implementaci dels procediments d'ABM realcionats directament amb els ciutadans
-- 			
-- Procediments:	p_altaCiutada, p_baixaCiutada, p_modificaCiutada_dataNaix, p_modificaCiutada_censLocal
-- 
-- Funcions:		--
-- ----------------------------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE BODY pk_ciutadans IS





	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaCiutada
	-- 			
	-- Descripci:		insereix un registre a la taula ciutadans i incrementa en una unitat el camp ciutadans_censats del cens local
	-- 			al qual est assignat el ciutad. Finalment, crida el procediment p_registrarLog del paquet pk_utils.
	-- 			Es copmprova la coherncia de les dades del ciutad.
	-- 
	-- 			
	-- Parmetres entrada:	n_aleatori	NUMBER(9)			PK
	-- 			cens_local	VARCHAR2(40 CHAR)		NOT NULL	REFERENCES censosLocals(nom_cens)
	-- 			id_local	NUMBER				NOT NULL	REFERENCES localitats(id)
	-- 			data_naix	DATE				NOT NULL
	-- 			pais		VARCHAR2(30 CHAR)		NOT NULL	REFERENCES paisos(nom_pais)
	-- 			baixa		NUMBER(1)			NOT NULL	CHECK(baixa IN(0,1))
	-- 			participacions	NUMBER		DEFAULT 0	NOT NULL	CHECK(participacions >= 0)
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada que no pot ser buit o NULL ho s
	-- 			valor_incorrecte	-> quan algun dels parmetres id_local, baixa i/o participacions t un valor inesperat
	-- 			menor_edat		-> quan el ciutad no ha complert 18 anys en el moment de registrar-se al sistema.
	-- 			cens_inexistent		-> quan cens_local no referencia cap cens donal d'alta al sistema
	-- 			localitat_inexistent	-> quan id_local no referencia cap localitat donada d'alta al sistema
	-- 			pais_inexistent		-> quan 'pais' no referencia cap pas donat d'alta al sistema.
	-- 			dades_incoherents	-> quan els parmetres cens_local i id_local no sn coherents entre si.
	-- 			pais_incoherent		-> quan pais no s coherent amb el pas al qual correspon el cens local.
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaCiutada (
	  n_aleatori		IN	ciutadans.num_aleatori%TYPE,
	  cens_local		IN	ciutadans.id_cens_local%TYPE,
	  id_local		IN	ciutadans.id_localitat%TYPE,
	  data_naix		IN	ciutadans.data_naixement%TYPE,
	  pais			IN	ciutadans.nom_pais%TYPE,
	  baixa			IN	ciutadans.baixa_logica%TYPE,
	  participacions	IN	ciutadans.num_participacions%TYPE,
	  RSP			OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_altaCiutada';
	  	parametres_entrada 	VARCHAR2(150 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		auxiliar_DATE		DATE;
		data_actuaL_DATE	DATE;
		data_actual_CHAR	VARCHAR2(10 CHAR);
		data_naix_mes_18	VARCHAR2(10 CHAR);
		controla_zero		VARCHAR2(2 CHAR);
		censats			NUMBER(10) := 0;
		localitat		NUMBER(10) :=0;
		cens_regional		VARCHAR2(40 CHAR);
		cens_estatal		VARCHAR2(30 CHAR);
		cod_pais		CHAR(2);
		pais_cens_local		VARCHAR2(30 CHAR);
	  	camp_obligatori 	EXCEPTION;
		valor_incorrecte 	EXCEPTION;
		menor_edat		EXCEPTION;
		cens_inexistent		EXCEPTION;
		localitat_inexistent	EXCEPTION;
		pais_inexistent		EXCEPTION;
		dades_incoherents	EXCEPTION;
		pais_incoherent		EXCEPTION;


		BEGIN

		  parametres_entrada := n_aleatori||', '||cens_local||', '||id_local||', '||data_naix||', '||pais||', '||baixa||', '||participacions;

		  IF (  n_aleatori IS NULL OR 
			cens_local IS NULL OR
			id_local IS NULL OR
			data_naix IS NULL OR
			pais IS NULL OR
			baixa IS NULL OR
			participacions IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (	id_local <= 0 OR
			baixa <> 0 OR
			participacions <> 0) THEN
				RAISE valor_incorrecte;
		  END IF;



		  -- es comprova que la nova data de naixement correspongui a un ciutad major d'edat (> 18 anys). El sistema noms
		  -- permet emmagatzemar ciutadans amb dret a vot a la data actual (i.e. majors d'edat respecte la data actual del sistema).
		  SELECT SYSDATE INTO data_actual_DATE FROM DUAL;
		  data_naix_mes_18 := TO_CHAR(data_naix, 'ddmmyyyy')+18;
		  controla_zero := TO_CHAR(data_naix, 'DD');
		  IF (controla_zero < 10) THEN

			data_naix_mes_18 := '0'||data_naix_mes_18;

		  END IF;

		  auxiliar_DATE := TO_DATE(data_naix_mes_18);
		  data_naix_mes_18 := TO_CHAR(auxiliar_DATE, 'yyyymmdd');
		  data_actual_CHAR := TO_CHAR(data_actual_DATE, 'yyyymmdd');


		  IF ( data_naix_mes_18 > data_actual_CHAR ) THEN
			RAISE menor_edat;
		  END IF;



		  -- es comprova que cens_local referenci a un cens local donat d'alta al sistema
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM censosLocals WHERE nom_cens = cens_local;
		  IF ( trobat = 0 ) THEN
			RAISE cens_inexistent;
		  END IF;

		  -- es comprova que id_local referenci a una localitat donada d'alta al sistema
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM localitats WHERE id = id_local;
		  IF ( trobat = 0 ) THEN
			RAISE localitat_inexistent;
		  END IF;


		  -- es comprova que pais referenci un pas donat d'alta al sistema
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM paisos WHERE nom_pais = pais;
		  IF ( trobat = 0 ) THEN
			RAISE pais_inexistent;
		  END IF;


		  -- es comprova que el cens al qual es vol donar d'alta al ciutad estigui associat a la localitat
		  -- a la qual es vol associar el ciutad, s a dir, que cens_local i id_local siguin mtuament coherents
		  SELECT id_localitat INTO localitat FROM censosLocals WHERE nom_cens = cens_local;
		  IF ( localitat <> id_local ) THEN
			RAISE dades_incoherents;
		  END IF;

		  
		  -- es comprova que el nom del pais del ciutad que es dna d'alta sigui coherent amb el pas del cens local del ciutad
		  SELECT id_cens_regional INTO cens_regional FROM censosLocals WHERE nom_cens = cens_local;
		  SELECT id_cens_estatal INTO cens_estatal FROM censosRegionals WHERE nom_cens = cens_regional;
		  SELECT codi_pais INTO cod_pais FROM censosEstatals WHERE nom_cens = cens_estatal;
		  SELECT nom_pais INTO pais_cens_local FROM paisos WHERE codi_pais = cod_pais;
		  IF ( pais <> pais_cens_local ) THEN
			RAISE pais_incoherent;
		  END IF;
		  

		  -- s'insereix el ciutad
		  INSERT INTO ciutadans(num_aleatori, id_cens_local, id_localitat, data_naixement, nom_pais, baixa_logica, num_participacions)
		  VALUES(n_aleatori, cens_local, id_local, data_naix, pais, baixa, participacions);

		  -- i s'actualitza el nmero de ciutadans censats del cens local corresponent
		  SELECT ciutadans_censats INTO censats FROM censosLocals WHERE id_localitat = id_local;
		  UPDATE censosLocals SET ciutadans_censats = censats+1 WHERE id_localitat = id_local;

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN valor_incorrecte THEN

		   RSP := 'ERROR: Revisi el valor dels camps segents. Han de complir: id_localitat > 0, baixa_logica = 0 i participacions = 0 ';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN menor_edat THEN

		   RSP := 'ERROR: el sistema no enregistra ciutadans menors de 18 anys respecte la data actual.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN cens_inexistent THEN

		   RSP := 'ERROR: l''identificador del cens no correspon a cap cens local del sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN localitat_inexistent THEN

		   RSP := 'ERROR: l''identificador de la localitat no correspon a cap localitat del sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN pais_inexistent THEN

		   RSP := 'ERROR: l''identificador del pais no correspon a cap pais del sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN dades_incoherents THEN

		   RSP := 'ERROR: el cens al qual es vol assignar el ciutada no es de la localitat del mateix.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN pais_incoherent THEN

		   RSP := 'ERROR: el nom del pais no es coherent amb el pais al que correspon el cens local.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;















	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_baixaCiutada
	-- 			
	-- Descripci:		el procediment dna de baixa lgica un ciutad, decrementa en una unitat el nmero de ciutadans censats del cens
	-- 			local al qual corresponia el ciutad i crida el procediment p_registrarLog del paquet pk_utils.
	-- 			
	-- 			
	-- Parmetres entrada:	n_aleatori	NUMBER(9)	PK
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan n_aleatori s buit o NULL
	-- 			valor_incorrecte	-> quan n_aleatori s <= 0 o quan t ms de 9 dgits de llargada
	-- 			ciutada_inexistent	-> quan n_aleatori no referencia cap ciutada de la BD
	-- 			ciutada_baixa		-> quan n_aleatori referencia un ciutad que ja estava donat de baixa
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_baixaCiutada (
	  n_aleatori		IN	ciutadans.num_aleatori%TYPE,
	  RSP			OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_baixaCiutada';
	  	parametres_entrada 	VARCHAR2(30 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		baixa			NUMBER(1) := 0;
		id_local		NUMBER := 0;
		censats			NUMBER := 0;
	  	camp_obligatori 	EXCEPTION;
		valor_incorrecte 	EXCEPTION;
		ciutada_inexistent	EXCEPTION;
		ciutada_baixa		EXCEPTION;

		BEGIN

		  parametres_entrada := n_aleatori;

		  IF (  n_aleatori IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (	n_aleatori <= 0 OR
			length(n_aleatori) > 9 ) THEN
			RAISE valor_incorrecte;
		  END IF;


		  -- es comprova que el ciutad existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM ciutadans WHERE num_aleatori = n_aleatori;
		  IF ( trobat = 0 ) THEN
			RAISE ciutada_inexistent;
		  END IF;

		  -- es comprova que el ciutad no estigus ja donat de baixa
		  baixa := 0;
		  SELECT baixa_logica INTO baixa FROM ciutadans WHERE num_aleatori = n_aleatori;
		  IF ( baixa = 1 ) THEN
			RAISE ciutada_baixa;
		  END IF;		  

		  -- es dna de baixa lgica el ciutad
		  UPDATE ciutadans SET baixa_logica = 1 WHERE num_aleatori = n_aleatori;

		  -- obtenim l'identificador de la localiat del ciutad
		  SELECT id_localitat INTO id_local FROM ciutadans WHERE num_aleatori = n_aleatori;

		  -- i s'actualitza el nmero de ciutadans censats del cens local corresponent (es decrementa en una unitat)
		  SELECT ciutadans_censats INTO censats FROM censosLocals WHERE id_localitat = id_local;
		  UPDATE censosLocals SET ciutadans_censats = censats-1 WHERE id_localitat = id_local;

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: el numero que ha d''identificar el ciutada es buit o NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN valor_incorrecte THEN

		   RSP := 'ERROR: el numero que ha d''identificar el ciutada ha de ser un nombre natural diferent de zero (i ha de tenir menys de 10 digits decimals) ';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN ciutada_inexistent THEN

		   RSP := 'ERROR: el numero no referencia cap ciutada de la BD.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN ciutada_baixa THEN

		   RSP := 'ERROR: el ciutada ja estava donat de baixa.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;








	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_modificaCiutada_dataNaix
	-- 			
	-- Descripci:		modifica la data de naixement d'un ciutada i crida el procediment p_registrarLog del paquet pk_utils.
	-- 			
	-- 			
	-- Parmetres entrada:	n_aleatori	NUMBER(9)	PK
	-- 			nova_data_naix	DATE
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan n_aleatori i/o  nova_data_naix sn buits o NULL
	-- 			valor_incorrecte	-> quan n_aleatori s <= 0 o quan t ms de 9 dgits de llargada
	-- 			ciutada_inexistent	-> quan n_aleatori no referencia cap ciutada de la BD
	-- 			ciutada_baixa		-> quan n_aleatori referencia un ciutad que ja estava donat de baixa
	-- 			menor_edat		-> quan la nova data de naixement correspondria a un menor d'edat respecte la data actual
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_modificaCiutada_dataNaix (
	  n_aleatori		IN	ciutadans.num_aleatori%TYPE,
	  nova_data_naix	IN	ciutadans.data_naixement%TYPE,
	  RSP			OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_modificaCiutada_dataNaix';
	  	parametres_entrada 	VARCHAR2(30 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		baixa			NUMBER(1) := 0;
		auxiliar_DATE		DATE;
		data_actuaL_DATE	DATE;
		data_actual_CHAR	VARCHAR2(10 CHAR);
		data_naix_mes_18	VARCHAR2(10 CHAR);
		controla_zero		VARCHAR2(2 CHAR);
	  	camp_obligatori 	EXCEPTION;
		valor_incorrecte 	EXCEPTION;
		ciutada_inexistent	EXCEPTION;
		ciutada_baixa		EXCEPTION;
		menor_edat		EXCEPTION;



		BEGIN

		  parametres_entrada := n_aleatori||', '||nova_data_naix;

		  IF (  n_aleatori IS NULL OR
			nova_data_naix IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (	n_aleatori <= 0 OR
			length(n_aleatori) > 9 ) THEN
			RAISE valor_incorrecte;
		  END IF;


		  -- es comprova que el ciutad existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM ciutadans WHERE num_aleatori = n_aleatori;
		  IF ( trobat = 0 ) THEN
			RAISE ciutada_inexistent;
		  END IF;

		  -- es comprova que el ciutad no estigus ja donat de baixa
		  baixa := 0;
		  SELECT baixa_logica INTO baixa FROM ciutadans WHERE num_aleatori = n_aleatori;
		  IF ( baixa = 1 ) THEN
			RAISE ciutada_baixa;
		  END IF;		  

		  -- es comprova que la nova data de naixement correspongui a un ciutad major d'edat (> 18 anys). El sistema noms
		  -- permet emmagatzemar ciutadans amb dret a vot a la data actual.
		  SELECT SYSDATE INTO data_actual_DATE FROM DUAL;
		  data_naix_mes_18 := TO_CHAR(nova_data_naix, 'ddmmyyyy')+18;
		  controla_zero := TO_CHAR(nova_data_naix, 'DD');
		  IF (controla_zero < 10) THEN

			data_naix_mes_18 := '0'||data_naix_mes_18;

		  END IF;

		  auxiliar_DATE := TO_DATE(data_naix_mes_18);
		  data_naix_mes_18 := TO_CHAR(auxiliar_DATE, 'yyyymmdd');
		  data_actual_CHAR := TO_CHAR(data_actual_DATE, 'yyyymmdd');


		  IF ( data_naix_mes_18 > data_actual_CHAR ) THEN
			RAISE menor_edat;
		  END IF;


		  -- fetes les verificacions prvies, es pot canviar la data de naixement del ciutad
		  UPDATE ciutadans SET data_naixement = nova_data_naix WHERE num_aleatori = n_aleatori;


		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: el numero que ha d''identificar el ciutada es buit o NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN valor_incorrecte THEN

		   RSP := 'ERROR: el numero que ha d''identificar el ciutada ha de ser un nombre natural diferent de zero (i ha de tenir menys de 10 digits decimals) ';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN ciutada_inexistent THEN

		   RSP := 'ERROR: no es pot canviar la data de naixement d''un ciutada que no existeix a la BD.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN ciutada_baixa THEN

		   RSP := 'ERROR: no es pot canviar la data de naixement d''un ciutada que ha estat donat de baixa.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN menor_edat THEN

		   RSP := 'ERROR: el sistema no permet ciutadans menors de 18 anys respecte la data actual. Revisi la nova data de naixement.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;









	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_modificaCiutada_censLocal
	-- 			
	-- Descripci:		sota determinades condicions, modifica el cens local d'un ciutad, actualitza la localitat i pas del ciutad 
	-- 			de manera que hi hagi coherncia amb el nou cens local i actualitza el nmero de ciutadans censats a l'antic
	-- 			i al nou cens local del ciutad. Com sempre, s'acaba cridant el procediment p_registrarLog del paquet pk_utils.
	-- 
	-- 			NOTA: no es permetr modificar el cens d'un ciutad si l'antic i/o el nou cens estan associats a alguna de les
	-- 			votacions no finalitzades (actives) que hi pugui haver al sistema. Aix es fa perqu no hi pugui haver cap dada
	-- 			incoherent (tot i que s'haurien de canviar els censos d'uns quants ciutadans perqu passs) a l'hora de calcular
	-- 			el percentatge de participaci d'una determinada votaci, ja que el nmero d'electors cridats a les urnes en una
	-- 			votaci no s dinmic, sin que s'actualitza noms a l'hora d'associar censos a les votacions. S'ha controlat
	-- 			aix malgrat que el client havia confirmat que no calia fer-ho.
	-- 			
	-- Parmetres entrada:	n_aleatori	NUMBER(9)		PK
	-- 			nou_cens_local	VARCHAR2(40 CHAR)	REFERENCES censosLocals(nom_cens) -- ha de referenciar-ne un, baja --
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori			-> quan n_aleatori s buit o NULL
	-- 			camp_incorrecte			-> quan nou_cens_local s massa llarg
	-- 			valor_incorrecte		-> quan n_aleatori s <= 0 o quan t ms de 9 dgits de llargada
	-- 			ciutada_inexistent		-> quan n_aleatori no referencia cap ciutada de la BD
	-- 			ciutada_baixa			-> quan n_aleatori referencia un ciutad que ja estava donat de baixa
	-- 			nou_cens_inexistent		-> quan nou_cens_local no referencia cap dels censos locals de la BD
	-- 			censos_a_votacions_actives	-> quan l'antic i/o el nou cens del ciutad estan associats a alguna votaci
	-- 							   no finalitzada.
	--			OTHERS				-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_modificaCiutada_censLocal (
	  n_aleatori		IN	ciutadans.num_aleatori%TYPE,
	  nou_cens_local	IN	censosLocals.nom_cens%TYPE,
	  RSP			OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_modificaCiutada_censLocal';
	  	parametres_entrada 	VARCHAR2(80 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		baixa			NUMBER(1) := 0;
		num_votacions_actives	NUMBER := 0;
		estat_final		NUMBER := 0;
		antic_cens_local	VARCHAR2(40 CHAR);
		id_local		NUMBER := 0;
		num_censats		NUMBER := 0;
		id_nova_localitat	NUMBER := 0;
		id_nova_regio		NUMBER := 0;
		id_nou_pais		VARCHAR2(30 CHAR);
	  	camp_obligatori 		EXCEPTION;
		camp_incorrecte			EXCEPTION;
		valor_incorrecte 		EXCEPTION;
		ciutada_inexistent		EXCEPTION;
		ciutada_baixa			EXCEPTION;
		nou_cens_inexistent		EXCEPTION;
		censos_a_votacions_actives	EXCEPTION;

		BEGIN

		  parametres_entrada := n_aleatori||', '||nou_cens_local;

		  IF (  n_aleatori IS NULL OR 
			nou_cens_local IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF ( length(nou_cens_local) > 40 ) THEN
			RAISE camp_incorrecte;
		  END IF;

		  IF (	n_aleatori <= 0 OR
			length(n_aleatori) > 9 ) THEN
				RAISE valor_incorrecte;
		  END IF;


		  -- es comprova que el ciutad existeixi a la BD
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM ciutadans WHERE num_aleatori = n_aleatori;
		  IF ( trobat = 0 ) THEN
			RAISE ciutada_inexistent;
		  END IF;

		  -- es comprova que el ciutad no estigui donat de baixa
		  baixa := 0;
		  SELECT baixa_logica INTO baixa FROM ciutadans WHERE num_aleatori = n_aleatori;
		  IF ( baixa = 1 ) THEN
			RAISE ciutada_baixa;
		  END IF;


		  -- es comprova que el nou cens local existeixi a la BD
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM censosLocals WHERE nom_cens = nou_cens_local;
		  IF ( trobat = 0 ) THEN
			RAISE nou_cens_inexistent;
		  END IF;

		  -- obtenim l'antic cens local del ciutad (ens far falta en ms d'un punt del procediment)
		  SELECT id_cens_local INTO antic_cens_local FROM ciutadans WHERE num_aleatori = n_aleatori;

		  -- es comprova que l'antic cens local del ciutad i/o el nou cens local no estiguin associats a alguna de les votacions
		  -- no finalitzades (actives). Si algun cens hi est associat, no es permetr el canvi de cens fins que no finalitzin les votacions.
		  trobat := 0;
		  SELECT id INTO estat_final FROM estatsVotacio WHERE nom_estat = 'finalitzada';
		  SELECT COUNT(*) INTO num_votacions_actives FROM votacions WHERE id_estat <> estat_final;
		  IF (num_votacions_actives = 0) THEN
			trobat := 0;
		  ELSE

			-- s'obtenen les votacions no finalitzades de la taula votacions i, per cadascuna, en un segon cursor
			-- es recorreran els censos locals que tinguin associats. Si algun d'ells coincideix amb l'antic
			-- o el nou cens local del ciutad, es posar trobat a 1 i no es permetr modificar el cens del ciutad.
			FOR votacio_activa IN (SELECT id FROM votacions WHERE id_estat <> estat_final) LOOP

				FOR cens_associat IN (SELECT id_cens_local FROM censosAssociats WHERE id_votacio = votacio_activa.id) LOOP

					trobat := trobat;
					IF ( trobat = 0 ) THEN

						IF ( cens_associat.id_cens_local = antic_cens_local OR cens_associat.id_cens_local = nou_cens_local ) THEN

							trobat := 1;

						END IF;
					END IF;

				END LOOP;

			END LOOP;

		  END IF;



		  IF ( trobat = 1 ) THEN
			RAISE censos_a_votacions_actives;
		  END IF;


		  -- superades les verificacions anteriors, es pot canviar el cens local del ciutad. Al mateix temps, es canviar (a la taula ciutadans) 
		  -- la localitat i el pas associats al nou cens del ciutad (pot ser que el pas sigui el mateix que l'anterior, o que no ho sigui).
		  SELECT id_localitat INTO id_nova_localitat FROM censosLocals WHERE nom_cens = nou_cens_local;
		  SELECT id_regio INTO id_nova_regio FROM localitats WHERE id = id_nova_localitat;
		  SELECT id_pais INTO id_nou_pais FROM regions WHERE id = id_nova_regio;
		  UPDATE ciutadans SET id_cens_local = nou_cens_local, id_localitat = id_nova_localitat, nom_pais = id_nou_pais 
			WHERE num_aleatori = n_aleatori;

		  -- Per altre costat, cal actualitzar el nmero de ciutadans censats de l'antic i del nou cens.
		  SELECT ciutadans_censats INTO num_censats FROM censosLocals WHERE nom_cens = antic_cens_local;
		  UPDATE censosLocals SET ciutadans_censats = (num_censats - 1) WHERE nom_cens = antic_cens_local;

		  SELECT ciutadans_censats INTO num_censats FROM censosLocals WHERE nom_cens = nou_cens_local;
		  UPDATE censosLocals SET ciutadans_censats = (num_censats + 1) WHERE nom_cens = nou_cens_local;


		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: el numero que ha d''identificar el ciutada i/o el nom del nou cens son buits o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN camp_incorrecte THEN

		   RSP := 'ERROR: el nom del nou cens local es massa llarg (maxim 40 caracters) ';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN valor_incorrecte THEN

		   RSP := 'ERROR: el numero que ha d''identificar el ciutada ha de ser un nombre natural diferent de zero (i ha de tenir menys de 10 digits decimals) ';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN ciutada_inexistent THEN

		   RSP := 'ERROR: no es pot canviar el cens d''un ciutada que no existeix a la BD. Revisi l''identificador del ciutada.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN ciutada_baixa THEN

		   RSP := 'ERROR: no es pot canviar de cens un ciutada que esta donat de baixa. Revisi l''identificador del ciutada.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN nou_cens_inexistent THEN

		   RSP := 'ERROR: intent d''associar un ciutada a un cens inexistent. Revisi el nom del nou cens.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN censos_a_votacions_actives THEN

		   RSP := 'ERROR: no es pot modificar el cens d''un ciutada mentre l''antic i/o el nou cens pertanyin a alguna votacio no finalitzada.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;







END; --fi implementaci paquet pk_ciutadans
/

