

-- Esborrat del paquet (especificaci i cos)
-- -----------------------------------------
-- DROP PACKAGE BODY pk_estadistiques;
-- DROP PACKAGE pk_estadistiques;

-- ******************************************************************************************************************************
-- ******************************************************************************************************************************
-- *																*
-- *						FASE D'ESPECIFICACI								*
-- *																*
-- ******************************************************************************************************************************
-- ******************************************************************************************************************************




-- ----------------------------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------------
-- Nom_paquet: 		pk_estadistiques
-- 
-- Descripci:		cont el procediment que actualitza totes les estadstiques que faci falta d'una votaci
-- 
-- Procediments:	p_estadistiquesVotacio
-- Funcions:		--
-- ----------------------------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE pk_estadistiques IS



	PROCEDURE p_estadistiquesVotacio (
	  id_vota	IN	votacions.id%TYPE,
	  RSP		OUT	VARCHAR2
	  );



END; --fi especificaci paquet pk_estadistiques
/










-- ******************************************************************************************************************************
-- ******************************************************************************************************************************
-- *																*
-- *						FASE D'IMPLEMENTACI								*
-- *																*
-- ******************************************************************************************************************************
-- ******************************************************************************************************************************




-- ----------------------------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------------
-- Nom_paquet: 		pk_estadistiques
-- 
-- Descripci:		cont el procediment que actualitza totes les estadstiques que faci falta d'una votaci
-- 
-- Procediments:	p_estadistiquesVotacio
-- Funcions:		--
-- ----------------------------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE BODY pk_estadistiques IS






	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_estadistiquesVotacio
	-- 			
	-- Descripci:		actualitza tots els camps i registres que calgui de totes les taules que formen el mdul estadstic. Finalment, 
	-- 			com sempre, es crida el procediment p_registrarLog del paquet pk_utils.
	-- 			
	-- Parmetres entrada:	id_vota		NUMBER
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori			-> quan id_vot s buit o NULL
	-- 			camp_incorrecte			-> quan id_vota no s un natural diferent de zero
	-- 			votacio_inexistent		-> quan id_vota no referencia cap votaci del sistema
	-- 			no_finalitzada			-> quan id_vota referencia una votaci que encara no ha finalitzat
	-- 			estadistiques_calculades	-> quan id_vota referncia una votaci de la qual ja se n'han calculat les estadstiques
	--			OTHERS				-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_estadistiquesVotacio (
	  id_vota	IN	votacions.id%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_estadistiquesVotacio';
	  	parametres_entrada 	VARCHAR2(10 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;

   
-- | per guardar dades de la votaci que es tracta
-- v ---------------------------------------------
		titol_votacio		VARCHAR2(100 CHAR) := NULL;
		estat_votacio		VARCHAR2(20 CHAR) := NULL;
		any_vota		NUMBER(4) := 0;
		num_electors_votacio	NUMBER := 0;

-- | per clculs intermitjos que es faran servir en diverses estadstiques
-- v ---------------------------------------------------------------------
		trobat			NUMBER(1) := 0;
		num			NUMBER := 0;
		t_vots			NUMBER := 0;
		t_electors		NUMBER := 0;
		nova_participacio	NUMBER(5,2) := 0;
		percent			NUMBER(5,2) := 0;
		percent_max		NUMBER(5,2) := 0;
		percent_min		NUMBER(5,2) := 0;
		max_vots		NUMBER := 0;



-- | per les estadstiques de censosLocalsAnys, censosRegionalsAnys, censosEstatalsAnys, censGlobalalAnys, 
-- v -----------------------------------------------------------------------------------------------------
		cens_global		VARCHAR2(30 CHAR) := NULL;
		trobat_cens_local	NUMBER(1) := 0;
		trobat_cens_regional	NUMBER(1) := 0;
		trobat_cens_estatal	NUMBER(1) := 0;
		trobat_cens_global	NUMBER(1) := 0;
		n_vots_cens		NUMBER := 0;
		n_electors_cens		NUMBER := 0;
		n_votacions_cens	NUMBER := 0;


-- | per les estadstiques de la taula  estadistiquesAnuals
-- v ------------------------------------------------------
		t_electors_han_votat	NUMBER := 0;
		t_electors_no_han_votat	NUMBER := 0;
		perc_no_han_votat	NUMBER(5,2) := 0;
		ciutadans_mai		NUMBER(5,2) := 0;
		antiga_participacio	NUMBER(5,2) := 0;
		ant_localitats_max_vots	VARCHAR2(360 CHAR) := NULL;
		localitats_max_vots	VARCHAR2(480 CHAR) := NULL;	-- guardar les localitats de la votaci amb un nombre mxim de vots (s d'esperar 
									-- que n'hi hagi una, per es declara de 480 CHAR per si de cas. Amb aquesta longitud
									-- hi podrem guardar un mnim de 12 noms de localitat). Una cosa semblant per
									-- ant_localitats_max_vots que servir en el cas que hi hagi coincidncia de mxim
									-- de vots entre localitats d'una votaci anterior a aquesta i d'aquesta. En cas
									-- que passs (poc probable) es concatenaran els noms de les localitats.

		votacions_mes_part	VARCHAR2(300 CHAR) := NULL;	-- guardar el ttol de les votacions d'un any en les qual hi hagi hagut ms participaci.
									-- tpicament ser una sola votaci (100 CHAR seria suficient), per pot passar que ms
									-- d'una votaci tinguin una mateixa participaci mxima (amb 300 CHAR hi podem guardar
									-- el ttol d'un mnim de 3 votacions)

-- | per les estadstiques de la taula estadistiquesVotacions
-- v --------------------------------------------------------
		id_opc			NUMBER := 0;
		id_opc_CHAR		VARCHAR2(20 CHAR) := NULL;
		nom_opc			VARCHAR2(150 CHAR) := NULL;	-- el nom d'una opci pot ocupar un mxim de 15 CHAR. nom_opc guardar el nom de les 
		nom			VARCHAR2(15 CHAR);		-- opcions amb un mxim i mnim de vots i, com que es pot donar el cas que hi hagi
		min_vots		NUMBER := 0;			-- empat entre dues o ms opcions, es reserven 150 CHAR. Aix, es podrien guardar
									-- els noms de 10 (o ms) opcions que empatessin a vots (mnims o mxims).

-- | per les estadstiques de la taula historicVotacions
-- v ----------------------------------------------------
		v_mes_part		VARCHAR2(300 CHAR) := NULL;	-- v_mes_part, v_max_dif, v_min_dif guardaran el ttol/s de les votacions de la taula 
		v_max_dif		VARCHAR2(300 CHAR) := NULL;	-- historicVotacions. Com que es podria donar que hi hagus ms d'una votaci en qu
		v_min_dif		VARCHAR2(300 CHAR) := NULL;	-- coincids el valor, s'han reservat 300 CHAR per guardar-hi un mnim de 3 ttols
									-- de votaci.

-- | per les estadstiques de la taula maxParticipacionsCiutada
-- v ----------------------------------------------------------
		id_ciutada		NUMBER := 0;
		n_pais			VARCHAR2(30 CHAR) := NULL;
		num_part		NUMBER := 0;
		n_files			NUMBER := 0;


-- | Excepcions controlades
-- v -----------------------
	  	camp_obligatori 		EXCEPTION;
		camp_incorrecte 		EXCEPTION;
		votacio_inexistent		EXCEPTION;
		no_finalitzada			EXCEPTION;
		estadistiques_calculades	EXCEPTION;



		BEGIN

		  parametres_entrada := id_vota;

		  IF (  id_vota IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (	id_vota <= 0 ) THEN
			RAISE camp_incorrecte;
		  END IF;


		  -- comprovem que l'identificador de votaci referenci una votaci existent
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM votacions WHERE id = id_vota;
		  IF ( trobat = 0 ) THEN
			RAISE votacio_inexistent;
		  END IF;

		  -- comprovem que la votaci estigui en estat 'finalitzada'
		  SELECT ev.nom_estat INTO estat_votacio
			FROM estatsVotacio ev, votacions v
			WHERE ev.id = v.id_estat AND v.id = id_vota;
		  IF ( estat_votacio <> 'finalitzada' ) THEN
			RAISE no_finalitzada;
		  END IF;

		  -- comprovem si ja s'han calculat les estadistiques de la votacio. Si la votaci id_vota existeix, tamb existir una entrada
		  -- a la taula estaditiquesVotacions per la votaci id_vota. En actualitzar les estadstiques t_electors > 0 (suposem que sempre
		  -- hi haur hagut com a mnim un elector) i ens servir per controlar que no es tornin a calcular les estadstiques. Una altra
		  -- opci seria jugar amb l'atribut 'finalitzada' per aquest atribut s'actualitza en un altre procediment i millor fer-ho aix.
		  SELECT  num_electors INTO t_electors FROM estadistiquesVotacions WHERE id_votacio = id_vota;
		  IF ( t_electors > 0 ) THEN
			RAISE estadistiques_calculades;
		  END IF;


		  -- obtenim dades de la votacio
		  SELECT titol, any_votacio, num_electors INTO titol_votacio, any_vota, num_electors_votacio 
			FROM votacions WHERE id = id_vota;



-- inici actualitzacions taula participacionsPaisosVotacions

		  -- recorrem un cursor no declarat que tindr el nom dels pasos que han participat a la votaci i, 
		  -- tenint en compte l'any de la votaci id_vota:
		  -- 1) els pasos que encara no siguin a la taula participacionsPaisosVotacions els afegirem a la taula 
		  -- 	posant el camp num_participacions a 1
		  -- 2) els pasos que ja siguin a la taula els incrementarem en 1 el camp num_participacions
		  FOR pais IN (SELECT DISTINCT nom_pais FROM censosAssociats WHERE id_votacio = id_vota) LOOP

			trobat := 0;
			SELECT COUNT(*) INTO trobat FROM participacionsPaisosVotacions WHERE lany = any_vota AND id_pais = pais.nom_pais;
			IF ( trobat = 0) THEN
				INSERT INTO participacionsPaisosVotacions(id_pais, lany, num_participacions) VALUES (pais.nom_pais, any_vota, 1);
			ELSIF ( trobat = 1 ) THEN
				num := 0;
				SELECT num_participacions INTO num FROM participacionsPaisosVotacions WHERE lany = any_vota AND id_pais = pais.nom_pais;
				UPDATE participacionsPaisosVotacions SET num_participacions = (num + 1) WHERE lany = any_vota AND id_pais = pais.nom_pais;
			END IF;

		  END LOOP;

-- fi actualitzacions taula participacionsPaisosVotacions




-- inici actualitzacions taules censosLocalsAnys, censosRegionalsAnys, censosEstatalsAnys, censGlobalAnys


		  -- recorrem un cursor no declarat que tindr les dades necessries per registrar i/o actualitzar dades de les taules 
		  -- censosLocalsAnys, censosRegionalsAnys, censosEstatalsAnys i censGlobalAnys.
		  FOR censAssociat IN (SELECT id_cens_local, cens_estatal, cens_regional, nom_localitat, ciutadans_censats, num_vots
					FROM censosAssociats WHERE id_votacio = id_vota) LOOP


			-- obtenim l'identificador de cens global (que no s a les dades del cursor i ens far falta a direrents parts)
			SELECT id_cens_global INTO cens_global FROM censosEstatals WHERE nom_cens = censAssociat.cens_estatal;

			n_vots_cens := 0;
			t_vots := 0;
			n_electors_cens := 0;
			t_electors := 0;
			nova_participacio := 0;
			n_votacions_cens := 0;
			

			-- comprovem si el cens local associat ja s a la taula censosLocalsAnys (hi ser si el cens ha participat en alguna altra
			-- votaci en el mateix any que la votacio que tractem). 
			trobat_cens_local := 0;
			SELECT COUNT(*) INTO trobat_cens_local FROM censosLocalsAnys WHERE lany = any_vota AND id_cens_local = censAssociat.id_cens_local;

			-- Si el cens local ja hi s, tamb hi seran, a les respectives taules, el cens regional/estatal/global al qual pertany
			-- i noms s'actualitzaran les dades de les taules corresponents
			IF ( trobat_cens_local = 1 ) THEN

				-- ----------------------- cens local -------------------------
				-- obtenci i reclcul de les dades acumulades en el cens local l'any de la votaci
				-- en el cas del cens local, podem incrementar en 1 el nmero de votacions en les que ha participat aquest any aqu
				SELECT total_electors, total_vots, num_votacions INTO n_electors_cens, n_vots_cens, n_votacions_cens FROM censosLocalsAnys 
					WHERE lany = any_vota AND id_cens_local = censAssociat.id_cens_local;

				t_vots := n_vots_cens + censAssociat.num_vots;
				t_electors := n_electors_cens + censAssociat.ciutadans_censats;
				nova_participacio := 100*t_vots/t_electors;

				-- actualitzaci de dades del cens local any votaci
				UPDATE censosLocalsAnys SET total_electors = t_electors, total_vots = t_vots, participacio = nova_participacio, 
					num_votacions = n_votacions_cens + 1
					WHERE lany = any_vota AND id_cens_local = censAssociat.id_cens_local;


				-- ----------------------- cens regional -----------------------
				SELECT total_electors, total_vots INTO n_electors_cens, n_vots_cens FROM censosRegionalsAnys 
					WHERE lany = any_vota AND id_cens_regional = censAssociat.cens_regional;

				t_vots := n_vots_cens + censAssociat.num_vots;
				t_electors := n_electors_cens + censAssociat.ciutadans_censats;
				nova_participacio := 100*t_vots/t_electors;

				UPDATE censosRegionalsAnys SET total_electors = t_electors, total_vots = t_vots, participacio = nova_participacio
					WHERE lany = any_vota AND id_cens_regional = censAssociat.cens_regional;


				-- ----------------------- cens estatal -----------------------
				SELECT total_electors, total_vots INTO n_electors_cens, n_vots_cens FROM censosEstatalsAnys 
					WHERE lany = any_vota AND id_cens_estatal = censAssociat.cens_estatal;

				t_vots := n_vots_cens + censAssociat.num_vots;
				t_electors := n_electors_cens + censAssociat.ciutadans_censats;
				nova_participacio := 100*t_vots/t_electors;

				UPDATE censosEstatalsAnys SET total_electors = t_electors, total_vots = t_vots, participacio = nova_participacio
					WHERE lany = any_vota AND id_cens_estatal = censAssociat.cens_estatal;

				-- ----------------------- cens global -----------------------
				SELECT total_electors, total_vots INTO n_electors_cens, n_vots_cens FROM censGlobalAnys 
					WHERE lany = any_vota AND id_cens_global = cens_global;

				t_vots := n_vots_cens + censAssociat.num_vots;
				t_electors := n_electors_cens + censAssociat.ciutadans_censats;
				nova_participacio := 100*t_vots/t_electors;

				UPDATE censGlobalAnys SET total_electors = t_electors, total_vots = t_vots, participacio = nova_participacio
					WHERE lany = any_vota AND id_cens_global = cens_global;



			-- si el cens local no s a la taula censosLocalsAnys (aix passa si s la primera votaci en qu participa el cens local a l'any 
			-- de la votaci que  tractem),  s'inserir el registre a la taula i llavors es comprovar si el censos regional/estatal/global 
			-- als qual pertany ja tenen algun registre per l'any de la votaci a les seves respectives taules.
			ELSIF ( trobat_cens_local = 0 ) THEN

				nova_participacio := 100*censAssociat.num_vots/censAssociat.ciutadans_censats;
				INSERT INTO censosLocalsAnys(lany, id_cens_local, total_electors, total_vots, participacio, num_votacions) 
				VALUES(any_vota, censAssociat.id_cens_local, censAssociat.ciutadans_censats, censAssociat.num_vots, nova_participacio, 1);


				-- comprovem si el cens regional al qual correspon el cens local ja s a la taula censosRegionalsAnys (hi ser si el
				-- cens local que estem tractant s la primera vegada que participa en una votaci l'any actual per un altre cens local
				-- del mateix cens regional ja ha participat a alguna altra votaci (o a aquesta mateixa) a l'any actual.
				-- Si hi s, tamb hi seran, a les respectives taules, els censos estatal/global i s'actualitzen dades de les taules
				-- corresponents.
				trobat_cens_regional := 0;
				SELECT COUNT(*) INTO trobat_cens_regional FROM censosRegionalsAnys 
					WHERE lany = any_vota AND id_cens_regional = censAssociat.cens_regional;

				IF ( trobat_cens_regional = 1 ) THEN

					-- ----------------------- cens regional -----------------------
					SELECT total_electors, total_vots INTO n_electors_cens, n_vots_cens FROM censosRegionalsAnys 
						WHERE lany = any_vota AND id_cens_regional = censAssociat.cens_regional;

					t_vots := n_vots_cens + censAssociat.num_vots;
					t_electors := n_electors_cens + censAssociat.ciutadans_censats;
					nova_participacio := 100*t_vots/t_electors;

					UPDATE censosRegionalsAnys SET total_electors = t_electors, total_vots = t_vots, participacio = nova_participacio
						WHERE lany = any_vota AND id_cens_regional = censAssociat.cens_regional;


					-- ----------------------- cens estatal -----------------------
					SELECT total_electors, total_vots INTO n_electors_cens, n_vots_cens FROM censosEstatalsAnys 
						WHERE lany = any_vota AND id_cens_estatal = censAssociat.cens_estatal;

					t_vots := n_vots_cens + censAssociat.num_vots;
					t_electors := n_electors_cens + censAssociat.ciutadans_censats;
					nova_participacio := 100*t_vots/t_electors;

					UPDATE censosEstatalsAnys SET total_electors = t_electors, total_vots = t_vots, participacio = nova_participacio
						WHERE lany = any_vota AND id_cens_estatal = censAssociat.cens_estatal;

					-- ----------------------- cens global -----------------------
					SELECT total_electors, total_vots INTO n_electors_cens, n_vots_cens FROM censGlobalAnys 
						WHERE lany = any_vota AND id_cens_global = cens_global;

					t_vots := n_vots_cens + censAssociat.num_vots;
					t_electors := n_electors_cens + censAssociat.ciutadans_censats;
					nova_participacio := 100*t_vots/t_electors;

					UPDATE censGlobalAnys SET total_electors = t_electors, total_vots = t_vots, participacio = nova_participacio
					WHERE lany = any_vota AND id_cens_global = cens_global;

				-- si el cens regional no s a la taula censosRegionalsAny (aix passa quan el cens local que tractem ni cap altre cens
				-- local de la regi a la qual pertany el primer no han participat a cap votaci aquest any; s a dir en tractar el
				-- primer cens local de cada regi a cada any), s'inserir un registre
				-- a aquesta taula i llavors es comprovar si el censos estatal/global als qual pertany ja tene algun registre
				-- per l'any de la votaci a les seves respectives taules.
				ELSIF ( trobat_cens_regional = 0 ) THEN


					nova_participacio := 100*censAssociat.num_vots/censAssociat.ciutadans_censats;
					INSERT INTO censosRegionalsAnys(lany, id_cens_regional, total_electors, total_vots, participacio, num_votacions) 
						VALUES(any_vota, censAssociat.cens_regional, censAssociat.ciutadans_censats, censAssociat.num_vots, 
							nova_participacio, 0);


					-- comprovem si el cens estatal al qual correspon el cens regional del cens local que tractem ja s a la taula 
					-- censosEstatalsAnys (hi ser si el cens local que estem tractant s la primera vegada que participa en una 
					-- votaci l'any actual per un altre cens local d'un cens regional diferent al del primer per del mateix pas 
					-- que aquest ja ha participat a alguna altra votaci (o a aquesta mateixa) a l'any actual. Si hi s, tamb hi 
					-- ser, a la taula censGlobalAnys, el cens global i s'actualitzen dades de les taules corresponents.
					trobat_cens_estatal := 0;
					SELECT COUNT(*) INTO trobat_cens_estatal FROM censosEstatalsAnys 
						WHERE lany = any_vota AND id_cens_estatal = censAssociat.cens_estatal;

					IF ( trobat_cens_estatal = 1 ) THEN

						-- ----------------------- cens estatal -----------------------
						SELECT total_electors, total_vots INTO n_electors_cens, n_vots_cens FROM censosEstatalsAnys 
							WHERE lany = any_vota AND id_cens_estatal = censAssociat.cens_estatal;

						t_vots := n_vots_cens + censAssociat.num_vots;
						t_electors := n_electors_cens + censAssociat.ciutadans_censats;
						nova_participacio := 100*t_vots/t_electors;

						UPDATE censosEstatalsAnys SET total_electors = t_electors, total_vots = t_vots, participacio = nova_participacio
							WHERE lany = any_vota AND id_cens_estatal = censAssociat.cens_estatal;

						-- ----------------------- cens global -----------------------
						SELECT total_electors, total_vots INTO n_electors_cens, n_vots_cens FROM censGlobalAnys 
							WHERE lany = any_vota AND id_cens_global = cens_global;

						t_vots := n_vots_cens + censAssociat.num_vots;
						t_electors := n_electors_cens + censAssociat.ciutadans_censats;
						nova_participacio := 100*t_vots/t_electors;

						UPDATE censGlobalAnys SET total_electors = t_electors, total_vots = t_vots, participacio = nova_participacio
						WHERE lany = any_vota AND id_cens_global = cens_global;

					-- si el cens estatal no s a la taula censosEstatalsAnys (aix passa quan el cens local que tractem ni cap altre 
					-- cens local del mateix pas no han participat a cap votaci aquest any; s a dir en tractar el primer cens local
					-- de cada pas a cada any), s'inserir el registre a la taula i llavors es comprovar si el cens global ja t
					-- registre per l'any de la votaci.
					ELSIF (trobat_cens_estatal = 0) THEN

						nova_participacio := 100*censAssociat.num_vots/censAssociat.ciutadans_censats;
						INSERT INTO censosEstatalsAnys(lany, id_cens_estatal, total_electors, total_vots, participacio, 
							num_votacions) 
							VALUES(any_vota, censAssociat.cens_estatal, censAssociat.ciutadans_censats, 
								censAssociat.num_vots, nova_participacio, 0);


						-- comprovem si el cens global ja s a la taula censGlobalAnys (hi ser una vegada tractat el primer cens
						-- local de l'any). Si hi s, tan sols s'actualitzen dades de la taula censGlobalAnys.
						trobat_cens_global := 0;
						SELECT COUNT(*) INTO trobat_cens_global FROM censGlobalAnys 
							WHERE lany = any_vota AND id_cens_global = cens_global;

						IF ( trobat_cens_global = 1 ) THEN

							-- ----------------------- cens global -----------------------
							SELECT total_electors, total_vots INTO n_electors_cens, n_vots_cens FROM censGlobalAnys 
								WHERE lany = any_vota AND id_cens_global = cens_global;

							t_vots := n_vots_cens + censAssociat.num_vots;
							t_electors := n_electors_cens + censAssociat.ciutadans_censats;
							nova_participacio := 100*t_vots/t_electors;

							UPDATE censGlobalAnys SET total_electors = t_electors, total_vots = t_vots, participacio = nova_participacio
								WHERE lany = any_vota AND id_cens_global = cens_global;

						-- si el cens global no s a la taula censGlobalAnys (aix noms passa un cop l'any i noms en tractar el
						-- primer cens local), s'inserir el registre a la taula censGlobaAnys.
						ELSIF ( trobat_cens_global = 0 ) THEN

							nova_participacio := 100*censAssociat.num_vots/censAssociat.ciutadans_censats;
							INSERT INTO censGlobalAnys(lany, id_cens_global, total_electors, total_vots, participacio, 
								num_votacions) 
								VALUES(any_vota, cens_global, censAssociat.ciutadans_censats, censAssociat.num_vots,
									 nova_participacio, 0);

						END IF; -- fi condicio trobat_cens_global

					END IF; -- fi condicio trobat_cens_estatal

				END IF; -- fi condicio trobat_cens_regional


			END IF; -- fi condicio trobat_cens_local



		  END LOOP; -- fi cursor censAssociat




		  -- recorrem un cursor no declarat per incrementar en 1 el nmero de votacions en les que han participat els censos regionals
		  -- que han participat a la votaci id_vota (les comptabiltizem a la taula censosRegionalsAnys)
		  -- Nota: el nmero de participacions dels censos locals ja s'ha actualitzat al recrrer el cursor anterior 'censAssociat'
		  FOR censRegional IN (SELECT DISTINCT cens_regional FROM censosAssociats WHERE id_votacio = id_vota AND any_votacio = any_vota) LOOP

			UPDATE censosRegionalsAnys SET num_votacions = num_votacions + 1 
				WHERE id_cens_regional = censRegional.cens_regional AND lany = any_vota;

		  END LOOP;


		  -- recorrem un cursor no declarat per incrementar en 1 el nmero de votacions en les que han participat els censos estatals
		  -- que han participat a la votaci id_vota (les comptabiltizem a la taula censosEstatalsAnys)
		  FOR censEstatal IN (SELECT DISTINCT cens_estatal FROM censosAssociats WHERE id_votacio = id_vota AND any_votacio = any_vota) LOOP

			UPDATE censosEstatalsAnys SET num_votacions = num_votacions + 1 
				WHERE id_cens_estatal = censEstatal.cens_estatal AND lany = any_vota;

		  END LOOP;


		  -- incrementem en 1 el nmero de votacions en les que ha participat el cens global l'any any_vota 
		  -- (les comptabiltizem a la taula censGlobalAnys)
		  UPDATE censGlobalAnys SET num_votacions = num_votacions + 1 WHERE id_cens_global = cens_global AND lany = any_vota;


-- fi actualitzacions taules censosLocalsAnys, censosRegionalsAnys, censosEstatalsAnys, censGlobalAnys




-- inici actualitzacions taula estadistiquesAnuals

	-- obtenci d'algunes dades d'aquesta votaci
		  -- obtenim la suma de vots de tots els censos associats a la votaci i calculem la participaci (el nmero d'electors 
		  -- ja l'hem obtingut al principi i est a la variable num_electors_votacio)
		  SELECT SUM(num_vots) INTO t_vots FROM censosAssociats WHERE id_votacio = id_vota;
		  nova_participacio := 100*t_vots/num_electors_votacio;
	
		  -- obtenim la/es localitat/s amb el nmero mxim de vots de la votaci
		  SELECT MAX(num_vots) INTO max_vots FROM censosAssociats WHERE id_votacio = id_vota;

		  -- com que podria passar que hi hagus ms d'una localitat amb el nombre mxim de vots, farem servir un cursor no declarat
		  localitats_max_vots := NULL;
		  FOR localitat IN (SELECT nom_localitat FROM censosAssociats WHERE id_votacio = id_vota AND num_vots = max_vots) LOOP

			localitats_max_vots := localitats_max_vots||localitat.nom_localitat||', ';

		  END LOOP;

		  -- obtenim el percentatge de ciutadans que no han votat mai durant l'any de la votaci que tractem (d'entre els 
		  -- que han estat cridats a les urnes; s a dir, no hem de comptabilitzar aquells ciutadans que estiguin de baixa 
		  -- ni aquells que no ho estiguin per que no hagin estat cridats a les urnes aquest any).
		  t_electors := 0;
		  t_electors_han_votat := 0;
		  t_electors_no_han_votat := 0;
		  perc_no_han_votat := 0;
		  SELECT COUNT(*) INTO t_electors FROM (SELECT DISTINCT id_ciutada FROM ciutadansVotacions 
			WHERE any_votacio = any_vota);

		  SELECT COUNT(*) INTO t_electors_han_votat FROM (SELECT DISTINCT id_ciutada FROM ciutadansVotacions 
			WHERE any_votacio = any_vota AND ha_participat = 1);

		  t_electors_no_han_votat := t_electors - t_electors_han_votat;
		  perc_no_han_votat := 100*t_electors_no_han_votat/t_electors;

	-- les dades obtingudes s'han guardat a: nova_participacio, localitats_max_vots, max_vots i perc_no_han_votat



		  -- comprovem si a la taula estadistiquesAnuals hi ha algun registre per aquest any. Si no n'hi ha cap (trobat  = 0),
		  -- n'inserim un amb les dades calculades, altrament, actualitzarem la taula (si cal)
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM estadistiquesAnuals WHERE lany = any_vota;

		  IF ( trobat = 0 ) THEN

			INSERT INTO estadistiquesAnuals(lany, votacions_mes_participacio, participacio, localitats_mes_vots, num_vots, ciutadans_no_han_votat_mai) 
				VALUES(any_vota, titol_votacio, nova_participacio,localitats_max_vots, max_vots, perc_no_han_votat);

		  ELSIF ( trobat = 1 ) THEN 

			-- obtenim dades del registre de la taula estadistiquesAnuals per aquest any
			SELECT votacions_mes_participacio, participacio, localitats_mes_vots, num_vots, ciutadans_no_han_votat_mai
				INTO votacions_mes_part, antiga_participacio, ant_localitats_max_vots, num, ciutadans_mai
				FROM estadistiquesAnuals WHERE lany = any_vota;

		      -- actualitzaci, si cal, de la/es votacio/ns amb mes participaci aquest any	
			-- si l'antiga mxima participaci d'una votaci coincideix amb la participaci de la votaci que tractem concatenarem els ttols
			IF ( antiga_participacio = nova_participacio ) THEN

				votacions_mes_part := votacions_mes_part||', '||titol_votacio;

			ELSE

				votacions_mes_part := titol_votacio;

			END IF;


			-- si l'antiga mxima participaci d'una votaci s <= que la participaci de la votaci que tractem actualitzarem la taula
			-- actualitzarem la taula, altrament no. 
			IF ( antiga_participacio <= nova_participacio ) THEN

				UPDATE estadistiquesAnuals SET votacions_mes_participacio = votacions_mes_part, participacio = nova_participacio
					WHERE lany = any_vota;

			END IF;

		     -- actualitzaci, si cal, de la/es localitat/s amb ms vots aquest any
			-- si l'antic nmero mxim de vots d'una localitat coincideix amb el mxim de la votaci que tractem (poc probacle) 
			-- concatenarem els noms de les localitats (la coma separadora ja la t la variable ant_localitats_max_vots)
			IF ( num = max_vots ) THEN

				localitats_max_vots := ant_localitats_max_vots||localitats_max_vots;

			END IF;


			-- si l'antic nmero mxim de vots d'una localitat d'una votaci del mateix any s <= que el mxim de vots de la votaci 
			-- que tractem actualitzarem la taula, altrament no. 
			IF ( num <= max_vots ) THEN

				UPDATE estadistiquesAnuals SET localitats_mes_vots = localitats_max_vots, num_vots = max_vots WHERE lany = any_vota;

			END IF;

		     -- actualitzaci, si cal, del percentatge de ciutadans que no han votat mai
			-- ciutadans_mai s el percentatge antic de la taula i perc_no_han_votat s el nou percentatge incloent els d'aquesta votaci.
			IF ( ciutadans_mai < perc_no_han_votat ) THEN

				UPDATE estadistiquesAnuals SET ciutadans_no_han_votat_mai = perc_no_han_votat WHERE lany = any_vota;


			END IF;


		  END IF; -- fi condicio trobat



-- fi actualitzacions taula estadistiquesAnuals





-- inici actualitzacions taula estadistiquesVotacions



	   -- actualitzaci de la participaci
		SELECT SUM(num_vots) INTO t_vots FROM censosAssociats WHERE id_votacio = id_vota;
		percent := 100*t_vots/num_electors_votacio;

		UPDATE estadistiquesVotacions SET num_electors = num_electors_votacio, num_vots = t_vots, participacio = percent
			WHERE id_votacio = id_vota;

	   -- actualitzaci de les dades de la opcio amb ms vots
		SELECT MAX(num_vots) INTO max_vots FROM opcionsVotacio WHERE id_votacio = id_vota;
		percent_max := 100*max_vots/t_vots;



		-- obtenim els identificadors i noms de les opcions que tenen el nmero mxim de vots
		-- NOTA: es pot donar el cas que en una votaci hi hagi empat de vots entre dos o ms partits.
		--	 Per controlar-ho, en cas que passi els identificadors i noms de les opcions es concatenaran.
		id_opc_CHAR := NULL;
		nom_opc := '';
		SELECT COUNT(*) INTO num FROM opcionsVotacio WHERE id_votacio = id_vota AND num_vots = max_vots;
		IF (num > 1) THEN

			-- si hi ha empat amb el mxim de vots, concatenem els identificadors i noms d'opci
			FOR opc IN (SELECT id_opcio FROM opcionsVotacio WHERE id_votacio = id_vota AND num_vots = max_vots) LOOP

				id_opc_CHAR := TO_CHAR(opc.id_opcio)||', '||id_opc_CHAR;

				SELECT nom_opcio INTO nom FROM opcions WHERE id = opc.id_opcio;
				nom_opc := nom||', '||nom_opc;

			END LOOP;

		ELSE
			SELECT id_opcio INTO id_opc FROM opcionsVotacio WHERE id_votacio = id_vota AND num_vots = max_vots;
			SELECT nom_opcio INTO nom_opc FROM opcions WHERE id = id_opc;
			id_opc_CHAR := TO_CHAR(id_opc);
		END IF;



		UPDATE estadistiquesVotacions SET id_opcio_mes_vots = id_opc_CHAR, nom_opcio_mes_vots = nom_opc, vots_opcio_mes_vots = max_vots,
			percentatge_opcio_mes_vots = percent_max WHERE id_votacio = id_vota;


	   -- actualitzaci de les dades de la opcio amb menys vots
		SELECT MIN(num_vots) INTO min_vots FROM opcionsVotacio WHERE id_votacio = id_vota;
		percent_min := 100*min_vots/t_vots;

		-- obtenim els identificadors i noms de les opcions que tenen el nmero mxim de vots
		-- NOTA: es pot donar el cas que en una votaci hi hagi empat de vots entre dos o ms partits.
		--	 Per controlar-ho, en cas que passi els identificadors i noms de les opcions es concatenaran.
		id_opc_CHAR := NULL;
		nom_opc := '';
		SELECT COUNT(*) INTO num FROM opcionsVotacio WHERE id_votacio = id_vota AND num_vots = min_vots;
		IF (num > 1) THEN

			-- si hi ha empat amb el mnim de vots, concatenem els identificadors i noms d'opci
			FOR opc IN (SELECT id_opcio FROM opcionsVotacio WHERE id_votacio = id_vota AND num_vots = min_vots) LOOP

				id_opc_CHAR := TO_CHAR(opc.id_opcio)||', '||id_opc_CHAR;

				SELECT nom_opcio INTO nom FROM opcions WHERE id = opc.id_opcio;
				nom_opc := nom||', '||nom_opc;

			END LOOP;

		ELSE
			SELECT id_opcio INTO id_opc FROM opcionsVotacio WHERE id_votacio = id_vota AND num_vots = min_vots;
			SELECT nom_opcio INTO nom_opc FROM opcions WHERE id = id_opc;
			id_opc_CHAR := TO_CHAR(id_opc);

		END IF;
		
		UPDATE estadistiquesVotacions SET id_opcio_menys_vots = id_opc_CHAR, nom_opcio_menys_vots = nom_opc, vots_opcio_menys_vots = min_vots,
			percentatge_opcio_menys_vots = percent_min WHERE id_votacio = id_vota;

	   -- actualitzaci de la diferncia percentual entre la opci ms i menys votada
		percent := percent_max - percent_min;
		UPDATE estadistiquesVotacions SET dif_perc_opc_mes_i_menys_vots = percent WHERE id_votacio = id_vota;



-- fi actualitzacions taula estadistiquesVotacions




-- inici actualitzacions taula historicVotacions

		-- obtenim les dades de la taula estadistiquesVotacions
		SELECT MAX(participacio) INTO percent FROM estadistiquesVotacions;
		SELECT MAX(dif_perc_opc_mes_i_menys_vots) INTO percent_max FROM estadistiquesVotacions;
		SELECT MIN(dif_perc_opc_mes_i_menys_vots) INTO percent_min FROM estadistiquesVotacions WHERE finalitzada = 1;

		-- obtenim els ttols de les votacions que compleixen amb les dades obtingudes
		-- NOTA: tot i que s poc probable, es considerar que podria haver-hi ms d'una votaci amb el mateix valor tant pel que fa
		--	 a la votaci amb ms participaci, com a la mxima i mnima diferncia entre les opcions de votaci. En cas que aix
		-- 	 sigui, es concatenaran els noms de les votacions (s'han reservat fins a 300 CHAR per guardar un mnim de 3 titols de
		-- 	 votaci en cas de coincidncia).
		SELECT COUNT(*) INTO num FROM estadistiquesVotacions WHERE participacio = percent;
		IF (num > 1) THEN
			FOR t_votacio IN (SELECT titol_votacio FROM estadistiquesVotacions WHERE participacio = percent) LOOP
				v_mes_part := v_mes_part||', '||t_votacio.titol_votacio;
			END LOOP;
		ELSE
			SELECT titol_votacio INTO v_mes_part FROM estadistiquesVotacions WHERE participacio = percent;
			v_mes_part := v_mes_part;
		END IF;


		SELECT COUNT(*) INTO num FROM estadistiquesVotacions WHERE dif_perc_opc_mes_i_menys_vots = percent_max;
		IF (num > 1) THEN
			FOR t_votacio IN (SELECT titol_votacio FROM estadistiquesVotacions WHERE dif_perc_opc_mes_i_menys_vots = percent_max) LOOP
				v_max_dif := v_max_dif||', '||t_votacio.titol_votacio;
			END LOOP;
		ELSE
			SELECT titol_votacio INTO v_max_dif FROM estadistiquesVotacions WHERE dif_perc_opc_mes_i_menys_vots = percent_max;
			v_max_dif := v_max_dif;
		END IF;


		SELECT COUNT(*) INTO num FROM estadistiquesVotacions WHERE dif_perc_opc_mes_i_menys_vots = percent_min;
		IF (num > 1) THEN
			FOR t_votacio IN (SELECT titol_votacio FROM estadistiquesVotacions WHERE dif_perc_opc_mes_i_menys_vots = percent_min) LOOP
				v_min_dif := v_min_dif||', '||t_votacio.titol_votacio;
			END LOOP;
		ELSE
			SELECT titol_votacio INTO v_min_dif FROM estadistiquesVotacions WHERE dif_perc_opc_mes_i_menys_vots = percent_min;
			v_min_dif := v_min_dif;
		END IF;


		-- una vegada tenim les dades i ttols actualitzarem la taula historicVotacions
		-- NOTA: com que el que tenim calculat sn les dades mximes/mnimes de totes les votacions, el ms fcil ser esborrar
		--	 el registre que hi ha (si n'hi ha algun) a la taula historicVotacions i inserir-n'hi un amb les noves dades.
		SELECT COUNT(*) INTO num FROM historicVotacions;
		IF (num > 0 ) THEN
			DELETE FROM historicVotacions;
		END IF;

		INSERT INTO historicVotacions (id, votacio_mes_participacio, participacio, votacio_max_diferencia_opcions, diferencia_max,
			votacio_min_diferencia_opcions, diferencia_min) VALUES(-1, v_mes_part, percent, v_max_dif, percent_max, v_min_dif, percent_min);


-- fi actualitzacions taula historicVotacions




-- inici actualitzacions taula maxParticipacionsPersona

		-- amb els vots de la votaci que estem tractant hi pot haver diversos ciutadans que igualin o superin el nmero de participacions
		-- mximes en una votaci. EL ms fcil s borrar les anteriors i tornar a omplir la taula amb les noves dades.
		SELECT COUNT(*) INTO n_files FROM maxParticipacionsCiutada;
		IF ( n_files > 0 ) THEN
			DELETE FROM maxParticipacionsCiutada;
		END IF;

		-- Com que s fora probable que hi hagi ms d'un ciutad amb un nmero mxim de participacions, els obtindrem tots
		-- en un cursor no declarat i els anirem afegint a la taula d'histrics.
		FOR ciutada IN (SELECT num_aleatori, nom_pais, num_participacions FROM ciutadans WHERE num_participacions = (SELECT MAX(num_participacions)
			FROM ciutadans)) LOOP

			INSERT INTO maxParticipacionsCiutada(id_aleatori_ciutada, pais, num_participacions)
				VALUES(ciutada.num_aleatori, ciutada.nom_pais, ciutada.num_participacions);

		END LOOP;


-- fi actualitzacions taula maxParticipacionsPersona


		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR al modul estadistic: l''identifcador de votacio es buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN camp_incorrecte THEN

		   RSP := 'ERROR al modul estadistic: l''identificador de la votacio ha de ser un nombre natural diferent de zero.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_inexistent THEN

		   RSP := 'ERROR al modul estadistic: l''identificador de votacio no correspon a cap votacio';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN no_finalitzada THEN

		   RSP := 'ERROR al modul estadistic: la votacio no esta finalitzada';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN estadistiques_calculades THEN

		   RSP := 'ERROR al modul estadistic: les estadistiques ja han estat calculades amb anterioritat';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL al : '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;








END; --fi implementaci paquet pk_estadistiques
/

