

-- Esborrat del paquet
-- -------------------
-- DROP PACKAGE BODY pk_votacions;
-- DROP PACKAGE pk_votacions;


-- ******************************************************************************************************************************
-- ******************************************************************************************************************************
-- *																*
-- *						FASE D'ESPECIFICACI								*
-- *																*
-- ******************************************************************************************************************************
-- ******************************************************************************************************************************



-- --------------------------------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------------------------------
-- Nom_paquet: 		pk_votacions
-- 
-- Descripci:		implementaci dels procediments d'ABM (Alta/Baixa/Modificacio) relacionats amb les votacions
-- 
-- Procediments:	p_alta_anys, p_altaEstatVotacio, p_altaOpcio, p_altaVotacio, p_altaOpcioVotacio, p_altaCensAssociat, 
-- 			p_actualitzaEstatVotacio, p_altaVotEmes, p_baixaVotacio, p_baixaOpcio, p_baixaCensAssociat, p_baixaOpcioVotacio, 
-- 			p_modificaOpcio_descripcio, p_modificaVotacio_titol, p_modificaVotacio_dataInici, p_modificaVotacio_dataFi,
-- 			p_modificaVotacio_web, p_modificaVotacio_president
-- Funcions:		--
-- --------------------------------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE pk_votacions IS


	PROCEDURE p_altaAnys(
	  anyx	IN	anys.lany%TYPE,
	  RSP	OUT	VARCHAR2
	  );


	PROCEDURE p_altaEstatVotacio (
	  id		IN	estatsVotacio.id%TYPE,
	  nom_estat	IN	estatsVotacio.nom_estat%TYPE,
	  RSP		OUT	VARCHAR2
	  );


	PROCEDURE p_altaOpcio (
	  id		IN	opcions.id%TYPE,
	  nom		IN	opcions.nom_opcio%TYPE,
	  descr		IN	opcions.descripcio%TYPE,
	  cod_pais	IN	opcions.codi_pais%TYPE,
	  baixa_log	IN	opcions.baixa_logica%TYPE,
	  RSP		OUT	VARCHAR2
	  );


	PROCEDURE p_altaVotacio (
	  id		IN	votacions.id%TYPE,
	  cens_global	IN	votacions.id_cens_global%TYPE,
	  id_estat	IN	votacions.id_estat%TYPE,
	  titol_v	IN	votacions.titol%TYPE,
	  data_inici	IN	votacions.data_inici%TYPE,
	  data_fi	IN	votacions.data_fi%TYPE,
	  any_votacio	IN	votacions.any_votacio%TYPE,
	  web		IN	votacions.adreca_web%TYPE,
	  president	IN	votacions.president_mesa%TYPE,
	  electors	IN	votacions.num_electors%TYPE,
	  RSP			OUT	VARCHAR2
	  );



	PROCEDURE p_altaOpcioVotacio (
	  id		IN	opcionsVotacio.id%TYPE,
	  id_votacio	IN	opcionsVotacio.id_votacio%TYPE,
	  id_opcio_v	IN	opcionsVotacio.id_opcio%TYPE,
	  n_vots	IN	opcionsVotacio.num_vots%TYPE,
	  RSP		OUT	VARCHAR2
	  );



	PROCEDURE p_altaCensAssociat (
	  id_vota	IN	censosAssociats.id_votacio%TYPE,
	  id_cens_loc	IN	censosAssociats.id_cens_local%TYPE,
	  cens_estat	IN	censosAssociats.cens_estatal%TYPE,
	  cens_regio	IN	censosAssociats.cens_regional%TYPE,
	  nom_local	IN	censosAssociats.nom_localitat%TYPE,
	  nom_pais	IN	censosAssociats.nom_pais%TYPE,
	  any_votacio	IN	censosAssociats.any_votacio%TYPE,
	  ciut_censats	IN	censosAssociats.ciutadans_censats%TYPE,
	  finalitzada	IN	censosAssociats.finalitzada%TYPE,
	  n_vots	IN	censosAssociats.num_vots%TYPE,
	  RSP		OUT	VARCHAR2
	  );



	PROCEDURE p_actualitzaEstatVotacio (
	  id_vota	IN	votacions.id%TYPE,
	  nou_estat	IN	estatsVotacio.nom_estat%TYPE,
	  RSP		OUT	VARCHAR2
	  );



	PROCEDURE p_altaVotEmes (
	  ciutada	IN	votsEmesos.id_ciutada%TYPE,
	  votacio	IN	votsEmesos.id_votacio%TYPE,
	  opcio		IN	votsEmesos.id_opcio%TYPE,
	  data_hora	IN	votsEmesos.data_hora%TYPE,
	  RSP		OUT	VARCHAR2
	  );


	PROCEDURE p_baixaVotacio (
	  titol_vota	IN	votacions.titol%TYPE,
	  RSP		OUT	VARCHAR2
	  );


	PROCEDURE p_baixaOpcio (
	  nom		IN	opcions.nom_opcio%TYPE,
	  cod_pais	IN	opcions.codi_pais%TYPE,
	  RSP		OUT	VARCHAR2
	  );


	PROCEDURE p_baixaCensAssociat (
	  id_vota	IN	censosAssociats.id_votacio%TYPE,
	  id_cens_loc	IN	censosAssociats.id_cens_local%TYPE,
	  RSP		OUT	VARCHAR2
	  );


	PROCEDURE p_baixaOpcioVotacio (
	  id_vota	IN	opcionsVotacio.id_votacio%TYPE,
	  id_opcio_v	IN	opcionsVotacio.id_opcio%TYPE,
	  RSP		OUT	VARCHAR2
	  );


	PROCEDURE p_modificaOpcio_descripcio (
	  nom		IN	opcions.nom_opcio%TYPE,
	  cod_pais	IN	opcions.codi_pais%TYPE,
	  nova_desc	IN	opcions.descripcio%TYPE,
	  RSP		OUT	VARCHAR2
	  );



	PROCEDURE p_modificaVotacio_titol (
	  titol_vota_vell	IN	votacions.titol%TYPE,
	  titol_vota_nou	IN	votacions.titol%TYPE,
	  RSP			OUT	VARCHAR2
	  );



	PROCEDURE p_modificaVotacio_dataInici (
	  titol_vota	IN	votacions.titol%TYPE,
	  data_in	IN	votacions.data_inici%TYPE,
	  RSP			OUT	VARCHAR2
	 );


	PROCEDURE p_modificaVotacio_dataFi (
	  titol_vota	IN	votacions.titol%TYPE,
	  data_fi_vota	IN	votacions.data_fi%TYPE,
	  RSP			OUT	VARCHAR2
	  );



	PROCEDURE p_modificaVotacio_web (
	  titol_vota	IN	votacions.titol%TYPE,
	  nova_web	IN	votacions.adreca_web%TYPE,
	  RSP			OUT	VARCHAR2
	  );


	PROCEDURE p_modificaVotacio_president (
	  titol_vota	IN	votacions.titol%TYPE,
	  nou_president	IN	votacions.president_mesa%TYPE,
	  RSP			OUT	VARCHAR2
	  );




END; --fi especificaci paquet pk_votacions
/















-- ******************************************************************************************************************************
-- ******************************************************************************************************************************
-- *																*
-- *						FASE D'IMPLEMENTACI								*
-- *																*
-- ******************************************************************************************************************************
-- ******************************************************************************************************************************




-- --------------------------------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------------------------------
-- Nom_paquet: 		pk_votacions
-- 
-- Descripci:		implementaci dels procediments d'ABM (Alta/Baixa/Modificacio) relacionats amb les votacions
-- 
-- Procediments:	p_alta_anys, p_altaEstatVotacio, p_altaOpcio, p_altaVotacio, p_altaOpcioVotacio, p_altaCensAssociat, 
-- 			p_actualitzaEstatVotacio, p_altaVotEmes, p_baixaVotacio, p_baixaOpcio, p_baixaCensAssociat, p_baixaOpcioVotacio, 
-- 			p_modificaOpcio_descripcio, p_modificaVotacio_titol, p_modificaVotacio_dataInici, p_modificaVotacio_dataFi,
-- 			p_modificaVotacio_web, p_modificaVotacio_president
-- Funcions:		--
-- --------------------------------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE BODY pk_votacions IS






	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom_procediment:	p_altaAnys
	-- 			
	-- Descripci:		insereix un registre a la taula anys i crida el procediment p_registrarLog del paquet pk_utils
	-- 			
	-- Parmetres entrada:	anyx	NUMBER(4)	PK	CHECK(anyx >= 2000)
	-- 			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori 	-> quan un parmetre d'entrada s buit o NULL
	-- 			camp_negatiu		-> quan s'intenta passar un any negatiu per parmetre
	-- 			camp_incorrecte		-> quan anyx no t exactament 4 dgits
	-- 			any_temps_passat	-> quan s'intenta donar d'alta un any anterior a l'any 2001
	-- 			any_existeix		-> quan l'any que s'intenta donar d'alta ja ho estava
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaAnys(
	  anyx	IN	anys.lany%TYPE,
	  RSP	OUT	VARCHAR2
	  ) IS


		nom_procedure		VARCHAR2(30 CHAR) := 'p_altaAnys';
	  	parametres_entrada	VARCHAR2(10 CHAR) := NULL;
		esCorrecte		NUMBER(1) := 1;
		confirmada		NUMBER(1) := 1;
		any_actual		NUMBER(4) := 0;
		trobat			NUMBER(1) := 0;
	  	camp_obligatori		EXCEPTION;
		camp_negatiu		EXCEPTION;
		camp_incorrecte		EXCEPTION;
		any_temps_passat	EXCEPTION;
		any_existeix		EXCEPTION;


		BEGIN

		   parametres_entrada := anyx;

		   IF (anyx IS NULL) THEN
			RAISE camp_obligatori;
		   END IF;

		   IF ( anyx < 0 ) THEN
			RAISE camp_negatiu;
		   END IF;

		   IF ( length(anyx) <> 4 ) THEN
			RAISE camp_incorrecte;
		   END IF;

		   IF ( anyx < 2000 ) THEN
			RAISE any_temps_passat;
		   END IF;

		  -- comprovem que l'any de la votaci no existeixi ja al sistema
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM anys WHERE lany = anyx;
		  IF ( trobat = 1 ) THEN
			RAISE any_existeix;
		  END IF;


		   INSERT INTO anys(lany) VALUES(anyx);

		   RSP := 'OK';
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: el camp any es buit o te el valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN camp_negatiu THEN

		   RSP := 'ERROR: el valor de l''any no pot ser mai negatiu.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN camp_incorrecte THEN

		   RSP := 'ERROR: l''any ha de tenir exactament 4 digits.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN any_temps_passat THEN

		   RSP := 'ERROR: l''any no pot ser anterior a l''any 2000.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN any_existeix THEN

		   RSP := 'ERROR: l''any que es vol donar d''alta ja existeix al sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN OTHERS THEN

		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;






	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaEstatVotacio
	-- 			
	-- Descripci:		insereix un registre a la taula estatsVotacio i crida el procediment p_registrarLog del paquet pk_utils
	-- 			
	-- Parmetres entrada:	id		NUMBER	PK		AUTO INCREMENT
	-- 			nom_estat	VARCHAR2(20 CHAR)	DEFAULT 'edicio'	NOT NULL	UNIQUE	
	-- 								CHECK(estat IN('edicio', 'pendent obertura', 'oberta', 'finalitzada'))
	--			
	-- Parmetres sortida:	RSP		VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 							al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori	-> quan un parmetre d'entrada que no pot ser buit o NULL ho s
	-- 			camp_incorrecte -> quan el parmetre id no s negatiu o quan nom_estat no tingui un dels valors esperats
	--			OTHERS		-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaEstatVotacio (
	  id		IN	estatsVotacio.id%TYPE,
	  nom_estat	IN	estatsVotacio.nom_estat%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_altaEstatVotacio';
	  	parametres_entrada 	VARCHAR2(50 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
	  	camp_obligatori 	EXCEPTION;
		camp_incorrecte 	EXCEPTION;



		BEGIN

		  parametres_entrada := id||', '||nom_estat;

		  IF (  id IS NULL OR 
			nom_estat IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;


		  IF (	id >= 0 OR
			nom_estat NOT IN ('edicio', 'pendent obertura', 'oberta', 'finalitzada')) THEN
				RAISE camp_incorrecte;
		  END IF;


		   INSERT INTO estatsVotacio(id, nom_estat) VALUES(id, nom_estat);

		   RSP := 'OK';
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN camp_incorrecte THEN

		   RSP := 'ERROR: a) el camp id no es negatiu o, b) el camp nom_estat no te un dels 4 estats esperats: ''edicio'', ''pendent obertura'', ''oberta'', ''finalitzada''';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		END;







	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaOpcio
	-- 			
	-- Descripci:		insereix un registre a la taula opcions i crida el procediment p_registrarLog del paquet pk_utils
	-- 			
	-- Parmetres entrada:	id 		NUMBER				PK
	-- 			nom		VARCHAR2(15 CHAR)		NOT NULL
	-- 			descr		VARCHAR2(100 CHAR)
	-- 			cod_pais	CHAR(2)				NOT NULL	REFERENCES codisISO(codi_pais)
	-- 			baixa_log	NUMBER(1)	DEFAULT 0	NOT NULL	CHECK (baixa_log IN(0,1))
	-- 										UNIQUE (nom, codi_pais)
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada que no pot ser buit o NULL ho s
	-- 			llargada_camp		-> quan els parmetres nom i/o descr tenen ms de 15 i 100 carcteres respectivament
	-- 						-> i quan el parmetre codi_pais no t exactament 2 carcters
	-- 			baixa_inapropiada	-> quan baixa_log no s zero
	-- 			codi_pais_incorrecte	-> quan cod_pais no referencia el codi ISO de cap pais (incls el pas neuter 'ZQ')
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaOpcio (
	  id		IN	opcions.id%TYPE,
	  nom		IN	opcions.nom_opcio%TYPE,
	  descr		IN	opcions.descripcio%TYPE,
	  cod_pais	IN	opcions.codi_pais%TYPE,
	  baixa_log	IN	opcions.baixa_logica%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_altaOpcio';
	  	parametres_entrada 	VARCHAR2(140 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER := 0;
	  	camp_obligatori 	EXCEPTION;
		llargada_camp		EXCEPTION;
		baixa_inapropiada	EXCEPTION;
		codi_pais_incorrecte	EXCEPTION;


		BEGIN

		  parametres_entrada := id||', '||nom||', '||descr||', '||cod_pais||', '||baixa_log;

		  IF (	id IS NULL OR
			nom IS NULL OR
			cod_pais IS NULL OR
			baixa_log IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (	length(nom) > 15 OR
			length(descr) > 100 OR
			length(cod_pais) <> 2 ) tHEN
			RAISE llargada_camp;
		  END IF;

		  IF ( baixa_log <> 0 ) THEN
			RAISE baixa_inapropiada;
		  END IF;

		  -- comprovem que codi_pais referenci un pas del sistema o al codi de pas neutre 'ZQ'
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM codisISO WHERE codi_pais = cod_pais;
		  IF ( trobat = 0 ) THEN
			RAISE codi_pais_incorrecte;
		  END IF;


		  -- fetes les verificacions anteriors, es pot donar d'alta la opci
		  INSERT INTO opcions(id, nom_opcio, descripcio, codi_pais, baixa_logica) VALUES(id, nom, descr, cod_pais, baixa_log);

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN llargada_camp THEN

		   RSP := 'ERROR: nombre de caracters inapropiat. Revisi la llargada dels camps nom_opcio, descripcio i/o codi_pais.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN baixa_inapropiada THEN

		   RSP := 'ERROR: el valor del camp baixa_logica ha de ser zero en el moment de donar d''alta una opcio.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN codi_pais_incorrecte THEN

		   RSP := 'ERROR: el codi del pais no referencia cap codi ISO.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		END;










	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaVotacio
	-- 			
	-- Descripci:		insereix un registre a la taula votacions i un registre a la taula estadistiquesVotacions amb els
	-- 			valors estadsitcs inicialitzats a zero. Finalment, crida el procediment p_registrarLog del paquet pk_utils
	-- 
	-- 			NOTA:	El procediment no deixa donar d'alta votacions que comencin (data inici) abans de la data actual
	-- 				del sistema. Tanmateix es fa una excepci per les votacions de prova que s'han creat.
	-- 
	-- Parmetres entrada:	id		NUMBER			PK
	-- 			cens_global	VARCHAR2(30 CHAR)	NOT NULL	REFERENCES censGlobal(nom_cens)
	-- 			id_estat	NUMBER			NOT NULL	REFERENCES estatsVotacio(id)
	-- 			titol_v		VARCHAR2(200 CHAR)	NOT NULL	UNIQUE
	-- 			data_inici	DATE			NOT NULL
	-- 			data_fi		DATE			NOT NULL	CHECK(data_fi > data_inici)
	-- 			any_votacio	NUMBER(4)		NOT NULL	REFERENCES anys(lany)
	-- 			web		VARCHAR2(200 CHAR)
	-- 			president	VARCHAR2(100 CHAR)	NOT NULL
	-- 			electors	NUMBER	DEFAULT 0	NOT NULL	CHECK(electors >= 0)
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada que no pot ser buit o NULL ho s
	-- 			camp_incorrecte		-> quan l'identificador de votaci s un nombre natural
	-- 			cens_global_inexistent	-> quan cens_global no referencia el cens global
	-- 			estat_inexistent	-> quan id_estat no referencia cap dels estats d'una vortaci
	-- 			estat_incorrecte	-> quan l'estat de la nova votaci no s 'edicio'
	-- 			num_electors_incorrecte	-> quan el nmero d'electors no s zero
	-- 			data_inici_incorrecte	-> quan la data d'inici s menor que la data actual (veure nota superior).
	-- 			data_fi_incorrecte 	-> quan la data de fi s menor que la data d'inici
	-- 			any_incorrecte		-> quan l'any de la votaci no es correspon amb l'any indicat per la data d'inici
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaVotacio (
	  id		IN	votacions.id%TYPE,
	  cens_global	IN	votacions.id_cens_global%TYPE,
	  id_estat	IN	votacions.id_estat%TYPE,
	  titol_v	IN	votacions.titol%TYPE,
	  data_inici	IN	votacions.data_inici%TYPE,
	  data_fi	IN	votacions.data_fi%TYPE,
	  any_votacio	IN	votacions.any_votacio%TYPE,
	  web		IN	votacions.adreca_web%TYPE,
	  president	IN	votacions.president_mesa%TYPE,
	  electors	IN	votacions.num_electors%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_altaVotacio';
	  	parametres_entrada 	VARCHAR2(600 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		votacio_prova		NUMBER(1) := 0;
		trobat			NUMBER(1) := 0;
		id_est			NUMBER := 0;
		estat			VARCHAR2(20 CHAR);
		inici			NUMBER;
		fi			NUMBER;
		actual			NUMBER;
		any_inici		NUMBER;
		num_sequencia		NUMBER;
	  	camp_obligatori 	EXCEPTION;
		camp_incorrecte 	EXCEPTION;
		cens_global_inexistent	EXCEPTION;
		estat_inexistent	EXCEPTION;
		estat_incorrecte	EXCEPTION;
		num_electors_incorrecte	EXCEPTION;
		data_inici_incorrecte	EXCEPTION;
		data_fi_incorrecte	EXCEPTION;
		any_incorrecte		EXCEPTION;


		BEGIN

		  parametres_entrada := id||', '||cens_global||', '||id_estat||', '||titol_v||', '||data_inici||', '||data_fi||', '||any_votacio||', '||web||', '||president||', '||electors;

		  -- convertim les dates d'inici i fi de la votaci i la data actual a un format adequat per fer comparacions
		  SELECT TO_NUMBER(TO_CHAR(data_inici, 'yyyymmdd')) INTO inici FROM DUAL;
		  SELECT TO_NUMBER(TO_CHAR(data_fi, 'yyyymmdd')) INTO fi FROM DUAL;
		  SELECT TO_NUMBER(TO_CHAR(SYSDATE, 'yyyymmdd')) INTO actual FROM DUAL;


		  IF (  id IS NULL OR 
			cens_global IS NULL OR
			id_estat IS NULL OR
			titol_v IS NULL OR
			data_inici IS NULL OR
			data_fi IS NULL OR
			any_votacio IS NULL OR
			web IS NULL OR
			president IS NULL OR
			electors IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF ( id >= 0 ) THEN
			RAISE camp_incorrecte;
		  END IF;

		  -- comprovem que el cens global estigui ben referenciat
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM censGlobal WHERE nom_cens = cens_global;
		  IF (trobat = 0) THEN
			RAISE cens_global_inexistent;
		  END IF;

		  -- comprovem que l'estat de la nova votaci existeixi i sigui 'edicio'
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM estatsVotacio WHERE id = id_estat;
		  IF ( trobat = 0 ) THEN
			RAISE estat_inexistent;
		  END IF;

		  SELECT nom_estat INTO estat FROM estatsVotacio WHERE id = id_estat;
		  IF ( estat <> 'edicio' ) THEN
			RAISE estat_incorrecte;
		  END IF;
		  
	
		  
		  -- el nmero d'electors en el moment d'inserir el registre ha de ser zero (s'actualitzar al associar censos a les votacions)
		  IF ( electors <> 0 ) THEN
			RAISE num_electors_incorrecte;
		  END IF;


	-- Votacions de prova (a les que es permetr que tinguin una data d'inici anterior a la data actual del sistema)
	IF ( titol_v  IN( 'Eleccions al municipi de Sevilla de 2 de gener de 2000',
			  'Eleccions al municipi de Munich de 3 de gener de 2000', 
			  'Eleccions al municipi de Roma de 2 de gener de 2001',
			  'Eleccions al municipi de Santander de 3 de gener de 2001',
			  'Eleccions al municipi de Berlin de 2 de gener de 2002',
			  'Eleccions al municipi de Toledo de 3 de gener de 2002',  
			  'Eleccions al municipi de Florencia de 2 de gener de 2003', 
			  'Eleccions al municipi de Madrid de 3 de gener de 2003', 
			  'Eleccions al municipi de Barcelona de 2 de gener de 2004',
 			  'Eleccions al municipi de Murcia de 3 de gener de 2004',  
			  'Eleccions al municipi de Dusseldorf de 2 de gener de 2005', 
			  'Eleccions al municipi de Girona de 3 de gener de 2005',
			  'Eleccions al municipi de Bolonya de 2 de gener de 2006', 
			  'Eleccions al municipi de Murcia de 3 de gener de 2006', 
			  'Eleccions al municipi de Hannover de 2 de gener de 2007',
			  'Eleccions al municipi de Florencia de 3 de gener de 2007',
			  'Eleccions al municipi de Barcelona de 2 de gener de 2008',
			  'Eleccions al municipi de Madrid de 3 de gener de 2008',  
			  'Eleccions al municipi d''Olot de 2 de gener de 2010',
			  'Eleccions al municipi de Girona de 3 de gener de 2010' ,
			  'Eleccions al Parlament de Catalunya de 1 de novembre de 2011', 
			  'Eleccions a Corts Generals de 2 de novembre de 2011', 
			  'Eleccions al Parlament Europeu de 3 de novembre de 2011' ) ) THEN
		votacio_prova := 1;
	END IF;


		  -- no es pot donar d'alta una votaci amb una data d'inici anterior a la data actual
		  -- 		NOTA:	les votacions amb el titol que es veu just aqu sobre sn per fer proves (posaran votacio_prova a 1)
		  -- 			i se'ls permetr una data d'inici anterior a la data actual del sistema.
		  IF ( inici < actual AND votacio_prova = 0 ) THEN
			RAISE data_inici_incorrecte;
		  END IF;


		  -- una votaci no pot finalitzar abans de comenar
		  IF ( fi < inici ) THEN
			RAISE data_fi_incorrecte;
		  END IF;

		  -- Obtenim l'any de la data d'inici per fer la segent comparaci.
		  -- L'obtenim de la variable 'inici' en comptes del parmetre data_inici perqu amb aquest no s'obtenia l'any correctament
		  SELECT TO_NUMBER(TO_CHAR(data_inici, 'yyyy')) INTO any_inici FROM DUAL;

		  -- l'any de la votaci no pot ser diferent a l'any de la data d'inici (la data d'inici i de final s'ha dit que seran el mateix any)
		  IF ( any_votacio <> any_inici ) THEN
			RAISE any_incorrecte;
		  END IF;


		  -- alta de la votaci
		  INSERT INTO votacions(id, id_cens_global, id_estat, titol, data_inici, data_fi, any_votacio, adreca_web, president_mesa, num_electors) 
		  VALUES(id, cens_global, id_estat, titol_v, data_inici, data_fi, any_votacio, web, president, electors);

		  -- consulta de l'identificador que la seqncia ha assignat a la votaci (el ttol s clau alternativa)
		  SELECT id INTO num_sequencia FROM votacions WHERE titol = titol_v;

		  -- i alta de les estadstiques de la votaci amb els valors estadstics inicialitzats a zero (valor per defecte)
		  INSERT INTO estadistiquesVotacions(id, id_votacio, titol_votacio, any_votacio, finalitzada, num_electors, num_vots, participacio,
			id_opcio_mes_vots, nom_opcio_mes_vots, vots_opcio_mes_vots, percentatge_opcio_mes_vots, id_opcio_menys_vots, 
			nom_opcio_menys_vots, vots_opcio_menys_vots, percentatge_opcio_menys_vots, dif_perc_opc_mes_i_menys_vots)
		  VALUES(-1, num_sequencia, titol_v, any_votacio, DEFAULT, DEFAULT, DEFAULT, DEFAULT, NULL, NULL, DEFAULT, DEFAULT, NULL, NULL, DEFAULT, DEFAULT, DEFAULT);
		  

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN camp_incorrecte THEN

		   RSP := 'ERROR: l''identificador de votacio ha de ser menor a zero (l''actualitza una sequencia).';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN cens_global_inexistent THEN

		   RSP := 'ERROR: el cens global no existeix';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN estat_inexistent THEN

		   RSP := 'ERROR: l''identificador de l''estat de la votacio no referencia cap dels estats possibles.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN estat_incorrecte THEN

		   RSP := 'ERROR: l''identificador de l''estat d''una nova votacio ha de referenciar l''estat ''edicio'' ';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN num_electors_incorrecte THEN

		   RSP := 'ERROR: el numero d''electors ha de ser zero.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN data_inici_incorrecte THEN

		   RSP := 'ERROR: la data d''inici ha de ser major que la data actual';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN data_fi_incorrecte THEN

		   RSP := 'ERROR: la data de fi no pot ser anterior a la data d''inici';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		WHEN any_incorrecte THEN

		   RSP := 'ERROR: l''any de la votacio no es correspon amb l''any de la data d''inici';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;







	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaOpcioVotacio
	-- 			
	-- Descripci:		insereix un registre a la taula opcionsVotacio i crida el procediment p_registrarLog del paquet pk_utils
	-- 			
	-- Parmetres entrada:	id		NUMBER			PK
	-- 			id_votacio	NUMBER			NOT NULL	REFERENCES votacions(id)
	-- 			id_opcio_v	NUMBER			NOT NULL	REFERENCES opcions(id)
	-- 			n_vots		NUMBER	DEFAULT 0	NOT NULL	CHECK(n_vots >=0)
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada que no pot ser buit o NULL ho s
	-- 			valor_incorrecte	-> quan id_votaci i id_opcio_v no sn enters positius i/o quan n_vots no s zero
	-- 			votacio_inexistent	-> quan id_votacio no referencia cap votaci del sistema
	-- 			opcio_inexistent	-> quan id_opcio_v no referencia cap opci del sistema
	-- 			opcio_de_baixa		-> quan id_opcio_v referencia una opcio que ha estat donada de baixa
	-- 			votacio_no_editable	-> quan id_votacio referencia una votaci a la qual ja no es poden afegir opcions
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaOpcioVotacio (
	  id		IN	opcionsVotacio.id%TYPE,
	  id_votacio	IN	opcionsVotacio.id_votacio%TYPE,
	  id_opcio_v	IN	opcionsVotacio.id_opcio%TYPE,
	  n_vots	IN	opcionsVotacio.num_vots%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_altaOpcioVotacio';
	  	parametres_entrada 	VARCHAR2(60 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		baixa			NUMBER(1) := 0;
		id_estat_vota		NUMBER := 0;
		estat_vota		VARCHAR2(20 CHAR);
	  	camp_obligatori 	EXCEPTION;
		valor_incorrecte	EXCEPTION;
		votacio_inexistent	EXCEPTION;
		opcio_inexistent	EXCEPTION;
		opcio_de_baixa		EXCEPTION;
		votacio_no_editable	EXCEPTION;



		BEGIN

		  parametres_entrada := id||', '||id_votacio||', '||id_opcio_v||', '||n_vots;

		  IF (  id IS NULL OR 
			id_votacio IS NULL OR
			id_opcio_v IS NULL OR
			n_vots IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (	id_votacio <= 0 OR
			id_opcio_v <= 0 OR
			n_vots <> 0 ) THEN
			RAISE valor_incorrecte;
		  END IF;

		  -- comprovem que la votaci existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM votacions WHERE id = id_votacio;
		  IF ( trobat = 0 ) THEN
			RAISE votacio_inexistent;
		  END IF;


		  -- comprovem que la opci existeixi
		  trobat := 0;
		  SELECT COUNT (*) INTO trobat FROM opcions WHERE id = id_opcio_v;
		  IF ( trobat = 0 ) THEN
			RAISE opcio_inexistent;
		  END IF;

		  -- comprovem que la opci no estigui de baixa
		  SELECT baixa_logica INTO baixa FROM opcions WHERE id = id_opcio_v;
		  IF ( baixa = 1 ) THEN
			RAISE opcio_de_baixa;
		  END IF;



		  -- comprovem que la votaci estigui en estat 'edicio'
		  SELECT id_estat INTO id_estat_vota FROM votacions WHERE id = id_votacio;
		  SELECT nom_estat INTO estat_vota FROM estatsVotacio	WHERE id = id_estat_vota;
		  IF ( estat_vota <> 'edicio' ) THEN
			RAISE votacio_no_editable;
		  END IF;


		  INSERT INTO opcionsVotacio(id, id_votacio, id_opcio, num_vots) VALUES (id, id_votacio, id_opcio_v, n_vots);

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN valor_incorrecte THEN

		   RSP := 'ERROR: comprovi que es compleixi el valor dels camps seguents: id_votacio > 0, id_opcio > 0 i/o num_vots = 0';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_inexistent THEN

		   RSP := 'ERROR: no es pot afegir una opcio de vot a una votacio que no existeix';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN opcio_inexistent THEN

		   RSP := 'ERROR: la opcio de vot no existeix al sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN opcio_de_baixa THEN

		   RSP := 'ERROR: la opcio de vot ha estat donada de baixa i no es pot associar a cap votacio';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_no_editable THEN

		   RSP := 'ERROR: no es pot associar cap opcio a una votacio que no estigui en estat d''edicio.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;







	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaCensAssociat
	-- 			
	-- Descripci:		Una vegada fetes unes quantes verificacions prvies, el procediment porta a terme diverses accions:
	-- 			   a) si encara no hi s, insereix un registre a la taula votacionsPaisos amb l'identificador del pas i
	-- 			      de la votaci (la taula registra els paisos que participen a cada votaci). Si el pas del censAssociat
	-- 			      que es dna d'alta ja existeix a votacionsPaisos, no s'inserir cap registre a aquesta taula.
	-- 			   b) Obt el n de ciutadans censats actualment en el cens local, id_cens_loc, que es vol associar a la 
	-- 			      votaci id_vota. Aquest valor servir a l'hora d'inserir el registre a la taula censosAssociats (punt c).
	-- 			   c) Insereix un registre a la taula censosAssociats (taula que registra els censos associats a cada votaci)
	-- 			      amb totes les dades rebudes per parmetre a excepci del parmetre ciut_censats que no es tindr en compte
	-- 			      ja que el nmero de ciutadans censats s'obt en aquest procediment (punt b).
	-- 			   d) Actualitza el nmero d'electors de la votaci.
	-- 			   e) Insereix dades de tots els ciutadans del cens local id_cens_loc que no estiguin donats de baixa a la taula 
	-- 			      ciutadansVotacions (taula que registra tots els electors de les votacions).
	-- 			   f) Crida el procediment p_registrarLog del paquet pk_utils.
	-- 
	-- 			NOTA:	El procediment noms deixa donar d'alta censos a les votacions que estan en estat d'edici 
	-- 
	-- 			
	-- Parmetres entrada:	id_vota		NUMBER				PK_composta1	REFERENCES votacions(id)
	-- 			id_cens_loc	VARCHAR2(40 CHAR)		PK_composta2	REFERENCES censosLocals(nom_cens)
	-- 			cens_estat	VARCHAR(30 CHAR)		NOT NULL
	-- 			cens_regio	VARCHAR(40 CHAR)		NOT NULL
	--			nom_local	VARCHAR(40 CHAR)		NOT NULL
	-- 			nom_pais	VARCHAR(30 CHAR)		NOT NULL
	-- 			any_votacio	NUMBER(4)			NOT NULL
	-- 			ciut_censats	NUMBER				NOT NULL	CHECK(ciut_censats >=0)
	-- 			finalitzada	NUMBER(1)	DEFAULT 0	NOT NULL	CHECK(finalitzada IN(0,1))
	-- 			n_vots		NUMBER		DEFAULT 0	NOT NULL	CHECK(n_vots >=0)
	-- 
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori			-> quan un parmetre d'entrada que no pot ser buit o NULL ho s
	-- 			numero_incorrecte		-> quan l'id_votacio no s un natural i/o quan ciut_censats, n_vots o
	-- 							   finalitzada no siguin zero
	-- 			votacio_inexistent		-> quan id_vota no correspon a cap votaci existent
	-- 			votacio_no_editable		-> quan la votaci no est en estat 'edicio' (en altres estats no s'hi pot associar 
	-- 							   cap cens)
	-- 			any_incorrecte			-> quan any_votaci no t 4 dgits o quan no es correspon amb l'any de la votaci
	-- 			cens_local_inexistent		-> quan id_cens_local no correspon a cap cens local existent
	-- 			cens_local_regional_erroni	-> quan el cens local no correspon al cens regional
	-- 			cens_regional_estatal_erroni	-> quan el cens regional no correspon al cens estatal
	-- 			nom_localitat_erroni		-> quan el nom de localitat no es correspon amb el cens local
	-- 			pais_incorrecte			-> quan el nom del pais no es correspon amb el del cens estatal
	--			OTHERS				-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaCensAssociat (
	  id_vota	IN	censosAssociats.id_votacio%TYPE,
	  id_cens_loc	IN	censosAssociats.id_cens_local%TYPE,
	  cens_estat	IN	censosAssociats.cens_estatal%TYPE,
	  cens_regio	IN	censosAssociats.cens_regional%TYPE,
	  nom_local	IN	censosAssociats.nom_localitat%TYPE,
	  nom_pais	IN	censosAssociats.nom_pais%TYPE,
	  any_votacio	IN	censosAssociats.any_votacio%TYPE,
	  ciut_censats	IN	censosAssociats.ciutadans_censats%TYPE,
	  finalitzada	IN	censosAssociats.finalitzada%TYPE,
	  n_vots	IN	censosAssociats.num_vots%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure		VARCHAR2(30 CHAR) := 'p_altaCensAssociat';
	  	parametres_entrada	VARCHAR2(240 CHAR) := NULL;
		esCorrecte		NUMBER(1) := 1;
		confirmada		NUMBER(1) := 1;
		any_v			NUMBER(4) := 0;
		trobat			NUMBER(1) := 0;
		titol_v			VARCHAR2(100 CHAR);
		estat_votacio		VARCHAR2(20 CHAR) := 0;
		pais			VARCHAR2(30 CHAR);
		cens_est		VARCHAR2(30 CHAR);
		cens_reg		VARCHAR2(40 CHAR);
		ciutat			VARCHAR2(40 CHAR);
		censats			NUMBER := 0;
		electors_votacio	NUMBER := 0;
	  	camp_obligatori			EXCEPTION;
		numero_incorrecte		EXCEPTION;
		votacio_inexistent		EXCEPTION;
		votacio_no_editable		EXCEPTION;
		any_incorrecte			EXCEPTION;
		cens_local_inexistent		EXCEPTION;
		cens_local_regional_erroni	EXCEPTION;
		cens_regional_estatal_erroni	EXCEPTION;
		nom_localitat_erroni		EXCEPTION;
		pais_incorrecte			EXCEPTION;


		-- declarem un cursor que obtindr els nmeros aleatoris del ciutadans associats al cens local
		-- i que ser til per carregar dades a la taula ciutadansVotacions.
		CURSOR ciutadans_cens IS SELECT num_aleatori, baixa_logica FROM ciutadans c WHERE c.id_cens_local = id_cens_loc;


		BEGIN

		  parametres_entrada := id_vota||', '||id_cens_loc||', '||cens_estat||', '||cens_regio||', '||nom_local||', '||nom_pais||', '||any_votacio||', '||ciut_censats||', '||finalitzada||', '||n_vots;

		  IF (  id_vota IS NULL OR 
			id_cens_loc IS NULL OR
			cens_estat IS NULL OR
			cens_regio IS NULL OR
			nom_local IS NULL OR
			nom_pais IS NULL OR
			any_votacio IS NULL OR
			ciut_censats IS NULL OR
			finalitzada IS NULL OR
			n_vots IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (	id_vota <= 0 OR
			ciut_censats <> 0 OR
			n_vots <> 0 OR
			finalitzada <> 0 ) THEN
			RAISE numero_incorrecte;
		  END IF;

		  IF( length(any_votacio) <> 4 ) THEN
			RAISE any_incorrecte;
		  END IF;


		  -- comprovem que l'identificador de la votaci existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM votacions WHERE id = id_vota;
		  IF ( trobat = 0 ) THEN
			RAISE votacio_inexistent;
		  END IF;



		  -- comprovem que la votaci encara sigui editable (en estat 'edicio'). Si no ho s, no s'hi podr afegir cap cens.
		  SELECT ev.nom_estat INTO estat_votacio 
			FROM estatsVotacio ev, votacions v
			WHERE ev.id = v.id_estat AND v.id = id_vota;
		  IF ( estat_votacio <> 'edicio' ) THEN
			RAISE votacio_no_editable;
		  END IF;

		  -- comprovem que hi hagi coherncia entre el valor del parmetre any_votacio i l'any real de la votaci
		  SELECT any_votacio INTO any_v FROM votacions WHERE id = id_vota;
		  IF ( any_votacio <> any_v ) THEN
			RAISE any_incorrecte;
		  END IF;


		  -- comprovem que l'identificador de cens local existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM censosLocals WHERE nom_cens = id_cens_loc;
		  IF ( trobat = 0 ) THEN
			RAISE cens_local_inexistent;
		  END IF;

		  -- comprovem que el cens local realment pertanyi al cens regional
		  SELECT id_cens_regional INTO cens_reg FROM censosLocals WHERE nom_cens = id_cens_loc;
		  IF ( cens_regio <> cens_reg ) THEN
			RAISE cens_local_regional_erroni;
		  END IF;

		  -- comprovem que el cens regional realment pertanyi al cens estatal
		  SELECT id_cens_estatal INTO cens_est FROM censosRegionals WHERE nom_cens = cens_regio;
		  IF ( cens_estat <> cens_est ) THEN
			RAISE cens_regional_estatal_erroni;
		  END IF;

		  -- comprovem que la localitat es correspongui amb el cens local
		  SELECT nom_localitat INTO ciutat FROM censosLocals WHERE nom_cens = id_cens_loc;
		  IF ( nom_local <> ciutat ) THEN
			RAISE nom_localitat_erroni;
		  END IF;

		  -- comprovem que el nom del pas es correspongui amb el pas del cens estatal
		  SELECT p.nom_pais INTO pais FROM paisos p, censosEstatals ce WHERE p.codi_pais = ce.codi_pais AND ce.nom_cens = cens_estat;
		  IF ( nom_pais <> pais) THEN
			RAISE pais_incorrecte;
		  END IF;


		  -- si el pas d'aquest cens i per aquesta votaci encara no s a la taula votacionsPaisos l'hi afegim
		  trobat := 0;
		  SELECT COUNT (*) INTO trobat FROM votacionsPaisos WHERE id_pais = nom_pais AND id_votacio = id_vota;
		  IF ( trobat = 0 ) THEN
			INSERT INTO votacionsPaisos(id_pais, id_votacio) VALUES(nom_pais, id_vota);
		  END IF;
		

		  -- Obtenim el nmero de ciutadans censats (en el moment actual) a aquest cens (el n de ciutadans censats del cens que associem 
		  -- a la votaci ser el n de ciutadans censats que hi hagi en el cens local en el moment de donar d'alta el cens associat)
		  SELECT ciutadans_censats INTO censats FROM censosLocals WHERE nom_cens = id_cens_loc;

		  -- Donem d'alta, a la taula censosAssociats,  el cens associat a la votaci
		  INSERT INTO censosAssociats(id_votacio, id_cens_local, cens_estatal, cens_regional, nom_localitat, nom_pais, any_votacio, 
			ciutadans_censats, finalitzada, num_vots) VALUES(id_vota, id_cens_loc, cens_estat, cens_regio, nom_local, nom_pais,
			any_votacio, censats, finalitzada, n_vots);

		  -- actualizem el nmero d'electors de la votaci
		  SELECT num_electors INTO electors_votacio FROM votacions WHERE id = id_vota;
		  UPDATE votacions SET num_electors = (electors_votacio + censats) WHERE id = id_vota;

		  -- Recorrem el cursor que hem declarat (que cont els ciutadans del cens local que es vol associar a la votaci). Els ciutadans 
		  -- que no estiguin donats de baixa s'inseriran a la taula ciutadansVotacions (que contindr els electors de les votacions).
		  FOR ciutada IN ciutadans_cens LOOP

			IF ( ciutada.baixa_logica = 0 ) THEN
				INSERT INTO ciutadansVotacions(id_ciutada, id_votacio, any_votacio, ha_participat)
					VALUES(ciutada.num_aleatori, id_vota, any_votacio, 0);
			END IF;
			
		  END LOOP;

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN numero_incorrecte THEN

		   RSP := 'ERROR: comprovi que es compleixi el valor dels camps seguents: id_votacio > 0, ciutadans_censats = 0,  num_vots = 0 i/o finalitzada = 0';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_inexistent THEN

		   RSP := 'ERROR: l''identificador de votacio no correspon a cap votacio';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_no_editable THEN

		   RSP := 'ERROR: la votacio no es troba en estat d''edicio. Ja no s''hi pot associar cap nou cens.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN any_incorrecte THEN

		   RSP := 'ERROR: comprovi que l''any tingui 4 digits i que es correspongui amb l''any de la votacio';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN cens_local_inexistent THEN

		   RSP := 'ERROR: l''identificador de cens local no correspon a cap cens local';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN cens_local_regional_erroni THEN

		   RSP := 'ERROR: el cens local no correspon a aquest cens regional. Revisi el cens regional.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN cens_regional_estatal_erroni THEN

		   RSP := 'ERROR: el cens regional no correspon al cens estatal. Revisi el cens estatal';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN nom_localitat_erroni THEN

		   RSP := 'ERROR: el nom de la localitat no es correspon amb el cens local. Revisi el nom de localitat.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN pais_incorrecte THEN

		   RSP := 'ERROR: el nom del pais no es correspon amb el cens estatal. Revisi el nom del pais.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;









	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_actualitzaEstatVotacio
	-- 			
	-- Descripci:		actualitza l'estat d'una votaci i, si el nou estat s 'finalitzada', actualitza un camp de les taules
	-- 			censosAssocitas i estadistiquesVotacions i crida el procediment p_estadistiquesVotacio del paquet pk_estadistiques;
	-- 			finalment, tamb crida el procediment registrarLog del paquet pk_utils. 
	-- 
	-- 			Per actualitzar l'estat d'una votaci, per, noms es podr fer d'un estat previ, a l'immediatament posterior. Aix,
	-- 			totes les votacions hauran de passar pels quatre estats i en aquest ordre:
	-- 			edicio -> pendent obertura -> oberta -> finalitzada
	-- 
	-- 			L'estat 'edicio' s l'estat que hauria de tenir (i que t per defecte) una nova votaci del sistema. Aix que
	-- 			el nou estat d'una votaci mai podr ser 'edicio' (s'ha considerat que no t sentit editar una votaci
	-- 			quan ja est pendent d'obertura (campanya electoral) i encara menys quan est oberta o finalitzada).
	-- 
	-- 			Per altre costat, tamb es tindran en compte les dates de les votacions abans d'actualitzar l'estat d'una
	-- 			votaci. Per exemple, no es pot finalitzar una votaci abans de la data de fi de la mateixa.
	-- 			
	-- 
	-- 			NOTA: Pel canvi d'estat de les votacions, com que tamb es tenen en compte les dades d'inici i de fi de la votaci,
	-- 			s'ha fet una excepci amb les votacions de prova que s'han creat. S'explica amb ms detall en el punt del codi on es fa
	-- 			l'excepci.
	-- 
	-- Parmetres entrada:	id_vota		NUMBER			NOT NULL	REFERENCES votacions(id)
	-- 			nou_estat	VARCHAR2(20 CHAR)	NOT NULL
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori			-> quan un parmetre d'entrada que no pot ser buit o NULL ho s
	-- 			estat_incorrecte		-> quan nou_estat no correspon a cap dels estats acceptats per una votaci
	-- 			votacio_inexistent		-> quan id_vota no correspon a cap votaci existent
	-- 			votacio_finalitzada		-> quan es vol canviar l'estat d'una votaci que ja ha finalitzat
	-- 			votacio_oberta			-> quan es vol canviar l'estat d'una votaci oberta a un estat diferent a 'finalitzada'
	-- 			votacio_pendent_obertura	-> quan es vol canviar l'estat d'una votaci pendent d'obertura a un estat diferent a 'oberta'
	-- 			votacio_edicio			-> quan es vol canviar l'estat d'una votaci en edici a un estat diferent a 'pendent obertura'
	-- 			edicio_pendent_error		-> quan es vol posar en estat 'pendent obertura'una votaci que segons la data d'inici
	-- 							   ja no hi pot estar (hauria d'estar oberta o finalitzada).
	-- 			pendent_oberta_error		-> quan es vol posar en estat 'oberta' una votaci que, d'acord amb les dates de la 
	-- 							   votaci, encara no es pot obrir o ja hauria d'haver acabat.
	-- 			oberta_finalitzada_error	-> quan es vol posar en estat 'finalitzada' una votaci que, segons la data de fi,
	-- 							   encara hauria d'estar 'oberta' (veure nota superior).
	--			OTHERS				-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_actualitzaEstatVotacio (
	  id_vota	IN	votacions.id%TYPE,
	  nou_estat	IN	estatsVotacio.nom_estat%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_actualitzaEstatVotacio';
	  	parametres_entrada 	VARCHAR2(25 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		estat_actual		estatsVotacio.nom_estat%TYPE;
		t_electors		NUMBER := 0;
		titol_v			VARCHAR2(100 CHAR);
		votacio_prova		NUMBER(1) := 0;
		dat_inici		VARCHAR2(10 CHAR);
		dat_fi			VARCHAR2(10 CHAR);
		dat_actual		VARCHAR2(10 CHAR);
		id_nou_estat		NUMBER := 0;
		RESP			VARCHAR2(200 CHAR);
	  	camp_obligatori 		EXCEPTION;
		estat_incorrecte 		EXCEPTION;
		votacio_inexistent		EXCEPTION;
		votacio_finalitzada		EXCEPTION;
		votacio_oberta			EXCEPTION;
		votacio_pendent_obertura	EXCEPTION;
		votacio_edicio			EXCEPTION;
		edicio_pendent_error		EXCEPTION;
		pendent_oberta_error		EXCEPTION;
		oberta_finalitzada_error	EXCEPTION;


		BEGIN

		  parametres_entrada := id_vota||', '||nou_estat;

		  IF (  id_vota IS NULL OR 
			nou_estat IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (	nou_estat NOT IN ('edicio', 'pendent obertura', 'oberta', 'finalitzada') ) THEN
			RAISE estat_incorrecte;
		  END IF;

		  -- comprovem que l'identificador de la votaci existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM votacions WHERE id = id_vota;
		  IF ( trobat = 0 ) THEN
			RAISE votacio_inexistent;
		  END IF;


		  -- obtenim l'estat actual de la votacio
		  SELECT ev.nom_estat INTO estat_actual
			FROM estatsVotacio ev, votacions v
			WHERE ev.id = v.id_estat AND v.id = id_vota;

		  -- comprovem que el nou estat correspongui a l'estat segent de l'estat actual (una votaci ha de tenir
		  -- els segents estats i en aquest ordre: edicio -> pendent obertura -> oberta -> finalitzada)
		  IF ( estat_actual = 'finalitzada' ) THEN
			RAISE votacio_finalitzada;

		  ELSIF ( estat_actual = 'oberta' AND nou_estat <> 'finalitzada' ) THEN
			RAISE votacio_oberta;

		  ELSIF ( estat_actual = 'pendent obertura' AND nou_estat <> 'oberta' ) THEN
			RAISE votacio_pendent_obertura;

		  ELSIF ( estat_actual = 'edicio' AND nou_estat <> 'pendent obertura' ) THEN
			RAISE votacio_edicio;

		  END IF;


	-- obtenim el titol de la votaci i, si coincideix amb el d'alguna de les votacions de prova de les que tenen censos
	-- associats, posem votacio_prova a 1 i aix podrem esquivar algunes les dues prximes condicions.
	-- Aix servir perqu es pugui posar l'estat 'oberta' i 'pendent obertura' a les votacions de prova que s'han creat i que
	-- tenen la data d'inici i de fi menors a la data actual. s a dir, per la resta de votacions, no es podr posar a estat
	-- 'pendent obertura' una votaci que segons la data d'inici ja hauria d'estar oberta ni a estat 'oberta' una votaci que
	-- segons la data d'inici encara no es pot obrir (o segons la data de fi la votaci ja hauria d'estar tancada).
	SELECT titol INTO titol_v FROM votacions WHERE id = id_vota;
	IF ( titol_v  IN( 'Eleccions al municipi de Sevilla de 2 de gener de 2000',
			  'Eleccions al municipi de Munich de 3 de gener de 2000', 
			  'Eleccions al municipi de Roma de 2 de gener de 2001',
			  'Eleccions al municipi de Santander de 3 de gener de 2001',
			  'Eleccions al municipi de Berlin de 2 de gener de 2002',
			  'Eleccions al municipi de Toledo de 3 de gener de 2002',  
			  'Eleccions al municipi de Florencia de 2 de gener de 2003', 
			  'Eleccions al municipi de Madrid de 3 de gener de 2003', 
			  'Eleccions al municipi de Barcelona de 2 de gener de 2004',
 			  'Eleccions al municipi de Murcia de 3 de gener de 2004',  
			  'Eleccions al municipi de Dusseldorf de 2 de gener de 2005', 
			  'Eleccions al municipi de Girona de 3 de gener de 2005',
			  'Eleccions al municipi de Bolonya de 2 de gener de 2006', 
			  'Eleccions al municipi de Murcia de 3 de gener de 2006', 
			  'Eleccions al municipi de Hannover de 2 de gener de 2007',
			  'Eleccions al municipi de Florencia de 3 de gener de 2007',
			  'Eleccions al municipi de Barcelona de 2 de gener de 2008',
			  'Eleccions al municipi de Madrid de 3 de gener de 2008',  
			  'Eleccions al municipi d''Olot de 2 de gener de 2010',
			  'Eleccions al municipi de Girona de 3 de gener de 2010',
			  'Eleccions al Parlament de Catalunya de 1 de novembre de 2011', 
			  'Eleccions a Corts Generals de 2 de novembre de 2011', 
			  'Eleccions al Parlament Europeu de 3 de novembre de 2011') ) THEN
		votacio_prova := 1;
	END IF;


		  -- obtenim el nmero d'electors
		  SELECT num_electors INTO t_electors FROM votacions WHERE id = id_vota;

		  -- obtenim les dates d'inici i de fi de la votaci i la data actual del sistema i els apliquem un format que ens permeti comparar-les.
		  SELECT data_inici INTO dat_inici FROM votacions WHERE id = id_vota;
		  SELECT data_fi INTO dat_fi FROM votacions WHERE id = id_vota;
		  SELECT SYSDATE INTO dat_actual FROM DUAL;
		  SELECT TO_CHAR(TO_DATE(dat_inici), 'yyyymmdd') INTO dat_inici FROM DUAL;
		  SELECT TO_CHAR(TO_DATE(dat_fi), 'yyyymmdd') INTO dat_fi FROM DUAL;
		  SELECT TO_CHAR(TO_DATE(dat_actual), 'yyyymmdd') INTO dat_actual FROM DUAL;


		  -- comprovem que el nou estat sigui aplicable en funci de la data actual del sistema, el nombre d'eletors i les dates d'inici 
		  -- i de fi de la votaci
		  IF ( nou_estat = 'pendent obertura' AND dat_actual > dat_inici AND votacio_prova = 0 ) THEN
			RAISE edicio_pendent_error;

		  ELSIF ( nou_estat = 'oberta' AND (t_electors = 0 OR dat_actual < dat_inici OR dat_actual > dat_fi) AND votacio_prova = 0) THEN
			RAISE pendent_oberta_error;

		  ELSIF ( nou_estat = 'finalitzada' AND dat_actual < dat_fi ) THEN
			RAISE oberta_finalitzada_error;

		  END IF;

		  -- obtenim l'identificador del nou estat que tindr la votaci
		  SELECT id INTO id_nou_estat FROM estatsVotacio WHERE nom_estat = nou_estat;


		  -- s'actualitza l'estat de la votaci
		  UPDATE votacions SET id_estat = id_nou_estat WHERE id = id_vota;

		  -- si el nou estat s 'finalitzada' s'actualitza aquest camp a les taules censosAssociats i estadistiquesVotacions
		  -- i es crida al procediment que actualitzar totes les estadstiques
		  IF ( nou_estat = 'finalitzada' ) THEN
			UPDATE censosAssociats SET finalitzada = 1 WHERE id_votacio = id_vota;
			UPDATE estadistiquesVotacions SET finalitzada = 1 WHERE id_votacio = id_vota;
			pk_estadistiques.p_estadistiquesVotacio(id_vota, RESP);
		  END IF;


		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN estat_incorrecte THEN

		   RSP := 'ERROR: nou estat incorrecte. Els estats que pot tenir una votacio son: edicio, pendent obertura, oberta o finalitzada';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_inexistent THEN

		   RSP := 'ERROR: l''identificador de votacio no correspon a cap votacio';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN votacio_finalitzada THEN

		   RSP := 'ERROR: La votacio esta finalitzada. No es pot canviar l''estat d''una votacio que ja s''ha celebrat.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_oberta THEN

		   RSP := 'ERROR: La votacio esta oberta. El nou estat nomes pot ser ''finalitzada''.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_pendent_obertura THEN

		   RSP := 'ERROR: La votacio esta pendent d''obertura. El nou estat nomes pot ser ''oberta''.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_edicio THEN

		   RSP := 'ERROR: La votacio esta en edicio. El nou estat nomes pot ser ''pendent obertura''.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN edicio_pendent_error THEN

		   RSP := 'ERROR: la votacio no es pot posar en estat ''pendent obertura''. Segons la data d''inici aquesta votacio s''hauria d''estar celebrant o ja hauria d''haver finalitzat.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN pendent_oberta_error THEN

		   RSP := 'ERROR: la votacio no es pot posar en estat ''oberta''. Pot ser: a) no te censos associats o b) les dates d''inici i/o fi no ho permeten.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN oberta_finalitzada_error THEN

		   RSP := 'ERROR: la votacio no es pot posar en estat ''finalitzada''. Segons la data de fi aquesta votacio encara no es pot finalitzar';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;







	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaVotEmes
	-- 			
	-- Descripci:		i crida el procediment p_registrarLog del paquet pk_utils
	-- 
	-- 			Nota: en registrar el vol a la taula vots emesos es far amb el valor de la variable id_opcio_final en comptes de fer-ho
	-- 			amb el valor del parmetre 'opcio'. En la majoria dels casos id_opcio_final tindr el mateix valor que el parmetre
	-- 			'opcio', per hi ha un cas en qu un vot vlid (dintre les opcions de la votaci) es considerar nul perqu l'est
	-- 			exercint un ciutad d'un pas en el qual no hi ha la possibilitat d'escollir aquella opci de vot. En aquests casos
	-- 			es considerar i comptabilitzar que l'elector ha votat, per el seu vot ser comptabilitzat com a nul.
	-- 			
	-- Parmetres entrada:	ciutada		NUMBER		PKcomposta1	REFERENCES ciutadans(num_aleatori)
	-- 			votacio		NUMBER		PKcomposta2	REFERENCES votacions(id)
	-- 			opcio		NUMBER		NOT NULL	REFERENCES opcionsVotacio(id)
	-- 			data_hota	TIMESTAMP(0)	NOT NULL
	-- 			
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori			-> quan un parmetre d'entrada que no pot ser buit o NULL ho s
	-- 			camp_incorrecte			-> quan els parmetres ciutada, votacio i/o opcio no sn nombres naturals diferents de zero
	-- 			votacio_inexistent		-> quan el parmetre 'votacio' no correspon a cap votaci
	-- 			elector_inexistent		-> quan el parmetre 'ciutada' no correspon a cap elector per la votaci
	-- 			opcio_inexistent		-> quan el parmetre 'opcio' no correspon a cap opcio de la votaci
	-- 			votacio_no_oberta		-> quan el parmetre 'votacio' correspon a una votaci que no est oberta
	-- 			ciutada_baixa			-> quan el parmetre 'ciutada' correspon a un ciutad que ha estat donat de baixa
	--			OTHERS				-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaVotEmes (
	  ciutada	IN	votsEmesos.id_ciutada%TYPE,
	  votacio	IN	votsEmesos.id_votacio%TYPE,
	  opcio		IN	votsEmesos.id_opcio%TYPE,
	  data_hora	IN	votsEmesos.data_hora%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_altaVotEmes';
	  	parametres_entrada 	VARCHAR2(50 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		elector_baixa		NUMBER(1) := 0;
		id_estat_votacio	NUMBER(1) := 0;
		estat_votacio		VARCHAR2(20 CHAR);
		nom_pais_elector	VARCHAR2(30 CHAR);
		cens_elector		VARCHAR2(40 CHAR);
		codi_pais_elector	CHAR(2);
		codi_pais_opcio		CHAR(2);
		id_opcio_final		NUMBER := 0;
		participacions_ciutada	NUMBER := 0;
		n_vots_censAssociat	NUMBER := 0;
		n_vots_opcioVotacio	NUMBER := 0;
	  	camp_obligatori 		EXCEPTION;
		camp_incorrecte 		EXCEPTION;
		votacio_inexistent		EXCEPTION;
		elector_inexistent		EXCEPTION;
		opcio_inexistent		EXCEPTION;
		votacio_no_oberta		EXCEPTION;
		ciutada_baixa			EXCEPTION;

		BEGIN

		  parametres_entrada := ciutada||', '||votacio||', '||opcio||', '||data_hora;


		  IF (  ciutada IS NULL OR 
			votacio IS NULL OR
			opcio IS NULL OR
			data_hora IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;


		  IF (	ciutada <= 0 OR
			votacio <= 0 OR
			opcio <= 0) THEN
			RAISE camp_incorrecte;
		  END IF;

		  -- comprovem que l'identificador de la votaci existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM votacions WHERE id = votacio;
		  IF ( trobat = 0 ) THEN
			RAISE votacio_inexistent;
		  END IF;

		  -- comprovem que el ciutad sigui un elector de la votaci. Es comprovar si el ciutad existeix a la taula ciutadansVotacions.
		  -- Aquesta taula s'omple automticament al associar censos a les votacions de manera que si el ciutad hi existeix ser elector i
		  -- el cens local del ciutad ser un cens associat a la votaci
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM ciutadansVotacions WHERE id_ciutada = ciutada AND id_votacio = votacio;
		  IF ( trobat = 0 ) THEN
			RAISE elector_inexistent;
		  END IF;

		  -- comprovem que la opci de vot estigui permesa en aquesta votaci		  
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM opcionsVotacio WHERE id_votacio = votacio AND id_opcio = opcio;
		  IF ( trobat = 0 ) THEN
			RAISE opcio_inexistent;
		  END IF;

		  -- comprovem que la votaci estigui en estat 'oberta'
		  SELECT id_estat INTO id_estat_votacio FROM votacions WHERE id = votacio;
		  SELECT nom_estat INTO estat_votacio FROM estatsVotacio WHERE id = id_estat_votacio;
		  IF ( estat_votacio <> 'oberta' ) THEN
			RAISE votacio_no_oberta;
		  END IF;

		  -- comprovem que el ciutada elector no estigui donat de baixa al sistema
		  SELECT baixa_logica, nom_pais, num_participacions, id_cens_local 
			INTO elector_baixa, nom_pais_elector, participacions_ciutada, cens_elector
			FROM ciutadans 
			WHERE num_aleatori = ciutada;
		  IF ( elector_baixa = 1 ) THEN
			RAISE ciutada_baixa;
		  END IF;

		  -- fetes les verificacions prvies (el ciutad es pot considerar un elector vlid, la votaci existeix i est oberta, 
		  -- la opci s permesa per la votaci), es fan un seguit d'actualitzacions abans de registrar el vot:

		  -- registrem, a la taula ciutadansVotacions on hi ha els electors de cada votaci, que l'elector ha participat.
		  UPDATE ciutadansVotacions SET ha_participat = 1 WHERE id_ciutada = ciutada AND id_votacio = votacio;

		  -- comprovem que la opci de vot (que ja s'ha comprovat que s vlida per aquesta votaci) pugui ser escollida per
		  -- l'elector que exerceix el vot (podr ser escollida si la opci s d'un partit del pas on pertany l'elector).
		  -- En cas que sigui una opci d'un pas diferent al de l'elector es comptabilitzar com un vot, per nul.
		  -- 	Notes:	1) nom_pais_elector s'ha obtingut a la consulta anterior
		  -- 		2) el codi pais 'ZQ' s el codi de pas neutre que fan servir les opcions de vot ('SI', 'NO', 'BLANC' i 'NUL').
		  SELECT codi_pais INTO codi_pais_elector FROM paisos WHERE nom_pais = nom_pais_elector;
		  SELECT codi_pais INTO codi_pais_opcio FROM opcions WHERE id = opcio;
		  IF ( codi_pais_elector <> codi_pais_opcio AND codi_pais_opcio <> 'ZQ' ) THEN
			SELECT id INTO id_opcio_final FROM opcions WHERE nom_opcio = 'NUL';
		  ELSE
			id_opcio_final := opcio;
		  END IF;


		  -- actualitzem el nmero de participacions del ciutad (participacions_ciutada s'ha obtingut en una consulta anterior)
		  UPDATE ciutadans SET num_participacions = (participacions_ciutada + 1) WHERE num_aleatori = ciutada;


		  -- incrementem en 1 el nmero de vots exercits en el censAssociat al qual pertany l'elector que emet el vot (cens elector 
		  -- s'ha obtingut en una consulta anterior).
		  SELECT num_vots INTO n_vots_censAssociat FROM censosAssociats WHERE id_cens_local = cens_elector AND id_votacio = votacio;
		  UPDATE censosAssociats SET num_vots = (n_vots_censAssociat + 1) WHERE id_cens_local = cens_elector AND id_votacio = votacio;


		  -- incrementem en 1 el nmero de vots obtinguts per la opci que ha escollit l'elector (o per la opci NUL, en cas que l'elector
		  -- hagi escollit una opci en la qual no podia exercir el vot -com s'ha comentat abans-).
		  SELECT num_vots INTO n_vots_opcioVotacio FROM opcionsVotacio WHERE id_opcio = id_opcio_final AND id_votacio = votacio;
		  UPDATE opcionsVotacio SET num_vots = (n_vots_opcioVotacio + 1) WHERE id_opcio = id_opcio_final AND id_votacio = votacio;

		  -- registrem el vot
		  INSERT INTO votsEmesos(id_ciutada, id_votacio, id_opcio, data_hora) VALUES(ciutada, votacio, opcio, data_hora);


		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN camp_incorrecte THEN

		   RSP := 'ERROR: El valor dels camps id_ciutada, id_votacio i/o id_opcio han de ser un nombre natural diferent de zero.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_inexistent THEN

		   RSP := 'ERROR: l''identificador de votacio no correspon a cap votacio';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN elector_inexistent THEN

		   RSP := 'ERROR: l''identificador aleatori del ciutada no correspon a cap elector.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN opcio_inexistent THEN

		   RSP := 'ERROR: la opcio de vot escollida no existeix per aquesta votacio.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_no_oberta THEN

		   RSP := 'ERROR: No es pot votar. La votacio no esta oberta.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN ciutada_baixa THEN

		   RSP := 'ERROR: intent de vot d''un ciutada que ha estat donat de baixa.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;










	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_baixaVotacio
	-- 			
	-- Descripci:		esborra un registre a la taula votacions que provocar l'esborrat d'un registre de la taula votacionsPaisos i
	-- 			d'un altre de la taula estadistiquesVotacions (que encara no estaven calculades). En, en el cas que la votaci 
	-- 			ja tingui opcions, censos i/o ciutadans associats, aquests s'esborraran per efecte de l'esborrat en cascada 
	-- 			(que permet preservar la integritat referencial) que es controlar pel que s'explica a la nota segent:
	-- 
	-- 			NOTA: Per tal de que, com a conseqncia de l'esborrat en cascada, no s'esborrin dades que haurien de ser fermes a 
	-- 			la BD (vots emesos, estadstiques, etc.), no es permetr esborrar cap votaci que estigui en estat 'oberta' ni en
	-- 			estat 'finalitzada'. En aquests dos estat la votaci ja pot tenir vots emesos (i en el cas de 'finalitzada' 
	-- 			resultats estadstics calculats) i no t cap sentit perdre aquesta informaci. Esborrar o modificar un vot ems
	-- 			seria un frau electoral i perdre dades estadstiques una llstima.
	-- 
	-- Parmetres entrada:	titol_vota	VARCHAR2(200 CHAR) 	Nota: el titol d'una votaci s clau alternativa
	-- 
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan titol_vota s buit o NULL
	-- 			llargada_incorrecte	-> quan el ttol de la votaci s massa llarg
	-- 			votacio_inexistent	-> quan titol_vota no referencia cap votacio
	-- 			estat_vota_no_permet	-> quan s'intenta esborrar una votaci que est 'oberta' o 'finaltzada'
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_baixaVotacio (
	  titol_vota	IN	votacions.titol%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_baixaVotacio';
	  	parametres_entrada 	VARCHAR2(400 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		id_estat_vota		NUMBER := 0;
		estat_vota		VARCHAR2(20 CHAR);
	  	camp_obligatori 	EXCEPTION;
		llargada_incorrecte 	EXCEPTION;
		votacio_inexistent	EXCEPTION;
		estat_vota_no_permet	EXCEPTION;


		BEGIN

		  parametres_entrada := titol_vota;


		  IF (  titol_vota IS NULL ) THEN
			RAISE camp_obligatori;
		  END IF;

		  IF ( length(titol_vota) > 200 ) THEN
			RAISE llargada_incorrecte;
		  END IF;

		  -- comprovem que la votacio existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM votacions WHERE titol = titol_vota;
		  IF (trobat = 0) THEN
			RAISE votacio_inexistent;
		  END IF;

		  -- comprovem que l'estat de la votaci permeti esborrar-la (no es permet en estat 'oberta' o 'finalitzada')
		  SELECT id_estat INTO id_estat_vota FROM votacions WHERE titol = titol_vota;
		  SELECT nom_estat INTO estat_vota FROM estatsVotacio WHERE id = id_estat_vota;

		  IF ( estat_vota IN ('oberta', 'finalitzada') ) THEN
			RAISE estat_vota_no_permet;
		  END IF;
		  

		  -- Si la votaci no est 'oberta' o 'finalitzada' es pot esborrar. Aix esborrar el seu registre corresponent de la
		  -- taula votacionsPaisos i el de la taula estaditiquesVotacions (que encara no estaven calculades perqu la votaci 
		  -- no havia finalitzat). Tamb implicar l'esborrat de les possibles associacions que se li haguessin fet de 
		  -- opcions/censos/ciutadans.

		  DELETE FROM votacions WHERE titol = titol_vota;


		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: el titol de la votacio es buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN llargada_incorrecte THEN

		   RSP := 'ERROR: el titol de la votacio no pot tenir mes de 200 caracters.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_inexistent THEN

		   RSP := 'ERROR:  no es pot donar de baixa una votacio que no existeix.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN estat_vota_no_permet THEN

		   RSP := 'ERROR: no es pot esborrar una votacio que s''esta celebrant o que hagi finalitzat.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;








	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_baixaOpcio
	-- 			
	-- Descripci:		posa a 1 l'atribut baixa_logica d'un registre de la taula opcions i crida el procediment p_registrarLog del 
	-- 			paquet pk_utils. S'ha considerat que no es pot donar de baixa una opci que est present en una votaci
	-- 			que s'estigui celebrant (en estat 'oberta'). Fetes unes quantes verificacions, crida el procediment
	-- 			p_registrarLog del paquet pk_utils.
	-- 			
	-- Parmetres entrada:	nom		VARCHAR2(15 CHAR)
	-- 			cod_pais	CHAR(2)			NOT NULL	REFERENCES codisISO(codi_pais)
	-- 			
	-- 			NOTA: (nom, cod_pais) s clau alternativa de 'opcions'
	-- 
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada que no pot ser buit o NULL ho s
	-- 			llargada_camp		-> quan el parmetre nom t ms de 15 i/o quan el parmetre codi_pais no t 
	-- 						   exactament 2 carcters
	-- 			codi_pais_inexistent	-> quan cod_pais no referencia el codi ISO de cap pais (incls el pas neuter 'ZQ')
	-- 			nom_opcio_inexistent	-> quan nom no referencia el nom de cap opci del sistema
	-- 			opcio_codi_inexistent	-> quan la combinaci nom+cod_pais no referencien la clau alternativa de 'opcions'
	-- 			opcio_baixa		-> quan la opci ja estava de baixa
	-- 			opcio_de_votacio_oberta	-> quan la opci que es vol donar de baixa est present en una votaci que s'est celebrant
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_baixaOpcio (
	  nom		IN	opcions.nom_opcio%TYPE,
	  cod_pais	IN	opcions.codi_pais%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_baixaOpcio';
	  	parametres_entrada 	VARCHAR2(40 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER := 0;
		baixa			NUMBER(1) := 0;
		id_opc			NUMBER := 0;
		id_estat_oberta		NUMBER := 0;
		id_estat_vota		NUMBER := 0;
	  	camp_obligatori 	EXCEPTION;
		llargada_camp		EXCEPTION;
		codi_pais_inexistent	EXCEPTION;
		nom_opcio_inexistent	EXCEPTION;
		opcio_codi_inexistent	EXCEPTION;
		opcio_baixa		EXCEPTION;
		opcio_de_votacio_oberta	EXCEPTION;

		BEGIN

		  parametres_entrada := nom||', '||cod_pais;

		  IF (	nom IS NULL OR
			cod_pais IS NULL ) THEN
			RAISE camp_obligatori;
		  END IF;

		  IF (	length(nom) > 15 OR
			length(cod_pais) <> 2 ) tHEN
			RAISE llargada_camp;
		  END IF;


		  -- comprovem que codi_pais referenci un pas del sistema o al codi de pas neutre 'ZQ'
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM codisISO WHERE codi_pais = cod_pais;
		  IF ( trobat = 0 ) THEN
			RAISE codi_pais_inexistent;
		  END IF;

		 
		  -- comprovem que 'nom' referenci un nom d'alguna opci de la taula 'opcions'
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM opcions WHERE nom_opcio = nom;
		  IF ( trobat = 0 ) THEN
			RAISE nom_opcio_inexistent;
		  END IF;


		  -- comprovem que nom+cod_pais referencin la clau alternativa d'opcions
		  trobat := 0;
		  FOR opcions IN ( SELECT nom_opcio FROM opcions WHERE codi_pais = cod_pais ) LOOP

			trobat := trobat;
			IF ( trobat = 0 ) THEN

				IF ( opcions.nom_opcio = nom ) THEN
					trobat := 1;
				END IF;

			END IF;

		  END LOOP;


		  IF ( trobat = 0 ) THEN
			RAISE opcio_codi_inexistent;
		  END IF;

		  -- comprovem que la opci no estigus ja de baixa
		  SELECT baixa_logica INTO baixa FROM opcions WHERE nom_opcio = nom AND codi_pais = cod_pais;
		  IF ( baixa = 1 ) THEN
			RAISE opcio_baixa;
		  END IF;


		  -- comprovem que no hi hagus cap votaci amb estat 'oberta' que tingus aquesta opci (considero
		  -- que no es pot donar de baixa una opci en ple procs de votaci)
		  SELECT id INTO id_estat_oberta FROM estatsVotacio WHERE nom_estat = 'oberta';
		  SELECT id INTO id_opc FROM opcions WHERE nom_opcio = nom AND codi_pais = cod_pais;

		  -- recorrem totes les votacions que tinguin associada la opci que es vol donar de baixa i 
		  -- comprovarem si alguna est en estat 'oberta'
		  trobat := 0;
		  FOR votacio IN ( SELECT id_votacio FROM opcionsVotacio WHERE id_opcio = id_opc) LOOP

			trobat := trobat;
			IF (trobat = 0) THEN

				SELECT id_estat INTO id_estat_vota FROM votacions WHERE id = votacio.id_votacio;
				IF ( id_estat_vota = id_estat_oberta ) THEN
					trobat := 1;
				END IF;

			END IF;			

		  END LOOP;


		  IF ( trobat = 1 ) THEN
			RAISE opcio_de_votacio_oberta;
		  END IF;


		  -- superades les verificacions anteriors, es pot donar de baixa lgica la opci
		  UPDATE opcions SET baixa_logica = 1 WHERE nom_opcio = nom AND codi_pais = cod_pais;



		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN llargada_camp THEN

		   RSP := 'ERROR: nombre de caracters inapropiat. Revisi la llargada dels camps nom_opcio i/o codi_pais.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN codi_pais_inexistent THEN

		   RSP := 'ERROR: el codi del pais no referencia cap dels codis ISO assignats als paisos, ni al codi del pais neutre ''ZQ''.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN nom_opcio_inexistent THEN

		   RSP := 'ERROR: el nom de la opcio no existeix a la taula ''opcions''';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN opcio_codi_inexistent THEN

		   RSP := 'ERROR: el nom de la opcio i el codi del pais no referencien cap opcio valida.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		WHEN opcio_baixa THEN

		   RSP := 'ERROR: quan la opcio ja havia estat donada de baixa.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN opcio_de_votacio_oberta THEN

		   RSP := 'ERROR: actualment no es pot donar de baixa la opcio ja que esta associada a una votacio en estat ''oberta'' ';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		END;













	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_baixaCensAssociat
	-- 			
	-- Descripci:		Una vegada fetes unes quantes verificacions prvies, el procediment porta a terme diverses accions:
	-- 			   a) si a la taula censosAssociats no hi ha cap altre cens associat a la votaci que sigui del mateix pas que
	-- 			      el cens id_cens_loc s'esborra un registre de la taula votacionsPaisos que tingui l'identificador del pas 
	-- 			      al qual pertany el cens id_cens_loc i l'identificador de la votaci id_vota.
	-- 			   b) Obt el n de ciutadans censats actualment en el cens associat que es vol donar de baixa, id_cens_loc. 
	-- 			      Aquest valor servir per actualitzar el nmero d'electors associats a la votaci (punt c)
	-- 			   c) Actualitza el nmero d'electors de la votaci.
	-- 			   d) Esborra tots els ciutadans de la taula ciutadansVotacions que fossin electors de la votaci  id_vota i 
	-- 			      ciutadans del cens id_cens_loc.
	-- 			   e) Dona de baixa el cens id_cens_loc a la taula censosAssociats (taula que registra els censos associats a 
	-- 			      cada votaci) per la votaci id_vota
	-- 			      amb totes les dades rebudes per parmetre a excepci del parmetre ciut_censats que no es tindr en compte
	-- 			      ja que el nmero de ciutadans censats s'obt en aquest procediment (punt b).
	-- 			   f) Crida el procediment p_registrarLog del paquet pk_utils.
	-- 
	-- 			NOTA:	El procediment noms deixa donar de baixa censos a les votacions que estan en estat d'edici 
	-- 
	-- 			
	-- Parmetres entrada:	id_vota		NUMBER				PK_composta1	REFERENCES votacions(id)
	-- 			id_cens_loc	VARCHAR2(40 CHAR)		PK_composta2	REFERENCES censosLocals(nom_cens)
	-- 
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori			-> quan un parmetre d'entrada que no pot ser buit o NULL ho s
	-- 			llargada_camp			-> quan id_cens_loc t ms de 40 carcters
	-- 			numero_incorrecte		-> quan l'id_votacio no s un natural i/o quan ciut_censats, n_vots o
	-- 							   finalitzada no siguin zero
	-- 			votacio_inexistent		-> quan id_vota no correspon a cap votaci existent
	-- 			cens_local_inexistent		-> quan id_cens_local no correspon a cap cens local existent
	-- 			cens_no_associat		-> quan el cens associat que es vol donar de baixa, realment no est associat a la votaci
	-- 			votacio_no_editable		-> quan la votaci no est en estat 'edicio' (en altres estats no s'hi pot associar 
	-- 							   cap cens)
	--			OTHERS				-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_baixaCensAssociat (
	  id_vota	IN	censosAssociats.id_votacio%TYPE,
	  id_cens_loc	IN	censosAssociats.id_cens_local%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure		VARCHAR2(30 CHAR) := 'p_baixaCensAssociat';
	  	parametres_entrada	VARCHAR2(80 CHAR) := NULL;
		esCorrecte		NUMBER(1) := 1;
		confirmada		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		num_censos_pais		NUMBER := 0;
		estat_votacio		VARCHAR2(20 CHAR) := 0;
		pais			VARCHAR2(30 CHAR);
		censats			NUMBER := 0;
		electors_votacio	NUMBER := 0;
	  	camp_obligatori			EXCEPTION;
		llargada_camp			EXCEPTION;
		numero_incorrecte		EXCEPTION;
		votacio_inexistent		EXCEPTION;
		cens_local_inexistent		EXCEPTION;
		cens_no_associat		EXCEPTION;
		votacio_no_editable		EXCEPTION;

		-- declarem un cursor que obtindr els nmeros aleatoris del ciutadans associats al cens local que es vol donar de baixa
		-- i que ser til per esborrar-los de la taula ciutadansVotacions.
		CURSOR ciutadans_cens IS SELECT num_aleatori FROM ciutadans WHERE id_cens_local = id_cens_loc;


		BEGIN

		  parametres_entrada := id_vota||', '||id_cens_loc;

		  IF (  id_vota IS NULL OR 
			id_cens_loc IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;


		  IF ( length(id_cens_loc) > 40 ) THEN
			RAISE llargada_camp;
		  END IF;


		  IF (	id_vota <= 0  ) THEN
			RAISE numero_incorrecte;
		  END IF;


		  -- comprovem que l'identificador de la votaci existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM votacions WHERE id = id_vota;
		  IF ( trobat = 0 ) THEN
			RAISE votacio_inexistent;
		  END IF;


		  -- comprovem que l'identificador de cens local existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM censosLocals WHERE nom_cens = id_cens_loc;
		  IF ( trobat = 0 ) THEN
			RAISE cens_local_inexistent;
		  END IF;


		  -- comprovem que el cens estigui associat a la votacio
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM censosAssociats WHERE id_votacio = id_vota AND id_cens_local = id_cens_loc;
		  IF ( trobat = 0 ) THEN
			RAISE cens_no_associat;
		  END IF;


		  -- comprovem que la votaci encara sigui editable (en estat 'edicio'). Si no ho s, no es podr donar de baixa el cens.
		  SELECT ev.nom_estat INTO estat_votacio 
			FROM estatsVotacio ev, votacions v
			WHERE ev.id = v.id_estat AND v.id = id_vota;
		  IF ( estat_votacio <> 'edicio' ) THEN
			RAISE votacio_no_editable;
		  END IF;


		  -- comprovem si hi ha algun altre cens del mateix pas que id_cens_loc que estigui associat a aquesta votaci. Si no 
		  -- n'hi ha cap ms, caldr esborrar el registre del pas del cens que es desassocia a la taula votacionsPaisos
		  SELECT nom_pais INTO pais FROM censosAssociats WHERE id_votacio = id_vota AND id_cens_local = id_cens_loc;
		  SELECT COUNT(*) INTO num_censos_pais FROM censosAssociats WHERE id_votacio = id_vota AND nom_pais = pais AND id_cens_local <> id_cens_loc;
		  IF ( num_censos_pais = 0 ) THEN
			DELETE FROM votacionsPaisos WHERE id_pais = pais AND id_votacio = id_vota;
		  END IF;


		  -- Obtenim el nmero de ciutadans censats que tenia el cens associat en el moment de donar-lo d'alta a la taula censosAssociats
		  SELECT ciutadans_censats INTO censats FROM censosAssociats WHERE id_votacio = id_vota AND id_cens_local = id_cens_loc;

		  -- actualizem el nmero d'electors de la votaci
		  SELECT num_electors INTO electors_votacio FROM votacions WHERE id = id_vota;
		  UPDATE votacions SET num_electors = (electors_votacio - censats) WHERE id = id_vota;

		  -- eliminiem, de la taula ciutadansVotacions, tots els electors la votaci id_vota del cens que donem de baixa
		  -- Recorrem el cursor que hem declarat (que cont l'identificador dels ciutadans del cens local que es desassociar de la votaci).
		  FOR ciutada IN ciutadans_cens LOOP

			trobat := 0;
			SELECT COUNT(*) INTO trobat FROM ciutadansVotacions WHERE id_ciutada = ciutada.num_aleatori AND id_votacio = id_vota;
			IF ( trobat = 1 ) THEN
				DELETE FROM ciutadansVotacions WHERE id_ciutada = ciutada.num_aleatori AND id_votacio = id_vota;
			END IF;
			
		  END LOOP;



		  -- Donem de baixa, a la taula censosAssociats,  el cens associat a la votaci
		  DELETE FROM censosAssociats WHERE id_votacio = id_vota AND id_cens_local = id_cens_loc;



		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN llargada_camp THEN

		   RSP := 'ERROR: l''identificador del cens es massa llarg.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN numero_incorrecte THEN

		   RSP := 'ERROR: l''identificador de la votacio no pot ser <= 0';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_inexistent THEN

		   RSP := 'ERROR: l''identificador de votacio no correspon a cap votacio';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN cens_local_inexistent THEN

		   RSP := 'ERROR: l''identificador de cens local no correspon a cap cens local';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN cens_no_associat THEN

		   RSP := 'ERROR: el cens que es vol ''desassociar'' no esta associat a la votacio.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_no_editable THEN

		   RSP := 'ERROR: la votacio no es troba en estat d''edicio. Ja no s''hi pot associar cap nou cens.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;













	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_baixaOpcioVotacio
	-- 			
	-- Descripci:		esborra un registre a la taula opcionsVotacio i crida el procediment p_registrarLog del paquet pk_utils
	-- 			
	-- Parmetres entrada:	id_vota		NUMBER			NOT NULL	REFERENCES votacions(id)
	-- 			id_opcio_v	NUMBER			NOT NULL	REFERENCES opcions(id)
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada que no pot ser buit o NULL ho s
	-- 			valor_incorrecte	-> quan id_vota i id_opcio_v no sn naturals > 0
	-- 			votacio_inexistent	-> quan id_vota no referencia cap votaci del sistema
	-- 			opcio_inexistent	-> quan id_opcio_v no referencia cap opci del sistema
	-- 			opcio_de_baixa		-> quan id_opcio_v referencia una opcio que ha estat donada de baixa
	-- 			opcio_no_associada	-> quan id_opcio_v no estava associada realment a id_vota
	-- 			votacio_no_editable	-> quan id_vota referencia una votaci a la qual ja no es poden esborrar opcions
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_baixaOpcioVotacio (
	  id_vota	IN	opcionsVotacio.id_votacio%TYPE,
	  id_opcio_v	IN	opcionsVotacio.id_opcio%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_baixaOpcioVotacio';
	  	parametres_entrada 	VARCHAR2(60 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		id_estat_vota		NUMBER := 0;
		estat_vota		VARCHAR2(20 CHAR);
	  	camp_obligatori 	EXCEPTION;
		valor_incorrecte	EXCEPTION;
		votacio_inexistent	EXCEPTION;
		opcio_inexistent	EXCEPTION;
		opcio_no_associada	EXCEPTION;
		votacio_no_editable	EXCEPTION;



		BEGIN

		  parametres_entrada := id_vota||', '||id_opcio_v;

		  IF (  id_vota IS NULL OR
			id_opcio_v IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (	id_vota <= 0 OR
			id_opcio_v <= 0 ) THEN
			RAISE valor_incorrecte;
		  END IF;

		  -- comprovem que la votaci existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM votacions WHERE id = id_vota;
		  IF ( trobat = 0 ) THEN
			RAISE votacio_inexistent;
		  END IF;


		  -- comprovem que la opci existeixi
		  trobat := 0;
		  SELECT COUNT (*) INTO trobat FROM opcions WHERE id = id_opcio_v;
		  IF ( trobat = 0 ) THEN
			RAISE opcio_inexistent;
		  END IF;

		  -- comprovem que la opci de vot realment estigui associada a la votaci
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM opcionsVotacio WHERE id_votacio = id_vota AND id_opcio = id_opcio_v;
		  IF ( trobat = 0 ) THEN
			RAISE opcio_no_associada;
		  END IF;


		  -- comprovem que la votaci estigui en estat 'edicio'
		  SELECT id_estat INTO id_estat_vota FROM votacions WHERE id = id_vota;
		  SELECT nom_estat INTO estat_vota FROM estatsVotacio	WHERE id = id_estat_vota;
		  IF ( estat_vota <> 'edicio' ) THEN
			RAISE votacio_no_editable;
		  END IF;


		  -- si tot s correcte donem de baixa la opci de la votaci
		  DELETE FROM opcionsVotacio WHERE id_votacio = id_vota AND id_opcio = id_opcio_v;

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN valor_incorrecte THEN

		   RSP := 'ERROR: els camps id_opcio i id_votacio han de ser nombres naturals diferents a zero';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_inexistent THEN

		   RSP := 'ERROR: no es pot afegir una opcio de vot a una votacio que no existeix';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN opcio_inexistent THEN

		   RSP := 'ERROR: la opcio de vot no existeix al sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN opcio_no_associada THEN

		   RSP := 'ERROR: la opcio de vot no esta associada a la votacio, per tant no es pot ''desassociar''.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_no_editable THEN

		   RSP := 'ERROR: no es pot ''desassociar'' cap opcio a una votacio que no estigui en estat d''edicio.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;















	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_modificaOpcio_descripcio
	-- 			
	-- Descripci:		canvia la descripci d'una opci i crida el procediment p_registrarLog del paquet pk_utils
	-- 			
	-- Parmetres entrada:	nom		VARCHAR2(15 CHAR)
	-- 			cod_pais	CHAR(2)			NOT NULL	REFERENCES codisISO(codi_pais)
	-- 			nova_desc	VARCHAR2(100 CHAR)
	-- 			NOTA: (nom, cod_pais) s clau alternativa de 'opcions'
	-- 
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada que no pot ser buit o NULL ho s
	-- 			llargada_camp		-> quan el parmetre nom t ms de 15 i/o quan el parmetre codi_pais no t 
	-- 						   exactament 2 carcters
	-- 			codi_pais_inexistent	-> quan cod_pais no referencia el codi ISO de cap pais (incls el pas neuter 'ZQ')
	-- 			nom_opcio_inexistent	-> quan nom no referencia el nom de cap opci del sistema
	-- 			opcio_codi_inexistent	-> quan la combinaci nom+cod_pais no referencien la clau alternativa de 'opcions'
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_modificaOpcio_descripcio (
	  nom		IN	opcions.nom_opcio%TYPE,
	  cod_pais	IN	opcions.codi_pais%TYPE,
	  nova_desc	IN	opcions.descripcio%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_modificaOpcio_descripcio';
	  	parametres_entrada 	VARCHAR2(140 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER := 0;
		id_opc			NUMBER := 0;
		id_estat_oberta		NUMBER := 0;
		id_estat_vota		NUMBER := 0;
	  	camp_obligatori 	EXCEPTION;
		llargada_camp		EXCEPTION;
		codi_pais_inexistent	EXCEPTION;
		nom_opcio_inexistent	EXCEPTION;
		opcio_codi_inexistent	EXCEPTION;


		BEGIN

		  parametres_entrada := nom||', '||cod_pais||', '||nova_desc;

		  IF (	nom IS NULL OR
			cod_pais IS NULL OR
			nova_desc IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (	length(nom) > 15 OR
			length(cod_pais) <> 2 OR
			length(nova_desc) > 100) tHEN
				RAISE llargada_camp;
		  END IF;


		  -- comprovem que codi_pais referenci un pas del sistema o al codi de pas neutre 'ZQ'
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM codisISO WHERE codi_pais = cod_pais;
		  IF ( trobat = 0 ) THEN
			RAISE codi_pais_inexistent;
		  END IF;

		 
		  -- comprovem que 'nom' referenci un nom d'alguna opci de la taula 'opcions'
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM opcions WHERE nom_opcio = nom;
		  IF ( trobat = 0 ) THEN
			RAISE nom_opcio_inexistent;
		  END IF;


		  -- comprovem que nom+cod_pais referencin la clau alternativa d'opcions
		  trobat := 0;
		  FOR opcions IN ( SELECT nom_opcio FROM opcions WHERE codi_pais = cod_pais ) LOOP

			trobat := trobat;
			IF ( trobat = 0 ) THEN

				IF ( opcions.nom_opcio = nom ) THEN
					trobat := 1;
				END IF;

			END IF;

		  END LOOP;


		  IF ( trobat = 0 ) THEN
			RAISE opcio_codi_inexistent;
		  END IF;




		  -- superades les verificacions anteriors, es pot canviar la decripcio de la opcio
		  UPDATE opcions SET descripcio = nova_desc WHERE nom_opcio = nom AND codi_pais = cod_pais;



		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN llargada_camp THEN

		   RSP := 'ERROR: nombre de caracters inapropiat. Revisi la llargada dels camps nom_opcio i/o codi_pais.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN codi_pais_inexistent THEN

		   RSP := 'ERROR: el codi del pais no referencia cap dels codis ISO assignats als paisos, ni al codi del pais neutre ''ZQ''.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN nom_opcio_inexistent THEN

		   RSP := 'ERROR: el nom de la opcio no existeix a la taula ''opcions''';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN opcio_codi_inexistent THEN

		   RSP := 'ERROR: el nom de la opcio i el codi del pais no referencien cap opcio valida.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		END;







	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_modificaVotacio_titol
	-- 			
	-- Descripci:		modifica el titol d'una votaci a dues taules (votacions i estadistiquesVotacions) i crida al procediment
	-- 			p_registrarLog de paquet pk_utils. No es permetr modificar el ttol de votacions que s'estiguin celebrant
	-- 			o que ja hagin finalitzat.
	-- 
	-- Parmetres entrada:	titol_vota_vell		VARCHAR2(200 CHAR) 	titol_vota_vell s clau alternativa de votacions
	-- 			titol_vota_nou		VARCHAR2(200 CHAR) 	titol_vota_vell ha de ser clau alternativa de votacions
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan titol_vota_vell i/o titol_vota_nou sn buits o NULL
	-- 			llargada_incorrecte	-> quan el ttol de la votaci (vell i/o nou) s massa llarg
	-- 			votacio_inexistent	-> quan titol_vota_vell no referencia cap votacio
	-- 			estat_vota_no_permet	-> quan s'intenta esborrar una votaci que est 'oberta' o 'finaltzada'
	-- 			titol_nou_invalid	-> quan titol_vota_nou s igual a un ttol ja existent a la taula votacions (ttol s UNIQUE)
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_modificaVotacio_titol (
	  titol_vota_vell	IN	votacions.titol%TYPE,
	  titol_vota_nou	IN	votacions.titol%TYPE,
	  RSP			OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_modificaVotacio_titol';
	  	parametres_entrada 	VARCHAR2(500 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		id_estat_vota		NUMBER := 0;
		estat_vota		VARCHAR2(20 CHAR);
	  	camp_obligatori 	EXCEPTION;
		llargada_incorrecte 	EXCEPTION;
		votacio_inexistent	EXCEPTION;
		estat_vota_no_permet	EXCEPTION;
		titol_nou_invalid	EXCEPTION;


		BEGIN

		  parametres_entrada := titol_vota_vell||', '||titol_vota_nou;


		  IF (  titol_vota_vell IS NULL OR
			titol_vota_nou IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF ( length(titol_vota_vell) > 200 OR
			length(titol_vota_nou) > 200 ) THEN
				RAISE llargada_incorrecte;
		  END IF;

		  -- comprovem que la votacio de la qual es vol modificar el titol existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM votacions WHERE titol = titol_vota_vell;
		  IF (trobat = 0) THEN
			RAISE votacio_inexistent;
		  END IF;

		  -- comprovem que l'estat de la votaci permeti modificar-ne el titol (no es permet si est en estat 'oberta' o 'finalitzada')
		  SELECT id_estat INTO id_estat_vota FROM votacions WHERE titol = titol_vota_vell;
		  SELECT nom_estat INTO estat_vota FROM estatsVotacio WHERE id = id_estat_vota;

		  IF ( estat_vota IN ('oberta', 'finalitzada') ) THEN
			RAISE estat_vota_no_permet;
		  END IF;
		  

		  -- comprovem que no hi hagi cap altra votaci que tingui el titol nou (el ttol d'una votaci s clau alternativa)
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM votacions WHERE titol = titol_vota_nou;
		  IF (trobat = 1) THEN
			RAISE titol_nou_invalid;
		  END IF;


		  -- superades les verificacions prvies es pot modificar el titol de la votaci titol_vota_vell a la taula votacions i 
		  -- a la taula estadistiquesVotacions.
		  UPDATE votacions SET titol = titol_vota_nou WHERE titol = titol_vota_vell;
		  UPDATE estadistiquesVotacions SET titol_votacio = titol_vota_nou WHERE titol_votacio = titol_vota_vell;

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: el titol (vell i/o nou) de la votacio es buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN llargada_incorrecte THEN

		   RSP := 'ERROR: el titol (vell i/o nou) de la votacio no pot tenir mes de 200 caracters.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_inexistent THEN

		   RSP := 'ERROR: no es pot canviar el titol d''una votacio que no existeix.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN estat_vota_no_permet THEN

		   RSP := 'ERROR: no es canviar el titol d''una votacio que s''esta celebrant o que ja ha finalitzat.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN titol_nou_invalid THEN

		   RSP := 'ERROR: hi ha una altra votacio que te el nou titol i un titol d''una votacio ha de ser unic.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;






	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_modificaVotacio_dataInici
	-- 			
	-- Descripci:		modifica la data d'inici d'una votaci i crida al procediment p_registrarLog de paquet pk_utils.
	-- 			No es permetr modificar la data de votacions que s'estiguin celebrant o que ja hagin finalitzat.
	-- 			No es permetr que data_inici < data_actual ni que data_inici > data_fi.
	-- 
	-- Parmetres entrada:	titol_vota		VARCHAR2(200 CHAR) 	titol_vota s clau alternativa de votacions
	-- 			data_in			DATE
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan titol_vota s buit o NULL
	-- 			llargada_incorrecte	-> quan el ttol de la votaci s massa llarg
	-- 			votacio_inexistent	-> quan titol_vota no referencia cap votacio
	-- 			estat_vota_no_permet	-> quan s'intenta esborrar una votaci que est 'oberta' o 'finaltzada'
	-- 			inici_menor_actual	-> quan data_in s menor que la data actual (en aquest cas ja s'hauria d'estar celebreant
	-- 						   la votaci o ja hauria d'haver acabat)
	-- 			inici_major_fi		-> quan data_in s major que la data de fi de la votaci amb ttol titol_vota
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_modificaVotacio_dataInici (
	  titol_vota	IN	votacions.titol%TYPE,
	  data_in	IN	votacions.data_inici%TYPE,
	  RSP			OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_modificaVotacio_dataInici';
	  	parametres_entrada 	VARCHAR2(300 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		id_estat_vota		NUMBER := 0;
		estat_vota		VARCHAR2(20 CHAR);
		data_fi_vota		DATE;
		data_actual		DATE;
	  	camp_obligatori 	EXCEPTION;
		llargada_incorrecte 	EXCEPTION;
		votacio_inexistent	EXCEPTION;
		estat_vota_no_permet	EXCEPTION;
		inici_menor_actual	EXCEPTION;
		inici_major_fi		EXCEPTION;



		BEGIN

		  parametres_entrada := titol_vota||', '||data_in;


		  IF (  titol_vota IS NULL OR
			data_in IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF ( length(titol_vota) > 200 ) THEN
				RAISE llargada_incorrecte;
		  END IF;

		  -- comprovem que la votacio (de la qual es vol modificar la data d'inici) existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM votacions WHERE titol = titol_vota;
		  IF (trobat = 0) THEN
			RAISE votacio_inexistent;
		  END IF;

		  -- comprovem que l'estat de la votaci permeti modificar-ne la data d'inici (no es permet si est en estat 'oberta' o
		  -- 'finalitzada')
		  SELECT id_estat INTO id_estat_vota FROM votacions WHERE titol = titol_vota;
		  SELECT nom_estat INTO estat_vota FROM estatsVotacio WHERE id = id_estat_vota;

		  IF ( estat_vota IN ('oberta', 'finalitzada') ) THEN
			RAISE estat_vota_no_permet;
		  END IF;
		  

		  -- comprovem que la nova data d'inici sigui major que la data actual i menor o igual que la data de fi de la votaci
		  SELECT data_fi INTO data_fi_vota FROM votacions WHERE titol = titol_vota;
		  SELECT SYSDATE INTO data_actual FROM DUAL;
		  IF ( data_in < data_actual ) THEN
			RAISE inici_menor_actual;
		  END IF;

		  IF ( data_in > data_fi_vota ) THEN
			RAISE inici_major_fi;
		  END IF;


		  -- superades les verificacions prvies, es pot modificar la data d'inici de la votaci de ttol titol_vota
		  UPDATE votacions SET data_inici = data_in WHERE titol = titol_vota;


		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: el titol de la votacio i/o la data d''inici son buits o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN llargada_incorrecte THEN

		   RSP := 'ERROR: el titol de la votacio no pot tenir mes de 200 caracters.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_inexistent THEN

		   RSP := 'ERROR: no es pot canviar la data d''inici d''una votacio que no existeix.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN estat_vota_no_permet THEN

		   RSP := 'ERROR: no es canviar la data d''inici d''una votacio que s''esta celebrant o que ja ha finalitzat.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN inici_menor_actual THEN

		   RSP := 'ERROR: la nova data d''inici d''una votacio no pot ser menor que la data actual.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN inici_major_fi THEN

		   RSP := 'ERROR: la nova data d''inici d''una votacio no pot ser major que la data de finalitzacio.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;








	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_modificaVotacio_dataFi
	-- 			
	-- Descripci:		modifica la data de finalitzaci d'una votaci i crida al procediment p_registrarLog de paquet pk_utils.
	-- 			No es permetr modificar la data de votacions que s'estiguin celebrant o que ja hagin finalitzat.
	-- 			No es permetr que data_fi_vota <= data_actual ni que data_fi_vota < data_inici.
	-- 			
	-- 
	-- Parmetres entrada:	titol_vota		VARCHAR2(200 CHAR) 	titol_vota s clau alternativa de votacions
	-- 			data_fi_vota		DATE
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan titol_vota s buit o NULL
	-- 			llargada_incorrecte	-> quan ttol_vota s massa llarg (ms de 200 carcters)
	-- 			votacio_inexistent	-> quan titol_vota no referencia cap votacio
	-- 			estat_vota_no_permet	-> quan s'intenta esborrar una votaci que est 'oberta' o 'finaltzada'
	-- 			fi_menor_actual		-> quan data_fi_vota s menor o igual que la data actual (la votacio ja hauria d'haver 
	-- 						-> acabat o s'estaria celebrant)
	-- 			fi_menor_inici		-> quan data_fi_vota s menor que la data d'inici de la votaci amb ttol titol_vota
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_modificaVotacio_dataFi (
	  titol_vota	IN	votacions.titol%TYPE,
	  data_fi_vota	IN	votacions.data_fi%TYPE,
	  RSP			OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_modificaVotacio_dataFi';
	  	parametres_entrada 	VARCHAR2(300 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		id_estat_vota		NUMBER := 0;
		estat_vota		VARCHAR2(20 CHAR);
		data_inici_vota		DATE;
		data_actual		DATE;
	  	camp_obligatori 	EXCEPTION;
		llargada_incorrecte 	EXCEPTION;
		votacio_inexistent	EXCEPTION;
		estat_vota_no_permet	EXCEPTION;
		fi_menor_actual		EXCEPTION;
		fi_menor_inici		EXCEPTION;



		BEGIN

		  parametres_entrada := titol_vota||', '||data_fi_vota;


		  IF (  titol_vota IS NULL OR
			data_fi_vota IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF ( length(titol_vota) > 200 ) THEN
				RAISE llargada_incorrecte;
		  END IF;

		  -- comprovem que la votacio (de la qual es vol modificar la data d'inici) existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM votacions WHERE titol = titol_vota;
		  IF (trobat = 0) THEN
			RAISE votacio_inexistent;
		  END IF;

		  -- comprovem que l'estat de la votaci permeti modificar-ne la data d'inici (no es permet si est en estat 'oberta' o
		  -- 'finalitzada')
		  SELECT id_estat INTO id_estat_vota FROM votacions WHERE titol = titol_vota;
		  SELECT nom_estat INTO estat_vota FROM estatsVotacio WHERE id = id_estat_vota;

		  IF ( estat_vota IN ('oberta', 'finalitzada') ) THEN
			RAISE estat_vota_no_permet;
		  END IF;
		  

		  -- comprovem que la nova data de fi sigui major que la data actual i major o igual que la data de fi de la votaci
		  SELECT data_inici INTO data_inici_vota FROM votacions WHERE titol = titol_vota;
		  SELECT SYSDATE INTO data_actual FROM DUAL;
		  IF ( data_fi_vota <= data_actual ) THEN
			RAISE fi_menor_actual;
		  END IF;

		  IF ( data_fi_vota < data_inici_vota ) THEN
			RAISE fi_menor_inici;
		  END IF;


		  -- superades les verificacions prvies, es pot modificar la data d'inici de la votaci de ttol titol_vota
		  UPDATE votacions SET data_fi = data_fi_vota WHERE titol = titol_vota;


		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: el titol de la votacio i/o la data de fi son buits o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN llargada_incorrecte THEN

		   RSP := 'ERROR: el titol de la votacio no pot tenir mes de 200 caracters.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_inexistent THEN

		   RSP := 'ERROR: no es pot canviar la data de fi d''una votacio que no existeix.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN estat_vota_no_permet THEN

		   RSP := 'ERROR: no es canviar la data de fi d''una votacio que s''esta celebrant o que ja ha finalitzat.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN fi_menor_actual THEN

		   RSP := 'ERROR: la nova data de fi d''una votacio no pot ser menor o igual que la data actual (ja s''hauria celebrat o s''estaria celebrant).';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN fi_menor_inici THEN

		   RSP := 'ERROR: la nova data de fi d''una votacio no pot ser menor que la data d''inici.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;






	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_modificaVotacio_web
	-- 			
	-- Descripci:		modifica la data de finalitzaci d'una votaci i crida al procediment p_registrarLog de paquet pk_utils.
	-- 			Es permetr modificar la web d'una votaci en qualsevol moment
	-- 
	-- Parmetres entrada:	titol_vota		VARCHAR2(200 CHAR) 	titol_vota s clau alternativa de votacions
	-- 			nova_web		VARCHAR2(200 CHAR)
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan titol_vota s buit o NULL
	-- 			llargada_incorrecte	-> quan titol_vota i/o l'URL de la nova web sn massa llargs (ms de 200 carcters)
	-- 			votacio_inexistent	-> quan titol_vota no referencia cap votacio
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_modificaVotacio_web (
	  titol_vota	IN	votacions.titol%TYPE,
	  nova_web	IN	votacions.adreca_web%TYPE,
	  RSP			OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_modificaVotacio_web';
	  	parametres_entrada 	VARCHAR2(500 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
	  	camp_obligatori 	EXCEPTION;
		llargada_incorrecte 	EXCEPTION;
		votacio_inexistent	EXCEPTION;



		BEGIN

		  parametres_entrada := titol_vota||', '||nova_web;


		  IF (  titol_vota IS NULL OR
			nova_web IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF ( 	length(titol_vota) > 200 OR
			length(nova_web) > 200 ) THEN
				RAISE llargada_incorrecte;
		  END IF;

		  -- comprovem que la votacio (de la qual es vol modificar la web) existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM votacions WHERE titol = titol_vota;
		  IF (trobat = 0) THEN
			RAISE votacio_inexistent;
		  END IF;


		  -- modifiquem l'URL de la votaci
		  UPDATE votacions SET adreca_web = nova_web WHERE titol = titol_vota;


		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: el titol de la votacio i/o la URL de la nova web son buits o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN llargada_incorrecte THEN

		   RSP := 'ERROR: el titol de la votacio i/o la URL de la nova web no poden tenir mes de 200 caracters.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_inexistent THEN

		   RSP := 'ERROR: no es pot canviar la web d''una votacio que no existeix.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;









	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_modificaVotacio_president
	-- 			
	-- Descripci:		modifica la data de finalitzaci d'una votaci i crida al procediment p_registrarLog de paquet pk_utils.
	-- 			Es permetr modificar el nom del president de la mesa d'una votaci en tot moment sempre que la votaci no
	-- 			estigui en estat 'finalitzada'
	-- 
	-- Parmetres entrada:	titol_vota		VARCHAR2(200 CHAR) 	titol_vota s clau alternativa de votacions
	-- 			nou_president		VARCHAR2(100 CHAR)
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan titol_vota s buit o NULL
	-- 			llargada_incorrecte	-> quan titol_vota i/o nou_president sn massa llargs (ms de 200 i 100 carcters respectivament)
	-- 			votacio_inexistent	-> quan titol_vota no referencia cap votacio
	-- 			estat_vota_no_permet	-> quan s'intenta canviar el nom del president d'una votaci finalitzada
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_modificaVotacio_president (
	  titol_vota	IN	votacions.titol%TYPE,
	  nou_president	IN	votacions.president_mesa%TYPE,
	  RSP			OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_modificaVotacio_president';
	  	parametres_entrada 	VARCHAR2(350 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		id_estat_vota		NUMBER := 0;
		estat_vota		VARCHAR2(20 CHAR);
	  	camp_obligatori 	EXCEPTION;
		llargada_incorrecte 	EXCEPTION;
		votacio_inexistent	EXCEPTION;
		estat_vota_no_permet	EXCEPTION;



		BEGIN

		  parametres_entrada := titol_vota||', '||nou_president;


		  IF (  titol_vota IS NULL OR
			nou_president IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF ( 	length(titol_vota) > 200 OR
			length(nou_president) > 100 ) THEN
				RAISE llargada_incorrecte;
		  END IF;

		  -- comprovem que la votacio (de la qual es vol modificar el nou president de mesa) existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM votacions WHERE titol = titol_vota;
		  IF (trobat = 0) THEN
			RAISE votacio_inexistent;
		  END IF;


		  -- comprovem que l'estat de la votaci permeti modificar-ne el president (no es permet si est en estat 'finalitzada')
		  SELECT id_estat INTO id_estat_vota FROM votacions WHERE titol = titol_vota;
		  SELECT nom_estat INTO estat_vota FROM estatsVotacio WHERE id = id_estat_vota;

		  IF ( estat_vota = 'finalitzada' ) THEN
			RAISE estat_vota_no_permet;
		  END IF;


		  -- modifiquem el nom del president de la mesa
		  UPDATE votacions SET president_mesa = nou_president WHERE titol = titol_vota;


		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: el titol de la votacio i/o el nom del nou president son buits o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN llargada_incorrecte THEN

		   RSP := 'ERROR: el titol de la votacio i/o el del nom del president no poden tenir mes de 200 i 100 caracters respectivament.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_inexistent THEN

		   RSP := 'ERROR: no es pot canviar el nom de president de mesa d''una votacio que no existeix.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN estat_vota_no_permet THEN

		   RSP := 'ERROR: no es canviar el president de mesa d''una votacio que ja ha finalitzat.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;








END; --fi implementaci paquet pk_votacions
/

