

-- Esborrat del paquet (especificaci i cos)
-- -----------------------------------------
-- DROP PACKAGE BODY pk_consultes;
-- DROP PACKAGE pk_consultes;



-- ******************************************************************************************************************************
-- ******************************************************************************************************************************
-- *																*
-- *						FASE D'ESPECIFICACI								*
-- *																*
-- ******************************************************************************************************************************
-- ******************************************************************************************************************************


-- ----------------------------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------------
-- Nom_paquet: 		pk_consultes
-- 
-- Descripci:		especificaci dels procediment que donen resposta a les consultes del requisit R6
-- 
-- Procediments:	p_consulta_votacionsPais, p_consulta_votacionsMesDif, p_consulta_votacionsMenysDif,
-- 			p_consulta_censosCiutada, p_consulta_votacionsCiutada
-- 
-- Funcions:		--
-- ----------------------------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE pk_consultes IS



	PROCEDURE p_consulta_votacionsPais (
	  pais		IN	paisos.nom_pais%TYPE,
	  RSP		OUT	VARCHAR2
	  );



	PROCEDURE p_consulta_votacionsMesDif (
	  RSP		OUT	VARCHAR2
	  );


	PROCEDURE p_consulta_votacionsMenysDif (
	  RSP		OUT	VARCHAR2
	  );


	PROCEDURE p_consulta_censosCiutada (
	  ciutada	IN	ciutadans.num_aleatori%TYPE,
	  RSP		OUT	VARCHAR2
	  );


	PROCEDURE p_consulta_votacionsCiutada (
	  ciutada	IN	ciutadans.num_aleatori%TYPE,
	  RSP		OUT	VARCHAR2
	  );


END; --fi especificaci paquet pk_consultes
/












-- ******************************************************************************************************************************
-- ******************************************************************************************************************************
-- *																*
-- *						FASE D'IMPLEMENTACI								*
-- *																*
-- ******************************************************************************************************************************
-- ******************************************************************************************************************************




-- ----------------------------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------------
-- Nom_paquet: 		pk_consultes
-- 
-- Descripci:		implementaci dels procediment que donen resposta a les consultes del requisit R6
-- 
-- Procediments:	p_consulta_votacionsPais, p_consulta_votacionsMesDif, p_consulta_votacionsMenysDif,
-- 			p_consulta_censosCiutada, p_consulta_votacionsCiutada
-- 
-- Funcions:		--
-- ----------------------------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE BODY pk_consultes IS






	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_consulta_votacionsPais
	-- 			
	-- Descripci:		procediment que mostra dades, pel canal de sortida, de les votacions a les que ha participat un pais rebut per 
	-- 			parmetre ordenat ascendentment per la data d'inici de les votacions. 
	-- 			En concret, mostra el ttol de la votaci, les dates d'inici i de fi de la mateixa, el nombre d'electors,
	-- 			l'estat de la votaci i, si la votaci ha finalitzat, el percentatge de participaci total, la opci (o opcions)
	-- 			guanyadora i el percentatge de vots d'aquesta/es.
	-- 			
	-- Parmetres entrada:	pais	VARCHAR2(30 CHAR)
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada que no pot ser buit o NULL ho s
	-- 			llargada_incorrecte	-> quan el parmetre pais t ms de 30 carcters
	-- 			pais_inexistent		-> quan el pas no existeix al sistema
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_consulta_votacionsPais (
	  pais		IN	paisos.nom_pais%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS



		nom_procedure 		VARCHAR2(30 CHAR) := 'p_consulta_votacionsPais';
	  	parametres_entrada 	VARCHAR2(30 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		num			NUMBER := 0;

		estat_vota		VARCHAR2(25 CHAR);
		opcio_guanyadora	VARCHAR2(25 CHAR);
		perc_part_total		VARCHAR2(5 CHAR);
		perc_opc_guanyadora	VARCHAR2(5 CHAR);
		titol			VARCHAR2(100 CHAR);
		electors		VARCHAR2(12 CHAR);
		particip		VARCHAR2(6 CHAR);
		estat			VARCHAR2(16 CHAR);
			
	  	camp_obligatori 	EXCEPTION;
		camp_incorrecte 	EXCEPTION;
		llargada_incorrecte	EXCEPTION;
		pais_inexistent		EXCEPTION;


		-- el cursor c_votacions t les dades sollicitades.

		CURSOR c_votacions IS SELECT vp.id_votacio, v.titol, v.data_inici, v.data_fi, v.num_electors, v.id_estat FROM votacionsPaisos vp, votacions v 
			WHERE vp.id_pais = pais AND vp.id_votacio = v.id ORDER BY v.data_inici;

		BEGIN

		  parametres_entrada := pais;

		  IF (  pais IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (	length(pais) > 30 ) THEN
			RAISE llargada_incorrecte;
		  END IF;

		  -- comprovem que el pas existeixi a la BD
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM paisos WHERE nom_pais = pais;
		  IF ( trobat = 0 ) THEN
			RAISE pais_inexistent;
		  END IF;

		  dbms_output.put_line(CHR(9)||CHR(9)||CHR(10)||CHR(13)||CHR(10)||CHR(13)||CHR(9)||CHR(9)||CHR(9)||'CONSULTA DE LES VOTACIONS QUE S''HAN PRODUIT A '||pais);
		  dbms_output.put_line(CHR(9)||CHR(9)||'---------------------------------------------------------------------------');


		  DBMS_OUTPUT.PUT_LINE(CHR(10)||CHR(13)||CHR(10)||CHR(13)||'TITOL DE LA VOTACIO'||CHR(9)||CHR(9)||CHR(9)||CHR(9)||CHR(9)||CHR(9)||'DATA INICI'||CHR(9)||'DATA FI'||CHR(9)||CHR(9)||'ELECTORS'||CHR(9)||'ESTAT'||CHR(9)||CHR(9)||CHR(9)||'PARTICIPACIO(%)'||' OPCIO GUANY.'||CHR(9)||'VOTS (%)');
		  DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------');



		  -- coprovem si el pais ha participat a alguna votaci
		  num := 0;
		  SELECT COUNT(*) INTO num FROM votacionsPaisos WHERE id_pais = pais;
		  -- si no ha participat, mostrem un missatge descriptiu
		  IF ( num = 0 ) THEN

			DBMS_OUTPUT.PUT_LINE(CHR(10)||CHR(13)||CHR(10)||CHR(13)||'SENSE RESULTATS. '||pais||' no ha participat a cap votacio');

		  -- Si el pas ha participat a alguna votaci, formategem els resultats i finalment, els mostrem
		  ELSIF ( num > 0 ) THEN

			-- recorrem el cursor que hem declarat abans
			FOR votacio IN c_votacions LOOP

				SELECT nom_estat INTO estat_vota FROM estatsVotacio WHERE id = votacio.id_estat ;
				IF ( estat_vota <> 'finalitzada' ) THEN

					opcio_guanyadora := '-';
					perc_part_total	:= '-';
 					perc_opc_guanyadora := '-';

				ELSE

					SELECT participacio, nom_opcio_mes_vots, percentatge_opcio_mes_vots 
						INTO perc_part_total, opcio_guanyadora, perc_opc_guanyadora FROM estadistiquesVotacions 
						WHERE id_votacio = votacio.id_votacio;

				END IF;

				-- alguns camps que sn numrics els transformarem a format carcter.
				titol := votacio.titol;
				electors := TO_CHAR(votacio.num_electors);
				particip := TO_CHAR(perc_part_total);

				-- formategem les dades de la fila actual (per tal de que totes les dades d'una mateixa columna tinguin una 
				-- llargada equivalent afegim espais en blanc al darrera fins arribar a una determinada longitud)

				WHILE (length(titol) < 60 ) LOOP

					titol := titol||' ';

				END LOOP;


				WHILE (length(electors) < 12 ) LOOP

					electors := electors||' ';

				END LOOP;

				WHILE (length(particip) < 6 ) LOOP

					particip := particip||' ';

				END LOOP;


				WHILE (length(opcio_guanyadora) < 18 ) LOOP

					opcio_guanyadora := opcio_guanyadora||' ';

				END LOOP;


				WHILE (length(estat_vota) < 16 ) LOOP

					estat_vota := estat_vota||' ';

				END LOOP;				

				DBMS_OUTPUT.PUT_LINE(titol||CHR(9)||votacio.data_inici||CHR(9)||votacio.data_fi||CHR(9)||electors||CHR(9)||estat_vota||CHR(9)||perc_part_total||CHR(9)||CHR(9)||opcio_guanyadora||perc_opc_guanyadora||CHR(10)||CHR(13));



			END LOOP;



		  END IF;

		  

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN llargada_incorrecte THEN

		   RSP := 'ERROR: el pais no pot tenir mes de 30 caracters.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN pais_inexistent THEN

		   RSP := 'ERROR: el pais no existeix a la BD. Revisi l''escriptura.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;














	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_consulta_votacionsMesDif
	-- 			
	-- Descripci:		procediment que mostra dades pel canal de sortida de les 10 votacions on hi ha hagut ms diferncia percentual
	-- 			entre la opci ms votada i la menys votada. En concret, mostra el ttol de la votaci, les dates d'inici i de fi
	-- 			de la mateixa, la opci (o opcions) amb major percentatge de vots i aquest percentatge, la opci (o opcions) amb 
	-- 			menor percentatge i aquest percentatge i, finalment, la diferncia percentual entre la opci ms votada i la menys 
	-- 			votada. Finalment crida al procediment p_registrarLog del paquet pk_utils.
	-- 			
	-- Parmetres entrada:	--
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument al procediment p_registrarLog)
	-- 			
	-- Excepcions:		OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_consulta_votacionsMesDif (
	  RSP		OUT	VARCHAR2
	  ) IS



		nom_procedure 		VARCHAR2(30 CHAR) := 'p_consulta_votacionsMesDif';
	  	parametres_entrada 	VARCHAR2(30 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;

		opcio_guanyadora	VARCHAR2(35 CHAR);
		opcio_perdedora		VARCHAR2(35 CHAR);
		perc_opc_guanyadora	VARCHAR2(6 CHAR);
		perc_opc_perdedora	VARCHAR2(6 CHAR);
		diferencia_perc		VARCHAR2(6 CHAR);
		titol			VARCHAR2(100 CHAR);
		compensar		NUMBER := 0;
			

		CURSOR c_votacions IS SELECT 	v.id, v.titol, v.data_inici, v.data_fi, ev.nom_opcio_mes_vots, ev.percentatge_opcio_mes_vots, 
						ev.nom_opcio_menys_vots, ev.percentatge_opcio_menys_vots, ev.dif_perc_opc_mes_i_menys_vots 
						FROM votacions v, estadistiquesVotacions ev 
						WHERE v.id = ev.id_votacio AND finalitzada = 1 ORDER BY ev.dif_perc_opc_mes_i_menys_vots DESC;

		-- 'votacio' ser del mateix tipus que un registre (fila) del cursor
		votacio c_votacions%ROWTYPE;

		BEGIN

		  parametres_entrada := ' ';

		  -- si el cursor no s obert, l'obrim
		  IF NOT c_votacions%ISOPEN THEN
			OPEN c_votacions;
		  END IF;

		  dbms_output.put_line(CHR(9)||CHR(9)||CHR(10)||CHR(13)||CHR(10)||CHR(13)||CHR(10)||CHR(13)||CHR(10)||CHR(13)||CHR(9)||CHR(9)||CHR(9)||'LLISTAT DE LES 10 VOTACIONS AMB MES DIFERENCIA PERCENTUAL ENTRE LA OPCIO GUANYADORA I LA PERDEDORA');
		  dbms_output.put_line(CHR(9)||CHR(9)||'----------------------------------------------------------------------------------------------------------------------');


		  DBMS_OUTPUT.PUT_LINE(CHR(10)||CHR(13)||CHR(10)||CHR(13)||'TITOL DE LA VOTACIO'||CHR(9)||CHR(9)||CHR(9)||CHR(9)||CHR(9)||CHR(9)||'DATA INICI'||CHR(9)||'DATA FI'||CHR(9)||CHR(9)||'OPCIO GUANYADORA'||CHR(9)||'VOTS(%)'||CHR(9)||'OPCIO PERDEDORA'||CHR(9)||CHR(9)||'VOTS(%)'||CHR(9)||'DIFERENCIA(%)');
		  DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------');


		  -- recorrem 10 files del cursor (si hi sn)
		  LOOP

			FETCH c_votacions INTO votacio;

			titol := votacio.titol;
			opcio_guanyadora := votacio.nom_opcio_mes_vots;
			opcio_perdedora := votacio.nom_opcio_menys_vots;
			perc_opc_guanyadora := TO_CHAR(votacio.percentatge_opcio_mes_vots);
			perc_opc_perdedora := TO_CHAR(votacio.percentatge_opcio_menys_vots);
			diferencia_perc := TO_CHAR(votacio.dif_perc_opc_mes_i_menys_vots);


			-- formategem les dades de la fila actual (per tal de que totes les dades d'una mateixa columna tinguin una 
			-- llargada equivalent afegim espais en blanc al darrera fins arribar a una determinada longitud)

			WHILE (length(titol) < 60 ) LOOP

				titol := titol||' ';

			END LOOP;

			compensar := 0;
			IF ( length(opcio_guanyadora) <= 4 ) THEN
				compensar := 1;
			END IF;

			WHILE (length(opcio_guanyadora) < 20 ) LOOP

				opcio_guanyadora := opcio_guanyadora||' ';

			END LOOP;


			IF ( compensar = 1 ) THEN
				opcio_guanyadora := opcio_guanyadora||'        ';
			END IF;


			WHILE (length(opcio_perdedora) < 20 ) LOOP

				opcio_perdedora := opcio_perdedora||' ';

			END LOOP;


			IF ( perc_opc_guanyadora > 0 AND perc_opc_guanyadora < 1 ) THEN 
				perc_opc_guanyadora := '0'||perc_opc_guanyadora; 
			END IF;

			WHILE (length(perc_opc_guanyadora) < 6 ) LOOP

				perc_opc_guanyadora := perc_opc_guanyadora||' ';

			END LOOP;



			IF ( perc_opc_perdedora > 0 AND perc_opc_perdedora < 1 ) THEN 
				perc_opc_perdedora := '0'||perc_opc_perdedora; 
			END IF;

			WHILE (length(perc_opc_perdedora) < 6 ) LOOP

				perc_opc_perdedora := perc_opc_perdedora||' ';

			END LOOP;



			IF ( diferencia_perc > 0 AND diferencia_perc < 1 ) THEN 
				diferencia_perc := '0'||diferencia_perc; 
			END IF;

			WHILE (length(diferencia_perc) < 6 ) LOOP

				diferencia_perc := diferencia_perc||' ';

			END LOOP;

			-- una vegada formatejades, mostrem les dades de la fila actual
			DBMS_OUTPUT.PUT_LINE(titol||CHR(9)||votacio.data_inici||CHR(9)||votacio.data_fi||CHR(9)||opcio_guanyadora||CHR(9)||perc_opc_guanyadora||CHR(9)||opcio_perdedora||CHR(9)||perc_opc_perdedora||CHR(9)||CHR(9)||diferencia_perc||CHR(10)||CHR(13));


			EXIT WHEN c_votacions%ROWCOUNT >= 10 OR c_votacions%NOTFOUND OR c_votacions%NOTFOUND IS NULL;
		  END LOOP;

		  IF ( c_votacions%ROWCOUNT < 10 ) THEN

			DBMS_OUTPUT.PUT_LINE(CHR(10)||CHR(13)||CHR(10)||CHR(13)||'No s''han pogut mostrar 10 votacions perque nomes n''hi ha: '||c_votacions%ROWCOUNT);

		  END IF;


		  -- si el cursor s obert, el tanquem
		  IF c_votacions%ISOPEN THEN
			CLOSE c_votacions;
		  END IF;


		  

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		EXCEPTION

	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;













	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_consulta_votacionsMenysDif
	-- 			
	-- Descripci:		procediment que mostra dades pel canal de sortida de les 10 votacions on hi ha hagut ms diferncia percentual
	-- 			entre la opci ms votada i la menys votada. En concret, mostra el ttol de la votaci, les dates d'inici i de fi
	-- 			de la mateixa, la opci (o opcions) amb major percentatge de vots i aquest percentatge, la opci (o opcions) amb 
	-- 			menor percentatge i aquest percentatge i, finalment, la diferncia percentual entre la opci ms votada i la menys 
	-- 			votada. Finalment crida al procediment p_registrarLog del paquet pk_utils.
	-- 			
	-- Parmetres entrada:	--
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument al procediment p_registrarLog)
	-- 			
	-- Excepcions:		OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_consulta_votacionsMenysDif (
	  RSP		OUT	VARCHAR2
	  ) IS



		nom_procedure 		VARCHAR2(30 CHAR) := 'p_consulta_votacionsMenysDif';
	  	parametres_entrada 	VARCHAR2(30 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;

		votacions_finalitzades	NUMBER := 0;
		votacions_a_llistar	NUMBER := 0;
		opcio_guanyadora	VARCHAR2(35 CHAR);
		opcio_perdedora		VARCHAR2(35 CHAR);
		perc_opc_guanyadora	VARCHAR2(6 CHAR);
		perc_opc_perdedora	VARCHAR2(6 CHAR);
		diferencia_perc		VARCHAR2(6 CHAR);
		titol			VARCHAR2(100 CHAR);
		compensar		NUMBER := 0;
			

		CURSOR c_votacions IS SELECT 	v.id, v.titol, v.data_inici, v.data_fi, ev.nom_opcio_mes_vots, ev.percentatge_opcio_mes_vots, 
						ev.nom_opcio_menys_vots, ev.percentatge_opcio_menys_vots, ev.dif_perc_opc_mes_i_menys_vots 
						FROM votacions v, estadistiquesVotacions ev 
						WHERE v.id = ev.id_votacio AND finalitzada = 1 ORDER BY ev.dif_perc_opc_mes_i_menys_vots DESC;

		-- 'votacio' ser del mateix tipus que un registre (fila) del cursor c_votacions
		votacio c_votacions%ROWTYPE;

		BEGIN

		  parametres_entrada := ' ';

		  -- comprovem quantes votacions finalitzades hi ha
		  SELECT COUNT(*) INTO votacions_finalitzades FROM estadistiquesVotacions WHERE finalitzada = 1;
		  votacions_a_llistar := votacions_finalitzades;

		  -- si el cursor no s obert, l'obrim
		  IF NOT c_votacions%ISOPEN THEN
			OPEN c_votacions;
		  END IF;

		  dbms_output.put_line(CHR(9)||CHR(9)||CHR(10)||CHR(13)||CHR(10)||CHR(13)||CHR(10)||CHR(13)||CHR(10)||CHR(13)||CHR(9)||CHR(9)||CHR(9)||'LLISTAT DE LES 10 VOTACIONS AMB MENYS DIFERENCIA PERCENTUAL ENTRE LA OPCIO GUANYADORA I LA PERDEDORA');
		  dbms_output.put_line(CHR(9)||CHR(9)||'----------------------------------------------------------------------------------------------------------------------');


		  DBMS_OUTPUT.PUT_LINE(CHR(10)||CHR(13)||CHR(10)||CHR(13)||'TITOL DE LA VOTACIO'||CHR(9)||CHR(9)||CHR(9)||CHR(9)||CHR(9)||CHR(9)||'DATA INICI'||CHR(9)||'DATA FI'||CHR(9)||CHR(9)||'OPCIO GUANYADORA'||CHR(9)||'VOTS(%)'||CHR(9)||'OPCIO PERDEDORA'||CHR(9)||CHR(9)||'VOTS(%)'||CHR(9)||'DIFERENCIA(%)');
		  DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------');


		  -- recorrem el cursor (ja t les dades ordenades descendentment)
		  LOOP

			-- si cal, descartem files del cursor fins que noms en quedin 10
			WHILE ( votacions_a_llistar > 10 ) LOOP

				FETCH c_votacions INTO votacio;
				votacions_a_llistar := votacions_a_llistar - 1;

			END LOOP;


			FETCH c_votacions INTO votacio;


			titol := votacio.titol;
			opcio_guanyadora := votacio.nom_opcio_mes_vots;
			opcio_perdedora := votacio.nom_opcio_menys_vots;
			perc_opc_guanyadora := TO_CHAR(votacio.percentatge_opcio_mes_vots);
			perc_opc_perdedora := TO_CHAR(votacio.percentatge_opcio_menys_vots);
			diferencia_perc := TO_CHAR(votacio.dif_perc_opc_mes_i_menys_vots);


			-- formategem les dades de la fila actual (per tal de que totes les dades d'una mateixa columna tinguin una 
			-- llargada equivalent afegim espais en blanc al darrera fins arribar a una determinada longitud)

			WHILE (length(titol) < 60 ) LOOP

				titol := titol||' ';

			END LOOP;

			compensar := 0;
			IF ( length(opcio_guanyadora) <= 4 ) THEN
				compensar := 1;
			END IF;

			WHILE (length(opcio_guanyadora) < 20 ) LOOP

				opcio_guanyadora := opcio_guanyadora||' ';

			END LOOP;


			IF ( compensar = 1 ) THEN
				opcio_guanyadora := opcio_guanyadora||'        ';
			END IF;


			WHILE (length(opcio_perdedora) < 20 ) LOOP

				opcio_perdedora := opcio_perdedora||' ';

			END LOOP;


			IF ( perc_opc_guanyadora > 0 AND perc_opc_guanyadora < 1 ) THEN 
				perc_opc_guanyadora := '0'||perc_opc_guanyadora; 
			END IF;

			WHILE (length(perc_opc_guanyadora) < 6 ) LOOP

				perc_opc_guanyadora := perc_opc_guanyadora||' ';

			END LOOP;



			IF ( perc_opc_perdedora > 0 AND perc_opc_perdedora < 1 ) THEN 
				perc_opc_perdedora := '0'||perc_opc_perdedora; 
			END IF;

			WHILE (length(perc_opc_perdedora) < 6 ) LOOP

				perc_opc_perdedora := perc_opc_perdedora||' ';

			END LOOP;


			IF ( diferencia_perc > 0 AND diferencia_perc < 1 ) THEN 
				diferencia_perc := '0'||diferencia_perc; 
			END IF;

			WHILE (length(diferencia_perc) < 6 ) LOOP

				diferencia_perc := diferencia_perc||' ';

			END LOOP;

			-- una vegada formatejades, mostrem les dades de la fila actual
			DBMS_OUTPUT.PUT_LINE(titol||CHR(9)||votacio.data_inici||CHR(9)||votacio.data_fi||CHR(9)||opcio_guanyadora||CHR(9)||perc_opc_guanyadora||CHR(9)||opcio_perdedora||CHR(9)||perc_opc_perdedora||CHR(9)||CHR(9)||diferencia_perc||CHR(10)||CHR(13));


			EXIT WHEN c_votacions%ROWCOUNT = votacions_finalitzades OR c_votacions%NOTFOUND OR c_votacions%NOTFOUND IS NULL;
		  END LOOP;

		  IF ( c_votacions%ROWCOUNT < 10 ) THEN

			DBMS_OUTPUT.PUT_LINE(CHR(10)||CHR(13)||CHR(10)||CHR(13)||'No s''han pogut mostrar 10 votacions perque nomes n''hi ha: '||c_votacions%ROWCOUNT);

		  END IF;


		  -- si el cursor s obert, el tanquem
		  IF c_votacions%ISOPEN THEN
			CLOSE c_votacions;
		  END IF;


		  

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		EXCEPTION

	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;











	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_consulta_censosCiutada
	-- 			
	-- Descripci:		procediment que mostra, pel canal de sortida, els censos als quals pertany un ciutad del qual se'n rep el
	-- 			nmero aleatori que l'identifica per parmetre.
	-- 			
	-- Parmetres entrada:	ciutada		NUMBER(10)	PK
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan el parmetre d'entrada s buit o NULL
	-- 			numero_incorrecte	-> quan el parmetre no s un nombre natural > 0 o quan t ms de 9 dgits decimals
	-- 			ciutada_inexistent	-> quan el nmero aleatori (parmetre d'entrada) no identifica cap ciutad de la BD
	-- 						   o quan el ciutad que identifica ha estat donat de baixa lgica.
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_consulta_censosCiutada (
	  ciutada	IN	ciutadans.num_aleatori%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS



		nom_procedure 		VARCHAR2(30 CHAR) := 'p_consulta_censosCiutada';
	  	parametres_entrada 	VARCHAR2(30 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		cens_local		VARCHAR2(40 CHAR) := NULL;
		cens_regional		VARCHAR2(40 CHAR) := NULL;
		cens_estatal		VARCHAR2(30 CHAR) := NULL;
		cens_global		VARCHAR2(30 CHAR) := NULL;
	  	camp_obligatori 	EXCEPTION;
		numero_incorrecte 	EXCEPTION;
		ciutada_inexistent	EXCEPTION;



		BEGIN


		  parametres_entrada := ciutada;

		  IF ( ciutada IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (	ciutada <= 0 OR
			length(ciutada) > 9 ) THEN
			RAISE numero_incorrecte;
		  END IF;

		  -- comprovem que el ciutad existeixi a la BD i no estigui donat de baixa
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM ciutadans WHERE num_aleatori = ciutada AND baixa_logica = 0;
		  IF ( trobat = 0 ) THEN
			RAISE ciutada_inexistent;
		  END IF;


		  dbms_output.put_line(CHR(9)||CHR(9)||CHR(10)||CHR(13)||CHR(10)||CHR(13)||CHR(9)||CHR(9)||CHR(9)||'CONSULTA DELS CENSOS ALS QUALS PERTANY EL CIUTADA AMB NUMERO ALEATORI '||ciutada);
		  dbms_output.put_line(CHR(9)||CHR(9)||'---------------------------------------------------------------------------------------------------');


		  DBMS_OUTPUT.PUT_LINE(CHR(10)||CHR(13)||CHR(10)||CHR(13)||'CENS GLOBAL'||CHR(9)||CHR(9)||CHR(9)||CHR(9)||'CENS ESTATAL'||CHR(9)||CHR(9)||CHR(9)||'CENS REGIONAL'||CHR(9)||CHR(9)||CHR(9)||CHR(9)||CHR(9)||'CENS LOCAL');
		  DBMS_OUTPUT.PUT_LINE('------------------------------------------------------------------------------------------------------------------------------------------------------');




		  -- obtenim els censos als quals pertany
		  SELECT id_cens_local INTO cens_local FROM ciutadans WHERE num_aleatori = ciutada;
		  SELECT id_cens_regional INTO cens_regional FROM censosLocals WHERE nom_cens = cens_local;
		  SELECT id_cens_estatal INTO cens_estatal FROM censosRegionals WHERE nom_cens = cens_regional;
		  SELECT id_cens_global INTO cens_global FROM censosEstatals WHERE nom_cens = cens_estatal;



		  -- formategem les dades (per tal de que totes les dades d'una mateixa columna tinguin una llargada equivalent 
		  -- afegim espais en blanc al darrera fins arribar a una determinada longitud)

		  WHILE (length(cens_global) < 29 ) LOOP

			cens_global := cens_global||' ';

		  END LOOP;


		  WHILE (length(cens_estatal) < 29 ) LOOP

			cens_estatal := cens_estatal||' ';

		  END LOOP;


		  WHILE (length(cens_regional) < 39 ) LOOP

			cens_regional := cens_regional||' ';

		  END LOOP;


		  WHILE (length(cens_local) < 39 ) LOOP

			cens_local := cens_local||' ';

		  END LOOP;

	

		  DBMS_OUTPUT.PUT_LINE(cens_global||CHR(9)||CHR(9)||cens_estatal||CHR(9)||cens_regional||CHR(9)||cens_local);


		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: el numero aleatori es NULL o sense valor.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN numero_incorrecte THEN

		   RSP := 'ERROR: el numero aleatori ha de ser un nombre natural diferent de zero (i ha de tenir menys de 10 digits).';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN ciutada_inexistent THEN

		   RSP := 'ERROR: el ciutada no existeix o esta donat de baixa.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;














	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_consulta_votacionsCiutada
	-- 			
	-- Descripci:		procediment que mostra, pel canal de sortida, els censos als quals pertany un ciutad del qual se'n rep el
	-- 			nmero aleatori que l'identifica per parmetre.
	-- 			
	-- Parmetres entrada:	ciutada		NUMBER(10)	PK
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan el parmetre d'entrada s buit o NULL
	-- 			numero_incorrecte	-> quan el parmetre no s un nombre natural > 0 o quan t ms de 9 dgits decimals
	-- 			ciutada_inexistent	-> quan el nmero aleatori (parmetre d'entrada) no identifica cap ciutad de la BD
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_consulta_votacionsCiutada (
	  ciutada	IN	ciutadans.num_aleatori%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS



		nom_procedure 		VARCHAR2(30 CHAR) := 'p_consulta_votacionsCiutada';
	  	parametres_entrada 	VARCHAR2(30 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		baixa			NUMBER(1) := 0;
		cens_local		VARCHAR2(40 CHAR) := NULL;
		cens_regional		VARCHAR2(40 CHAR) := NULL;
		cens_estatal		VARCHAR2(30 CHAR) := NULL;
		pais			VARCHAR2(30 CHAR) := NULL;
		cod_pais		CHAR(2) := NULL;
		titol_SI		VARCHAR2(250 CHAR);
		titol_NO		VARCHAR2(100 CHAR);
		n_votacions_NO		NUMBER := 0;
		n_votacions_SI		NUMBER := 0;
	  	iteracions 		NUMBER := 0;
		n_cridat_urnes		NUMBER := 0;
		camp_obligatori		EXCEPTION;
		numero_incorrecte 	EXCEPTION;
		ciutada_inexistent	EXCEPTION;



		-- c_votacions_NO tindr els identificadors de les voacions en les quals no ha participat el ciutad
		-- c_votacions_SI tindr els identificadors de les voacions en les quals ha participat el ciutad

		CURSOR c_votacions_NO IS SELECT id_votacio FROM ciutadansVotacions WHERE id_ciutada = ciutada AND ha_participat = 0;
		CURSOR c_votacions_SI IS SELECT id_votacio FROM ciutadansVotacions WHERE id_ciutada = ciutada AND ha_participat = 1;


		-- 'votacio_NO' i 'votacio_SI' seran del mateix tipus que un registre (fila) del seu cursor corresponent
		votacio_NO c_votacions_NO%ROWTYPE;	
		votacio_SI c_votacions_SI%ROWTYPE;	

		BEGIN


		  parametres_entrada := ciutada;

		  IF ( ciutada IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (	ciutada <= 0 OR
			length(ciutada) > 9 ) THEN
			RAISE numero_incorrecte;
		  END IF;

		  -- comprovem que el ciutad existeixi a la BD
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM ciutadans WHERE num_aleatori = ciutada;
		  IF ( trobat = 0 ) THEN
			RAISE ciutada_inexistent;
		  END IF;

		  -- mirem si el ciutad est de baixa (per presentar unes o altres dades ms endavant)
		  SELECT baixa_logica INTO baixa FROM ciutadans WHERE num_aleatori = ciutada;
		  


		  dbms_output.put_line(CHR(9)||CHR(9)||CHR(10)||CHR(13)||CHR(10)||CHR(13)||CHR(9)||CHR(9)||CHR(9)||'CONSULTA DE L''HISTORIC DE PARTICIPACIONS A LES VOTACIONS DEL CIUTADA AMB NUMERO ALEATORI '||ciutada);
		  dbms_output.put_line(CHR(9)||CHR(9)||'---------------------------------------------------------------------------------------------------------------------'||CHR(10)||CHR(13)||CHR(10)||CHR(13));


		  -- calculem les iteracions que haurem de fer al bucle de ms endavant (ja que vull mostrar les votacions en dues columnes, 
		  -- en comptes d'una)
		  SELECT COUNT(*) INTO n_votacions_NO FROM ciutadansVotacions WHERE id_ciutada = ciutada AND ha_participat = 0;
		  SELECT COUNT(*) INTO n_votacions_SI FROM ciutadansVotacions WHERE id_ciutada = ciutada AND ha_participat = 1;
		  IF ( n_votacions_NO > n_votacions_SI ) THEN
			iteracions := n_votacions_NO;
		  ELSE
			iteracions := n_votacions_SI;			
		  END IF;

		  -- n de vegades que ha estat cridat a les urnes
		  n_cridat_urnes := n_votacions_SI + n_votacions_NO;


		  -- obtenim el pas al qual pertany el ciutad (per mostrar informaci complementria)
		  SELECT id_cens_local INTO cens_local FROM ciutadans WHERE num_aleatori = ciutada;
		  SELECT id_cens_regional INTO cens_regional FROM censosLocals WHERE nom_cens = cens_local;
		  SELECT id_cens_estatal INTO cens_estatal FROM censosRegionals WHERE nom_cens = cens_regional;
		  SELECT codi_pais INTO cod_pais FROM censosEstatals WHERE nom_cens = cens_estatal;
		  SELECT nom_pais INTO pais FROM paisos WHERE codi_pais = cod_pais;

		  IF ( baixa = 0 ) THEN

			  DBMS_OUTPUT.PUT_LINE('Pais del ciutada: '||pais||CHR(10)||CHR(13));
			  DBMS_OUTPUT.PUT_LINE('Cens local: '||cens_local||CHR(10)||CHR(13));

		  ELSIF (baixa = 1) THEN

			  DBMS_OUTPUT.PUT_LINE('Pais del ciutada: El ciutada ha estat donat de baixa.'||CHR(10)||CHR(13));
			  DBMS_OUTPUT.PUT_LINE('Cens local: El ciutada ha estat donat de baixa.'||CHR(10)||CHR(13));

		  END IF;

		  DBMS_OUTPUT.PUT_LINE('Numero de vegades que ha estat cridat a les urnes: '||n_cridat_urnes||CHR(10)||CHR(13));

 

		  DBMS_OUTPUT.PUT_LINE(CHR(10)||CHR(13)||CHR(10)||CHR(13)||'VOTACIONS EN LES QUALS HA PARTICIPAT'||CHR(9)||CHR(9)||CHR(9)||CHR(9)||CHR(9)||'VOTACIONS EN LES QUALS NO HA PARTICIPAT');
		  DBMS_OUTPUT.PUT_LINE('----------------------------------------------------------------------------------------------------------------------------------');


		  -- si els cursors no estan oberts, els obrim
		  IF NOT c_votacions_SI%ISOPEN THEN
			OPEN c_votacions_SI;
		  END IF;

		  IF NOT c_votacions_NO%ISOPEN THEN
			OPEN c_votacions_NO;
		  END IF;

		  -- recorrem les files dels cursors i anem formatejant les dades a mostrar (sn de longitud variable i hi afegirem espais en 
		  -- blanc fins a una determinada longitud)
		  LOOP

			-- obtenim una fila de cada cursor
			FETCH c_votacions_NO INTO votacio_NO;
			FETCH c_votacions_SI INTO votacio_SI;

			-- comprovem si la fila del cursor c_votacions_NO t dades o no, i actuem en conseqncia
			IF ( c_votacions_NO%NOTFOUND OR c_votacions_NO%NOTFOUND IS NULL ) THEN
				titol_NO := '-';

			ELSE
				SELECT titol INTO titol_NO FROM votacions WHERE id = votacio_NO.id_votacio;
	
			END IF;
 


			-- comprovem si la fila del cursor c_votacions_SI t dades o no, i actuem en conseqncia
			IF ( c_votacions_SI%NOTFOUND OR c_votacions_SI%NOTFOUND IS NULL ) THEN

				titol_SI := '-';

			ELSE

				SELECT titol INTO titol_SI FROM votacions WHERE id = votacio_SI.id_votacio;
	
			END IF;


			-- formategem les dades (per tal de que totes tinguin una llargada equivalent afegim espais en blanc al darrera
			-- fins arribar a una determinada longitud)

			WHILE ( length(titol_NO) < 60 ) LOOP

				titol_NO := titol_NO||' ';

		  	END LOOP;


			WHILE ( length(titol_SI) < 60 ) LOOP

				titol_SI := titol_SI||' ';

		  	END LOOP;


			DBMS_OUTPUT.PUT_LINE(titol_SI||CHR(9)||CHR(9)||titol_NO);
 


			IF ( n_votacions_NO > n_votacions_SI) THEN

				EXIT WHEN c_votacions_NO%ROWCOUNT = iteracions OR c_votacions_NO%NOTFOUND OR c_votacions_NO%NOTFOUND IS NULL;
			ELSE

				EXIT WHEN c_votacions_SI%ROWCOUNT = iteracions OR c_votacions_SI%NOTFOUND OR c_votacions_SI%NOTFOUND IS NULL;

			END IF;


		  END LOOP;


		  -- si els cursors estan oberts, els tanquem
		  IF c_votacions_SI%ISOPEN THEN
			CLOSE c_votacions_SI;
		  END IF;

		  IF c_votacions_NO%ISOPEN THEN
			CLOSE c_votacions_NO;
		  END IF;




		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: el numero aleatori es NULL o sense valor.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN numero_incorrecte THEN

		   RSP := 'ERROR: el numero aleatori ha de ser un nombre natural diferent de zero (i ha de tenir menys de 10 digits).';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN ciutada_inexistent THEN

		   RSP := 'ERROR: el ciutada no existeix a la BD';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;












END; --fi implementaci paquet pk_consultes
/

