
-- Esborrat dels paquets (especificaci i cos)
-- -------------------------------------------
-- DROP PACKAGE BODY pk_utils;
-- DROP PACKAGE pk_utils;

-- DROP PACKAGE BODY pk_territoris;
-- DROP PACKAGE pk_territoris;

-- DROP PACKAGE BODY pk_ciutadans;
-- DROP PACKAGE pk_ciutadans;

-- DROP PACKAGE BODY pk_estadistiques;
-- DROP PACKAGE pk_estadistiques;

-- DROP PACKAGE BODY pk_votacions;
-- DROP PACKAGE pk_votacions;

-- DROP PACKAGE BODY pk_consultes;
-- DROP PACKAGE pk_consultes;




-- ******************************************************************************************************************************
-- ******************************************************************************************************************************
-- *																*
-- *						FASE D'ESPECIFICACI								*
-- *																*
-- ******************************************************************************************************************************
-- ******************************************************************************************************************************


-- --------------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------------
-- Nom_paquet: 		pk_utils
-- Fase: 
-- Descripci:		paquet amb un procediment cridat per tots els altres procediments i tres funcions
-- 			molt tils per la crrega de dades inicial
-- 
-- Procediments:	p_registrarLog, 
-- Funcions:		f_ num_aleatori, f_ data_aleatoria, f_opcio_aleatoria
-- --------------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE pk_utils IS



	PROCEDURE p_registrarLog (
	  data		IN logProcediments.data%TYPE,
	  nom_proc	IN logProcediments.nom_procediment%TYPE,
	  pEntrada	IN logProcediments.parametres_entrada%TYPE,
	  pSortida	IN logProcediments.sortida%TYPE,
	  correcte	IN logProcediments.correcte%TYPE,
	  confirmar	IN logProcediments.confirmar%TYPE
  	);


	FUNCTION f_num_aleatori RETURN NUMBER;


	FUNCTION f_data_aleatoria RETURN VARCHAR2;


	FUNCTION f_opcio_aleatoria(i_votacio IN NUMBER, cod_pais CHAR) RETURN NUMBER;


END; --fi especificaci paquet pk_utils
/







-- ###################################################################   pk_utils    ########################################################################










-- ******************************************************************************************************************************
-- ******************************************************************************************************************************
-- *																*
-- *						FASE D'IMPLEMENTACI								*
-- *																*
-- ******************************************************************************************************************************
-- ******************************************************************************************************************************




-- --------------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------------
-- Nom_paquet: 		pk_utils
-- Fase: 
-- Descripci:		paquet amb un procediment cridat per tots els altres procediments i tres funcions
-- 			molt tils per la crrega de dades inicial
-- 
-- Procediments:	p_registrarLog, 
-- Funcions:		f_ num_aleatori, f_ data_aleatoria, f_opcio_aleatoria
-- --------------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE BODY pk_utils IS


	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom_procediment:	p_registrarLog
	-- 
	-- Descripci:		insereix un registre a la taula logProcediments i confirma la transacci o b 
	--			la desf (si hi ha excepcions o el parmetre confirmar = 0). En cas de desfer
	--			una transacci tamb mostra un missatge descriptiu.
	-- Parmetres entrada:	data 		TIMESTAMP(0)
	--			nom_proc	VARCHAR(30 CHAR)	NOT NULL
	--			pEntrada	VARCHAR(650 CHAR)	NOT NULL
	--			pSortida	VARCHAR(200 CHAR)	NOT NULL
	--			correcte	NUMBER(1)		NOT NULL CHECK(correcte IN(0,1))
	--			confirmar	NUMBER(1)		NOT NULL CHECK(confirmar IN(0,1))
	-- Parmetres sortida:	--
	-- Excepcions:		camp_obligatori	-> quan un parmetre d'entrada s buit o NULL
	--			camp_incorrecte	-> quan els parmetres correcte i/o confirmar no sn 0 o 1
	--			OTHERS		-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_registrarLog (
	  data		IN logProcediments.data%TYPE,
	  nom_proc	IN logProcediments.nom_procediment%TYPE,
	  pEntrada	IN logProcediments.parametres_entrada%TYPE,
	  pSortida	IN logProcediments.sortida%TYPE,
	  correcte	IN logProcediments.correcte%TYPE,
	  confirmar	IN logProcediments.confirmar%TYPE
	  ) IS

	  	camp_obligatori		EXCEPTION;
		camp_incorrecte		EXCEPTION;

		BEGIN

		  IF (  data IS NULL OR 
			nom_proc IS NULL OR 
			pSortida IS NULL OR 
			correcte IS NULL OR 
			confirmar IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (	correcte < 0 OR
			correcte > 1 OR
			confirmar < 0 OR
			confirmar > 1 OR
			correcte <> confirmar) THEN
				RAISE camp_incorrecte;
		  END IF;

		  IF (confirmar = 0) THEN
			ROLLBACK;

		  END IF;

		  -- Tant si la transacci s'ha desfet (confirmar = 0), com si no, es registrar el procediment a la taula de log
		  INSERT INTO logProcediments (id, data, nom_procediment, parametres_entrada, sortida, correcte, confirmar)
		  VALUES (-1, data, nom_proc, pEntrada, pSortida, correcte, confirmar);
		  COMMIT;


		EXCEPTION

		WHEN camp_obligatori THEN

		   ROLLBACK;
		   DBMS_OUTPUT.PUT_LINE('Error al registrar el log, algun camp s buit o NULL.');
		   DBMS_OUTPUT.PUT_LINE('S''ha desfet la transacci');

		WHEN camp_incorrecte THEN

		   ROLLBACK;
		   DBMS_OUTPUT.PUT_LINE('Error al registrar el log, els arguments correcte i/o confirmar no tenen un valor vlid.');
		   DBMS_OUTPUT.PUT_LINE('S''ha desfet la transacci');
	
		WHEN OTHERS THEN

		   ROLLBACK;
		   DBMS_OUTPUT.PUT_LINE('Error al registrar el log. Error SQL: '||SQLCODE||'-'||SQLERRM);
		   DBMS_OUTPUT.PUT_LINE('S''ha desfet la transacci');
	
		END;








	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom_funci:		f_num_aleatori
	-- 
	-- Descripci:		calcula un nombre aleatori d'un mxim de 9 xifres que no sigui a la taula ciutadans i el retorna.
	--			
	-- Parmetres entrada:	--

	-- Retorn:		un nombre aleatori d'un mxim de 9 xifres
	-- 
	-- Excepcions:		OTHERS	-> qualsevol esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	FUNCTION f_num_aleatori RETURN NUMBER IS

		num 	NUMBER(9) := 0;
		trobat	NUMBER(1) := 0;
		err	VARCHAR2(200 CHAR);

		BEGIN

		  -- es genera un valor aleatori d'un mxim de 9 dgits i es mira si ja existeixi a la taula ciutadans (si existeix, trobat = 1)
		  num := DBMS_RANDOM.value * 1000000000;
		  SELECT COUNT(*) INTO trobat FROM ciutadans WHERE num_aleatori = num;

		  -- mentre el valor existeixi a la taula ciutadans ( trobat = 1 ), s'aniran generant nous aleatoris 
		  WHILE ( trobat = 1 ) LOOP
			num := DBMS_RANDOM.value * 1000000000;
		  	SELECT COUNT(*) INTO trobat FROM ciutadans WHERE num_aleatori = num;
		  END LOOP;

		  RETURN num;

		EXCEPTION

		WHEN OTHERS THEN
	
		   err := 'Error: '||SQLCODE||'-'||SQLERRM;
		   DBMS_OUTPUT.PUT_LINE(err);
		   RETURN num;

		END;










	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom_funci:		f_data_aleatoria
	-- 
	-- Descripci:		Genera una cadena aleatria que representa una data en format dd/mm/yyyy 
	-- 			dintre del rang 01/01/1901 - 31/12/1992 (excloent els 29 de febrer)
	--			
	-- Parmetres entrada:	--

	-- Retorn:		una cadena aleatria que representa una data en format dd/mm/yyyy
	-- 
	-- Excepcions:		OTHERS	-> qualsevol esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	FUNCTION f_data_aleatoria RETURN VARCHAR2 IS

		data_x	VARCHAR2(12 CHAR);
		dia_x	NUMBER(3) := 0;
		mes_x	NUMBER(3) := 0;
		any_x	NUMBER(5) := 0;
		trobat	NUMBER(1) := 0;
		n_dies	NUMBER(2) := 0;
		err	VARCHAR2(200 CHAR);

		BEGIN

		   -- obtenim un nmero aleatori de dos dgits dins el rang {1, 12} que representaran un mes
		   trobat := 0;
		   WHILE (trobat = 0) LOOP

			mes_x := DBMS_RANDOM.value * 100;
			IF (mes_x > 0 AND mes_x < 13) THEN trobat := 1; END IF;

		   END LOOP;

		   -- en funci del mes obtingut determinem els dies que t		
		   n_dies := 31;
		   IF (mes_x = 2) THEN n_dies := 28;
		   ELSIF (mes_x = 4) THEN n_dies := 30;
		   ELSIF (mes_x = 6) THEN n_dies := 30;
		   ELSIF (mes_x = 9) THEN n_dies := 30;
		   ELSIF (mes_x = 11) THEN n_dies := 30;
		   END IF;

		   -- obtenim un nmero aleatori de dos dgits dins el rang {1, n_dies} que representaran un dia del mes
		   trobat := 0;
		   WHILE (trobat = 0) LOOP

			dia_x := DBMS_RANDOM.value * 100;
			IF (dia_x > 0 AND dia_x <= n_dies ) THEN trobat := 1; END IF;

		   END LOOP;

		   -- obtenim un nmero alatori de quatre dgits dins el rang {1901, 1992} que representaran un any
		   trobat := 0;
		   WHILE (trobat = 0) LOOP

			any_x := DBMS_RANDOM.value * 10000;
			IF ( any_x > 1900 AND any_x <= 1992 ) THEN trobat := 1; END IF;

		   END LOOP;

		   -- formatem la cadena de retorn amb els valors aleatoris obtinguts
		   data_x := dia_x||'/'||mes_x||'/'||any_x;


		   RETURN data_X;


		EXCEPTION

		WHEN OTHERS THEN
	
		   err := 'Error: '||SQLCODE||'-'||SQLERRM;
		   DBMS_OUTPUT.PUT_LINE(err);
		   RETURN data_x;

		END;




	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom_funci:		f_opcio_aleatoria
	-- 
	-- Descripci:		Donat el codi d'una votaci i d'un codi ISO d'un pas que particip a la votaci, la funci genera un nombre 
	-- 			aleatori que representa una opci de vot vlida dins la votaci i vlida pels electors del pas (electors de
	-- 			pasos diferents que participin a una mateixa votaci tindran un codi de pas diferent que determinar quines
	-- 			opcions de vot tenen disponibles. Aix servir, per exemple, perqu un ciutad alemany no pugui votar ERC).
	-- 
	-- 			Notes: 
	-- 			1) Es considera que les opcions de vot que pot escollir un elector seran opcions que pertanyin al mateix
	-- 			pas que l'elector o b determinades opcions que pertanyen a tots els pasos. Aquestes darreres sn les
	-- 			opcions de vot 'SI' i 'NO', en el cas que la votaci fos una consulta, i 'BLANC' i 'NUL' en el cas d'una
	-- 			votaci normal. Totes quatre estan representades pel codi ISO 'ZQ' que no est assignat a cap pas i que 
	-- 			l'he fet servir com a codi de 'pas neutre'.
	-- 			
	-- 			2) Com que, escollides aleatriament, les opcions de vot 'BLANC' i 'NUL' tenen la mateixa probabilitat de
	-- 			ser escollides que les altres opcions i podrien guanyar fcilment les eleccions, per fer-ho ms realista s'ha
	-- 			moderat aquesta aleatorietat fent que aproximadament el 90% de les vegades que el nmero aleatori representi
	-- 			una d'aquestes opcions, es descarti el nmero i se'n generi un altre.
	-- 
	--			
	-- Parmetres entrada:	i_votacio	NUMBER		(hauria de repesentar l'identificador d'una votaci amb opcions de vot associades)
	-- 			cod_pais	CHAR		(hauria de representar el codi ISO del pas d'un elector)
	-- 
	-- Retorn:		Si i_votaci referencia una votaci amb opcions de vot associades i cod_pais referencia el codi ISO d'un pas, 
	-- 			retornar un nmero natural que representa el codi d'una opci de vot vlida per la votaci i pels electors 
	-- 			d'aquell pas (incloent opcions de vot 'BLANC' i 'NUL'). En cas d'error, retorna zero i l'excepci.
	-- 
	-- Excepcions:		votacio_inexistent	-> quan i_votacio no referencia cap votaci.
	-- 			codi_pais_inexistent	-> quan cod_pais no referencia el codi ISO de cap pas del sistema
	-- 			no_opcions		-> quan la votaci existeix per encara no t opcions de vot associades
	-- 			pais_votacio_inexistent	-> quan la votaci existeix i el codi ISO referencia un pas del sistema
	-- 						   per aquest pas no participa a la votaci (la votaci no t cap cens
	-- 						   d'aquest pas).
	-- 			OTHERS			-> qualsevol esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	FUNCTION f_opcio_aleatoria(i_votacio IN NUMBER, cod_pais CHAR) RETURN NUMBER IS

		aleatori	NUMBER(4) := 0;
		trobat		NUMBER(1) := 0;
		moderador	NUMBER := 0;
		opcions		NUMBER := 0;
		opc_min		NUMBER(2) := 0;
		opc_max		NUMBER(2) := 0;
		pais		VARCHAR2(30 CHAR);
		err		VARCHAR2(200 CHAR);
		votacio_inexistent	EXCEPTION;
		codi_pais_inexistent	EXCEPTION;
		no_opcions		EXCEPTION;
		pais_votacio_inexistent	EXCEPTION;

		BEGIN

		-- comprovem que l'identificador de votaci existeixi
		trobat := 0;
		SELECT COUNT(*) INTO trobat FROM votacions WHERE id = i_votacio;
		IF ( trobat = 0 ) THEN
			RAISE votacio_inexistent;
		END IF;

		-- comprovem que el codi de pas de l'elector existeixi
		trobat := 0;
		SELECT COUNT(*) INTO trobat FROM paisos WHERE codi_pais = cod_pais;
		IF ( trobat = 0 ) THEN
			RAISE codi_pais_inexistent;
		END IF;


		-- comprovem que la votacio tingui opcions associades
		SELECT COUNT(*) INTO opcions FROM opcionsVotacio WHERE id_votacio = i_votacio;
		IF ( opcions = 0) THEN
			RAISE no_opcions;
		END IF;


		-- comprovem que el codi del pais estigui associat a la votaci
		SELECT nom_pais INTO pais FROM paisos WHERE codi_pais = cod_pais;
		trobat := 0;
		SELECT COUNT(*) INTO trobat FROM votacionsPaisos WHERE id_pais = pais AND id_votacio = i_votacio;
		IF ( trobat = 0 ) THEN
			RAISE pais_Votacio_inexistent;
		END IF;


		-- determinem l'identificador d'opci mxim i mnim de la votaci en qesti
		SELECT MIN(id_opcio) INTO opc_min FROM opcionsVotacio WHERE id_votacio = i_votacio;
		SELECT MAX(id_opcio) INTO opc_max FROM opcionsVotacio WHERE id_votacio = i_votacio;


		trobat := 0;
		moderador := 0;
		WHILE ( trobat = 0 ) LOOP

			moderador := moderador+1;

			-- es genera un aleatori d'un mxim de dues xifres (per ara considerem que no hi haur ms 
			-- de 99 opcions de vot diferent en una votaci)
			aleatori := DBMS_RANDOM.value * 100;

			-- si l'aleatori est dintre el rang d'opcions possibles per la votaci, llavors...
			IF (aleatori >= opc_min AND aleatori <= opc_max ) THEN

				-- comprovem si representa una opci que correspon realment a la votaci (en cas afirmatiu, posar trobat a 1)
				SELECT COUNT(*) INTO trobat FROM opcionsVotacio WHERE id_votacio = i_votacio AND id_opcio = aleatori;

				IF ( trobat = 1 ) THEN

					-- comprovem si representa una opci del pas de l'elector (si no ho s posar trobat a 0)
					SELECT COUNT(*) INTO trobat FROM opcions WHERE id = aleatori AND codi_pais = cod_pais;

					IF ( trobat = 0 ) THEN

						-- Aqu dins, l'aleatori no representa cap opci del pas de l'elector. Tenim dos casos:
						-- 1) representa una opci que no pot escollir l'elector (no s del seu pas)
						-- 2) representa una opci que poden escollir els electors de tots els pasos. Es tracta de
						--    les opcions de vot 'SI', 'NO', 'BLANC' o  'NUL' i que estan representades
						--    per un codi de pas neutre 'ZQ'.

						-- comprovem si representa una opci del 'pas neutre' ZQ.
						-- En cas afirmatiu trobat valdr 1
						SELECT COUNT(*) INTO trobat FROM opcions WHERE id = aleatori AND codi_pais = 'ZQ';

						-- Com que aleatriament les opcions de vot del pas neutre ZQ tenen la mateixa probabilitat
						-- que la resta d'opcions de vot, podrien arribar a guanyar les eleccions fcilment opcions de
						-- de vot en BLANC o opcions de vot NUL. Per fer-ho ms realista, moderarem la quantitat d'aquests
						-- vots. Considerarem vlids un 10% (de promig) dels nmeros aleatoris que representin aquestes
						-- opcions de vot. Es consideraran vlides aquelles opcions de vot del pas neutre que s'obtinguin
						-- en les iteracions del bucle acabades en 1 {1, 11, 21, ..., n10+1}
						IF ( trobat = 1 ) THEN

							IF ( MOD(moderador, 10) <> 1) THEN
								trobat := 0;
							END IF;

						END IF;

					END IF;

				END IF;

			-- l'aleatori no est dintre el rang d'opcions possibles per la votaci
			ELSE
				trobat := 0;
			END IF;

		END LOOP;

		RETURN aleatori;


		EXCEPTION

		WHEN votacio_inexistent THEN
	
		   err := 'Error: l''identificador de votacio no correspon a cap votacio del sistema.';
		   DBMS_OUTPUT.PUT_LINE(err);
		   RETURN aleatori;

		WHEN codi_pais_inexistent THEN
	
		   err := 'Error: el codi del pais de l''elector no correspon a cap pais del sistema.';
		   DBMS_OUTPUT.PUT_LINE(err);
		   RETURN aleatori;

		WHEN no_opcions THEN
	
		   err := 'Error: la votacio no te opcions associades.';
		   DBMS_OUTPUT.PUT_LINE(err);
		   RETURN aleatori;

		WHEN pais_votacio_inexistent THEN
	
		   err := 'Error: el pais representat pel seu codi no participa a la votacio.';
		   DBMS_OUTPUT.PUT_LINE(err);
		   RETURN aleatori;

		WHEN OTHERS THEN
	
		   err := 'Error: '||SQLCODE||'-'||SQLERRM;
		   DBMS_OUTPUT.PUT_LINE(err);
		   RETURN aleatori;

		END;











END; --fi implementaci paquet pk_utils
/








-- ###################################################################   pk_territoris    ########################################################################




-- ******************************************************************************************************************************
-- ******************************************************************************************************************************
-- *																*
-- *						FASE D'ESPECIFICACI								*
-- *																*
-- ******************************************************************************************************************************
-- ******************************************************************************************************************************



-- -----------------------------------------------------------------------------------------------------------------------------
-- -----------------------------------------------------------------------------------------------------------------------------
-- Nom_paquet: 		pk_territoris
-- 
-- Descripci:		especificaci de diferents procediments relacionats amb els territoris i els seus censos
-- 
-- Procediments:	p_altaCodisISO, p_altaPais, p_altaRegio, p_altaLocalitat, p_altaCensGlobal, p_altaCensEstatal
-- 			p_altaCensRegional, p_altaCensLocal, p_baixaCensGlobal, p_baixaCensEstatl, p_baixaCensRegional,
-- 			p_baixaCensLocal
-- Funcions:		--
-- ------------------------------------------------------------------------------------------------------------------------------
-- ------------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE pk_territoris IS



	PROCEDURE p_altaCodisISO (
	  codi			IN	codisISO.codi_pais%TYPE,
	  pais			IN	codisISO.nom_pais%TYPE,
	  RSP			OUT	VARCHAR2
	  );




	PROCEDURE p_altaPais (
	  pais			IN	paisos.nom_pais%TYPE,
	  codi			IN	paisos.codi_pais%TYPE,
	  RSP			OUT	VARCHAR2
	  );



	PROCEDURE p_altaRegio (
	  id_regio		IN	regions.id%TYPE,
	  pais			IN	regions.id_pais%TYPE,
	  regio			IN	regions.nom_regio%TYPE,
	  RSP			OUT	VARCHAR2
	  );



	PROCEDURE p_altaLocalitat (
	  id			IN	localitats.id%TYPE,
	  regio			IN	localitats.id_regio%TYPE,
	  localitat		IN	localitats.nom_localitat%TYPE,
	  RSP			OUT	VARCHAR2
	  );



	PROCEDURE p_altaCensGlobal (
	  n_cens		IN	censGlobal.nom_cens%TYPE,
	  RSP			OUT	VARCHAR2
	  );



	PROCEDURE p_altaCensEstatal (
	  nom_cens		IN	censosEstatals.nom_cens%TYPE,
	  id_cens_global	IN	censosEstatals.id_cens_global%TYPE,
	  cod_pais		IN	censosEstatals.codi_pais%TYPE,
	  RSP			OUT	VARCHAR2
	  );



	PROCEDURE p_altaCensRegional (
	  nom_cens		IN	censosRegionals.nom_cens%TYPE,
	  id_cens_estat		IN	censosRegionals.id_cens_estatal%TYPE,
	  id_regio		IN	censosRegionals.id_regio%TYPE,
	  nom_regio		IN	censosRegionals.nom_regio%TYPE,
	  RSP			OUT	VARCHAR2
	  );


	PROCEDURE p_altaCensLocal (
	  nom_cens		IN	censosLocals.nom_cens%TYPE,
	  id_cens_regio		IN	censosLocals.id_cens_regional%TYPE,
	  id_localitat		IN	censosLocals.id_localitat%TYPE,
	  nom_localitat		IN	censosLocals.nom_localitat%TYPE,
	  num_censats		IN	censosLocals.ciutadans_censats%TYPE,
	  RSP			OUT	VARCHAR2
	  );



	PROCEDURE p_baixaCensGlobal (
	  nom_cens_g	IN	censGlobal.nom_cens%TYPE,
	  RSP		OUT	VARCHAR2
	  );


	PROCEDURE p_baixaCensEstatal (
	  nom_cens_e	IN	censosEstatals.nom_cens%TYPE,
	  RSP		OUT	VARCHAR2
	  );


	PROCEDURE p_baixaCensRegional (
	  nom_cens_r	IN	censosRegionals.nom_cens%TYPE,
	  RSP		OUT	VARCHAR2
	  );


	PROCEDURE p_baixaCensLocal (
	  nom_cens_l	IN	censosLocals.nom_cens%TYPE,
	  RSP		OUT	VARCHAR2
	  );




END; --fi especificaci paquet pk_territoris
/









-- ******************************************************************************************************************************
-- ******************************************************************************************************************************
-- *																*
-- *						FASE D'IMPLEMENTACI								*
-- *																*
-- ******************************************************************************************************************************
-- ******************************************************************************************************************************




-- -----------------------------------------------------------------------------------------------------------------------------
-- -----------------------------------------------------------------------------------------------------------------------------
-- Nom_paquet: 		pk_territoris
-- 
-- Descripci:		implementaci de diferents procediments relacionats amb els territoris i els seus censos
-- 
-- Procediments:	p_altaCodisISO, p_altaPais, p_altaRegio, p_altaLocalitat, p_altaCensGlobal, p_altaCensEstatal
-- 			p_altaCensRegional, p_altaCensLocal, p_baixaCensGlobal, p_baixaCensEstatl, p_baixaCensRegional,
-- 			p_baixaCensLocal
-- Funcions:		--
-- ------------------------------------------------------------------------------------------------------------------------------
-- ------------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE BODY pk_territoris IS



	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaCodisISO
	-- 			
	-- Descripci:		insereix un registre a la taula codisISO i crida el procediment p_registrarLog del paquet pk_utils
	-- 			
	-- Parmetres entrada:	codi	CHAR(2)		PK
	-- 			pais	VARCHAR2(30)	NOT NULL	UNIQUE
	-- 			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori	-> quan un parmetre d'entrada s buit o NULL
	-- 			camp_incorrecte -> quan el codi ISO no t exactament 2 carcters
	-- 			codi_existent	-> codi referencia un codi que ja existeix a la taula codisISO
	-- 			pais_existent	-> quan pais referencia el nom d'un pas que ja existeix a la taula codisISO (nom pas s UNIQUE)
	--			OTHERS		-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaCodisISO (
	  codi		IN	codisISO.codi_pais%TYPE,
	  pais		IN	codisISO.nom_pais%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_altaCodisISO';
	  	parametres_entrada 	VARCHAR2(40 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
	  	camp_obligatori 	EXCEPTION;
		camp_incorrecte 	EXCEPTION;
		codi_existent		EXCEPTION;
		pais_existent		EXCEPTION;


		BEGIN

		  parametres_entrada := codi||', '||pais;

		  IF (  codi IS NULL OR 
			pais IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (length(codi) <> 2) THEN
				RAISE camp_incorrecte;
		  END IF;



		  -- comprovem que el codi del pas no existeixi ja a la taula codisISO
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM codisISO WHERE codi_pais = codi;
		  IF ( trobat = 1 ) THEN
			RAISE codi_existent;
		  END IF;

		  -- comprovem que el nom del pas no existeixi ja a la taula codisISO (el nom del pas s clau alternativa)
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM codisISO WHERE nom_pais = pais;
		  IF ( trobat = 1 ) THEN
			RAISE pais_existent;
		  END IF;


		  INSERT INTO codisISO(codi_pais, nom_pais) VALUES(codi, pais);

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN camp_incorrecte THEN

		   RSP := 'ERROR: el codi ha de tenir 2 caracters.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN codi_existent THEN

		   RSP := 'ERROR: el codi del pais ja existeix a la taula codisISO.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN pais_existent THEN

		   RSP := 'ERROR: el nom del pais ja existeix a la taula codisISO. El nom ha de ser unic.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;






	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaPais
	-- 			
	-- Descripci:		insereix un registre a la taula paisos i crida el procediment p_registrarLog del paquet pk_utils
	-- 			
	-- Parmetres entrada:	nom_pais	VARCHAR2(30 CHAR)	PK
	-- 			codi_pais	CHAR(2)			NOT NULL	UNIQUE	REFERENCES codisISO(codi_pais)
	-- 			
	-- Parmetres sortida:	RSP		VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 							al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada s buit o NULL
	-- 			camp_incorrecte 	-> quan el codi ISO no t exactament 2 carcters
	-- 			codiISOException	-> quan codi no referencia cap codi ISO
	-- 			pais_existent		-> quan pais referencia el nom d'un pas que ja existeix a la taula paisos
	-- 			codi_existent		-> codi referencia un codi que ja existeix a la taula paisos  (codi_pais s UNIQUE)
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaPais (
	  pais		IN	paisos.nom_pais%TYPE,
	  codi		IN	paisos.codi_pais%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_altaPais';
	  	parametres_entrada 	VARCHAR2(40 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
	  	camp_obligatori 	EXCEPTION;
		camp_incorrecte 	EXCEPTION;
		codiISO_inexistent	EXCEPTION;
		pais_existent		EXCEPTION;
		codi_existent		EXCEPTION;


		BEGIN

		  parametres_entrada := pais||', '||codi;

		  IF (  pais IS NULL OR 
			codi IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (length(codi) <> 2) THEN
				RAISE camp_incorrecte;
		  END IF;

		  -- comprovem que 'codi' referenci un codiISO
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM codisISO WHERE codi_pais = codi;
		  IF ( trobat = 0 ) THEN
			RAISE codiISO_inexistent;
		  END IF;

		  -- comprovem que el nom del pas no existeixi ja a la taula paisos
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM paisos WHERE nom_pais = pais;
		  IF ( trobat = 1 ) THEN
			RAISE pais_existent;
		  END IF;


		  -- comprovem que el codi del pas no existeixi ja a la taula paisos  (el codi del pas s clau alternativa)
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM paisos WHERE codi_pais = codi;
		  IF ( trobat = 1 ) THEN
			RAISE codi_existent;
		  END IF;


		  -- donem d'alta el pas
		  INSERT INTO paisos(nom_pais, codi_pais) VALUES(pais, codi);

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN camp_incorrecte THEN

		   RSP := 'ERROR: el codi ha de tenir 2 carcters.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN codiISO_inexistent THEN

		   RSP := 'ERROR: l''identificador de codi ISO no correspon a cap codi ISO del sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN pais_existent THEN

		   RSP := 'ERROR: el nom del pais ja existeix a la taula paisos.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	

		WHEN codi_existent THEN

		   RSP := 'ERROR: el codi del pais ja existeix a la taula paisos. El codi del pais ha de ser unic.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;








	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaRegio
	-- 			
	-- Descripci:		insereix un registre a la taula Regions i crida el procediment p_registrarLog del paquet pk_utils
	-- 			
	-- Parmetres entrada:	id_regio	NUMBER		PK		AUTO INCREMENT
	-- 			pais		VARCHAR2(30)	NOT NULL	REFERENCES paisos(nom_pais)
	-- 			regio		VARCHAR2(40)	NOT NULL	UNIQUE(pais, regio)
	-- 
	-- Parmetres sortida:	RSP		VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 							al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori	-> quan un parmetre d'entrada s buit o NULL
	-- 			camp_incorrecte -> quan el parmetre d'entrada id s >= 0
	-- 			pais_inexistent	-> quan pais no refeencia cap pas
	--			OTHERS		-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaRegio (
	  id_regio	IN	regions.id%TYPE,
	  pais		IN	regions.id_pais%TYPE,
	  regio		IN	regions.nom_regio%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_altaRegio';
	  	parametres_entrada 	VARCHAR2(80 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
	  	camp_obligatori 	EXCEPTION;
		camp_incorrecte 	EXCEPTION;
		pais_inexistent		EXCEPTION;


		BEGIN

		  parametres_entrada := id_regio||', '||pais||', '||regio;

		  IF (  id_regio IS NULL OR
			pais IS NULL OR 
			regio IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF ( id_regio >=0 ) THEN
				RAISE camp_incorrecte;
		  END IF;


		  -- comprovem pais referenci un pais
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM paisos WHERE nom_pais = pais;
		  IF ( trobat = 0 ) THEN
			RAISE pais_inexistent;
		  END IF;


		  INSERT INTO regions(id, id_pais, nom_regio) VALUES(id_regio, pais, regio);

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN camp_incorrecte THEN

		   RSP := 'ERROR: el primer parametre no es correcte.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN pais_inexistent THEN

		   RSP := 'ERROR: l''identificador del pais no correspon a cap pais del sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;










	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaLocalitat
	-- 			
	-- Descripci:		insereix un registre a la taula Localitats i crida el procediment p_registrarLog del paquet pk_utils
	-- 			
	-- Parmetres entrada:	id		NUMBER		PK		AUTO INCREMENT
	-- 			regio		VARCHAR2(40)	NOT NULL	REFERENCES regions(id)
	-- 			localitat	VARCHAR2(40)	NOT NULL	UNIQUE(regio, localitat)
	-- 
	-- Parmetres sortida:	RSP		VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 							al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada s buit o NULL
	-- 			camp_incorrecte 	-> quan el parmetre d'entrada id s >= 0
	-- 			regio_inexistent	-> quan regio no referencia cap regi del sistema
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaLocalitat (
	  id		IN	localitats.id%TYPE,
	  regio		IN	localitats.id_regio%TYPE,
	  localitat	IN	localitats.nom_localitat%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_altaLocalitat';
	  	parametres_entrada 	VARCHAR2(100 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
	  	camp_obligatori 	EXCEPTION;
		camp_incorrecte 	EXCEPTION;
		regio_inexistent	EXCEPTION;


		BEGIN

		  parametres_entrada := id||', '||regio||', '||localitat;

		  IF (  id IS NULL OR
			regio IS NULL OR 
			localitat IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF ( id >=0 ) THEN
				RAISE camp_incorrecte;
		  END IF;


		  -- comprovem 	que regio referenci una regio existent
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM regions WHERE id = regio;
		  IF ( trobat = 0 ) THEN
			RAISE regio_inexistent;
		  END IF;


		   INSERT INTO localitats(id, id_regio, nom_localitat) VALUES(id, regio, localitat);

		   RSP := 'OK';
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN camp_incorrecte THEN

		   RSP := 'ERROR: el primer parametre no es correcte.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN regio_inexistent THEN

		   RSP := 'ERROR: l''identificador de regio no correspon a cap regio del sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;









	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaCensGlobal
	-- 			
	-- Descripci:		insereix un registre a la taula censGlobal i crida el procediment p_registrarLog del paquet pk_utils
	-- 			
	-- Parmetres entrada:	n_cens		VARCHAR2(30)	PK
	-- 
	-- Parmetres sortida:	RSP		VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 							al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori	-> quan un parmetre d'entrada s buit o NULL
	-- 			camp_incorrecte -> quan el nom del cens supera els 30 carcters
	-- 			cens_existent	-> quan el nom del cens ja existeix a la taula censGlobal
	--			OTHERS		-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaCensGlobal (
	  n_cens	IN	censGlobal.nom_cens%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_altaCensGlobal';
	  	parametres_entrada 	VARCHAR2(60 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
	  	camp_obligatori 	EXCEPTION;
		camp_incorrecte 	EXCEPTION;
		cens_existent		EXCEPTION;


		BEGIN

		  parametres_entrada := n_cens;

		  IF ( n_cens IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;


		  IF ( length(n_cens) > 30 ) THEN
				RAISE camp_incorrecte;
		  END IF;


		  -- comprovem que el nom del cens no existeixi ja a la taula censGlobal
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM censGlobal WHERE nom_cens = n_cens;
		  IF ( trobat = 1 ) THEN
			RAISE cens_existent;
		  END IF;


		  INSERT INTO censGlobal(nom_cens) VALUES(n_cens);

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: El nom del cens es buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN camp_incorrecte THEN

		   RSP := 'ERROR: El nom del cens es massa llarg (maxim 30 caracters).';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN cens_existent THEN

		   RSP := 'ERROR: El nom del cens ja existeix a la taula censGlobal.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;






	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaCensEstatal
	-- 			
	-- Descripci:		insereix un registre a la taula censosEstatals i crida el procediment p_registrarLog del paquet pk_utils
	-- 			
	-- Parmetres entrada:	nom_cens	VARCHAR2(30)	PK
	-- 			id_cens_global	VARCHAR2(30)	NOT NULL	REFERENCES censGlobal(nom_cens)
	-- 			cod_pais	CHAR(2)		NOT NULL	REFERENCES paisos(codi_pais)
	-- 
	-- Parmetres sortida:	RSP		VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 							al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada s buit o NULL
	-- 			camp_incorrecte 	-> quan el codi del pas no t exactament 2 carcters
	-- 			cens_global_inexistent	-> quan id_cens_global no referencia el cens global del sistema
	-- 			pais_inexistent		-> quan cod_pais no referencia el codi de cap pas del sistema
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaCensEstatal (
	  nom_cens		IN	censosEstatals.nom_cens%TYPE,
	  id_cens_global	IN	censosEstatals.id_cens_global%TYPE,
	  cod_pais		IN	censosEstatals.codi_pais%TYPE,
	  RSP			OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_altaCensEstatal';
	  	parametres_entrada 	VARCHAR2(70 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
	  	camp_obligatori 	EXCEPTION;
		camp_incorrecte 	EXCEPTION;
		cens_global_inexistent	EXCEPTION;
		pais_inexistent		EXCEPTION;


		BEGIN


		  parametres_entrada := nom_cens||', '||id_cens_global||', '||cod_pais;

		  IF (  nom_cens IS NULL OR
			id_cens_global IS NULL OR 
			cod_pais IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF ( length(cod_pais) <> 2 ) THEN
				RAISE camp_incorrecte;
		  END IF;



		  -- comprovem que el cens global existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM censGlobal WHERE nom_cens = id_cens_global;
		  IF ( trobat = 0 ) THEN
			RAISE cens_global_inexistent;
		  END IF;
		  

		  -- comprovem el codi_pais referenci un pais
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM paisos WHERE codi_pais = cod_pais;
		  IF ( trobat = 0 ) THEN
			RAISE pais_inexistent;
		  END IF;

		  -- afegim el cens estatal
		  INSERT INTO censosEstatals(nom_cens, id_cens_global, codi_pais) VALUES(nom_cens, id_cens_global, cod_pais);

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN camp_incorrecte THEN

		   RSP := 'ERROR: el codi del pais ha de tenir 2 caracters.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN cens_global_inexistent THEN

		   RSP := 'ERROR: l''identificador de cens global no es correspon amb el cens global del sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN pais_inexistent THEN

		   RSP := 'ERROR: l''identificador del codi del pais no correspon a cap pais del sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;








	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaCensRegional
	-- 			
	-- Descripci:		insereix un registre a la taula censosRegionals i crida el procediment p_registrarLog del paquet pk_utils
	-- 			
	-- Parmetres entrada:	nom_cens	VARCHAR2(40)	PK
	-- 			id_cens_estat	VARCHAR2(30)	NOT NULL	REFERENCES censosEstatals(nom_cens)
	-- 			id_regio	NUMBER		NOT NULL	REFERENCES regions(id)
	-- 			nom_regio	VARCHAR2(40)
	-- 
	-- Parmetres sortida:	RSP		VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 							al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada s buit o NULL
	-- 			camp_incorrecte 	-> quan id_regio t un valor negatiu o zero
	-- 			cens_estatal_inexistent	-> quan l'id_cens_estat no referencia cap cens estatal existent
	-- 			regio_inexistent	-> quan id_regio no referencia cap regio existent
	-- 			n_regio_incoherent	-> quan nom_regio i id_regio sn mtuament incoherents
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaCensRegional (
	  nom_cens		IN	censosRegionals.nom_cens%TYPE,
	  id_cens_estat		IN	censosRegionals.id_cens_estatal%TYPE,
	  id_regio		IN	censosRegionals.id_regio%TYPE,
	  nom_regio		IN	censosRegionals.nom_regio%TYPE,
	  RSP			OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_altaCensRegional';
	  	parametres_entrada 	VARCHAR2(130 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		nom_reg			VARCHAR2(40 CHAR);
	  	camp_obligatori 	EXCEPTION;
		camp_incorrecte 	EXCEPTION;
		cens_estatal_inexistent	EXCEPTION;
		regio_inexistent	EXCEPTION;
		n_regio_incoherent	EXCEPTION;


		BEGIN


		  parametres_entrada := nom_cens||', '||id_cens_estat||', '||id_regio||', '||nom_regio;

		  IF (  nom_cens IS NULL OR
			id_cens_estat IS NULL OR 
			id_regio IS NULL OR 
			nom_regio IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF ( id_regio <= 0 ) THEN
				RAISE camp_incorrecte;
		  END IF;

		  -- comprovem que el cens estatal existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM censosEstatals WHERE nom_cens = id_cens_estat;
		  IF ( trobat = 0 ) THEN
			RAISE cens_estatal_inexistent;
		  END IF;

		  -- comprovem 	que id_regio referenci una regio existent
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM regions WHERE id = id_regio;
		  IF ( trobat = 0 ) THEN
			RAISE regio_inexistent;
		  END IF;

		  -- comprovem que el nom de la regi sigui coherent amb el cens regional
		  SELECT nom_regio INTO nom_reg FROM regions WHERE id = id_regio;
		  IF ( nom_regio <> nom_reg ) THEN
			RAISE n_regio_incoherent;
		  END IF;

		  -- afegim el cens regional
		  INSERT INTO censosRegionals(nom_cens, id_cens_estatal, id_regio, nom_regio) VALUES(nom_cens, id_cens_estat, id_regio, nom_regio);

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN camp_incorrecte THEN

		   RSP := 'ERROR: l''identificador de regio no es un enter positiu.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN cens_estatal_inexistent THEN

		   RSP := 'ERROR: l''identificador de cens estatal no correspon a cap cens estatal del sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN regio_inexistent THEN

		   RSP := 'ERROR: l''identificador de regio no correspon a cap regio del sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN n_regio_incoherent THEN

		   RSP := 'ERROR: el nom de regio no es coherent amb el cens regional.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;










	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaCensLocal
	-- 			
	-- Descripci:		insereix un registre a la taula censosLocals i crida el procediment p_registrarLog del paquet pk_utils
	-- 			
	-- Parmetres entrada:	nom_cens	VARCHAR2(40)	PK
	-- 			id_cens_regio	VARCHAR2(40)	NOT NULL	REFERENCES censosRegionals(nom_cens)
	-- 			id_localitat	NUMBER		NOT NULL	REFERENCES localitats(id)
	-- 			nom_localitat	VARCHAR2(40)	NOT NULL
	-- 			num_censats	NUMBER		NOT NULL	DEFAULT 0	CHECK(num_censats >=0)
	-- 			
	-- Parmetres sortida:	RSP		VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 							al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada s buit o NULL
	-- 			camp_incorrecte 	-> quan id_localitat t un valor negatiu o zero i/o quan num_censats no s zero
	-- 			cens_regio_inexistent	-> quan l'id_cens_regio no referencia cap cens regional existent
	-- 			localitat_inexistent	-> quan id_localitat no referencia cap localitat existent
	-- 			n_localitat_incoherent	-> quan nom_localitat i id_localitat sn mtuament incoherents
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaCensLocal (
	  nom_cens		IN	censosLocals.nom_cens%TYPE,
	  id_cens_regio		IN	censosLocals.id_cens_regional%TYPE,
	  id_localitat		IN	censosLocals.id_localitat%TYPE,
	  nom_localitat		IN	censosLocals.nom_localitat%TYPE,
	  num_censats		IN	censosLocals.ciutadans_censats%TYPE,
	  RSP			OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_altaCensLocal';
	  	parametres_entrada 	VARCHAR2(150 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		nom_local		VARCHAR2(40 CHAR);
	  	camp_obligatori 	EXCEPTION;
		camp_incorrecte 	EXCEPTION;
		cens_regio_inexistent	EXCEPTION;
		localitat_inexistent	EXCEPTION;
		n_localitat_incoherent	EXCEPTION;


		BEGIN


		  parametres_entrada := nom_cens||', '||id_cens_regio||', '||id_localitat||', '||nom_localitat||', '||num_censats;

		  IF (  nom_cens IS NULL OR 
			id_cens_regio IS NULL OR 
			id_localitat IS NULL OR 
			nom_localitat IS NULL OR
			num_censats IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF ( id_localitat <= 0 OR
			num_censats < 0 OR
			num_censats > 0 ) THEN
				RAISE camp_incorrecte;
		  END IF;

		  -- comprovem que el cens regional existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM censosRegionals WHERE nom_cens = id_cens_regio;
		  IF ( trobat = 0 ) THEN
			RAISE cens_regio_inexistent;
		  END IF;

		  -- comprovem 	que la localitat existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM localitats WHERE id = id_localitat;
		  IF ( trobat = 0 ) THEN
			RAISE localitat_inexistent;
		  END IF;

		  -- comprovem que el nom de la localitat sigui coherent amb el cens
		  SELECT nom_localitat INTO nom_local FROM localitats WHERE id = id_localitat;
		  IF ( nom_localitat <> nom_local ) THEN
			RAISE n_localitat_incoherent;
		  END IF;

		  -- afegim el cens local
		  INSERT INTO censosLocals(nom_cens, id_cens_regional, id_localitat, nom_localitat, ciutadans_censats) 
		  VALUES(nom_cens, id_cens_regio, id_localitat, nom_localitat, num_censats);

		   RSP := 'OK';
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN camp_incorrecte THEN

		   RSP := 'ERROR: l''id de localitat ha de ser un enter positiu i el nombre de ciutadans censats zero.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		WHEN cens_regio_inexistent THEN

		   RSP := 'ERROR: l''identificador de cens regional no correspon a cap cens regional del sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN localitat_inexistent THEN

		   RSP := 'ERROR: l''identificador de localitat no correspon a cap localitat del sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN n_localitat_incoherent THEN

		   RSP := 'ERROR: el nom de localitat no es coherent amb el cens local.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;


 






	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_baixaCensGlobal
	-- 			
	-- Descripci:		esborra un registre (l'nic) de la taula censGlobal (noms en el cas que no s'hi hagi celebrat cap votaci*) i
	-- 			crida el procediment p_registrarLog del paquet pk_utils.
	-- 
	-- 			*Nota: En el cas que no es controls, l'esborrat d'un cens global provacaria l'esborrat en cascada de gaireb 
	-- 			totes les dades relacionades amb les votacions del cens; la qual cosa es considera (considero) que no t cap
	-- 			sentit. Per tant, per no perdre informaci de votacions que ja s'hagin celebrat, noms es permetr donar de
	-- 			baixa un cens global si no s'hi ha celebrat cap votaci (o si no n'hi ha cap associada al cens que s'estigui 
	-- 			celebrant en aquest moment).
	-- 			
	-- Parmetres entrada:	nom_cens_g	VARCHAR2(30)	
	-- 
	-- Parmetres sortida:	RSP		VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 							al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada s buit o NULL
	-- 			cens_inexistent		-> quan nom_cens_g no referencia cap cens
	-- 			votacions_obertes	-> quan el cens t alguna votaci associada en estat 'oberta'
	-- 			votacions_finalitzades	-> quan el cens t alguna votaci associada en estat 'finalitzada'
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_baixaCensGlobal (
	  nom_cens_g	IN	censGlobal.nom_cens%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_baixaCensGlobal';
	  	parametres_entrada 	VARCHAR2(35 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		i_estat			NUMBER := 0;
		n_votacions_estat	NUMBER := 0;
	  	camp_obligatori 	EXCEPTION;
		cens_inexistent		EXCEPTION;
		votacions_obertes	EXCEPTION;
		votacions_finalitzades	EXCEPTION;


		BEGIN

		  parametres_entrada := nom_cens_g;

		  IF ( nom_cens_g IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;

		  -- comprovem que el cens existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM censGlobal WHERE nom_cens = nom_cens_g;
		  IF ( trobat = 0 ) THEN
			RAISE cens_inexistent;
		  END IF;

		  -- comprovem que hi hagi cap votaci en estat 'oberta' o 'finalitzada' que tingui associat el cens
		  n_votacions_estat := 0;
		  SELECT id INTO i_estat FROM estatsVotacio WHERE nom_estat = 'oberta';
		  SELECT COUNT(*) INTO n_votacions_estat FROM votacions WHERE id_cens_global = nom_cens_g AND id_estat = i_estat;
		  IF (n_votacions_estat > 0) THEN
			RAISE votacions_obertes;
		  END IF;
		  
		  n_votacions_estat := 0;
		  SELECT id INTO i_estat FROM estatsVotacio WHERE nom_estat = 'finalitzada';
		  SELECT COUNT(*) INTO n_votacions_estat FROM votacions WHERE id_cens_global = nom_cens_g AND id_estat = i_estat;
		  IF (n_votacions_estat > 0) THEN
			RAISE votacions_finalitzades;
		  END IF;
		  

		  DELETE FROM censGlobal WHERE nom_cens = nom_cens_g;

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: El nom del cens es buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);



		WHEN cens_inexistent THEN

		   RSP := 'ERROR: no es pot eliminar un cens que no existeix.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN votacions_obertes THEN

		   RSP := 'ERROR: no es pot eliminar un cens que tingui alguna votacio celebrant-se';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN votacions_finalitzades THEN

		   RSP := 'ERROR: no es pot eliminar un cens on ja s''hi hagin celebrat votacions.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;








	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_baixaCensEstatal
	-- 			
	-- Descripci:		esborra un registre de la taula censosEstatals (noms en el cas que no s'hi hagi celebrat cap votaci*) i
	-- 			crida el procediment p_registrarLog del paquet pk_utils.
	-- 
	-- 			*Nota: En el cas que no es controls, l'esborrat d'un cens estatal provacaria l'esborrat en cascada de gaireb 
	-- 			totes les dades relacionades amb les votacions del cens; la qual cosa es considera (considero) que no t cap
	-- 			sentit. Per tant, per no perdre informaci de votacions que ja s'hagin celebrat, noms es permetr donar de
	-- 			baixa un cens estatal si no s'hi ha celebrat cap votaci (o si no n'hi ha cap associada al cens que s'estigui 
	-- 			celebrant en aquest moment).
	-- 			
	-- Parmetres entrada:	nom_cens_e	VARCHAR2(30 CHAR)	
	-- 
	-- Parmetres sortida:	RSP		VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 							al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada s buit o NULL
	-- 			cens_inexistent		-> quan nom_cens_e no referencia cap cens
	-- 			votacions_obert_final	-> quan el cens t alguna votaci associada en estat 'oberta' i/o 'finalitzada'
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_baixaCensEstatal (
	  nom_cens_e	IN	censosEstatals.nom_cens%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_baixaCensEstatal';
	  	parametres_entrada 	VARCHAR2(35 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		i_estat_oberta		NUMBER := 0;
		i_estat_final		NUMBER := 0;
		i_estat_votacio		NUMBER := 0;
		n_censos_associats	NUMBER := 0;
	  	camp_obligatori 	EXCEPTION;
		cens_inexistent		EXCEPTION;
		votacions_obert_final	EXCEPTION;


		BEGIN

		  parametres_entrada := nom_cens_e;

		  IF ( nom_cens_e IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;

		  -- comprovem que el cens existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM censosEstatals WHERE nom_cens = nom_cens_e;
		  IF ( trobat = 0 ) THEN
			RAISE cens_inexistent;
		  END IF;

		  -- comprovem que hi hagi cap votaci en estat 'oberta' o 'finalitzada' que tingui associat el cens
		  trobat := 0;
		  n_censos_associats := 0;
		  SELECT COUNT(*) INTO n_censos_associats FROM censosAssociats WHERE cens_estatal = nom_cens_e;
		  IF ( n_censos_associats > 0 ) THEN


			SELECT id INTO i_estat_oberta FROM estatsVotacio WHERE nom_estat = 'oberta';
			SELECT id INTO i_estat_final FROM estatsVotacio WHERE nom_estat = 'finalitzada';

			-- recorrem les votacions que tenen el cens associat
			FOR votacio IN (SELECT DISTINCT id_votacio FROM censosAssociats WHERE cens_estatal = nom_cens_e) LOOP
				
				trobat := trobat;
				IF ( trobat = 0 ) THEN

					SELECT id_estat INTO i_estat_votacio FROM votacions WHERE id = votacio.id_votacio;
					IF ( i_estat_votacio = i_estat_oberta OR i_estat_votacio = i_estat_final ) THEN
						trobat := 1;
					END IF;

				END IF;
				
			END LOOP;

		  END IF;


		  IF ( trobat = 1 ) THEN
			RAISE votacions_obert_final;
		  END IF;

		  
		  -- Si el cens no ha estat associat a cap votaci i/o no s'ha associat a cap votaci que ja hagi comenat o que
		  -- hagi finalitzat, es podr esborrar

		  DELETE FROM censosEstatals WHERE nom_cens = nom_cens_e;


		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: El nom del cens es buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN cens_inexistent THEN

		   RSP := 'ERROR: no es pot eliminar un cens que no existeix.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN votacions_obert_final THEN

		   RSP := 'ERROR: no es pot eliminar un cens que tingui alguna votacio celebrant-se o finalitzada';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;









	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_baixaCensRegional
	-- 			
	-- Descripci:		esborra un registre de la taula censosRegionals (noms en el cas que no s'hi hagi celebrat cap votaci*) i
	-- 			crida el procediment p_registrarLog del paquet pk_utils.
	-- 
	-- 			*Nota: En el cas que no es controls, l'esborrat d'un cens regional provacaria l'esborrat en cascada de gaireb 
	-- 			totes les dades relacionades amb les votacions del cens; la qual cosa es considera (considero) que no t cap
	-- 			sentit. Per tant, per no perdre informaci de votacions que ja s'hagin celebrat, noms es permetr donar de
	-- 			baixa un cens regional si no s'hi ha celebrat cap votaci (o si no n'hi ha cap associada al cens que s'estigui 
	-- 			celebrant en aquest moment).
	-- 			
	-- Parmetres entrada:	nom_cens_r	VARCHAR2(40 CHAR)	
	-- 
	-- Parmetres sortida:	RSP		VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 							al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada s buit o NULL
	-- 			cens_inexistent		-> quan nom_cens_r no referencia cap cens
	-- 			votacions_obert_final	-> quan el cens t alguna votaci associada en estat 'oberta' i/o 'finalitzada'
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_baixaCensRegional (
	  nom_cens_r	IN	censosRegionals.nom_cens%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_baixaCensRegional';
	  	parametres_entrada 	VARCHAR2(35 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		i_estat_oberta		NUMBER := 0;
		i_estat_final		NUMBER := 0;
		i_estat_votacio		NUMBER := 0;
		n_censos_associats	NUMBER := 0;
	  	camp_obligatori 	EXCEPTION;
		cens_inexistent		EXCEPTION;
		votacions_obert_final	EXCEPTION;


		BEGIN

		  parametres_entrada := nom_cens_r;

		  IF ( nom_cens_r IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;

		  -- comprovem que el cens existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM censosRegionals WHERE nom_cens = nom_cens_r;
		  IF ( trobat = 0 ) THEN
			RAISE cens_inexistent;
		  END IF;

		  -- comprovem que hi hagi cap votaci en estat 'oberta' o 'finalitzada' que tingui associat el cens
		  trobat := 0;
		  n_censos_associats := 0;
		  SELECT COUNT(*) INTO n_censos_associats FROM censosAssociats WHERE cens_regional = nom_cens_r;
		  IF ( n_censos_associats > 0 ) THEN


			SELECT id INTO i_estat_oberta FROM estatsVotacio WHERE nom_estat = 'oberta';
			SELECT id INTO i_estat_final FROM estatsVotacio WHERE nom_estat = 'finalitzada';

			-- recorrem les votacions que tenen el cens associat
			FOR votacio IN (SELECT DISTINCT id_votacio FROM censosAssociats WHERE cens_regional = nom_cens_r) LOOP
				
				trobat := trobat;
				IF ( trobat = 0 ) THEN

					SELECT id_estat INTO i_estat_votacio FROM votacions WHERE id = votacio.id_votacio;
					IF ( i_estat_votacio = i_estat_oberta OR i_estat_votacio = i_estat_final ) THEN
						trobat := 1;
					END IF;

				END IF;
				
			END LOOP;

		  END IF;


		  IF ( trobat = 1 ) THEN
			RAISE votacions_obert_final;
		  END IF;

		  
		  -- Si el cens no ha estat associat a cap votaci i/o no s'ha associat a cap votaci que ja hagi comenat o que
		  -- hagi finalitzat, es podr esborrar

		  DELETE FROM censosRegionals WHERE nom_cens = nom_cens_r;


		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: El nom del cens es buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN cens_inexistent THEN

		   RSP := 'ERROR: no es pot eliminar un cens que no existeix.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN votacions_obert_final THEN

		   RSP := 'ERROR: no es pot eliminar un cens que tingui alguna votacio celebrant-se o finalitzada';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;









	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_baixaCensLocal
	-- 			
	-- Descripci:		esborra un registre de la taula censosLocals (noms en el cas que no s'hi hagi celebrat cap votaci*) i
	-- 			crida el procediment p_registrarLog del paquet pk_utils.
	-- 
	-- 			*Nota: En el cas que no es controls, l'esborrat d'un cens local provacaria l'esborrat en cascada de fora
	-- 			dades relacionades amb el cens (ciutadans, vots emesos, etc.); la qual cosa es considera (considero) que no t
	-- 			cap sentit per a votacions celebrades o celebrant-se. Per no perdre informaci d'aquestes votacions, noms es 
	-- 			permetr donar de baixa un cens local si no s'hi ha celebrat cap votaci (o si no n'hi ha cap associada al cens 
	-- 			que s'estigui celebrant en aquest moment).
	-- 			
	-- Parmetres entrada:	nom_cens_l	VARCHAR2(40 CHAR)	
	-- 
	-- Parmetres sortida:	RSP		VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 							al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada s buit o NULL
	-- 			cens_inexistent		-> quan nom_cens_r no referencia cap cens
	-- 			votacions_obert_final	-> quan el cens t alguna votaci associada en estat 'oberta' i/o 'finalitzada'
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_baixaCensLocal (
	  nom_cens_l	IN	censosLocals.nom_cens%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_baixaCensLocal';
	  	parametres_entrada 	VARCHAR2(35 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		i_estat_oberta		NUMBER := 0;
		i_estat_final		NUMBER := 0;
		i_estat_votacio		NUMBER := 0;
		n_censos_associats	NUMBER := 0;
	  	camp_obligatori 	EXCEPTION;
		cens_inexistent		EXCEPTION;
		votacions_obert_final	EXCEPTION;


		BEGIN

		  parametres_entrada := nom_cens_l;

		  IF ( nom_cens_l IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;

		  -- comprovem que el cens existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM censosLocals WHERE nom_cens = nom_cens_l;
		  IF ( trobat = 0 ) THEN
			RAISE cens_inexistent;
		  END IF;

		  -- comprovem que hi hagi cap votaci en estat 'oberta' o 'finalitzada' que tingui associat el cens
		  trobat := 0;
		  n_censos_associats := 0;
		  SELECT COUNT(*) INTO n_censos_associats FROM censosAssociats WHERE id_cens_local = nom_cens_l;
		  IF ( n_censos_associats > 0 ) THEN


			SELECT id INTO i_estat_oberta FROM estatsVotacio WHERE nom_estat = 'oberta';
			SELECT id INTO i_estat_final FROM estatsVotacio WHERE nom_estat = 'finalitzada';

			-- recorrem les votacions que tenen el cens associat
			FOR votacio IN (SELECT DISTINCT id_votacio FROM censosAssociats WHERE id_cens_local = nom_cens_l) LOOP
				
				trobat := trobat;
				IF ( trobat = 0 ) THEN

					SELECT id_estat INTO i_estat_votacio FROM votacions WHERE id = votacio.id_votacio;
					IF ( i_estat_votacio = i_estat_oberta OR i_estat_votacio = i_estat_final ) THEN
						trobat := 1;
					END IF;

				END IF;
				
			END LOOP;

		  END IF;


		  IF ( trobat = 1 ) THEN
			RAISE votacions_obert_final;
		  END IF;

		  
		  -- Si el cens no ha estat associat a cap votaci i/o no s'ha associat a cap votaci que ja hagi comenat o que
		  -- hagi finalitzat, es podr esborrar

		  DELETE FROM censosLocals WHERE nom_cens = nom_cens_l;


		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: El nom del cens es buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN cens_inexistent THEN

		   RSP := 'ERROR: no es pot eliminar un cens que no existeix.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN votacions_obert_final THEN

		   RSP := 'ERROR: no es pot eliminar un cens que tingui alguna votacio celebrant-se o finalitzada';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;





END; --fi implementaci paquet pk_territoris
/













-- ###################################################################   pk_ciutadans    ########################################################################





-- ******************************************************************************************************************************
-- ******************************************************************************************************************************
-- *																*
-- *						FASE D'ESPECIFICACI								*
-- *																*
-- ******************************************************************************************************************************
-- ******************************************************************************************************************************



-- ----------------------------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------------
-- Nom_paquet: 		pk_ciutadans
-- 
-- Descripci:		especificaci dels procediments d'ABM realcionats directament amb els ciutadans
-- 			
-- Procediments:	p_altaCiutada, p_baixaCiutada, p_modificaCiutada_dataNaix, p_modificaCiutada_censLocal
-- 
-- Funcions:		--
-- ----------------------------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE pk_ciutadans IS




	PROCEDURE p_altaCiutada (
	  n_aleatori		IN	ciutadans.num_aleatori%TYPE,
	  cens_local		IN	ciutadans.id_cens_local%TYPE,
	  id_local		IN	ciutadans.id_localitat%TYPE,
	  data_naix		IN	ciutadans.data_naixement%TYPE,
	  pais			IN	ciutadans.nom_pais%TYPE,
	  baixa			IN	ciutadans.baixa_logica%TYPE,
	  participacions	IN	ciutadans.num_participacions%TYPE,
	  RSP			OUT	VARCHAR2
	  );



	PROCEDURE p_baixaCiutada (
	  n_aleatori		IN	ciutadans.num_aleatori%TYPE,
	  RSP			OUT	VARCHAR2
	  );


	PROCEDURE p_modificaCiutada_dataNaix (
	  n_aleatori		IN	ciutadans.num_aleatori%TYPE,
	  nova_data_naix	IN	ciutadans.data_naixement%TYPE,
	  RSP			OUT	VARCHAR2
	  );


	PROCEDURE p_modificaCiutada_censLocal (
	  n_aleatori		IN	ciutadans.num_aleatori%TYPE,
	  nou_cens_local	IN	censosLocals.nom_cens%TYPE,
	  RSP			OUT	VARCHAR2
	  );


END; --fi especificaci paquet pk_ciutadans
/










-- ******************************************************************************************************************************
-- ******************************************************************************************************************************
-- *																*
-- *						FASE D'IMPLEMENTACI								*
-- *																*
-- ******************************************************************************************************************************
-- ******************************************************************************************************************************


-- ----------------------------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------------
-- Nom_paquet: 		pk_ciutadans
-- 
-- Descripci:		implementaci dels procediments d'ABM realcionats directament amb els ciutadans
-- 			
-- Procediments:	p_altaCiutada, p_baixaCiutada, p_modificaCiutada_dataNaix, p_modificaCiutada_censLocal
-- 
-- Funcions:		--
-- ----------------------------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE BODY pk_ciutadans IS





	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaCiutada
	-- 			
	-- Descripci:		insereix un registre a la taula ciutadans i incrementa en una unitat el camp ciutadans_censats del cens local
	-- 			al qual est assignat el ciutad. Finalment, crida el procediment p_registrarLog del paquet pk_utils.
	-- 			Es copmprova la coherncia de les dades del ciutad.
	-- 
	-- 			
	-- Parmetres entrada:	n_aleatori	NUMBER(9)			PK
	-- 			cens_local	VARCHAR2(40 CHAR)		NOT NULL	REFERENCES censosLocals(nom_cens)
	-- 			id_local	NUMBER				NOT NULL	REFERENCES localitats(id)
	-- 			data_naix	DATE				NOT NULL
	-- 			pais		VARCHAR2(30 CHAR)		NOT NULL	REFERENCES paisos(nom_pais)
	-- 			baixa		NUMBER(1)			NOT NULL	CHECK(baixa IN(0,1))
	-- 			participacions	NUMBER		DEFAULT 0	NOT NULL	CHECK(participacions >= 0)
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada que no pot ser buit o NULL ho s
	-- 			valor_incorrecte	-> quan algun dels parmetres id_local, baixa i/o participacions t un valor inesperat
	-- 			menor_edat		-> quan el ciutad no ha complert 18 anys en el moment de registrar-se al sistema.
	-- 			cens_inexistent		-> quan cens_local no referencia cap cens donal d'alta al sistema
	-- 			localitat_inexistent	-> quan id_local no referencia cap localitat donada d'alta al sistema
	-- 			pais_inexistent		-> quan 'pais' no referencia cap pas donat d'alta al sistema.
	-- 			dades_incoherents	-> quan els parmetres cens_local i id_local no sn coherents entre si.
	-- 			pais_incoherent		-> quan pais no s coherent amb el pas al qual correspon el cens local.
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaCiutada (
	  n_aleatori		IN	ciutadans.num_aleatori%TYPE,
	  cens_local		IN	ciutadans.id_cens_local%TYPE,
	  id_local		IN	ciutadans.id_localitat%TYPE,
	  data_naix		IN	ciutadans.data_naixement%TYPE,
	  pais			IN	ciutadans.nom_pais%TYPE,
	  baixa			IN	ciutadans.baixa_logica%TYPE,
	  participacions	IN	ciutadans.num_participacions%TYPE,
	  RSP			OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_altaCiutada';
	  	parametres_entrada 	VARCHAR2(150 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		auxiliar_DATE		DATE;
		data_actuaL_DATE	DATE;
		data_actual_CHAR	VARCHAR2(10 CHAR);
		data_naix_mes_18	VARCHAR2(10 CHAR);
		controla_zero		VARCHAR2(2 CHAR);
		censats			NUMBER(10) := 0;
		localitat		NUMBER(10) :=0;
		cens_regional		VARCHAR2(40 CHAR);
		cens_estatal		VARCHAR2(30 CHAR);
		cod_pais		CHAR(2);
		pais_cens_local		VARCHAR2(30 CHAR);
	  	camp_obligatori 	EXCEPTION;
		valor_incorrecte 	EXCEPTION;
		menor_edat		EXCEPTION;
		cens_inexistent		EXCEPTION;
		localitat_inexistent	EXCEPTION;
		pais_inexistent		EXCEPTION;
		dades_incoherents	EXCEPTION;
		pais_incoherent		EXCEPTION;


		BEGIN

		  parametres_entrada := n_aleatori||', '||cens_local||', '||id_local||', '||data_naix||', '||pais||', '||baixa||', '||participacions;

		  IF (  n_aleatori IS NULL OR 
			cens_local IS NULL OR
			id_local IS NULL OR
			data_naix IS NULL OR
			pais IS NULL OR
			baixa IS NULL OR
			participacions IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (	id_local <= 0 OR
			baixa <> 0 OR
			participacions <> 0) THEN
				RAISE valor_incorrecte;
		  END IF;



		  -- es comprova que la nova data de naixement correspongui a un ciutad major d'edat (> 18 anys). El sistema noms
		  -- permet emmagatzemar ciutadans amb dret a vot a la data actual (i.e. majors d'edat respecte la data actual del sistema).
		  SELECT SYSDATE INTO data_actual_DATE FROM DUAL;
		  data_naix_mes_18 := TO_CHAR(data_naix, 'ddmmyyyy')+18;
		  controla_zero := TO_CHAR(data_naix, 'DD');
		  IF (controla_zero < 10) THEN

			data_naix_mes_18 := '0'||data_naix_mes_18;

		  END IF;

		  auxiliar_DATE := TO_DATE(data_naix_mes_18);
		  data_naix_mes_18 := TO_CHAR(auxiliar_DATE, 'yyyymmdd');
		  data_actual_CHAR := TO_CHAR(data_actual_DATE, 'yyyymmdd');


		  IF ( data_naix_mes_18 > data_actual_CHAR ) THEN
			RAISE menor_edat;
		  END IF;



		  -- es comprova que cens_local referenci a un cens local donat d'alta al sistema
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM censosLocals WHERE nom_cens = cens_local;
		  IF ( trobat = 0 ) THEN
			RAISE cens_inexistent;
		  END IF;

		  -- es comprova que id_local referenci a una localitat donada d'alta al sistema
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM localitats WHERE id = id_local;
		  IF ( trobat = 0 ) THEN
			RAISE localitat_inexistent;
		  END IF;


		  -- es comprova que pais referenci un pas donat d'alta al sistema
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM paisos WHERE nom_pais = pais;
		  IF ( trobat = 0 ) THEN
			RAISE pais_inexistent;
		  END IF;


		  -- es comprova que el cens al qual es vol donar d'alta al ciutad estigui associat a la localitat
		  -- a la qual es vol associar el ciutad, s a dir, que cens_local i id_local siguin mtuament coherents
		  SELECT id_localitat INTO localitat FROM censosLocals WHERE nom_cens = cens_local;
		  IF ( localitat <> id_local ) THEN
			RAISE dades_incoherents;
		  END IF;

		  
		  -- es comprova que el nom del pais del ciutad que es dna d'alta sigui coherent amb el pas del cens local del ciutad
		  SELECT id_cens_regional INTO cens_regional FROM censosLocals WHERE nom_cens = cens_local;
		  SELECT id_cens_estatal INTO cens_estatal FROM censosRegionals WHERE nom_cens = cens_regional;
		  SELECT codi_pais INTO cod_pais FROM censosEstatals WHERE nom_cens = cens_estatal;
		  SELECT nom_pais INTO pais_cens_local FROM paisos WHERE codi_pais = cod_pais;
		  IF ( pais <> pais_cens_local ) THEN
			RAISE pais_incoherent;
		  END IF;
		  

		  -- s'insereix el ciutad
		  INSERT INTO ciutadans(num_aleatori, id_cens_local, id_localitat, data_naixement, nom_pais, baixa_logica, num_participacions)
		  VALUES(n_aleatori, cens_local, id_local, data_naix, pais, baixa, participacions);

		  -- i s'actualitza el nmero de ciutadans censats del cens local corresponent
		  SELECT ciutadans_censats INTO censats FROM censosLocals WHERE id_localitat = id_local;
		  UPDATE censosLocals SET ciutadans_censats = censats+1 WHERE id_localitat = id_local;

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN valor_incorrecte THEN

		   RSP := 'ERROR: Revisi el valor dels camps segents. Han de complir: id_localitat > 0, baixa_logica = 0 i participacions = 0 ';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN menor_edat THEN

		   RSP := 'ERROR: el sistema no enregistra ciutadans menors de 18 anys respecte la data actual.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN cens_inexistent THEN

		   RSP := 'ERROR: l''identificador del cens no correspon a cap cens local del sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN localitat_inexistent THEN

		   RSP := 'ERROR: l''identificador de la localitat no correspon a cap localitat del sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN pais_inexistent THEN

		   RSP := 'ERROR: l''identificador del pais no correspon a cap pais del sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN dades_incoherents THEN

		   RSP := 'ERROR: el cens al qual es vol assignar el ciutada no es de la localitat del mateix.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN pais_incoherent THEN

		   RSP := 'ERROR: el nom del pais no es coherent amb el pais al que correspon el cens local.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;















	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_baixaCiutada
	-- 			
	-- Descripci:		el procediment dna de baixa lgica un ciutad, decrementa en una unitat el nmero de ciutadans censats del cens
	-- 			local al qual corresponia el ciutad i crida el procediment p_registrarLog del paquet pk_utils.
	-- 			
	-- 			
	-- Parmetres entrada:	n_aleatori	NUMBER(9)	PK
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan n_aleatori s buit o NULL
	-- 			valor_incorrecte	-> quan n_aleatori s <= 0 o quan t ms de 9 dgits de llargada
	-- 			ciutada_inexistent	-> quan n_aleatori no referencia cap ciutada de la BD
	-- 			ciutada_baixa		-> quan n_aleatori referencia un ciutad que ja estava donat de baixa
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_baixaCiutada (
	  n_aleatori		IN	ciutadans.num_aleatori%TYPE,
	  RSP			OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_baixaCiutada';
	  	parametres_entrada 	VARCHAR2(30 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		baixa			NUMBER(1) := 0;
		id_local		NUMBER := 0;
		censats			NUMBER := 0;
	  	camp_obligatori 	EXCEPTION;
		valor_incorrecte 	EXCEPTION;
		ciutada_inexistent	EXCEPTION;
		ciutada_baixa		EXCEPTION;

		BEGIN

		  parametres_entrada := n_aleatori;

		  IF (  n_aleatori IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (	n_aleatori <= 0 OR
			length(n_aleatori) > 9 ) THEN
			RAISE valor_incorrecte;
		  END IF;


		  -- es comprova que el ciutad existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM ciutadans WHERE num_aleatori = n_aleatori;
		  IF ( trobat = 0 ) THEN
			RAISE ciutada_inexistent;
		  END IF;

		  -- es comprova que el ciutad no estigus ja donat de baixa
		  baixa := 0;
		  SELECT baixa_logica INTO baixa FROM ciutadans WHERE num_aleatori = n_aleatori;
		  IF ( baixa = 1 ) THEN
			RAISE ciutada_baixa;
		  END IF;		  

		  -- es dna de baixa lgica el ciutad
		  UPDATE ciutadans SET baixa_logica = 1 WHERE num_aleatori = n_aleatori;

		  -- obtenim l'identificador de la localiat del ciutad
		  SELECT id_localitat INTO id_local FROM ciutadans WHERE num_aleatori = n_aleatori;

		  -- i s'actualitza el nmero de ciutadans censats del cens local corresponent (es decrementa en una unitat)
		  SELECT ciutadans_censats INTO censats FROM censosLocals WHERE id_localitat = id_local;
		  UPDATE censosLocals SET ciutadans_censats = censats-1 WHERE id_localitat = id_local;

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: el numero que ha d''identificar el ciutada es buit o NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN valor_incorrecte THEN

		   RSP := 'ERROR: el numero que ha d''identificar el ciutada ha de ser un nombre natural diferent de zero (i ha de tenir menys de 10 digits decimals) ';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN ciutada_inexistent THEN

		   RSP := 'ERROR: el numero no referencia cap ciutada de la BD.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN ciutada_baixa THEN

		   RSP := 'ERROR: el ciutada ja estava donat de baixa.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;








	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_modificaCiutada_dataNaix
	-- 			
	-- Descripci:		modifica la data de naixement d'un ciutada i crida el procediment p_registrarLog del paquet pk_utils.
	-- 			
	-- 			
	-- Parmetres entrada:	n_aleatori	NUMBER(9)	PK
	-- 			nova_data_naix	DATE
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan n_aleatori i/o  nova_data_naix sn buits o NULL
	-- 			valor_incorrecte	-> quan n_aleatori s <= 0 o quan t ms de 9 dgits de llargada
	-- 			ciutada_inexistent	-> quan n_aleatori no referencia cap ciutada de la BD
	-- 			ciutada_baixa		-> quan n_aleatori referencia un ciutad que ja estava donat de baixa
	-- 			menor_edat		-> quan la nova data de naixement correspondria a un menor d'edat respecte la data actual
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_modificaCiutada_dataNaix (
	  n_aleatori		IN	ciutadans.num_aleatori%TYPE,
	  nova_data_naix	IN	ciutadans.data_naixement%TYPE,
	  RSP			OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_modificaCiutada_dataNaix';
	  	parametres_entrada 	VARCHAR2(30 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		baixa			NUMBER(1) := 0;
		auxiliar_DATE		DATE;
		data_actuaL_DATE	DATE;
		data_actual_CHAR	VARCHAR2(10 CHAR);
		data_naix_mes_18	VARCHAR2(10 CHAR);
		controla_zero		VARCHAR2(2 CHAR);
	  	camp_obligatori 	EXCEPTION;
		valor_incorrecte 	EXCEPTION;
		ciutada_inexistent	EXCEPTION;
		ciutada_baixa		EXCEPTION;
		menor_edat		EXCEPTION;



		BEGIN

		  parametres_entrada := n_aleatori||', '||nova_data_naix;

		  IF (  n_aleatori IS NULL OR
			nova_data_naix IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (	n_aleatori <= 0 OR
			length(n_aleatori) > 9 ) THEN
			RAISE valor_incorrecte;
		  END IF;


		  -- es comprova que el ciutad existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM ciutadans WHERE num_aleatori = n_aleatori;
		  IF ( trobat = 0 ) THEN
			RAISE ciutada_inexistent;
		  END IF;

		  -- es comprova que el ciutad no estigus ja donat de baixa
		  baixa := 0;
		  SELECT baixa_logica INTO baixa FROM ciutadans WHERE num_aleatori = n_aleatori;
		  IF ( baixa = 1 ) THEN
			RAISE ciutada_baixa;
		  END IF;		  

		  -- es comprova que la nova data de naixement correspongui a un ciutad major d'edat (> 18 anys). El sistema noms
		  -- permet emmagatzemar ciutadans amb dret a vot a la data actual.
		  SELECT SYSDATE INTO data_actual_DATE FROM DUAL;
		  data_naix_mes_18 := TO_CHAR(nova_data_naix, 'ddmmyyyy')+18;
		  controla_zero := TO_CHAR(nova_data_naix, 'DD');
		  IF (controla_zero < 10) THEN

			data_naix_mes_18 := '0'||data_naix_mes_18;

		  END IF;

		  auxiliar_DATE := TO_DATE(data_naix_mes_18);
		  data_naix_mes_18 := TO_CHAR(auxiliar_DATE, 'yyyymmdd');
		  data_actual_CHAR := TO_CHAR(data_actual_DATE, 'yyyymmdd');


		  IF ( data_naix_mes_18 > data_actual_CHAR ) THEN
			RAISE menor_edat;
		  END IF;


		  -- fetes les verificacions prvies, es pot canviar la data de naixement del ciutad
		  UPDATE ciutadans SET data_naixement = nova_data_naix WHERE num_aleatori = n_aleatori;


		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: el numero que ha d''identificar el ciutada es buit o NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN valor_incorrecte THEN

		   RSP := 'ERROR: el numero que ha d''identificar el ciutada ha de ser un nombre natural diferent de zero (i ha de tenir menys de 10 digits decimals) ';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN ciutada_inexistent THEN

		   RSP := 'ERROR: no es pot canviar la data de naixement d''un ciutada que no existeix a la BD.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN ciutada_baixa THEN

		   RSP := 'ERROR: no es pot canviar la data de naixement d''un ciutada que ha estat donat de baixa.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN menor_edat THEN

		   RSP := 'ERROR: el sistema no permet ciutadans menors de 18 anys respecte la data actual. Revisi la nova data de naixement.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;









	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_modificaCiutada_censLocal
	-- 			
	-- Descripci:		sota determinades condicions, modifica el cens local d'un ciutad, actualitza la localitat i pas del ciutad 
	-- 			de manera que hi hagi coherncia amb el nou cens local i actualitza el nmero de ciutadans censats a l'antic
	-- 			i al nou cens local del ciutad. Com sempre, s'acaba cridant el procediment p_registrarLog del paquet pk_utils.
	-- 
	-- 			NOTA: no es permetr modificar el cens d'un ciutad si l'antic i/o el nou cens estan associats a alguna de les
	-- 			votacions no finalitzades (actives) que hi pugui haver al sistema. Aix es fa perqu no hi pugui haver cap dada
	-- 			incoherent (tot i que s'haurien de canviar els censos d'uns quants ciutadans perqu passs) a l'hora de calcular
	-- 			el percentatge de participaci d'una determinada votaci, ja que el nmero d'electors cridats a les urnes en una
	-- 			votaci no s dinmic, sin que s'actualitza noms a l'hora d'associar censos a les votacions. S'ha controlat
	-- 			aix malgrat que el client havia confirmat que no calia fer-ho.
	-- 			
	-- Parmetres entrada:	n_aleatori	NUMBER(9)		PK
	-- 			nou_cens_local	VARCHAR2(40 CHAR)	REFERENCES censosLocals(nom_cens) -- ha de referenciar-ne un, baja --
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori			-> quan n_aleatori s buit o NULL
	-- 			camp_incorrecte			-> quan nou_cens_local s massa llarg
	-- 			valor_incorrecte		-> quan n_aleatori s <= 0 o quan t ms de 9 dgits de llargada
	-- 			ciutada_inexistent		-> quan n_aleatori no referencia cap ciutada de la BD
	-- 			ciutada_baixa			-> quan n_aleatori referencia un ciutad que ja estava donat de baixa
	-- 			nou_cens_inexistent		-> quan nou_cens_local no referencia cap dels censos locals de la BD
	-- 			censos_a_votacions_actives	-> quan l'antic i/o el nou cens del ciutad estan associats a alguna votaci
	-- 							   no finalitzada.
	--			OTHERS				-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_modificaCiutada_censLocal (
	  n_aleatori		IN	ciutadans.num_aleatori%TYPE,
	  nou_cens_local	IN	censosLocals.nom_cens%TYPE,
	  RSP			OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_modificaCiutada_censLocal';
	  	parametres_entrada 	VARCHAR2(80 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		baixa			NUMBER(1) := 0;
		num_votacions_actives	NUMBER := 0;
		estat_final		NUMBER := 0;
		antic_cens_local	VARCHAR2(40 CHAR);
		id_local		NUMBER := 0;
		num_censats		NUMBER := 0;
		id_nova_localitat	NUMBER := 0;
		id_nova_regio		NUMBER := 0;
		id_nou_pais		VARCHAR2(30 CHAR);
	  	camp_obligatori 		EXCEPTION;
		camp_incorrecte			EXCEPTION;
		valor_incorrecte 		EXCEPTION;
		ciutada_inexistent		EXCEPTION;
		ciutada_baixa			EXCEPTION;
		nou_cens_inexistent		EXCEPTION;
		censos_a_votacions_actives	EXCEPTION;

		BEGIN

		  parametres_entrada := n_aleatori||', '||nou_cens_local;

		  IF (  n_aleatori IS NULL OR 
			nou_cens_local IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF ( length(nou_cens_local) > 40 ) THEN
			RAISE camp_incorrecte;
		  END IF;

		  IF (	n_aleatori <= 0 OR
			length(n_aleatori) > 9 ) THEN
				RAISE valor_incorrecte;
		  END IF;


		  -- es comprova que el ciutad existeixi a la BD
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM ciutadans WHERE num_aleatori = n_aleatori;
		  IF ( trobat = 0 ) THEN
			RAISE ciutada_inexistent;
		  END IF;

		  -- es comprova que el ciutad no estigui donat de baixa
		  baixa := 0;
		  SELECT baixa_logica INTO baixa FROM ciutadans WHERE num_aleatori = n_aleatori;
		  IF ( baixa = 1 ) THEN
			RAISE ciutada_baixa;
		  END IF;


		  -- es comprova que el nou cens local existeixi a la BD
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM censosLocals WHERE nom_cens = nou_cens_local;
		  IF ( trobat = 0 ) THEN
			RAISE nou_cens_inexistent;
		  END IF;

		  -- obtenim l'antic cens local del ciutad (ens far falta en ms d'un punt del procediment)
		  SELECT id_cens_local INTO antic_cens_local FROM ciutadans WHERE num_aleatori = n_aleatori;

		  -- es comprova que l'antic cens local del ciutad i/o el nou cens local no estiguin associats a alguna de les votacions
		  -- no finalitzades (actives). Si algun cens hi est associat, no es permetr el canvi de cens fins que no finalitzin les votacions.
		  trobat := 0;
		  SELECT id INTO estat_final FROM estatsVotacio WHERE nom_estat = 'finalitzada';
		  SELECT COUNT(*) INTO num_votacions_actives FROM votacions WHERE id_estat <> estat_final;
		  IF (num_votacions_actives = 0) THEN
			trobat := 0;
		  ELSE

			-- s'obtenen les votacions no finalitzades de la taula votacions i, per cadascuna, en un segon cursor
			-- es recorreran els censos locals que tinguin associats. Si algun d'ells coincideix amb l'antic
			-- o el nou cens local del ciutad, es posar trobat a 1 i no es permetr modificar el cens del ciutad.
			FOR votacio_activa IN (SELECT id FROM votacions WHERE id_estat <> estat_final) LOOP

				FOR cens_associat IN (SELECT id_cens_local FROM censosAssociats WHERE id_votacio = votacio_activa.id) LOOP

					trobat := trobat;
					IF ( trobat = 0 ) THEN

						IF ( cens_associat.id_cens_local = antic_cens_local OR cens_associat.id_cens_local = nou_cens_local ) THEN

							trobat := 1;

						END IF;
					END IF;

				END LOOP;

			END LOOP;

		  END IF;



		  IF ( trobat = 1 ) THEN
			RAISE censos_a_votacions_actives;
		  END IF;


		  -- superades les verificacions anteriors, es pot canviar el cens local del ciutad. Al mateix temps, es canviar (a la taula ciutadans) 
		  -- la localitat i el pas associats al nou cens del ciutad (pot ser que el pas sigui el mateix que l'anterior, o que no ho sigui).
		  SELECT id_localitat INTO id_nova_localitat FROM censosLocals WHERE nom_cens = nou_cens_local;
		  SELECT id_regio INTO id_nova_regio FROM localitats WHERE id = id_nova_localitat;
		  SELECT id_pais INTO id_nou_pais FROM regions WHERE id = id_nova_regio;
		  UPDATE ciutadans SET id_cens_local = nou_cens_local, id_localitat = id_nova_localitat, nom_pais = id_nou_pais 
			WHERE num_aleatori = n_aleatori;

		  -- Per altre costat, cal actualitzar el nmero de ciutadans censats de l'antic i del nou cens.
		  SELECT ciutadans_censats INTO num_censats FROM censosLocals WHERE nom_cens = antic_cens_local;
		  UPDATE censosLocals SET ciutadans_censats = (num_censats - 1) WHERE nom_cens = antic_cens_local;

		  SELECT ciutadans_censats INTO num_censats FROM censosLocals WHERE nom_cens = nou_cens_local;
		  UPDATE censosLocals SET ciutadans_censats = (num_censats + 1) WHERE nom_cens = nou_cens_local;


		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: el numero que ha d''identificar el ciutada i/o el nom del nou cens son buits o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN camp_incorrecte THEN

		   RSP := 'ERROR: el nom del nou cens local es massa llarg (maxim 40 caracters) ';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN valor_incorrecte THEN

		   RSP := 'ERROR: el numero que ha d''identificar el ciutada ha de ser un nombre natural diferent de zero (i ha de tenir menys de 10 digits decimals) ';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN ciutada_inexistent THEN

		   RSP := 'ERROR: no es pot canviar el cens d''un ciutada que no existeix a la BD. Revisi l''identificador del ciutada.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN ciutada_baixa THEN

		   RSP := 'ERROR: no es pot canviar de cens un ciutada que esta donat de baixa. Revisi l''identificador del ciutada.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN nou_cens_inexistent THEN

		   RSP := 'ERROR: intent d''associar un ciutada a un cens inexistent. Revisi el nom del nou cens.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN censos_a_votacions_actives THEN

		   RSP := 'ERROR: no es pot modificar el cens d''un ciutada mentre l''antic i/o el nou cens pertanyin a alguna votacio no finalitzada.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;





END; --fi implementaci paquet pk_ciutadans
/










-- ###################################################################   pk_estadistiques    ########################################################################








-- ******************************************************************************************************************************
-- ******************************************************************************************************************************
-- *																*
-- *						FASE D'ESPECIFICACI								*
-- *																*
-- ******************************************************************************************************************************
-- ******************************************************************************************************************************




-- ----------------------------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------------
-- Nom_paquet: 		pk_estadistiques
-- 
-- Descripci:		cont el procediment que actualitza totes les estadstiques que faci falta d'una votaci
-- 
-- Procediments:	p_estadistiquesVotacio
-- Funcions:		--
-- ----------------------------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE pk_estadistiques IS



	PROCEDURE p_estadistiquesVotacio (
	  id_vota	IN	votacions.id%TYPE,
	  RSP		OUT	VARCHAR2
	  );



END; --fi especificaci paquet pk_estadistiques
/










-- ******************************************************************************************************************************
-- ******************************************************************************************************************************
-- *																*
-- *						FASE D'IMPLEMENTACI								*
-- *																*
-- ******************************************************************************************************************************
-- ******************************************************************************************************************************




-- ----------------------------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------------
-- Nom_paquet: 		pk_estadistiques
-- 
-- Descripci:		cont el procediment que actualitza totes les estadstiques que faci falta d'una votaci
-- 
-- Procediments:	p_estadistiquesVotacio
-- Funcions:		--
-- ----------------------------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE BODY pk_estadistiques IS






	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_estadistiquesVotacio
	-- 			
	-- Descripci:		actualitza tots els camps i registres que calgui de totes les taules que formen el mdul estadstic. Finalment, 
	-- 			com sempre, es crida el procediment p_registrarLog del paquet pk_utils.
	-- 			
	-- Parmetres entrada:	id_vota		NUMBER
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori			-> quan id_vot s buit o NULL
	-- 			camp_incorrecte			-> quan id_vota no s un natural diferent de zero
	-- 			votacio_inexistent		-> quan id_vota no referencia cap votaci del sistema
	-- 			no_finalitzada			-> quan id_vota referencia una votaci que encara no ha finalitzat
	-- 			estadistiques_calculades	-> quan id_vota referncia una votaci de la qual ja se n'han calculat les estadstiques
	--			OTHERS				-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_estadistiquesVotacio (
	  id_vota	IN	votacions.id%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_estadistiquesVotacio';
	  	parametres_entrada 	VARCHAR2(10 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;

   
-- | per guardar dades de la votaci que es tracta
-- v ---------------------------------------------
		titol_votacio		VARCHAR2(100 CHAR) := NULL;
		estat_votacio		VARCHAR2(20 CHAR) := NULL;
		any_vota		NUMBER(4) := 0;
		num_electors_votacio	NUMBER := 0;

-- | per clculs intermitjos que es faran servir en diverses estadstiques
-- v ---------------------------------------------------------------------
		trobat			NUMBER(1) := 0;
		num			NUMBER := 0;
		t_vots			NUMBER := 0;
		t_electors		NUMBER := 0;
		nova_participacio	NUMBER(5,2) := 0;
		percent			NUMBER(5,2) := 0;
		percent_max		NUMBER(5,2) := 0;
		percent_min		NUMBER(5,2) := 0;
		max_vots		NUMBER := 0;



-- | per les estadstiques de censosLocalsAnys, censosRegionalsAnys, censosEstatalsAnys, censGlobalalAnys, 
-- v -----------------------------------------------------------------------------------------------------
		cens_global		VARCHAR2(30 CHAR) := NULL;
		trobat_cens_local	NUMBER(1) := 0;
		trobat_cens_regional	NUMBER(1) := 0;
		trobat_cens_estatal	NUMBER(1) := 0;
		trobat_cens_global	NUMBER(1) := 0;
		n_vots_cens		NUMBER := 0;
		n_electors_cens		NUMBER := 0;
		n_votacions_cens	NUMBER := 0;


-- | per les estadstiques de la taula  estadistiquesAnuals
-- v ------------------------------------------------------
		t_electors_han_votat	NUMBER := 0;
		t_electors_no_han_votat	NUMBER := 0;
		perc_no_han_votat	NUMBER(5,2) := 0;
		ciutadans_mai		NUMBER(5,2) := 0;
		antiga_participacio	NUMBER(5,2) := 0;
		ant_localitats_max_vots	VARCHAR2(360 CHAR) := NULL;
		localitats_max_vots	VARCHAR2(480 CHAR) := NULL;	-- guardar les localitats de la votaci amb un nombre mxim de vots (s d'esperar 
									-- que n'hi hagi una, per es declara de 480 CHAR per si de cas. Amb aquesta longitud
									-- hi podrem guardar un mnim de 12 noms de localitat). Una cosa semblant per
									-- ant_localitats_max_vots que servir en el cas que hi hagi coincidncia de mxim
									-- de vots entre localitats d'una votaci anterior a aquesta i d'aquesta. En cas
									-- que passs (poc probable) es concatenaran els noms de les localitats.

		votacions_mes_part	VARCHAR2(300 CHAR) := NULL;	-- guardar el ttol de les votacions d'un any en les qual hi hagi hagut ms participaci.
									-- tpicament ser una sola votaci (100 CHAR seria suficient), per pot passar que ms
									-- d'una votaci tinguin una mateixa participaci mxima (amb 300 CHAR hi podem guardar
									-- el ttol d'un mnim de 3 votacions)

-- | per les estadstiques de la taula estadistiquesVotacions
-- v --------------------------------------------------------
		id_opc			NUMBER := 0;
		id_opc_CHAR		VARCHAR2(20 CHAR) := NULL;
		nom_opc			VARCHAR2(150 CHAR) := NULL;	-- el nom d'una opci pot ocupar un mxim de 15 CHAR. nom_opc guardar el nom de les 
		nom			VARCHAR2(15 CHAR);		-- opcions amb un mxim i mnim de vots i, com que es pot donar el cas que hi hagi
		min_vots		NUMBER := 0;			-- empat entre dues o ms opcions, es reserven 150 CHAR. Aix, es podrien guardar
									-- els noms de 10 (o ms) opcions que empatessin a vots (mnims o mxims).

-- | per les estadstiques de la taula historicVotacions
-- v ----------------------------------------------------
		v_mes_part		VARCHAR2(300 CHAR) := NULL;	-- v_mes_part, v_max_dif, v_min_dif guardaran el ttol/s de les votacions de la taula 
		v_max_dif		VARCHAR2(300 CHAR) := NULL;	-- historicVotacions. Com que es podria donar que hi hagus ms d'una votaci en qu
		v_min_dif		VARCHAR2(300 CHAR) := NULL;	-- coincids el valor, s'han reservat 300 CHAR per guardar-hi un mnim de 3 ttols
									-- de votaci.

-- | per les estadstiques de la taula maxParticipacionsCiutada
-- v ----------------------------------------------------------
		id_ciutada		NUMBER := 0;
		n_pais			VARCHAR2(30 CHAR) := NULL;
		num_part		NUMBER := 0;
		n_files			NUMBER := 0;


-- | Excepcions controlades
-- v -----------------------
	  	camp_obligatori 		EXCEPTION;
		camp_incorrecte 		EXCEPTION;
		votacio_inexistent		EXCEPTION;
		no_finalitzada			EXCEPTION;
		estadistiques_calculades	EXCEPTION;



		BEGIN

		  parametres_entrada := id_vota;

		  IF (  id_vota IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (	id_vota <= 0 ) THEN
			RAISE camp_incorrecte;
		  END IF;


		  -- comprovem que l'identificador de votaci referenci una votaci existent
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM votacions WHERE id = id_vota;
		  IF ( trobat = 0 ) THEN
			RAISE votacio_inexistent;
		  END IF;

		  -- comprovem que la votaci estigui en estat 'finalitzada'
		  SELECT ev.nom_estat INTO estat_votacio
			FROM estatsVotacio ev, votacions v
			WHERE ev.id = v.id_estat AND v.id = id_vota;
		  IF ( estat_votacio <> 'finalitzada' ) THEN
			RAISE no_finalitzada;
		  END IF;

		  -- comprovem si ja s'han calculat les estadistiques de la votacio. Si la votaci id_vota existeix, tamb existir una entrada
		  -- a la taula estaditiquesVotacions per la votaci id_vota. En actualitzar les estadstiques t_electors > 0 (suposem que sempre
		  -- hi haur hagut com a mnim un elector) i ens servir per controlar que no es tornin a calcular les estadstiques. Una altra
		  -- opci seria jugar amb l'atribut 'finalitzada' per aquest atribut s'actualitza en un altre procediment i millor fer-ho aix.
		  SELECT  num_electors INTO t_electors FROM estadistiquesVotacions WHERE id_votacio = id_vota;
		  IF ( t_electors > 0 ) THEN
			RAISE estadistiques_calculades;
		  END IF;


		  -- obtenim dades de la votacio
		  SELECT titol, any_votacio, num_electors INTO titol_votacio, any_vota, num_electors_votacio 
			FROM votacions WHERE id = id_vota;



-- inici actualitzacions taula participacionsPaisosVotacions

		  -- recorrem un cursor no declarat que tindr el nom dels pasos que han participat a la votaci i, 
		  -- tenint en compte l'any de la votaci id_vota:
		  -- 1) els pasos que encara no siguin a la taula participacionsPaisosVotacions els afegirem a la taula 
		  -- 	posant el camp num_participacions a 1
		  -- 2) els pasos que ja siguin a la taula els incrementarem en 1 el camp num_participacions
		  FOR pais IN (SELECT DISTINCT nom_pais FROM censosAssociats WHERE id_votacio = id_vota) LOOP

			trobat := 0;
			SELECT COUNT(*) INTO trobat FROM participacionsPaisosVotacions WHERE lany = any_vota AND id_pais = pais.nom_pais;
			IF ( trobat = 0) THEN
				INSERT INTO participacionsPaisosVotacions(id_pais, lany, num_participacions) VALUES (pais.nom_pais, any_vota, 1);
			ELSIF ( trobat = 1 ) THEN
				num := 0;
				SELECT num_participacions INTO num FROM participacionsPaisosVotacions WHERE lany = any_vota AND id_pais = pais.nom_pais;
				UPDATE participacionsPaisosVotacions SET num_participacions = (num + 1) WHERE lany = any_vota AND id_pais = pais.nom_pais;
			END IF;

		  END LOOP;

-- fi actualitzacions taula participacionsPaisosVotacions




-- inici actualitzacions taules censosLocalsAnys, censosRegionalsAnys, censosEstatalsAnys, censGlobalAnys


		  -- recorrem un cursor no declarat que tindr les dades necessries per registrar i/o actualitzar dades de les taules 
		  -- censosLocalsAnys, censosRegionalsAnys, censosEstatalsAnys i censGlobalAnys.
		  FOR censAssociat IN (SELECT id_cens_local, cens_estatal, cens_regional, nom_localitat, ciutadans_censats, num_vots
					FROM censosAssociats WHERE id_votacio = id_vota) LOOP


			-- obtenim l'identificador de cens global (que no s a les dades del cursor i ens far falta a direrents parts)
			SELECT id_cens_global INTO cens_global FROM censosEstatals WHERE nom_cens = censAssociat.cens_estatal;

			n_vots_cens := 0;
			t_vots := 0;
			n_electors_cens := 0;
			t_electors := 0;
			nova_participacio := 0;
			n_votacions_cens := 0;
			

			-- comprovem si el cens local associat ja s a la taula censosLocalsAnys (hi ser si el cens ha participat en alguna altra
			-- votaci en el mateix any que la votacio que tractem). 
			trobat_cens_local := 0;
			SELECT COUNT(*) INTO trobat_cens_local FROM censosLocalsAnys WHERE lany = any_vota AND id_cens_local = censAssociat.id_cens_local;

			-- Si el cens local ja hi s, tamb hi seran, a les respectives taules, el cens regional/estatal/global al qual pertany
			-- i noms s'actualitzaran les dades de les taules corresponents
			IF ( trobat_cens_local = 1 ) THEN

				-- ----------------------- cens local -------------------------
				-- obtenci i reclcul de les dades acumulades en el cens local l'any de la votaci
				-- en el cas del cens local, podem incrementar en 1 el nmero de votacions en les que ha participat aquest any aqu
				SELECT total_electors, total_vots, num_votacions INTO n_electors_cens, n_vots_cens, n_votacions_cens FROM censosLocalsAnys 
					WHERE lany = any_vota AND id_cens_local = censAssociat.id_cens_local;

				t_vots := n_vots_cens + censAssociat.num_vots;
				t_electors := n_electors_cens + censAssociat.ciutadans_censats;
				nova_participacio := 100*t_vots/t_electors;

				-- actualitzaci de dades del cens local any votaci
				UPDATE censosLocalsAnys SET total_electors = t_electors, total_vots = t_vots, participacio = nova_participacio, 
					num_votacions = n_votacions_cens + 1
					WHERE lany = any_vota AND id_cens_local = censAssociat.id_cens_local;


				-- ----------------------- cens regional -----------------------
				SELECT total_electors, total_vots INTO n_electors_cens, n_vots_cens FROM censosRegionalsAnys 
					WHERE lany = any_vota AND id_cens_regional = censAssociat.cens_regional;

				t_vots := n_vots_cens + censAssociat.num_vots;
				t_electors := n_electors_cens + censAssociat.ciutadans_censats;
				nova_participacio := 100*t_vots/t_electors;

				UPDATE censosRegionalsAnys SET total_electors = t_electors, total_vots = t_vots, participacio = nova_participacio
					WHERE lany = any_vota AND id_cens_regional = censAssociat.cens_regional;


				-- ----------------------- cens estatal -----------------------
				SELECT total_electors, total_vots INTO n_electors_cens, n_vots_cens FROM censosEstatalsAnys 
					WHERE lany = any_vota AND id_cens_estatal = censAssociat.cens_estatal;

				t_vots := n_vots_cens + censAssociat.num_vots;
				t_electors := n_electors_cens + censAssociat.ciutadans_censats;
				nova_participacio := 100*t_vots/t_electors;

				UPDATE censosEstatalsAnys SET total_electors = t_electors, total_vots = t_vots, participacio = nova_participacio
					WHERE lany = any_vota AND id_cens_estatal = censAssociat.cens_estatal;

				-- ----------------------- cens global -----------------------
				SELECT total_electors, total_vots INTO n_electors_cens, n_vots_cens FROM censGlobalAnys 
					WHERE lany = any_vota AND id_cens_global = cens_global;

				t_vots := n_vots_cens + censAssociat.num_vots;
				t_electors := n_electors_cens + censAssociat.ciutadans_censats;
				nova_participacio := 100*t_vots/t_electors;

				UPDATE censGlobalAnys SET total_electors = t_electors, total_vots = t_vots, participacio = nova_participacio
					WHERE lany = any_vota AND id_cens_global = cens_global;



			-- si el cens local no s a la taula censosLocalsAnys (aix passa si s la primera votaci en qu participa el cens local a l'any 
			-- de la votaci que  tractem),  s'inserir el registre a la taula i llavors es comprovar si el censos regional/estatal/global 
			-- als qual pertany ja tenen algun registre per l'any de la votaci a les seves respectives taules.
			ELSIF ( trobat_cens_local = 0 ) THEN

				nova_participacio := 100*censAssociat.num_vots/censAssociat.ciutadans_censats;
				INSERT INTO censosLocalsAnys(lany, id_cens_local, total_electors, total_vots, participacio, num_votacions) 
				VALUES(any_vota, censAssociat.id_cens_local, censAssociat.ciutadans_censats, censAssociat.num_vots, nova_participacio, 1);


				-- comprovem si el cens regional al qual correspon el cens local ja s a la taula censosRegionalsAnys (hi ser si el
				-- cens local que estem tractant s la primera vegada que participa en una votaci l'any actual per un altre cens local
				-- del mateix cens regional ja ha participat a alguna altra votaci (o a aquesta mateixa) a l'any actual.
				-- Si hi s, tamb hi seran, a les respectives taules, els censos estatal/global i s'actualitzen dades de les taules
				-- corresponents.
				trobat_cens_regional := 0;
				SELECT COUNT(*) INTO trobat_cens_regional FROM censosRegionalsAnys 
					WHERE lany = any_vota AND id_cens_regional = censAssociat.cens_regional;

				IF ( trobat_cens_regional = 1 ) THEN

					-- ----------------------- cens regional -----------------------
					SELECT total_electors, total_vots INTO n_electors_cens, n_vots_cens FROM censosRegionalsAnys 
						WHERE lany = any_vota AND id_cens_regional = censAssociat.cens_regional;

					t_vots := n_vots_cens + censAssociat.num_vots;
					t_electors := n_electors_cens + censAssociat.ciutadans_censats;
					nova_participacio := 100*t_vots/t_electors;

					UPDATE censosRegionalsAnys SET total_electors = t_electors, total_vots = t_vots, participacio = nova_participacio
						WHERE lany = any_vota AND id_cens_regional = censAssociat.cens_regional;


					-- ----------------------- cens estatal -----------------------
					SELECT total_electors, total_vots INTO n_electors_cens, n_vots_cens FROM censosEstatalsAnys 
						WHERE lany = any_vota AND id_cens_estatal = censAssociat.cens_estatal;

					t_vots := n_vots_cens + censAssociat.num_vots;
					t_electors := n_electors_cens + censAssociat.ciutadans_censats;
					nova_participacio := 100*t_vots/t_electors;

					UPDATE censosEstatalsAnys SET total_electors = t_electors, total_vots = t_vots, participacio = nova_participacio
						WHERE lany = any_vota AND id_cens_estatal = censAssociat.cens_estatal;

					-- ----------------------- cens global -----------------------
					SELECT total_electors, total_vots INTO n_electors_cens, n_vots_cens FROM censGlobalAnys 
						WHERE lany = any_vota AND id_cens_global = cens_global;

					t_vots := n_vots_cens + censAssociat.num_vots;
					t_electors := n_electors_cens + censAssociat.ciutadans_censats;
					nova_participacio := 100*t_vots/t_electors;

					UPDATE censGlobalAnys SET total_electors = t_electors, total_vots = t_vots, participacio = nova_participacio
					WHERE lany = any_vota AND id_cens_global = cens_global;

				-- si el cens regional no s a la taula censosRegionalsAny (aix passa quan el cens local que tractem ni cap altre cens
				-- local de la regi a la qual pertany el primer no han participat a cap votaci aquest any; s a dir en tractar el
				-- primer cens local de cada regi a cada any), s'inserir un registre
				-- a aquesta taula i llavors es comprovar si el censos estatal/global als qual pertany ja tene algun registre
				-- per l'any de la votaci a les seves respectives taules.
				ELSIF ( trobat_cens_regional = 0 ) THEN


					nova_participacio := 100*censAssociat.num_vots/censAssociat.ciutadans_censats;
					INSERT INTO censosRegionalsAnys(lany, id_cens_regional, total_electors, total_vots, participacio, num_votacions) 
						VALUES(any_vota, censAssociat.cens_regional, censAssociat.ciutadans_censats, censAssociat.num_vots, 
							nova_participacio, 0);


					-- comprovem si el cens estatal al qual correspon el cens regional del cens local que tractem ja s a la taula 
					-- censosEstatalsAnys (hi ser si el cens local que estem tractant s la primera vegada que participa en una 
					-- votaci l'any actual per un altre cens local d'un cens regional diferent al del primer per del mateix pas 
					-- que aquest ja ha participat a alguna altra votaci (o a aquesta mateixa) a l'any actual. Si hi s, tamb hi 
					-- ser, a la taula censGlobalAnys, el cens global i s'actualitzen dades de les taules corresponents.
					trobat_cens_estatal := 0;
					SELECT COUNT(*) INTO trobat_cens_estatal FROM censosEstatalsAnys 
						WHERE lany = any_vota AND id_cens_estatal = censAssociat.cens_estatal;

					IF ( trobat_cens_estatal = 1 ) THEN

						-- ----------------------- cens estatal -----------------------
						SELECT total_electors, total_vots INTO n_electors_cens, n_vots_cens FROM censosEstatalsAnys 
							WHERE lany = any_vota AND id_cens_estatal = censAssociat.cens_estatal;

						t_vots := n_vots_cens + censAssociat.num_vots;
						t_electors := n_electors_cens + censAssociat.ciutadans_censats;
						nova_participacio := 100*t_vots/t_electors;

						UPDATE censosEstatalsAnys SET total_electors = t_electors, total_vots = t_vots, participacio = nova_participacio
							WHERE lany = any_vota AND id_cens_estatal = censAssociat.cens_estatal;

						-- ----------------------- cens global -----------------------
						SELECT total_electors, total_vots INTO n_electors_cens, n_vots_cens FROM censGlobalAnys 
							WHERE lany = any_vota AND id_cens_global = cens_global;

						t_vots := n_vots_cens + censAssociat.num_vots;
						t_electors := n_electors_cens + censAssociat.ciutadans_censats;
						nova_participacio := 100*t_vots/t_electors;

						UPDATE censGlobalAnys SET total_electors = t_electors, total_vots = t_vots, participacio = nova_participacio
						WHERE lany = any_vota AND id_cens_global = cens_global;

					-- si el cens estatal no s a la taula censosEstatalsAnys (aix passa quan el cens local que tractem ni cap altre 
					-- cens local del mateix pas no han participat a cap votaci aquest any; s a dir en tractar el primer cens local
					-- de cada pas a cada any), s'inserir el registre a la taula i llavors es comprovar si el cens global ja t
					-- registre per l'any de la votaci.
					ELSIF (trobat_cens_estatal = 0) THEN

						nova_participacio := 100*censAssociat.num_vots/censAssociat.ciutadans_censats;
						INSERT INTO censosEstatalsAnys(lany, id_cens_estatal, total_electors, total_vots, participacio, 
							num_votacions) 
							VALUES(any_vota, censAssociat.cens_estatal, censAssociat.ciutadans_censats, 
								censAssociat.num_vots, nova_participacio, 0);


						-- comprovem si el cens global ja s a la taula censGlobalAnys (hi ser una vegada tractat el primer cens
						-- local de l'any). Si hi s, tan sols s'actualitzen dades de la taula censGlobalAnys.
						trobat_cens_global := 0;
						SELECT COUNT(*) INTO trobat_cens_global FROM censGlobalAnys 
							WHERE lany = any_vota AND id_cens_global = cens_global;

						IF ( trobat_cens_global = 1 ) THEN

							-- ----------------------- cens global -----------------------
							SELECT total_electors, total_vots INTO n_electors_cens, n_vots_cens FROM censGlobalAnys 
								WHERE lany = any_vota AND id_cens_global = cens_global;

							t_vots := n_vots_cens + censAssociat.num_vots;
							t_electors := n_electors_cens + censAssociat.ciutadans_censats;
							nova_participacio := 100*t_vots/t_electors;

							UPDATE censGlobalAnys SET total_electors = t_electors, total_vots = t_vots, participacio = nova_participacio
								WHERE lany = any_vota AND id_cens_global = cens_global;

						-- si el cens global no s a la taula censGlobalAnys (aix noms passa un cop l'any i noms en tractar el
						-- primer cens local), s'inserir el registre a la taula censGlobaAnys.
						ELSIF ( trobat_cens_global = 0 ) THEN

							nova_participacio := 100*censAssociat.num_vots/censAssociat.ciutadans_censats;
							INSERT INTO censGlobalAnys(lany, id_cens_global, total_electors, total_vots, participacio, 
								num_votacions) 
								VALUES(any_vota, cens_global, censAssociat.ciutadans_censats, censAssociat.num_vots,
									 nova_participacio, 0);

						END IF; -- fi condicio trobat_cens_global

					END IF; -- fi condicio trobat_cens_estatal

				END IF; -- fi condicio trobat_cens_regional


			END IF; -- fi condicio trobat_cens_local



		  END LOOP; -- fi cursor censAssociat




		  -- recorrem un cursor no declarat per incrementar en 1 el nmero de votacions en les que han participat els censos regionals
		  -- que han participat a la votaci id_vota (les comptabiltizem a la taula censosRegionalsAnys)
		  -- Nota: el nmero de participacions dels censos locals ja s'ha actualitzat al recrrer el cursor anterior 'censAssociat'
		  FOR censRegional IN (SELECT DISTINCT cens_regional FROM censosAssociats WHERE id_votacio = id_vota AND any_votacio = any_vota) LOOP

			UPDATE censosRegionalsAnys SET num_votacions = num_votacions + 1 
				WHERE id_cens_regional = censRegional.cens_regional AND lany = any_vota;

		  END LOOP;


		  -- recorrem un cursor no declarat per incrementar en 1 el nmero de votacions en les que han participat els censos estatals
		  -- que han participat a la votaci id_vota (les comptabiltizem a la taula censosEstatalsAnys)
		  FOR censEstatal IN (SELECT DISTINCT cens_estatal FROM censosAssociats WHERE id_votacio = id_vota AND any_votacio = any_vota) LOOP

			UPDATE censosEstatalsAnys SET num_votacions = num_votacions + 1 
				WHERE id_cens_estatal = censEstatal.cens_estatal AND lany = any_vota;

		  END LOOP;


		  -- incrementem en 1 el nmero de votacions en les que ha participat el cens global l'any any_vota 
		  -- (les comptabiltizem a la taula censGlobalAnys)
		  UPDATE censGlobalAnys SET num_votacions = num_votacions + 1 WHERE id_cens_global = cens_global AND lany = any_vota;


-- fi actualitzacions taules censosLocalsAnys, censosRegionalsAnys, censosEstatalsAnys, censGlobalAnys




-- inici actualitzacions taula estadistiquesAnuals

	-- obtenci d'algunes dades d'aquesta votaci
		  -- obtenim la suma de vots de tots els censos associats a la votaci i calculem la participaci (el nmero d'electors 
		  -- ja l'hem obtingut al principi i est a la variable num_electors_votacio)
		  SELECT SUM(num_vots) INTO t_vots FROM censosAssociats WHERE id_votacio = id_vota;
		  nova_participacio := 100*t_vots/num_electors_votacio;
	
		  -- obtenim la/es localitat/s amb el nmero mxim de vots de la votaci
		  SELECT MAX(num_vots) INTO max_vots FROM censosAssociats WHERE id_votacio = id_vota;

		  -- com que podria passar que hi hagus ms d'una localitat amb el nombre mxim de vots, farem servir un cursor no declarat
		  localitats_max_vots := NULL;
		  FOR localitat IN (SELECT nom_localitat FROM censosAssociats WHERE id_votacio = id_vota AND num_vots = max_vots) LOOP

			localitats_max_vots := localitats_max_vots||localitat.nom_localitat||', ';

		  END LOOP;

		  -- obtenim el percentatge de ciutadans que no han votat mai durant l'any de la votaci que tractem (d'entre els 
		  -- que han estat cridats a les urnes; s a dir, no hem de comptabilitzar aquells ciutadans que estiguin de baixa 
		  -- ni aquells que no ho estiguin per que no hagin estat cridats a les urnes aquest any).
		  t_electors := 0;
		  t_electors_han_votat := 0;
		  t_electors_no_han_votat := 0;
		  perc_no_han_votat := 0;
		  SELECT COUNT(*) INTO t_electors FROM (SELECT DISTINCT id_ciutada FROM ciutadansVotacions 
			WHERE any_votacio = any_vota);

		  SELECT COUNT(*) INTO t_electors_han_votat FROM (SELECT DISTINCT id_ciutada FROM ciutadansVotacions 
			WHERE any_votacio = any_vota AND ha_participat = 1);

		  t_electors_no_han_votat := t_electors - t_electors_han_votat;
		  perc_no_han_votat := 100*t_electors_no_han_votat/t_electors;

	-- les dades obtingudes s'han guardat a: nova_participacio, localitats_max_vots, max_vots i perc_no_han_votat



		  -- comprovem si a la taula estadistiquesAnuals hi ha algun registre per aquest any. Si no n'hi ha cap (trobat  = 0),
		  -- n'inserim un amb les dades calculades, altrament, actualitzarem la taula (si cal)
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM estadistiquesAnuals WHERE lany = any_vota;

		  IF ( trobat = 0 ) THEN

			INSERT INTO estadistiquesAnuals(lany, votacions_mes_participacio, participacio, localitats_mes_vots, num_vots, ciutadans_no_han_votat_mai) 
				VALUES(any_vota, titol_votacio, nova_participacio,localitats_max_vots, max_vots, perc_no_han_votat);

		  ELSIF ( trobat = 1 ) THEN 

			-- obtenim dades del registre de la taula estadistiquesAnuals per aquest any
			SELECT votacions_mes_participacio, participacio, localitats_mes_vots, num_vots, ciutadans_no_han_votat_mai
				INTO votacions_mes_part, antiga_participacio, ant_localitats_max_vots, num, ciutadans_mai
				FROM estadistiquesAnuals WHERE lany = any_vota;

		      -- actualitzaci, si cal, de la/es votacio/ns amb mes participaci aquest any	
			-- si l'antiga mxima participaci d'una votaci coincideix amb la participaci de la votaci que tractem concatenarem els ttols
			IF ( antiga_participacio = nova_participacio ) THEN

				votacions_mes_part := votacions_mes_part||', '||titol_votacio;

			ELSE

				votacions_mes_part := titol_votacio;

			END IF;


			-- si l'antiga mxima participaci d'una votaci s <= que la participaci de la votaci que tractem actualitzarem la taula
			-- actualitzarem la taula, altrament no. 
			IF ( antiga_participacio <= nova_participacio ) THEN

				UPDATE estadistiquesAnuals SET votacions_mes_participacio = votacions_mes_part, participacio = nova_participacio
					WHERE lany = any_vota;

			END IF;

		     -- actualitzaci, si cal, de la/es localitat/s amb ms vots aquest any
			-- si l'antic nmero mxim de vots d'una localitat coincideix amb el mxim de la votaci que tractem (poc probacle) 
			-- concatenarem els noms de les localitats (la coma separadora ja la t la variable ant_localitats_max_vots)
			IF ( num = max_vots ) THEN

				localitats_max_vots := ant_localitats_max_vots||localitats_max_vots;

			END IF;


			-- si l'antic nmero mxim de vots d'una localitat d'una votaci del mateix any s <= que el mxim de vots de la votaci 
			-- que tractem actualitzarem la taula, altrament no. 
			IF ( num <= max_vots ) THEN

				UPDATE estadistiquesAnuals SET localitats_mes_vots = localitats_max_vots, num_vots = max_vots WHERE lany = any_vota;

			END IF;

		     -- actualitzaci, si cal, del percentatge de ciutadans que no han votat mai
			-- ciutadans_mai s el percentatge antic de la taula i perc_no_han_votat s el nou percentatge incloent els d'aquesta votaci.
			IF ( ciutadans_mai < perc_no_han_votat ) THEN

				UPDATE estadistiquesAnuals SET ciutadans_no_han_votat_mai = perc_no_han_votat WHERE lany = any_vota;


			END IF;


		  END IF; -- fi condicio trobat



-- fi actualitzacions taula estadistiquesAnuals





-- inici actualitzacions taula estadistiquesVotacions



	   -- actualitzaci de la participaci
		SELECT SUM(num_vots) INTO t_vots FROM censosAssociats WHERE id_votacio = id_vota;
		percent := 100*t_vots/num_electors_votacio;

		UPDATE estadistiquesVotacions SET num_electors = num_electors_votacio, num_vots = t_vots, participacio = percent
			WHERE id_votacio = id_vota;

	   -- actualitzaci de les dades de la opcio amb ms vots
		SELECT MAX(num_vots) INTO max_vots FROM opcionsVotacio WHERE id_votacio = id_vota;
		percent_max := 100*max_vots/t_vots;



		-- obtenim els identificadors i noms de les opcions que tenen el nmero mxim de vots
		-- NOTA: es pot donar el cas que en una votaci hi hagi empat de vots entre dos o ms partits.
		--	 Per controlar-ho, en cas que passi els identificadors i noms de les opcions es concatenaran.
		id_opc_CHAR := NULL;
		nom_opc := '';
		SELECT COUNT(*) INTO num FROM opcionsVotacio WHERE id_votacio = id_vota AND num_vots = max_vots;
		IF (num > 1) THEN

			-- si hi ha empat amb el mxim de vots, concatenem els identificadors i noms d'opci
			FOR opc IN (SELECT id_opcio FROM opcionsVotacio WHERE id_votacio = id_vota AND num_vots = max_vots) LOOP

				id_opc_CHAR := TO_CHAR(opc.id_opcio)||', '||id_opc_CHAR;

				SELECT nom_opcio INTO nom FROM opcions WHERE id = opc.id_opcio;
				nom_opc := nom||', '||nom_opc;

			END LOOP;

		ELSE
			SELECT id_opcio INTO id_opc FROM opcionsVotacio WHERE id_votacio = id_vota AND num_vots = max_vots;
			SELECT nom_opcio INTO nom_opc FROM opcions WHERE id = id_opc;
			id_opc_CHAR := TO_CHAR(id_opc);
		END IF;



		UPDATE estadistiquesVotacions SET id_opcio_mes_vots = id_opc_CHAR, nom_opcio_mes_vots = nom_opc, vots_opcio_mes_vots = max_vots,
			percentatge_opcio_mes_vots = percent_max WHERE id_votacio = id_vota;


	   -- actualitzaci de les dades de la opcio amb menys vots
		SELECT MIN(num_vots) INTO min_vots FROM opcionsVotacio WHERE id_votacio = id_vota;
		percent_min := 100*min_vots/t_vots;

		-- obtenim els identificadors i noms de les opcions que tenen el nmero mxim de vots
		-- NOTA: es pot donar el cas que en una votaci hi hagi empat de vots entre dos o ms partits.
		--	 Per controlar-ho, en cas que passi els identificadors i noms de les opcions es concatenaran.
		id_opc_CHAR := NULL;
		nom_opc := '';
		SELECT COUNT(*) INTO num FROM opcionsVotacio WHERE id_votacio = id_vota AND num_vots = min_vots;
		IF (num > 1) THEN

			-- si hi ha empat amb el mnim de vots, concatenem els identificadors i noms d'opci
			FOR opc IN (SELECT id_opcio FROM opcionsVotacio WHERE id_votacio = id_vota AND num_vots = min_vots) LOOP

				id_opc_CHAR := TO_CHAR(opc.id_opcio)||', '||id_opc_CHAR;

				SELECT nom_opcio INTO nom FROM opcions WHERE id = opc.id_opcio;
				nom_opc := nom||', '||nom_opc;

			END LOOP;

		ELSE
			SELECT id_opcio INTO id_opc FROM opcionsVotacio WHERE id_votacio = id_vota AND num_vots = min_vots;
			SELECT nom_opcio INTO nom_opc FROM opcions WHERE id = id_opc;
			id_opc_CHAR := TO_CHAR(id_opc);

		END IF;
		
		UPDATE estadistiquesVotacions SET id_opcio_menys_vots = id_opc_CHAR, nom_opcio_menys_vots = nom_opc, vots_opcio_menys_vots = min_vots,
			percentatge_opcio_menys_vots = percent_min WHERE id_votacio = id_vota;

	   -- actualitzaci de la diferncia percentual entre la opci ms i menys votada
		percent := percent_max - percent_min;
		UPDATE estadistiquesVotacions SET dif_perc_opc_mes_i_menys_vots = percent WHERE id_votacio = id_vota;



-- fi actualitzacions taula estadistiquesVotacions




-- inici actualitzacions taula historicVotacions

		-- obtenim les dades de la taula estadistiquesVotacions
		SELECT MAX(participacio) INTO percent FROM estadistiquesVotacions;
		SELECT MAX(dif_perc_opc_mes_i_menys_vots) INTO percent_max FROM estadistiquesVotacions;
		SELECT MIN(dif_perc_opc_mes_i_menys_vots) INTO percent_min FROM estadistiquesVotacions WHERE finalitzada = 1;

		-- obtenim els ttols de les votacions que compleixen amb les dades obtingudes
		-- NOTA: tot i que s poc probable, es considerar que podria haver-hi ms d'una votaci amb el mateix valor tant pel que fa
		--	 a la votaci amb ms participaci, com a la mxima i mnima diferncia entre les opcions de votaci. En cas que aix
		-- 	 sigui, es concatenaran els noms de les votacions (s'han reservat fins a 300 CHAR per guardar un mnim de 3 titols de
		-- 	 votaci en cas de coincidncia).
		SELECT COUNT(*) INTO num FROM estadistiquesVotacions WHERE participacio = percent;
		IF (num > 1) THEN
			FOR t_votacio IN (SELECT titol_votacio FROM estadistiquesVotacions WHERE participacio = percent) LOOP
				v_mes_part := v_mes_part||', '||t_votacio.titol_votacio;
			END LOOP;
		ELSE
			SELECT titol_votacio INTO v_mes_part FROM estadistiquesVotacions WHERE participacio = percent;
			v_mes_part := v_mes_part;
		END IF;


		SELECT COUNT(*) INTO num FROM estadistiquesVotacions WHERE dif_perc_opc_mes_i_menys_vots = percent_max;
		IF (num > 1) THEN
			FOR t_votacio IN (SELECT titol_votacio FROM estadistiquesVotacions WHERE dif_perc_opc_mes_i_menys_vots = percent_max) LOOP
				v_max_dif := v_max_dif||', '||t_votacio.titol_votacio;
			END LOOP;
		ELSE
			SELECT titol_votacio INTO v_max_dif FROM estadistiquesVotacions WHERE dif_perc_opc_mes_i_menys_vots = percent_max;
			v_max_dif := v_max_dif;
		END IF;


		SELECT COUNT(*) INTO num FROM estadistiquesVotacions WHERE dif_perc_opc_mes_i_menys_vots = percent_min;
		IF (num > 1) THEN
			FOR t_votacio IN (SELECT titol_votacio FROM estadistiquesVotacions WHERE dif_perc_opc_mes_i_menys_vots = percent_min) LOOP
				v_min_dif := v_min_dif||', '||t_votacio.titol_votacio;
			END LOOP;
		ELSE
			SELECT titol_votacio INTO v_min_dif FROM estadistiquesVotacions WHERE dif_perc_opc_mes_i_menys_vots = percent_min;
			v_min_dif := v_min_dif;
		END IF;


		-- una vegada tenim les dades i ttols actualitzarem la taula historicVotacions
		-- NOTA: com que el que tenim calculat sn les dades mximes/mnimes de totes les votacions, el ms fcil ser esborrar
		--	 el registre que hi ha (si n'hi ha algun) a la taula historicVotacions i inserir-n'hi un amb les noves dades.
		SELECT COUNT(*) INTO num FROM historicVotacions;
		IF (num > 0 ) THEN
			DELETE FROM historicVotacions;
		END IF;

		INSERT INTO historicVotacions (id, votacio_mes_participacio, participacio, votacio_max_diferencia_opcions, diferencia_max,
			votacio_min_diferencia_opcions, diferencia_min) VALUES(-1, v_mes_part, percent, v_max_dif, percent_max, v_min_dif, percent_min);


-- fi actualitzacions taula historicVotacions




-- inici actualitzacions taula maxParticipacionsPersona

		-- amb els vots de la votaci que estem tractant hi pot haver diversos ciutadans que igualin o superin el nmero de participacions
		-- mximes en una votaci. EL ms fcil s borrar les anteriors i tornar a omplir la taula amb les noves dades.
		SELECT COUNT(*) INTO n_files FROM maxParticipacionsCiutada;
		IF ( n_files > 0 ) THEN
			DELETE FROM maxParticipacionsCiutada;
		END IF;

		-- Com que s fora probable que hi hagi ms d'un ciutad amb un nmero mxim de participacions, els obtindrem tots
		-- en un cursor no declarat i els anirem afegint a la taula d'histrics.
		FOR ciutada IN (SELECT num_aleatori, nom_pais, num_participacions FROM ciutadans WHERE num_participacions = (SELECT MAX(num_participacions)
			FROM ciutadans)) LOOP

			INSERT INTO maxParticipacionsCiutada(id_aleatori_ciutada, pais, num_participacions)
				VALUES(ciutada.num_aleatori, ciutada.nom_pais, ciutada.num_participacions);

		END LOOP;


-- fi actualitzacions taula maxParticipacionsPersona


		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR al modul estadistic: l''identifcador de votacio es buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN camp_incorrecte THEN

		   RSP := 'ERROR al modul estadistic: l''identificador de la votacio ha de ser un nombre natural diferent de zero.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_inexistent THEN

		   RSP := 'ERROR al modul estadistic: l''identificador de votacio no correspon a cap votacio';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN no_finalitzada THEN

		   RSP := 'ERROR al modul estadistic: la votacio no esta finalitzada';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN estadistiques_calculades THEN

		   RSP := 'ERROR al modul estadistic: les estadistiques ja han estat calculades amb anterioritat';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL al : '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;




END; --fi implementaci paquet pk_estadistiques
/











-- ###################################################################   pk_votacions    ########################################################################








-- ******************************************************************************************************************************
-- ******************************************************************************************************************************
-- *																*
-- *						FASE D'ESPECIFICACI								*
-- *																*
-- ******************************************************************************************************************************
-- ******************************************************************************************************************************



-- --------------------------------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------------------------------
-- Nom_paquet: 		pk_votacions
-- 
-- Descripci:		implementaci dels procediments d'ABM (Alta/Baixa/Modificacio) relacionats amb les votacions
-- 
-- Procediments:	p_alta_anys, p_altaEstatVotacio, p_altaOpcio, p_altaVotacio, p_altaOpcioVotacio, p_altaCensAssociat, 
-- 			p_actualitzaEstatVotacio, p_altaVotEmes, p_baixaVotacio, p_baixaOpcio, p_baixaCensAssociat, p_baixaOpcioVotacio, 
-- 			p_modificaOpcio_descripcio, p_modificaVotacio_titol, p_modificaVotacio_dataInici, p_modificaVotacio_dataFi,
-- 			p_modificaVotacio_web, p_modificaVotacio_president
-- Funcions:		--
-- --------------------------------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE pk_votacions IS


	PROCEDURE p_altaAnys(
	  anyx	IN	anys.lany%TYPE,
	  RSP	OUT	VARCHAR2
	  );


	PROCEDURE p_altaEstatVotacio (
	  id		IN	estatsVotacio.id%TYPE,
	  nom_estat	IN	estatsVotacio.nom_estat%TYPE,
	  RSP		OUT	VARCHAR2
	  );


	PROCEDURE p_altaOpcio (
	  id		IN	opcions.id%TYPE,
	  nom		IN	opcions.nom_opcio%TYPE,
	  descr		IN	opcions.descripcio%TYPE,
	  cod_pais	IN	opcions.codi_pais%TYPE,
	  baixa_log	IN	opcions.baixa_logica%TYPE,
	  RSP		OUT	VARCHAR2
	  );


	PROCEDURE p_altaVotacio (
	  id		IN	votacions.id%TYPE,
	  cens_global	IN	votacions.id_cens_global%TYPE,
	  id_estat	IN	votacions.id_estat%TYPE,
	  titol_v	IN	votacions.titol%TYPE,
	  data_inici	IN	votacions.data_inici%TYPE,
	  data_fi	IN	votacions.data_fi%TYPE,
	  any_votacio	IN	votacions.any_votacio%TYPE,
	  web		IN	votacions.adreca_web%TYPE,
	  president	IN	votacions.president_mesa%TYPE,
	  electors	IN	votacions.num_electors%TYPE,
	  RSP			OUT	VARCHAR2
	  );



	PROCEDURE p_altaOpcioVotacio (
	  id		IN	opcionsVotacio.id%TYPE,
	  id_votacio	IN	opcionsVotacio.id_votacio%TYPE,
	  id_opcio_v	IN	opcionsVotacio.id_opcio%TYPE,
	  n_vots	IN	opcionsVotacio.num_vots%TYPE,
	  RSP		OUT	VARCHAR2
	  );



	PROCEDURE p_altaCensAssociat (
	  id_vota	IN	censosAssociats.id_votacio%TYPE,
	  id_cens_loc	IN	censosAssociats.id_cens_local%TYPE,
	  cens_estat	IN	censosAssociats.cens_estatal%TYPE,
	  cens_regio	IN	censosAssociats.cens_regional%TYPE,
	  nom_local	IN	censosAssociats.nom_localitat%TYPE,
	  nom_pais	IN	censosAssociats.nom_pais%TYPE,
	  any_votacio	IN	censosAssociats.any_votacio%TYPE,
	  ciut_censats	IN	censosAssociats.ciutadans_censats%TYPE,
	  finalitzada	IN	censosAssociats.finalitzada%TYPE,
	  n_vots	IN	censosAssociats.num_vots%TYPE,
	  RSP		OUT	VARCHAR2
	  );



	PROCEDURE p_actualitzaEstatVotacio (
	  id_vota	IN	votacions.id%TYPE,
	  nou_estat	IN	estatsVotacio.nom_estat%TYPE,
	  RSP		OUT	VARCHAR2
	  );



	PROCEDURE p_altaVotEmes (
	  ciutada	IN	votsEmesos.id_ciutada%TYPE,
	  votacio	IN	votsEmesos.id_votacio%TYPE,
	  opcio		IN	votsEmesos.id_opcio%TYPE,
	  data_hora	IN	votsEmesos.data_hora%TYPE,
	  RSP		OUT	VARCHAR2
	  );


	PROCEDURE p_baixaVotacio (
	  titol_vota	IN	votacions.titol%TYPE,
	  RSP		OUT	VARCHAR2
	  );


	PROCEDURE p_baixaOpcio (
	  nom		IN	opcions.nom_opcio%TYPE,
	  cod_pais	IN	opcions.codi_pais%TYPE,
	  RSP		OUT	VARCHAR2
	  );


	PROCEDURE p_baixaCensAssociat (
	  id_vota	IN	censosAssociats.id_votacio%TYPE,
	  id_cens_loc	IN	censosAssociats.id_cens_local%TYPE,
	  RSP		OUT	VARCHAR2
	  );


	PROCEDURE p_baixaOpcioVotacio (
	  id_vota	IN	opcionsVotacio.id_votacio%TYPE,
	  id_opcio_v	IN	opcionsVotacio.id_opcio%TYPE,
	  RSP		OUT	VARCHAR2
	  );


	PROCEDURE p_modificaOpcio_descripcio (
	  nom		IN	opcions.nom_opcio%TYPE,
	  cod_pais	IN	opcions.codi_pais%TYPE,
	  nova_desc	IN	opcions.descripcio%TYPE,
	  RSP		OUT	VARCHAR2
	  );



	PROCEDURE p_modificaVotacio_titol (
	  titol_vota_vell	IN	votacions.titol%TYPE,
	  titol_vota_nou	IN	votacions.titol%TYPE,
	  RSP			OUT	VARCHAR2
	  );



	PROCEDURE p_modificaVotacio_dataInici (
	  titol_vota	IN	votacions.titol%TYPE,
	  data_in	IN	votacions.data_inici%TYPE,
	  RSP			OUT	VARCHAR2
	 );


	PROCEDURE p_modificaVotacio_dataFi (
	  titol_vota	IN	votacions.titol%TYPE,
	  data_fi_vota	IN	votacions.data_fi%TYPE,
	  RSP			OUT	VARCHAR2
	  );



	PROCEDURE p_modificaVotacio_web (
	  titol_vota	IN	votacions.titol%TYPE,
	  nova_web	IN	votacions.adreca_web%TYPE,
	  RSP			OUT	VARCHAR2
	  );


	PROCEDURE p_modificaVotacio_president (
	  titol_vota	IN	votacions.titol%TYPE,
	  nou_president	IN	votacions.president_mesa%TYPE,
	  RSP			OUT	VARCHAR2
	  );




END; --fi especificaci paquet pk_votacions
/















-- ******************************************************************************************************************************
-- ******************************************************************************************************************************
-- *																*
-- *						FASE D'IMPLEMENTACI								*
-- *																*
-- ******************************************************************************************************************************
-- ******************************************************************************************************************************




-- --------------------------------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------------------------------
-- Nom_paquet: 		pk_votacions
-- 
-- Descripci:		implementaci dels procediments d'ABM (Alta/Baixa/Modificacio) relacionats amb les votacions
-- 
-- Procediments:	p_alta_anys, p_altaEstatVotacio, p_altaOpcio, p_altaVotacio, p_altaOpcioVotacio, p_altaCensAssociat, 
-- 			p_actualitzaEstatVotacio, p_altaVotEmes, p_baixaVotacio, p_baixaOpcio, p_baixaCensAssociat, p_baixaOpcioVotacio, 
-- 			p_modificaOpcio_descripcio, p_modificaVotacio_titol, p_modificaVotacio_dataInici, p_modificaVotacio_dataFi,
-- 			p_modificaVotacio_web, p_modificaVotacio_president
-- Funcions:		--
-- --------------------------------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE BODY pk_votacions IS






	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom_procediment:	p_altaAnys
	-- 			
	-- Descripci:		insereix un registre a la taula anys i crida el procediment p_registrarLog del paquet pk_utils
	-- 			
	-- Parmetres entrada:	anyx	NUMBER(4)	PK	CHECK(anyx >= 2000)
	-- 			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori 	-> quan un parmetre d'entrada s buit o NULL
	-- 			camp_negatiu		-> quan s'intenta passar un any negatiu per parmetre
	-- 			camp_incorrecte		-> quan anyx no t exactament 4 dgits
	-- 			any_temps_passat	-> quan s'intenta donar d'alta un any anterior a l'any 2001
	-- 			any_existeix		-> quan l'any que s'intenta donar d'alta ja ho estava
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaAnys(
	  anyx	IN	anys.lany%TYPE,
	  RSP	OUT	VARCHAR2
	  ) IS


		nom_procedure		VARCHAR2(30 CHAR) := 'p_altaAnys';
	  	parametres_entrada	VARCHAR2(10 CHAR) := NULL;
		esCorrecte		NUMBER(1) := 1;
		confirmada		NUMBER(1) := 1;
		any_actual		NUMBER(4) := 0;
		trobat			NUMBER(1) := 0;
	  	camp_obligatori		EXCEPTION;
		camp_negatiu		EXCEPTION;
		camp_incorrecte		EXCEPTION;
		any_temps_passat	EXCEPTION;
		any_existeix		EXCEPTION;


		BEGIN

		   parametres_entrada := anyx;

		   IF (anyx IS NULL) THEN
			RAISE camp_obligatori;
		   END IF;

		   IF ( anyx < 0 ) THEN
			RAISE camp_negatiu;
		   END IF;

		   IF ( length(anyx) <> 4 ) THEN
			RAISE camp_incorrecte;
		   END IF;

		   IF ( anyx < 2000 ) THEN
			RAISE any_temps_passat;
		   END IF;

		  -- comprovem que l'any de la votaci no existeixi ja al sistema
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM anys WHERE lany = anyx;
		  IF ( trobat = 1 ) THEN
			RAISE any_existeix;
		  END IF;


		   INSERT INTO anys(lany) VALUES(anyx);

		   RSP := 'OK';
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: el camp any es buit o te el valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN camp_negatiu THEN

		   RSP := 'ERROR: el valor de l''any no pot ser mai negatiu.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN camp_incorrecte THEN

		   RSP := 'ERROR: l''any ha de tenir exactament 4 digits.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN any_temps_passat THEN

		   RSP := 'ERROR: l''any no pot ser anterior a l''any 2000.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN any_existeix THEN

		   RSP := 'ERROR: l''any que es vol donar d''alta ja existeix al sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN OTHERS THEN

		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;






	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaEstatVotacio
	-- 			
	-- Descripci:		insereix un registre a la taula estatsVotacio i crida el procediment p_registrarLog del paquet pk_utils
	-- 			
	-- Parmetres entrada:	id		NUMBER	PK		AUTO INCREMENT
	-- 			nom_estat	VARCHAR2(20 CHAR)	DEFAULT 'edicio'	NOT NULL	UNIQUE	
	-- 								CHECK(estat IN('edicio', 'pendent obertura', 'oberta', 'finalitzada'))
	--			
	-- Parmetres sortida:	RSP		VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 							al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori	-> quan un parmetre d'entrada que no pot ser buit o NULL ho s
	-- 			camp_incorrecte -> quan el parmetre id no s negatiu o quan nom_estat no tingui un dels valors esperats
	--			OTHERS		-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaEstatVotacio (
	  id		IN	estatsVotacio.id%TYPE,
	  nom_estat	IN	estatsVotacio.nom_estat%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_altaEstatVotacio';
	  	parametres_entrada 	VARCHAR2(50 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
	  	camp_obligatori 	EXCEPTION;
		camp_incorrecte 	EXCEPTION;



		BEGIN

		  parametres_entrada := id||', '||nom_estat;

		  IF (  id IS NULL OR 
			nom_estat IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;


		  IF (	id >= 0 OR
			nom_estat NOT IN ('edicio', 'pendent obertura', 'oberta', 'finalitzada')) THEN
				RAISE camp_incorrecte;
		  END IF;


		   INSERT INTO estatsVotacio(id, nom_estat) VALUES(id, nom_estat);

		   RSP := 'OK';
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN camp_incorrecte THEN

		   RSP := 'ERROR: a) el camp id no es negatiu o, b) el camp nom_estat no te un dels 4 estats esperats: ''edicio'', ''pendent obertura'', ''oberta'', ''finalitzada''';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		END;







	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaOpcio
	-- 			
	-- Descripci:		insereix un registre a la taula opcions i crida el procediment p_registrarLog del paquet pk_utils
	-- 			
	-- Parmetres entrada:	id 		NUMBER				PK
	-- 			nom		VARCHAR2(15 CHAR)		NOT NULL
	-- 			descr		VARCHAR2(100 CHAR)
	-- 			cod_pais	CHAR(2)				NOT NULL	REFERENCES codisISO(codi_pais)
	-- 			baixa_log	NUMBER(1)	DEFAULT 0	NOT NULL	CHECK (baixa_log IN(0,1))
	-- 										UNIQUE (nom, codi_pais)
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada que no pot ser buit o NULL ho s
	-- 			llargada_camp		-> quan els parmetres nom i/o descr tenen ms de 15 i 100 carcteres respectivament
	-- 						-> i quan el parmetre codi_pais no t exactament 2 carcters
	-- 			baixa_inapropiada	-> quan baixa_log no s zero
	-- 			codi_pais_incorrecte	-> quan cod_pais no referencia el codi ISO de cap pais (incls el pas neuter 'ZQ')
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaOpcio (
	  id		IN	opcions.id%TYPE,
	  nom		IN	opcions.nom_opcio%TYPE,
	  descr		IN	opcions.descripcio%TYPE,
	  cod_pais	IN	opcions.codi_pais%TYPE,
	  baixa_log	IN	opcions.baixa_logica%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_altaOpcio';
	  	parametres_entrada 	VARCHAR2(140 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER := 0;
	  	camp_obligatori 	EXCEPTION;
		llargada_camp		EXCEPTION;
		baixa_inapropiada	EXCEPTION;
		codi_pais_incorrecte	EXCEPTION;


		BEGIN

		  parametres_entrada := id||', '||nom||', '||descr||', '||cod_pais||', '||baixa_log;

		  IF (	id IS NULL OR
			nom IS NULL OR
			cod_pais IS NULL OR
			baixa_log IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (	length(nom) > 15 OR
			length(descr) > 100 OR
			length(cod_pais) <> 2 ) tHEN
			RAISE llargada_camp;
		  END IF;

		  IF ( baixa_log <> 0 ) THEN
			RAISE baixa_inapropiada;
		  END IF;

		  -- comprovem que codi_pais referenci un pas del sistema o al codi de pas neutre 'ZQ'
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM codisISO WHERE codi_pais = cod_pais;
		  IF ( trobat = 0 ) THEN
			RAISE codi_pais_incorrecte;
		  END IF;


		  -- fetes les verificacions anteriors, es pot donar d'alta la opci
		  INSERT INTO opcions(id, nom_opcio, descripcio, codi_pais, baixa_logica) VALUES(id, nom, descr, cod_pais, baixa_log);

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN llargada_camp THEN

		   RSP := 'ERROR: nombre de caracters inapropiat. Revisi la llargada dels camps nom_opcio, descripcio i/o codi_pais.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN baixa_inapropiada THEN

		   RSP := 'ERROR: el valor del camp baixa_logica ha de ser zero en el moment de donar d''alta una opcio.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN codi_pais_incorrecte THEN

		   RSP := 'ERROR: el codi del pais no referencia cap codi ISO.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		END;










	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaVotacio
	-- 			
	-- Descripci:		insereix un registre a la taula votacions i un registre a la taula estadistiquesVotacions amb els
	-- 			valors estadsitcs inicialitzats a zero. Finalment, crida el procediment p_registrarLog del paquet pk_utils
	-- 
	-- 			NOTA:	El procediment no deixa donar d'alta votacions que comencin (data inici) abans de la data actual
	-- 				del sistema. Tanmateix es fa una excepci per les votacions de prova que s'han creat.
	-- 
	-- Parmetres entrada:	id		NUMBER			PK
	-- 			cens_global	VARCHAR2(30 CHAR)	NOT NULL	REFERENCES censGlobal(nom_cens)
	-- 			id_estat	NUMBER			NOT NULL	REFERENCES estatsVotacio(id)
	-- 			titol_v		VARCHAR2(200 CHAR)	NOT NULL	UNIQUE
	-- 			data_inici	DATE			NOT NULL
	-- 			data_fi		DATE			NOT NULL	CHECK(data_fi > data_inici)
	-- 			any_votacio	NUMBER(4)		NOT NULL	REFERENCES anys(lany)
	-- 			web		VARCHAR2(200 CHAR)
	-- 			president	VARCHAR2(100 CHAR)	NOT NULL
	-- 			electors	NUMBER	DEFAULT 0	NOT NULL	CHECK(electors >= 0)
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada que no pot ser buit o NULL ho s
	-- 			camp_incorrecte		-> quan l'identificador de votaci s un nombre natural
	-- 			cens_global_inexistent	-> quan cens_global no referencia el cens global
	-- 			estat_inexistent	-> quan id_estat no referencia cap dels estats d'una vortaci
	-- 			estat_incorrecte	-> quan l'estat de la nova votaci no s 'edicio'
	-- 			num_electors_incorrecte	-> quan el nmero d'electors no s zero
	-- 			data_inici_incorrecte	-> quan la data d'inici s menor que la data actual (veure nota superior).
	-- 			data_fi_incorrecte 	-> quan la data de fi s menor que la data d'inici
	-- 			any_incorrecte		-> quan l'any de la votaci no es correspon amb l'any indicat per la data d'inici
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaVotacio (
	  id		IN	votacions.id%TYPE,
	  cens_global	IN	votacions.id_cens_global%TYPE,
	  id_estat	IN	votacions.id_estat%TYPE,
	  titol_v	IN	votacions.titol%TYPE,
	  data_inici	IN	votacions.data_inici%TYPE,
	  data_fi	IN	votacions.data_fi%TYPE,
	  any_votacio	IN	votacions.any_votacio%TYPE,
	  web		IN	votacions.adreca_web%TYPE,
	  president	IN	votacions.president_mesa%TYPE,
	  electors	IN	votacions.num_electors%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_altaVotacio';
	  	parametres_entrada 	VARCHAR2(600 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		votacio_prova		NUMBER(1) := 0;
		trobat			NUMBER(1) := 0;
		id_est			NUMBER := 0;
		estat			VARCHAR2(20 CHAR);
		inici			NUMBER;
		fi			NUMBER;
		actual			NUMBER;
		any_inici		NUMBER;
		num_sequencia		NUMBER;
	  	camp_obligatori 	EXCEPTION;
		camp_incorrecte 	EXCEPTION;
		cens_global_inexistent	EXCEPTION;
		estat_inexistent	EXCEPTION;
		estat_incorrecte	EXCEPTION;
		num_electors_incorrecte	EXCEPTION;
		data_inici_incorrecte	EXCEPTION;
		data_fi_incorrecte	EXCEPTION;
		any_incorrecte		EXCEPTION;


		BEGIN

		  parametres_entrada := id||', '||cens_global||', '||id_estat||', '||titol_v||', '||data_inici||', '||data_fi||', '||any_votacio||', '||web||', '||president||', '||electors;

		  -- convertim les dates d'inici i fi de la votaci i la data actual a un format adequat per fer comparacions
		  SELECT TO_NUMBER(TO_CHAR(data_inici, 'yyyymmdd')) INTO inici FROM DUAL;
		  SELECT TO_NUMBER(TO_CHAR(data_fi, 'yyyymmdd')) INTO fi FROM DUAL;
		  SELECT TO_NUMBER(TO_CHAR(SYSDATE, 'yyyymmdd')) INTO actual FROM DUAL;


		  IF (  id IS NULL OR 
			cens_global IS NULL OR
			id_estat IS NULL OR
			titol_v IS NULL OR
			data_inici IS NULL OR
			data_fi IS NULL OR
			any_votacio IS NULL OR
			web IS NULL OR
			president IS NULL OR
			electors IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF ( id >= 0 ) THEN
			RAISE camp_incorrecte;
		  END IF;

		  -- comprovem que el cens global estigui ben referenciat
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM censGlobal WHERE nom_cens = cens_global;
		  IF (trobat = 0) THEN
			RAISE cens_global_inexistent;
		  END IF;

		  -- comprovem que l'estat de la nova votaci existeixi i sigui 'edicio'
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM estatsVotacio WHERE id = id_estat;
		  IF ( trobat = 0 ) THEN
			RAISE estat_inexistent;
		  END IF;

		  SELECT nom_estat INTO estat FROM estatsVotacio WHERE id = id_estat;
		  IF ( estat <> 'edicio' ) THEN
			RAISE estat_incorrecte;
		  END IF;
		  
	
		  
		  -- el nmero d'electors en el moment d'inserir el registre ha de ser zero (s'actualitzar al associar censos a les votacions)
		  IF ( electors <> 0 ) THEN
			RAISE num_electors_incorrecte;
		  END IF;


	-- Votacions de prova (a les que es permetr que tinguin una data d'inici anterior a la data actual del sistema)
	IF ( titol_v  IN( 'Eleccions al municipi de Sevilla de 2 de gener de 2000',
			  'Eleccions al municipi de Munich de 3 de gener de 2000', 
			  'Eleccions al municipi de Roma de 2 de gener de 2001',
			  'Eleccions al municipi de Santander de 3 de gener de 2001',
			  'Eleccions al municipi de Berlin de 2 de gener de 2002',
			  'Eleccions al municipi de Toledo de 3 de gener de 2002',  
			  'Eleccions al municipi de Florencia de 2 de gener de 2003', 
			  'Eleccions al municipi de Madrid de 3 de gener de 2003', 
			  'Eleccions al municipi de Barcelona de 2 de gener de 2004',
 			  'Eleccions al municipi de Murcia de 3 de gener de 2004',  
			  'Eleccions al municipi de Dusseldorf de 2 de gener de 2005', 
			  'Eleccions al municipi de Girona de 3 de gener de 2005',
			  'Eleccions al municipi de Bolonya de 2 de gener de 2006', 
			  'Eleccions al municipi de Murcia de 3 de gener de 2006', 
			  'Eleccions al municipi de Hannover de 2 de gener de 2007',
			  'Eleccions al municipi de Florencia de 3 de gener de 2007',
			  'Eleccions al municipi de Barcelona de 2 de gener de 2008',
			  'Eleccions al municipi de Madrid de 3 de gener de 2008',  
			  'Eleccions al municipi d''Olot de 2 de gener de 2010',
			  'Eleccions al municipi de Girona de 3 de gener de 2010' ,
			  'Eleccions al Parlament de Catalunya de 1 de novembre de 2011', 
			  'Eleccions a Corts Generals de 2 de novembre de 2011', 
			  'Eleccions al Parlament Europeu de 3 de novembre de 2011' ) ) THEN
		votacio_prova := 1;
	END IF;


		  -- no es pot donar d'alta una votaci amb una data d'inici anterior a la data actual
		  -- 		NOTA:	les votacions amb el titol que es veu just aqu sobre sn per fer proves (posaran votacio_prova a 1)
		  -- 			i se'ls permetr una data d'inici anterior a la data actual del sistema.
		  IF ( inici < actual AND votacio_prova = 0 ) THEN
			RAISE data_inici_incorrecte;
		  END IF;


		  -- una votaci no pot finalitzar abans de comenar
		  IF ( fi < inici ) THEN
			RAISE data_fi_incorrecte;
		  END IF;

		  -- Obtenim l'any de la data d'inici per fer la segent comparaci.
		  -- L'obtenim de la variable 'inici' en comptes del parmetre data_inici perqu amb aquest no s'obtenia l'any correctament
		  SELECT TO_NUMBER(TO_CHAR(data_inici, 'yyyy')) INTO any_inici FROM DUAL;

		  -- l'any de la votaci no pot ser diferent a l'any de la data d'inici (la data d'inici i de final s'ha dit que seran el mateix any)
		  IF ( any_votacio <> any_inici ) THEN
			RAISE any_incorrecte;
		  END IF;


		  -- alta de la votaci
		  INSERT INTO votacions(id, id_cens_global, id_estat, titol, data_inici, data_fi, any_votacio, adreca_web, president_mesa, num_electors) 
		  VALUES(id, cens_global, id_estat, titol_v, data_inici, data_fi, any_votacio, web, president, electors);

		  -- consulta de l'identificador que la seqncia ha assignat a la votaci (el ttol s clau alternativa)
		  SELECT id INTO num_sequencia FROM votacions WHERE titol = titol_v;

		  -- i alta de les estadstiques de la votaci amb els valors estadstics inicialitzats a zero (valor per defecte)
		  INSERT INTO estadistiquesVotacions(id, id_votacio, titol_votacio, any_votacio, finalitzada, num_electors, num_vots, participacio,
			id_opcio_mes_vots, nom_opcio_mes_vots, vots_opcio_mes_vots, percentatge_opcio_mes_vots, id_opcio_menys_vots, 
			nom_opcio_menys_vots, vots_opcio_menys_vots, percentatge_opcio_menys_vots, dif_perc_opc_mes_i_menys_vots)
		  VALUES(-1, num_sequencia, titol_v, any_votacio, DEFAULT, DEFAULT, DEFAULT, DEFAULT, NULL, NULL, DEFAULT, DEFAULT, NULL, NULL, DEFAULT, DEFAULT, DEFAULT);
		  

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN camp_incorrecte THEN

		   RSP := 'ERROR: l''identificador de votacio ha de ser menor a zero (l''actualitza una sequencia).';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN cens_global_inexistent THEN

		   RSP := 'ERROR: el cens global no existeix';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN estat_inexistent THEN

		   RSP := 'ERROR: l''identificador de l''estat de la votacio no referencia cap dels estats possibles.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN estat_incorrecte THEN

		   RSP := 'ERROR: l''identificador de l''estat d''una nova votacio ha de referenciar l''estat ''edicio'' ';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN num_electors_incorrecte THEN

		   RSP := 'ERROR: el numero d''electors ha de ser zero.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN data_inici_incorrecte THEN

		   RSP := 'ERROR: la data d''inici ha de ser major que la data actual';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN data_fi_incorrecte THEN

		   RSP := 'ERROR: la data de fi no pot ser anterior a la data d''inici';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		WHEN any_incorrecte THEN

		   RSP := 'ERROR: l''any de la votacio no es correspon amb l''any de la data d''inici';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;







	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaOpcioVotacio
	-- 			
	-- Descripci:		insereix un registre a la taula opcionsVotacio i crida el procediment p_registrarLog del paquet pk_utils
	-- 			
	-- Parmetres entrada:	id		NUMBER			PK
	-- 			id_votacio	NUMBER			NOT NULL	REFERENCES votacions(id)
	-- 			id_opcio_v	NUMBER			NOT NULL	REFERENCES opcions(id)
	-- 			n_vots		NUMBER	DEFAULT 0	NOT NULL	CHECK(n_vots >=0)
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada que no pot ser buit o NULL ho s
	-- 			valor_incorrecte	-> quan id_votaci i id_opcio_v no sn enters positius i/o quan n_vots no s zero
	-- 			votacio_inexistent	-> quan id_votacio no referencia cap votaci del sistema
	-- 			opcio_inexistent	-> quan id_opcio_v no referencia cap opci del sistema
	-- 			opcio_de_baixa		-> quan id_opcio_v referencia una opcio que ha estat donada de baixa
	-- 			votacio_no_editable	-> quan id_votacio referencia una votaci a la qual ja no es poden afegir opcions
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaOpcioVotacio (
	  id		IN	opcionsVotacio.id%TYPE,
	  id_votacio	IN	opcionsVotacio.id_votacio%TYPE,
	  id_opcio_v	IN	opcionsVotacio.id_opcio%TYPE,
	  n_vots	IN	opcionsVotacio.num_vots%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_altaOpcioVotacio';
	  	parametres_entrada 	VARCHAR2(60 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		baixa			NUMBER(1) := 0;
		id_estat_vota		NUMBER := 0;
		estat_vota		VARCHAR2(20 CHAR);
	  	camp_obligatori 	EXCEPTION;
		valor_incorrecte	EXCEPTION;
		votacio_inexistent	EXCEPTION;
		opcio_inexistent	EXCEPTION;
		opcio_de_baixa		EXCEPTION;
		votacio_no_editable	EXCEPTION;



		BEGIN

		  parametres_entrada := id||', '||id_votacio||', '||id_opcio_v||', '||n_vots;

		  IF (  id IS NULL OR 
			id_votacio IS NULL OR
			id_opcio_v IS NULL OR
			n_vots IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (	id_votacio <= 0 OR
			id_opcio_v <= 0 OR
			n_vots <> 0 ) THEN
			RAISE valor_incorrecte;
		  END IF;

		  -- comprovem que la votaci existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM votacions WHERE id = id_votacio;
		  IF ( trobat = 0 ) THEN
			RAISE votacio_inexistent;
		  END IF;


		  -- comprovem que la opci existeixi
		  trobat := 0;
		  SELECT COUNT (*) INTO trobat FROM opcions WHERE id = id_opcio_v;
		  IF ( trobat = 0 ) THEN
			RAISE opcio_inexistent;
		  END IF;

		  -- comprovem que la opci no estigui de baixa
		  SELECT baixa_logica INTO baixa FROM opcions WHERE id = id_opcio_v;
		  IF ( baixa = 1 ) THEN
			RAISE opcio_de_baixa;
		  END IF;



		  -- comprovem que la votaci estigui en estat 'edicio'
		  SELECT id_estat INTO id_estat_vota FROM votacions WHERE id = id_votacio;
		  SELECT nom_estat INTO estat_vota FROM estatsVotacio	WHERE id = id_estat_vota;
		  IF ( estat_vota <> 'edicio' ) THEN
			RAISE votacio_no_editable;
		  END IF;


		  INSERT INTO opcionsVotacio(id, id_votacio, id_opcio, num_vots) VALUES (id, id_votacio, id_opcio_v, n_vots);

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN valor_incorrecte THEN

		   RSP := 'ERROR: comprovi que es compleixi el valor dels camps seguents: id_votacio > 0, id_opcio > 0 i/o num_vots = 0';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_inexistent THEN

		   RSP := 'ERROR: no es pot afegir una opcio de vot a una votacio que no existeix';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN opcio_inexistent THEN

		   RSP := 'ERROR: la opcio de vot no existeix al sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN opcio_de_baixa THEN

		   RSP := 'ERROR: la opcio de vot ha estat donada de baixa i no es pot associar a cap votacio';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_no_editable THEN

		   RSP := 'ERROR: no es pot associar cap opcio a una votacio que no estigui en estat d''edicio.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;







	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaCensAssociat
	-- 			
	-- Descripci:		Una vegada fetes unes quantes verificacions prvies, el procediment porta a terme diverses accions:
	-- 			   a) si encara no hi s, insereix un registre a la taula votacionsPaisos amb l'identificador del pas i
	-- 			      de la votaci (la taula registra els paisos que participen a cada votaci). Si el pas del censAssociat
	-- 			      que es dna d'alta ja existeix a votacionsPaisos, no s'inserir cap registre a aquesta taula.
	-- 			   b) Obt el n de ciutadans censats actualment en el cens local, id_cens_loc, que es vol associar a la 
	-- 			      votaci id_vota. Aquest valor servir a l'hora d'inserir el registre a la taula censosAssociats (punt c).
	-- 			   c) Insereix un registre a la taula censosAssociats (taula que registra els censos associats a cada votaci)
	-- 			      amb totes les dades rebudes per parmetre a excepci del parmetre ciut_censats que no es tindr en compte
	-- 			      ja que el nmero de ciutadans censats s'obt en aquest procediment (punt b).
	-- 			   d) Actualitza el nmero d'electors de la votaci.
	-- 			   e) Insereix dades de tots els ciutadans del cens local id_cens_loc que no estiguin donats de baixa a la taula 
	-- 			      ciutadansVotacions (taula que registra tots els electors de les votacions).
	-- 			   f) Crida el procediment p_registrarLog del paquet pk_utils.
	-- 
	-- 			NOTA:	El procediment noms deixa donar d'alta censos a les votacions que estan en estat d'edici 
	-- 
	-- 			
	-- Parmetres entrada:	id_vota		NUMBER				PK_composta1	REFERENCES votacions(id)
	-- 			id_cens_loc	VARCHAR2(40 CHAR)		PK_composta2	REFERENCES censosLocals(nom_cens)
	-- 			cens_estat	VARCHAR(30 CHAR)		NOT NULL
	-- 			cens_regio	VARCHAR(40 CHAR)		NOT NULL
	--			nom_local	VARCHAR(40 CHAR)		NOT NULL
	-- 			nom_pais	VARCHAR(30 CHAR)		NOT NULL
	-- 			any_votacio	NUMBER(4)			NOT NULL
	-- 			ciut_censats	NUMBER				NOT NULL	CHECK(ciut_censats >=0)
	-- 			finalitzada	NUMBER(1)	DEFAULT 0	NOT NULL	CHECK(finalitzada IN(0,1))
	-- 			n_vots		NUMBER		DEFAULT 0	NOT NULL	CHECK(n_vots >=0)
	-- 
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori			-> quan un parmetre d'entrada que no pot ser buit o NULL ho s
	-- 			numero_incorrecte		-> quan l'id_votacio no s un natural i/o quan ciut_censats, n_vots o
	-- 							   finalitzada no siguin zero
	-- 			votacio_inexistent		-> quan id_vota no correspon a cap votaci existent
	-- 			votacio_no_editable		-> quan la votaci no est en estat 'edicio' (en altres estats no s'hi pot associar 
	-- 							   cap cens)
	-- 			any_incorrecte			-> quan any_votaci no t 4 dgits o quan no es correspon amb l'any de la votaci
	-- 			cens_local_inexistent		-> quan id_cens_local no correspon a cap cens local existent
	-- 			cens_local_regional_erroni	-> quan el cens local no correspon al cens regional
	-- 			cens_regional_estatal_erroni	-> quan el cens regional no correspon al cens estatal
	-- 			nom_localitat_erroni		-> quan el nom de localitat no es correspon amb el cens local
	-- 			pais_incorrecte			-> quan el nom del pais no es correspon amb el del cens estatal
	--			OTHERS				-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaCensAssociat (
	  id_vota	IN	censosAssociats.id_votacio%TYPE,
	  id_cens_loc	IN	censosAssociats.id_cens_local%TYPE,
	  cens_estat	IN	censosAssociats.cens_estatal%TYPE,
	  cens_regio	IN	censosAssociats.cens_regional%TYPE,
	  nom_local	IN	censosAssociats.nom_localitat%TYPE,
	  nom_pais	IN	censosAssociats.nom_pais%TYPE,
	  any_votacio	IN	censosAssociats.any_votacio%TYPE,
	  ciut_censats	IN	censosAssociats.ciutadans_censats%TYPE,
	  finalitzada	IN	censosAssociats.finalitzada%TYPE,
	  n_vots	IN	censosAssociats.num_vots%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure		VARCHAR2(30 CHAR) := 'p_altaCensAssociat';
	  	parametres_entrada	VARCHAR2(240 CHAR) := NULL;
		esCorrecte		NUMBER(1) := 1;
		confirmada		NUMBER(1) := 1;
		any_v			NUMBER(4) := 0;
		trobat			NUMBER(1) := 0;
		titol_v			VARCHAR2(100 CHAR);
		estat_votacio		VARCHAR2(20 CHAR) := 0;
		pais			VARCHAR2(30 CHAR);
		cens_est		VARCHAR2(30 CHAR);
		cens_reg		VARCHAR2(40 CHAR);
		ciutat			VARCHAR2(40 CHAR);
		censats			NUMBER := 0;
		electors_votacio	NUMBER := 0;
	  	camp_obligatori			EXCEPTION;
		numero_incorrecte		EXCEPTION;
		votacio_inexistent		EXCEPTION;
		votacio_no_editable		EXCEPTION;
		any_incorrecte			EXCEPTION;
		cens_local_inexistent		EXCEPTION;
		cens_local_regional_erroni	EXCEPTION;
		cens_regional_estatal_erroni	EXCEPTION;
		nom_localitat_erroni		EXCEPTION;
		pais_incorrecte			EXCEPTION;


		-- declarem un cursor que obtindr els nmeros aleatoris del ciutadans associats al cens local
		-- i que ser til per carregar dades a la taula ciutadansVotacions.
		CURSOR ciutadans_cens IS SELECT num_aleatori, baixa_logica FROM ciutadans c WHERE c.id_cens_local = id_cens_loc;


		BEGIN

		  parametres_entrada := id_vota||', '||id_cens_loc||', '||cens_estat||', '||cens_regio||', '||nom_local||', '||nom_pais||', '||any_votacio||', '||ciut_censats||', '||finalitzada||', '||n_vots;

		  IF (  id_vota IS NULL OR 
			id_cens_loc IS NULL OR
			cens_estat IS NULL OR
			cens_regio IS NULL OR
			nom_local IS NULL OR
			nom_pais IS NULL OR
			any_votacio IS NULL OR
			ciut_censats IS NULL OR
			finalitzada IS NULL OR
			n_vots IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (	id_vota <= 0 OR
			ciut_censats <> 0 OR
			n_vots <> 0 OR
			finalitzada <> 0 ) THEN
			RAISE numero_incorrecte;
		  END IF;

		  IF( length(any_votacio) <> 4 ) THEN
			RAISE any_incorrecte;
		  END IF;


		  -- comprovem que l'identificador de la votaci existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM votacions WHERE id = id_vota;
		  IF ( trobat = 0 ) THEN
			RAISE votacio_inexistent;
		  END IF;



		  -- comprovem que la votaci encara sigui editable (en estat 'edicio'). Si no ho s, no s'hi podr afegir cap cens.
		  SELECT ev.nom_estat INTO estat_votacio 
			FROM estatsVotacio ev, votacions v
			WHERE ev.id = v.id_estat AND v.id = id_vota;
		  IF ( estat_votacio <> 'edicio' ) THEN
			RAISE votacio_no_editable;
		  END IF;

		  -- comprovem que hi hagi coherncia entre el valor del parmetre any_votacio i l'any real de la votaci
		  SELECT any_votacio INTO any_v FROM votacions WHERE id = id_vota;
		  IF ( any_votacio <> any_v ) THEN
			RAISE any_incorrecte;
		  END IF;


		  -- comprovem que l'identificador de cens local existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM censosLocals WHERE nom_cens = id_cens_loc;
		  IF ( trobat = 0 ) THEN
			RAISE cens_local_inexistent;
		  END IF;

		  -- comprovem que el cens local realment pertanyi al cens regional
		  SELECT id_cens_regional INTO cens_reg FROM censosLocals WHERE nom_cens = id_cens_loc;
		  IF ( cens_regio <> cens_reg ) THEN
			RAISE cens_local_regional_erroni;
		  END IF;

		  -- comprovem que el cens regional realment pertanyi al cens estatal
		  SELECT id_cens_estatal INTO cens_est FROM censosRegionals WHERE nom_cens = cens_regio;
		  IF ( cens_estat <> cens_est ) THEN
			RAISE cens_regional_estatal_erroni;
		  END IF;

		  -- comprovem que la localitat es correspongui amb el cens local
		  SELECT nom_localitat INTO ciutat FROM censosLocals WHERE nom_cens = id_cens_loc;
		  IF ( nom_local <> ciutat ) THEN
			RAISE nom_localitat_erroni;
		  END IF;

		  -- comprovem que el nom del pas es correspongui amb el pas del cens estatal
		  SELECT p.nom_pais INTO pais FROM paisos p, censosEstatals ce WHERE p.codi_pais = ce.codi_pais AND ce.nom_cens = cens_estat;
		  IF ( nom_pais <> pais) THEN
			RAISE pais_incorrecte;
		  END IF;


		  -- si el pas d'aquest cens i per aquesta votaci encara no s a la taula votacionsPaisos l'hi afegim
		  trobat := 0;
		  SELECT COUNT (*) INTO trobat FROM votacionsPaisos WHERE id_pais = nom_pais AND id_votacio = id_vota;
		  IF ( trobat = 0 ) THEN
			INSERT INTO votacionsPaisos(id_pais, id_votacio) VALUES(nom_pais, id_vota);
		  END IF;
		

		  -- Obtenim el nmero de ciutadans censats (en el moment actual) a aquest cens (el n de ciutadans censats del cens que associem 
		  -- a la votaci ser el n de ciutadans censats que hi hagi en el cens local en el moment de donar d'alta el cens associat)
		  SELECT ciutadans_censats INTO censats FROM censosLocals WHERE nom_cens = id_cens_loc;

		  -- Donem d'alta, a la taula censosAssociats,  el cens associat a la votaci
		  INSERT INTO censosAssociats(id_votacio, id_cens_local, cens_estatal, cens_regional, nom_localitat, nom_pais, any_votacio, 
			ciutadans_censats, finalitzada, num_vots) VALUES(id_vota, id_cens_loc, cens_estat, cens_regio, nom_local, nom_pais,
			any_votacio, censats, finalitzada, n_vots);

		  -- actualizem el nmero d'electors de la votaci
		  SELECT num_electors INTO electors_votacio FROM votacions WHERE id = id_vota;
		  UPDATE votacions SET num_electors = (electors_votacio + censats) WHERE id = id_vota;

		  -- Recorrem el cursor que hem declarat (que cont els ciutadans del cens local que es vol associar a la votaci). Els ciutadans 
		  -- que no estiguin donats de baixa s'inseriran a la taula ciutadansVotacions (que contindr els electors de les votacions).
		  FOR ciutada IN ciutadans_cens LOOP

			IF ( ciutada.baixa_logica = 0 ) THEN
				INSERT INTO ciutadansVotacions(id_ciutada, id_votacio, any_votacio, ha_participat)
					VALUES(ciutada.num_aleatori, id_vota, any_votacio, 0);
			END IF;
			
		  END LOOP;

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN numero_incorrecte THEN

		   RSP := 'ERROR: comprovi que es compleixi el valor dels camps seguents: id_votacio > 0, ciutadans_censats = 0,  num_vots = 0 i/o finalitzada = 0';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_inexistent THEN

		   RSP := 'ERROR: l''identificador de votacio no correspon a cap votacio';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_no_editable THEN

		   RSP := 'ERROR: la votacio no es troba en estat d''edicio. Ja no s''hi pot associar cap nou cens.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN any_incorrecte THEN

		   RSP := 'ERROR: comprovi que l''any tingui 4 digits i que es correspongui amb l''any de la votacio';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN cens_local_inexistent THEN

		   RSP := 'ERROR: l''identificador de cens local no correspon a cap cens local';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN cens_local_regional_erroni THEN

		   RSP := 'ERROR: el cens local no correspon a aquest cens regional. Revisi el cens regional.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN cens_regional_estatal_erroni THEN

		   RSP := 'ERROR: el cens regional no correspon al cens estatal. Revisi el cens estatal';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN nom_localitat_erroni THEN

		   RSP := 'ERROR: el nom de la localitat no es correspon amb el cens local. Revisi el nom de localitat.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN pais_incorrecte THEN

		   RSP := 'ERROR: el nom del pais no es correspon amb el cens estatal. Revisi el nom del pais.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;









	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_actualitzaEstatVotacio
	-- 			
	-- Descripci:		actualitza l'estat d'una votaci i, si el nou estat s 'finalitzada', actualitza un camp de les taules
	-- 			censosAssocitas i estadistiquesVotacions i crida el procediment p_estadistiquesVotacio del paquet pk_estadistiques;
	-- 			finalment, tamb crida el procediment registrarLog del paquet pk_utils. 
	-- 
	-- 			Per actualitzar l'estat d'una votaci, per, noms es podr fer d'un estat previ, a l'immediatament posterior. Aix,
	-- 			totes les votacions hauran de passar pels quatre estats i en aquest ordre:
	-- 			edicio -> pendent obertura -> oberta -> finalitzada
	-- 
	-- 			L'estat 'edicio' s l'estat que hauria de tenir (i que t per defecte) una nova votaci del sistema. Aix que
	-- 			el nou estat d'una votaci mai podr ser 'edicio' (s'ha considerat que no t sentit editar una votaci
	-- 			quan ja est pendent d'obertura (campanya electoral) i encara menys quan est oberta o finalitzada).
	-- 
	-- 			Per altre costat, tamb es tindran en compte les dates de les votacions abans d'actualitzar l'estat d'una
	-- 			votaci. Per exemple, no es pot finalitzar una votaci abans de la data de fi de la mateixa.
	-- 			
	-- 
	-- 			NOTA: Pel canvi d'estat de les votacions, com que tamb es tenen en compte les dades d'inici i de fi de la votaci,
	-- 			s'ha fet una excepci amb les votacions de prova que s'han creat. S'explica amb ms detall en el punt del codi on es fa
	-- 			l'excepci.
	-- 
	-- Parmetres entrada:	id_vota		NUMBER			NOT NULL	REFERENCES votacions(id)
	-- 			nou_estat	VARCHAR2(20 CHAR)	NOT NULL
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori			-> quan un parmetre d'entrada que no pot ser buit o NULL ho s
	-- 			estat_incorrecte		-> quan nou_estat no correspon a cap dels estats acceptats per una votaci
	-- 			votacio_inexistent		-> quan id_vota no correspon a cap votaci existent
	-- 			votacio_finalitzada		-> quan es vol canviar l'estat d'una votaci que ja ha finalitzat
	-- 			votacio_oberta			-> quan es vol canviar l'estat d'una votaci oberta a un estat diferent a 'finalitzada'
	-- 			votacio_pendent_obertura	-> quan es vol canviar l'estat d'una votaci pendent d'obertura a un estat diferent a 'oberta'
	-- 			votacio_edicio			-> quan es vol canviar l'estat d'una votaci en edici a un estat diferent a 'pendent obertura'
	-- 			edicio_pendent_error		-> quan es vol posar en estat 'pendent obertura'una votaci que segons la data d'inici
	-- 							   ja no hi pot estar (hauria d'estar oberta o finalitzada).
	-- 			pendent_oberta_error		-> quan es vol posar en estat 'oberta' una votaci que, d'acord amb les dates de la 
	-- 							   votaci, encara no es pot obrir o ja hauria d'haver acabat.
	-- 			oberta_finalitzada_error	-> quan es vol posar en estat 'finalitzada' una votaci que, segons la data de fi,
	-- 							   encara hauria d'estar 'oberta' (veure nota superior).
	--			OTHERS				-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_actualitzaEstatVotacio (
	  id_vota	IN	votacions.id%TYPE,
	  nou_estat	IN	estatsVotacio.nom_estat%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_actualitzaEstatVotacio';
	  	parametres_entrada 	VARCHAR2(25 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		estat_actual		estatsVotacio.nom_estat%TYPE;
		t_electors		NUMBER := 0;
		titol_v			VARCHAR2(100 CHAR);
		votacio_prova		NUMBER(1) := 0;
		dat_inici		VARCHAR2(10 CHAR);
		dat_fi			VARCHAR2(10 CHAR);
		dat_actual		VARCHAR2(10 CHAR);
		id_nou_estat		NUMBER := 0;
		RESP			VARCHAR2(200 CHAR);
	  	camp_obligatori 		EXCEPTION;
		estat_incorrecte 		EXCEPTION;
		votacio_inexistent		EXCEPTION;
		votacio_finalitzada		EXCEPTION;
		votacio_oberta			EXCEPTION;
		votacio_pendent_obertura	EXCEPTION;
		votacio_edicio			EXCEPTION;
		edicio_pendent_error		EXCEPTION;
		pendent_oberta_error		EXCEPTION;
		oberta_finalitzada_error	EXCEPTION;


		BEGIN

		  parametres_entrada := id_vota||', '||nou_estat;

		  IF (  id_vota IS NULL OR 
			nou_estat IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (	nou_estat NOT IN ('edicio', 'pendent obertura', 'oberta', 'finalitzada') ) THEN
			RAISE estat_incorrecte;
		  END IF;

		  -- comprovem que l'identificador de la votaci existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM votacions WHERE id = id_vota;
		  IF ( trobat = 0 ) THEN
			RAISE votacio_inexistent;
		  END IF;


		  -- obtenim l'estat actual de la votacio
		  SELECT ev.nom_estat INTO estat_actual
			FROM estatsVotacio ev, votacions v
			WHERE ev.id = v.id_estat AND v.id = id_vota;

		  -- comprovem que el nou estat correspongui a l'estat segent de l'estat actual (una votaci ha de tenir
		  -- els segents estats i en aquest ordre: edicio -> pendent obertura -> oberta -> finalitzada)
		  IF ( estat_actual = 'finalitzada' ) THEN
			RAISE votacio_finalitzada;

		  ELSIF ( estat_actual = 'oberta' AND nou_estat <> 'finalitzada' ) THEN
			RAISE votacio_oberta;

		  ELSIF ( estat_actual = 'pendent obertura' AND nou_estat <> 'oberta' ) THEN
			RAISE votacio_pendent_obertura;

		  ELSIF ( estat_actual = 'edicio' AND nou_estat <> 'pendent obertura' ) THEN
			RAISE votacio_edicio;

		  END IF;


	-- obtenim el titol de la votaci i, si coincideix amb el d'alguna de les votacions de prova de les que tenen censos
	-- associats, posem votacio_prova a 1 i aix podrem esquivar algunes les dues prximes condicions.
	-- Aix servir perqu es pugui posar l'estat 'oberta' i 'pendent obertura' a les votacions de prova que s'han creat i que
	-- tenen la data d'inici i de fi menors a la data actual. s a dir, per la resta de votacions, no es podr posar a estat
	-- 'pendent obertura' una votaci que segons la data d'inici ja hauria d'estar oberta ni a estat 'oberta' una votaci que
	-- segons la data d'inici encara no es pot obrir (o segons la data de fi la votaci ja hauria d'estar tancada).
	SELECT titol INTO titol_v FROM votacions WHERE id = id_vota;
	IF ( titol_v  IN( 'Eleccions al municipi de Sevilla de 2 de gener de 2000',
			  'Eleccions al municipi de Munich de 3 de gener de 2000', 
			  'Eleccions al municipi de Roma de 2 de gener de 2001',
			  'Eleccions al municipi de Santander de 3 de gener de 2001',
			  'Eleccions al municipi de Berlin de 2 de gener de 2002',
			  'Eleccions al municipi de Toledo de 3 de gener de 2002',  
			  'Eleccions al municipi de Florencia de 2 de gener de 2003', 
			  'Eleccions al municipi de Madrid de 3 de gener de 2003', 
			  'Eleccions al municipi de Barcelona de 2 de gener de 2004',
 			  'Eleccions al municipi de Murcia de 3 de gener de 2004',  
			  'Eleccions al municipi de Dusseldorf de 2 de gener de 2005', 
			  'Eleccions al municipi de Girona de 3 de gener de 2005',
			  'Eleccions al municipi de Bolonya de 2 de gener de 2006', 
			  'Eleccions al municipi de Murcia de 3 de gener de 2006', 
			  'Eleccions al municipi de Hannover de 2 de gener de 2007',
			  'Eleccions al municipi de Florencia de 3 de gener de 2007',
			  'Eleccions al municipi de Barcelona de 2 de gener de 2008',
			  'Eleccions al municipi de Madrid de 3 de gener de 2008',  
			  'Eleccions al municipi d''Olot de 2 de gener de 2010',
			  'Eleccions al municipi de Girona de 3 de gener de 2010',
			  'Eleccions al Parlament de Catalunya de 1 de novembre de 2011', 
			  'Eleccions a Corts Generals de 2 de novembre de 2011', 
			  'Eleccions al Parlament Europeu de 3 de novembre de 2011') ) THEN
		votacio_prova := 1;
	END IF;


		  -- obtenim el nmero d'electors
		  SELECT num_electors INTO t_electors FROM votacions WHERE id = id_vota;

		  -- obtenim les dates d'inici i de fi de la votaci i la data actual del sistema i els apliquem un format que ens permeti comparar-les.
		  SELECT data_inici INTO dat_inici FROM votacions WHERE id = id_vota;
		  SELECT data_fi INTO dat_fi FROM votacions WHERE id = id_vota;
		  SELECT SYSDATE INTO dat_actual FROM DUAL;
		  SELECT TO_CHAR(TO_DATE(dat_inici), 'yyyymmdd') INTO dat_inici FROM DUAL;
		  SELECT TO_CHAR(TO_DATE(dat_fi), 'yyyymmdd') INTO dat_fi FROM DUAL;
		  SELECT TO_CHAR(TO_DATE(dat_actual), 'yyyymmdd') INTO dat_actual FROM DUAL;


		  -- comprovem que el nou estat sigui aplicable en funci de la data actual del sistema, el nombre d'eletors i les dates d'inici 
		  -- i de fi de la votaci
		  IF ( nou_estat = 'pendent obertura' AND dat_actual > dat_inici AND votacio_prova = 0 ) THEN
			RAISE edicio_pendent_error;

		  ELSIF ( nou_estat = 'oberta' AND (t_electors = 0 OR dat_actual < dat_inici OR dat_actual > dat_fi) AND votacio_prova = 0) THEN
			RAISE pendent_oberta_error;

		  ELSIF ( nou_estat = 'finalitzada' AND dat_actual < dat_fi ) THEN
			RAISE oberta_finalitzada_error;

		  END IF;

		  -- obtenim l'identificador del nou estat que tindr la votaci
		  SELECT id INTO id_nou_estat FROM estatsVotacio WHERE nom_estat = nou_estat;


		  -- s'actualitza l'estat de la votaci
		  UPDATE votacions SET id_estat = id_nou_estat WHERE id = id_vota;

		  -- si el nou estat s 'finalitzada' s'actualitza aquest camp a les taules censosAssociats i estadistiquesVotacions
		  -- i es crida al procediment que actualitzar totes les estadstiques
		  IF ( nou_estat = 'finalitzada' ) THEN
			UPDATE censosAssociats SET finalitzada = 1 WHERE id_votacio = id_vota;
			UPDATE estadistiquesVotacions SET finalitzada = 1 WHERE id_votacio = id_vota;
			pk_estadistiques.p_estadistiquesVotacio(id_vota, RESP);
		  END IF;


		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN estat_incorrecte THEN

		   RSP := 'ERROR: nou estat incorrecte. Els estats que pot tenir una votacio son: edicio, pendent obertura, oberta o finalitzada';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_inexistent THEN

		   RSP := 'ERROR: l''identificador de votacio no correspon a cap votacio';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN votacio_finalitzada THEN

		   RSP := 'ERROR: La votacio esta finalitzada. No es pot canviar l''estat d''una votacio que ja s''ha celebrat.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_oberta THEN

		   RSP := 'ERROR: La votacio esta oberta. El nou estat nomes pot ser ''finalitzada''.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_pendent_obertura THEN

		   RSP := 'ERROR: La votacio esta pendent d''obertura. El nou estat nomes pot ser ''oberta''.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_edicio THEN

		   RSP := 'ERROR: La votacio esta en edicio. El nou estat nomes pot ser ''pendent obertura''.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN edicio_pendent_error THEN

		   RSP := 'ERROR: la votacio no es pot posar en estat ''pendent obertura''. Segons la data d''inici aquesta votacio s''hauria d''estar celebrant o ja hauria d''haver finalitzat.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN pendent_oberta_error THEN

		   RSP := 'ERROR: la votacio no es pot posar en estat ''oberta''. Pot ser: a) no te censos associats o b) les dates d''inici i/o fi no ho permeten.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN oberta_finalitzada_error THEN

		   RSP := 'ERROR: la votacio no es pot posar en estat ''finalitzada''. Segons la data de fi aquesta votacio encara no es pot finalitzar';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;







	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_altaVotEmes
	-- 			
	-- Descripci:		i crida el procediment p_registrarLog del paquet pk_utils
	-- 
	-- 			Nota: en registrar el vol a la taula vots emesos es far amb el valor de la variable id_opcio_final en comptes de fer-ho
	-- 			amb el valor del parmetre 'opcio'. En la majoria dels casos id_opcio_final tindr el mateix valor que el parmetre
	-- 			'opcio', per hi ha un cas en qu un vot vlid (dintre les opcions de la votaci) es considerar nul perqu l'est
	-- 			exercint un ciutad d'un pas en el qual no hi ha la possibilitat d'escollir aquella opci de vot. En aquests casos
	-- 			es considerar i comptabilitzar que l'elector ha votat, per el seu vot ser comptabilitzat com a nul.
	-- 			
	-- Parmetres entrada:	ciutada		NUMBER		PKcomposta1	REFERENCES ciutadans(num_aleatori)
	-- 			votacio		NUMBER		PKcomposta2	REFERENCES votacions(id)
	-- 			opcio		NUMBER		NOT NULL	REFERENCES opcionsVotacio(id)
	-- 			data_hota	TIMESTAMP(0)	NOT NULL
	-- 			
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori			-> quan un parmetre d'entrada que no pot ser buit o NULL ho s
	-- 			camp_incorrecte			-> quan els parmetres ciutada, votacio i/o opcio no sn nombres naturals diferents de zero
	-- 			votacio_inexistent		-> quan el parmetre 'votacio' no correspon a cap votaci
	-- 			elector_inexistent		-> quan el parmetre 'ciutada' no correspon a cap elector per la votaci
	-- 			opcio_inexistent		-> quan el parmetre 'opcio' no correspon a cap opcio de la votaci
	-- 			votacio_no_oberta		-> quan el parmetre 'votacio' correspon a una votaci que no est oberta
	-- 			ciutada_baixa			-> quan el parmetre 'ciutada' correspon a un ciutad que ha estat donat de baixa
	--			OTHERS				-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_altaVotEmes (
	  ciutada	IN	votsEmesos.id_ciutada%TYPE,
	  votacio	IN	votsEmesos.id_votacio%TYPE,
	  opcio		IN	votsEmesos.id_opcio%TYPE,
	  data_hora	IN	votsEmesos.data_hora%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_altaVotEmes';
	  	parametres_entrada 	VARCHAR2(50 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		elector_baixa		NUMBER(1) := 0;
		id_estat_votacio	NUMBER(1) := 0;
		estat_votacio		VARCHAR2(20 CHAR);
		nom_pais_elector	VARCHAR2(30 CHAR);
		cens_elector		VARCHAR2(40 CHAR);
		codi_pais_elector	CHAR(2);
		codi_pais_opcio		CHAR(2);
		id_opcio_final		NUMBER := 0;
		participacions_ciutada	NUMBER := 0;
		n_vots_censAssociat	NUMBER := 0;
		n_vots_opcioVotacio	NUMBER := 0;
	  	camp_obligatori 		EXCEPTION;
		camp_incorrecte 		EXCEPTION;
		votacio_inexistent		EXCEPTION;
		elector_inexistent		EXCEPTION;
		opcio_inexistent		EXCEPTION;
		votacio_no_oberta		EXCEPTION;
		ciutada_baixa			EXCEPTION;

		BEGIN

		  parametres_entrada := ciutada||', '||votacio||', '||opcio||', '||data_hora;


		  IF (  ciutada IS NULL OR 
			votacio IS NULL OR
			opcio IS NULL OR
			data_hora IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;


		  IF (	ciutada <= 0 OR
			votacio <= 0 OR
			opcio <= 0) THEN
			RAISE camp_incorrecte;
		  END IF;

		  -- comprovem que l'identificador de la votaci existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM votacions WHERE id = votacio;
		  IF ( trobat = 0 ) THEN
			RAISE votacio_inexistent;
		  END IF;

		  -- comprovem que el ciutad sigui un elector de la votaci. Es comprovar si el ciutad existeix a la taula ciutadansVotacions.
		  -- Aquesta taula s'omple automticament al associar censos a les votacions de manera que si el ciutad hi existeix ser elector i
		  -- el cens local del ciutad ser un cens associat a la votaci
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM ciutadansVotacions WHERE id_ciutada = ciutada AND id_votacio = votacio;
		  IF ( trobat = 0 ) THEN
			RAISE elector_inexistent;
		  END IF;

		  -- comprovem que la opci de vot estigui permesa en aquesta votaci		  
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM opcionsVotacio WHERE id_votacio = votacio AND id_opcio = opcio;
		  IF ( trobat = 0 ) THEN
			RAISE opcio_inexistent;
		  END IF;

		  -- comprovem que la votaci estigui en estat 'oberta'
		  SELECT id_estat INTO id_estat_votacio FROM votacions WHERE id = votacio;
		  SELECT nom_estat INTO estat_votacio FROM estatsVotacio WHERE id = id_estat_votacio;
		  IF ( estat_votacio <> 'oberta' ) THEN
			RAISE votacio_no_oberta;
		  END IF;

		  -- comprovem que el ciutada elector no estigui donat de baixa al sistema
		  SELECT baixa_logica, nom_pais, num_participacions, id_cens_local 
			INTO elector_baixa, nom_pais_elector, participacions_ciutada, cens_elector
			FROM ciutadans 
			WHERE num_aleatori = ciutada;
		  IF ( elector_baixa = 1 ) THEN
			RAISE ciutada_baixa;
		  END IF;

		  -- fetes les verificacions prvies (el ciutad es pot considerar un elector vlid, la votaci existeix i est oberta, 
		  -- la opci s permesa per la votaci), es fan un seguit d'actualitzacions abans de registrar el vot:

		  -- registrem, a la taula ciutadansVotacions on hi ha els electors de cada votaci, que l'elector ha participat.
		  UPDATE ciutadansVotacions SET ha_participat = 1 WHERE id_ciutada = ciutada AND id_votacio = votacio;

		  -- comprovem que la opci de vot (que ja s'ha comprovat que s vlida per aquesta votaci) pugui ser escollida per
		  -- l'elector que exerceix el vot (podr ser escollida si la opci s d'un partit del pas on pertany l'elector).
		  -- En cas que sigui una opci d'un pas diferent al de l'elector es comptabilitzar com un vot, per nul.
		  -- 	Notes:	1) nom_pais_elector s'ha obtingut a la consulta anterior
		  -- 		2) el codi pais 'ZQ' s el codi de pas neutre que fan servir les opcions de vot ('SI', 'NO', 'BLANC' i 'NUL').
		  SELECT codi_pais INTO codi_pais_elector FROM paisos WHERE nom_pais = nom_pais_elector;
		  SELECT codi_pais INTO codi_pais_opcio FROM opcions WHERE id = opcio;
		  IF ( codi_pais_elector <> codi_pais_opcio AND codi_pais_opcio <> 'ZQ' ) THEN
			SELECT id INTO id_opcio_final FROM opcions WHERE nom_opcio = 'NUL';
		  ELSE
			id_opcio_final := opcio;
		  END IF;


		  -- actualitzem el nmero de participacions del ciutad (participacions_ciutada s'ha obtingut en una consulta anterior)
		  UPDATE ciutadans SET num_participacions = (participacions_ciutada + 1) WHERE num_aleatori = ciutada;


		  -- incrementem en 1 el nmero de vots exercits en el censAssociat al qual pertany l'elector que emet el vot (cens elector 
		  -- s'ha obtingut en una consulta anterior).
		  SELECT num_vots INTO n_vots_censAssociat FROM censosAssociats WHERE id_cens_local = cens_elector AND id_votacio = votacio;
		  UPDATE censosAssociats SET num_vots = (n_vots_censAssociat + 1) WHERE id_cens_local = cens_elector AND id_votacio = votacio;


		  -- incrementem en 1 el nmero de vots obtinguts per la opci que ha escollit l'elector (o per la opci NUL, en cas que l'elector
		  -- hagi escollit una opci en la qual no podia exercir el vot -com s'ha comentat abans-).
		  SELECT num_vots INTO n_vots_opcioVotacio FROM opcionsVotacio WHERE id_opcio = id_opcio_final AND id_votacio = votacio;
		  UPDATE opcionsVotacio SET num_vots = (n_vots_opcioVotacio + 1) WHERE id_opcio = id_opcio_final AND id_votacio = votacio;

		  -- registrem el vot
		  INSERT INTO votsEmesos(id_ciutada, id_votacio, id_opcio, data_hora) VALUES(ciutada, votacio, opcio, data_hora);


		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN camp_incorrecte THEN

		   RSP := 'ERROR: El valor dels camps id_ciutada, id_votacio i/o id_opcio han de ser un nombre natural diferent de zero.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_inexistent THEN

		   RSP := 'ERROR: l''identificador de votacio no correspon a cap votacio';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN elector_inexistent THEN

		   RSP := 'ERROR: l''identificador aleatori del ciutada no correspon a cap elector.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN opcio_inexistent THEN

		   RSP := 'ERROR: la opcio de vot escollida no existeix per aquesta votacio.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_no_oberta THEN

		   RSP := 'ERROR: No es pot votar. La votacio no esta oberta.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN ciutada_baixa THEN

		   RSP := 'ERROR: intent de vot d''un ciutada que ha estat donat de baixa.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;










	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_baixaVotacio
	-- 			
	-- Descripci:		esborra un registre a la taula votacions que provocar l'esborrat d'un registre de la taula votacionsPaisos i
	-- 			d'un altre de la taula estadistiquesVotacions (que encara no estaven calculades). En, en el cas que la votaci 
	-- 			ja tingui opcions, censos i/o ciutadans associats, aquests s'esborraran per efecte de l'esborrat en cascada 
	-- 			(que permet preservar la integritat referencial) que es controlar pel que s'explica a la nota segent:
	-- 
	-- 			NOTA: Per tal de que, com a conseqncia de l'esborrat en cascada, no s'esborrin dades que haurien de ser fermes a 
	-- 			la BD (vots emesos, estadstiques, etc.), no es permetr esborrar cap votaci que estigui en estat 'oberta' ni en
	-- 			estat 'finalitzada'. En aquests dos estat la votaci ja pot tenir vots emesos (i en el cas de 'finalitzada' 
	-- 			resultats estadstics calculats) i no t cap sentit perdre aquesta informaci. Esborrar o modificar un vot ems
	-- 			seria un frau electoral i perdre dades estadstiques una llstima.
	-- 
	-- Parmetres entrada:	titol_vota	VARCHAR2(200 CHAR) 	Nota: el titol d'una votaci s clau alternativa
	-- 
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan titol_vota s buit o NULL
	-- 			llargada_incorrecte	-> quan el ttol de la votaci s massa llarg
	-- 			votacio_inexistent	-> quan titol_vota no referencia cap votacio
	-- 			estat_vota_no_permet	-> quan s'intenta esborrar una votaci que est 'oberta' o 'finaltzada'
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_baixaVotacio (
	  titol_vota	IN	votacions.titol%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_baixaVotacio';
	  	parametres_entrada 	VARCHAR2(400 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		id_estat_vota		NUMBER := 0;
		estat_vota		VARCHAR2(20 CHAR);
	  	camp_obligatori 	EXCEPTION;
		llargada_incorrecte 	EXCEPTION;
		votacio_inexistent	EXCEPTION;
		estat_vota_no_permet	EXCEPTION;


		BEGIN

		  parametres_entrada := titol_vota;


		  IF (  titol_vota IS NULL ) THEN
			RAISE camp_obligatori;
		  END IF;

		  IF ( length(titol_vota) > 200 ) THEN
			RAISE llargada_incorrecte;
		  END IF;

		  -- comprovem que la votacio existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM votacions WHERE titol = titol_vota;
		  IF (trobat = 0) THEN
			RAISE votacio_inexistent;
		  END IF;

		  -- comprovem que l'estat de la votaci permeti esborrar-la (no es permet en estat 'oberta' o 'finalitzada')
		  SELECT id_estat INTO id_estat_vota FROM votacions WHERE titol = titol_vota;
		  SELECT nom_estat INTO estat_vota FROM estatsVotacio WHERE id = id_estat_vota;

		  IF ( estat_vota IN ('oberta', 'finalitzada') ) THEN
			RAISE estat_vota_no_permet;
		  END IF;
		  

		  -- Si la votaci no est 'oberta' o 'finalitzada' es pot esborrar. Aix esborrar el seu registre corresponent de la
		  -- taula votacionsPaisos i el de la taula estaditiquesVotacions (que encara no estaven calculades perqu la votaci 
		  -- no havia finalitzat). Tamb implicar l'esborrat de les possibles associacions que se li haguessin fet de 
		  -- opcions/censos/ciutadans.

		  DELETE FROM votacions WHERE titol = titol_vota;


		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: el titol de la votacio es buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN llargada_incorrecte THEN

		   RSP := 'ERROR: el titol de la votacio no pot tenir mes de 200 caracters.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_inexistent THEN

		   RSP := 'ERROR:  no es pot donar de baixa una votacio que no existeix.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN estat_vota_no_permet THEN

		   RSP := 'ERROR: no es pot esborrar una votacio que s''esta celebrant o que hagi finalitzat.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;








	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_baixaOpcio
	-- 			
	-- Descripci:		posa a 1 l'atribut baixa_logica d'un registre de la taula opcions i crida el procediment p_registrarLog del 
	-- 			paquet pk_utils. S'ha considerat que no es pot donar de baixa una opci que est present en una votaci
	-- 			que s'estigui celebrant (en estat 'oberta'). Fetes unes quantes verificacions, crida el procediment
	-- 			p_registrarLog del paquet pk_utils.
	-- 			
	-- Parmetres entrada:	nom		VARCHAR2(15 CHAR)
	-- 			cod_pais	CHAR(2)			NOT NULL	REFERENCES codisISO(codi_pais)
	-- 			
	-- 			NOTA: (nom, cod_pais) s clau alternativa de 'opcions'
	-- 
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada que no pot ser buit o NULL ho s
	-- 			llargada_camp		-> quan el parmetre nom t ms de 15 i/o quan el parmetre codi_pais no t 
	-- 						   exactament 2 carcters
	-- 			codi_pais_inexistent	-> quan cod_pais no referencia el codi ISO de cap pais (incls el pas neuter 'ZQ')
	-- 			nom_opcio_inexistent	-> quan nom no referencia el nom de cap opci del sistema
	-- 			opcio_codi_inexistent	-> quan la combinaci nom+cod_pais no referencien la clau alternativa de 'opcions'
	-- 			opcio_baixa		-> quan la opci ja estava de baixa
	-- 			opcio_de_votacio_oberta	-> quan la opci que es vol donar de baixa est present en una votaci que s'est celebrant
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_baixaOpcio (
	  nom		IN	opcions.nom_opcio%TYPE,
	  cod_pais	IN	opcions.codi_pais%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_baixaOpcio';
	  	parametres_entrada 	VARCHAR2(40 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER := 0;
		baixa			NUMBER(1) := 0;
		id_opc			NUMBER := 0;
		id_estat_oberta		NUMBER := 0;
		id_estat_vota		NUMBER := 0;
	  	camp_obligatori 	EXCEPTION;
		llargada_camp		EXCEPTION;
		codi_pais_inexistent	EXCEPTION;
		nom_opcio_inexistent	EXCEPTION;
		opcio_codi_inexistent	EXCEPTION;
		opcio_baixa		EXCEPTION;
		opcio_de_votacio_oberta	EXCEPTION;

		BEGIN

		  parametres_entrada := nom||', '||cod_pais;

		  IF (	nom IS NULL OR
			cod_pais IS NULL ) THEN
			RAISE camp_obligatori;
		  END IF;

		  IF (	length(nom) > 15 OR
			length(cod_pais) <> 2 ) tHEN
			RAISE llargada_camp;
		  END IF;


		  -- comprovem que codi_pais referenci un pas del sistema o al codi de pas neutre 'ZQ'
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM codisISO WHERE codi_pais = cod_pais;
		  IF ( trobat = 0 ) THEN
			RAISE codi_pais_inexistent;
		  END IF;

		 
		  -- comprovem que 'nom' referenci un nom d'alguna opci de la taula 'opcions'
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM opcions WHERE nom_opcio = nom;
		  IF ( trobat = 0 ) THEN
			RAISE nom_opcio_inexistent;
		  END IF;


		  -- comprovem que nom+cod_pais referencin la clau alternativa d'opcions
		  trobat := 0;
		  FOR opcions IN ( SELECT nom_opcio FROM opcions WHERE codi_pais = cod_pais ) LOOP

			trobat := trobat;
			IF ( trobat = 0 ) THEN

				IF ( opcions.nom_opcio = nom ) THEN
					trobat := 1;
				END IF;

			END IF;

		  END LOOP;


		  IF ( trobat = 0 ) THEN
			RAISE opcio_codi_inexistent;
		  END IF;

		  -- comprovem que la opci no estigus ja de baixa
		  SELECT baixa_logica INTO baixa FROM opcions WHERE nom_opcio = nom AND codi_pais = cod_pais;
		  IF ( baixa = 1 ) THEN
			RAISE opcio_baixa;
		  END IF;


		  -- comprovem que no hi hagus cap votaci amb estat 'oberta' que tingus aquesta opci (considero
		  -- que no es pot donar de baixa una opci en ple procs de votaci)
		  SELECT id INTO id_estat_oberta FROM estatsVotacio WHERE nom_estat = 'oberta';
		  SELECT id INTO id_opc FROM opcions WHERE nom_opcio = nom AND codi_pais = cod_pais;

		  -- recorrem totes les votacions que tinguin associada la opci que es vol donar de baixa i 
		  -- comprovarem si alguna est en estat 'oberta'
		  trobat := 0;
		  FOR votacio IN ( SELECT id_votacio FROM opcionsVotacio WHERE id_opcio = id_opc) LOOP

			trobat := trobat;
			IF (trobat = 0) THEN

				SELECT id_estat INTO id_estat_vota FROM votacions WHERE id = votacio.id_votacio;
				IF ( id_estat_vota = id_estat_oberta ) THEN
					trobat := 1;
				END IF;

			END IF;			

		  END LOOP;


		  IF ( trobat = 1 ) THEN
			RAISE opcio_de_votacio_oberta;
		  END IF;


		  -- superades les verificacions anteriors, es pot donar de baixa lgica la opci
		  UPDATE opcions SET baixa_logica = 1 WHERE nom_opcio = nom AND codi_pais = cod_pais;



		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN llargada_camp THEN

		   RSP := 'ERROR: nombre de caracters inapropiat. Revisi la llargada dels camps nom_opcio i/o codi_pais.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN codi_pais_inexistent THEN

		   RSP := 'ERROR: el codi del pais no referencia cap dels codis ISO assignats als paisos, ni al codi del pais neutre ''ZQ''.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN nom_opcio_inexistent THEN

		   RSP := 'ERROR: el nom de la opcio no existeix a la taula ''opcions''';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN opcio_codi_inexistent THEN

		   RSP := 'ERROR: el nom de la opcio i el codi del pais no referencien cap opcio valida.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		WHEN opcio_baixa THEN

		   RSP := 'ERROR: quan la opcio ja havia estat donada de baixa.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN opcio_de_votacio_oberta THEN

		   RSP := 'ERROR: actualment no es pot donar de baixa la opcio ja que esta associada a una votacio en estat ''oberta'' ';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		END;













	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_baixaCensAssociat
	-- 			
	-- Descripci:		Una vegada fetes unes quantes verificacions prvies, el procediment porta a terme diverses accions:
	-- 			   a) si a la taula censosAssociats no hi ha cap altre cens associat a la votaci que sigui del mateix pas que
	-- 			      el cens id_cens_loc s'esborra un registre de la taula votacionsPaisos que tingui l'identificador del pas 
	-- 			      al qual pertany el cens id_cens_loc i l'identificador de la votaci id_vota.
	-- 			   b) Obt el n de ciutadans censats actualment en el cens associat que es vol donar de baixa, id_cens_loc. 
	-- 			      Aquest valor servir per actualitzar el nmero d'electors associats a la votaci (punt c)
	-- 			   c) Actualitza el nmero d'electors de la votaci.
	-- 			   d) Esborra tots els ciutadans de la taula ciutadansVotacions que fossin electors de la votaci  id_vota i 
	-- 			      ciutadans del cens id_cens_loc.
	-- 			   e) Dona de baixa el cens id_cens_loc a la taula censosAssociats (taula que registra els censos associats a 
	-- 			      cada votaci) per la votaci id_vota
	-- 			      amb totes les dades rebudes per parmetre a excepci del parmetre ciut_censats que no es tindr en compte
	-- 			      ja que el nmero de ciutadans censats s'obt en aquest procediment (punt b).
	-- 			   f) Crida el procediment p_registrarLog del paquet pk_utils.
	-- 
	-- 			NOTA:	El procediment noms deixa donar de baixa censos a les votacions que estan en estat d'edici 
	-- 
	-- 			
	-- Parmetres entrada:	id_vota		NUMBER				PK_composta1	REFERENCES votacions(id)
	-- 			id_cens_loc	VARCHAR2(40 CHAR)		PK_composta2	REFERENCES censosLocals(nom_cens)
	-- 
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori			-> quan un parmetre d'entrada que no pot ser buit o NULL ho s
	-- 			llargada_camp			-> quan id_cens_loc t ms de 40 carcters
	-- 			numero_incorrecte		-> quan l'id_votacio no s un natural i/o quan ciut_censats, n_vots o
	-- 							   finalitzada no siguin zero
	-- 			votacio_inexistent		-> quan id_vota no correspon a cap votaci existent
	-- 			cens_local_inexistent		-> quan id_cens_local no correspon a cap cens local existent
	-- 			cens_no_associat		-> quan el cens associat que es vol donar de baixa, realment no est associat a la votaci
	-- 			votacio_no_editable		-> quan la votaci no est en estat 'edicio' (en altres estats no s'hi pot associar 
	-- 							   cap cens)
	--			OTHERS				-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_baixaCensAssociat (
	  id_vota	IN	censosAssociats.id_votacio%TYPE,
	  id_cens_loc	IN	censosAssociats.id_cens_local%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure		VARCHAR2(30 CHAR) := 'p_baixaCensAssociat';
	  	parametres_entrada	VARCHAR2(80 CHAR) := NULL;
		esCorrecte		NUMBER(1) := 1;
		confirmada		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		num_censos_pais		NUMBER := 0;
		estat_votacio		VARCHAR2(20 CHAR) := 0;
		pais			VARCHAR2(30 CHAR);
		censats			NUMBER := 0;
		electors_votacio	NUMBER := 0;
	  	camp_obligatori			EXCEPTION;
		llargada_camp			EXCEPTION;
		numero_incorrecte		EXCEPTION;
		votacio_inexistent		EXCEPTION;
		cens_local_inexistent		EXCEPTION;
		cens_no_associat		EXCEPTION;
		votacio_no_editable		EXCEPTION;

		-- declarem un cursor que obtindr els nmeros aleatoris del ciutadans associats al cens local que es vol donar de baixa
		-- i que ser til per esborrar-los de la taula ciutadansVotacions.
		CURSOR ciutadans_cens IS SELECT num_aleatori FROM ciutadans WHERE id_cens_local = id_cens_loc;


		BEGIN

		  parametres_entrada := id_vota||', '||id_cens_loc;

		  IF (  id_vota IS NULL OR 
			id_cens_loc IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;


		  IF ( length(id_cens_loc) > 40 ) THEN
			RAISE llargada_camp;
		  END IF;


		  IF (	id_vota <= 0  ) THEN
			RAISE numero_incorrecte;
		  END IF;


		  -- comprovem que l'identificador de la votaci existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM votacions WHERE id = id_vota;
		  IF ( trobat = 0 ) THEN
			RAISE votacio_inexistent;
		  END IF;


		  -- comprovem que l'identificador de cens local existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM censosLocals WHERE nom_cens = id_cens_loc;
		  IF ( trobat = 0 ) THEN
			RAISE cens_local_inexistent;
		  END IF;


		  -- comprovem que el cens estigui associat a la votacio
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM censosAssociats WHERE id_votacio = id_vota AND id_cens_local = id_cens_loc;
		  IF ( trobat = 0 ) THEN
			RAISE cens_no_associat;
		  END IF;


		  -- comprovem que la votaci encara sigui editable (en estat 'edicio'). Si no ho s, no es podr donar de baixa el cens.
		  SELECT ev.nom_estat INTO estat_votacio 
			FROM estatsVotacio ev, votacions v
			WHERE ev.id = v.id_estat AND v.id = id_vota;
		  IF ( estat_votacio <> 'edicio' ) THEN
			RAISE votacio_no_editable;
		  END IF;


		  -- comprovem si hi ha algun altre cens del mateix pas que id_cens_loc que estigui associat a aquesta votaci. Si no 
		  -- n'hi ha cap ms, caldr esborrar el registre del pas del cens que es desassocia a la taula votacionsPaisos
		  SELECT nom_pais INTO pais FROM censosAssociats WHERE id_votacio = id_vota AND id_cens_local = id_cens_loc;
		  SELECT COUNT(*) INTO num_censos_pais FROM censosAssociats WHERE id_votacio = id_vota AND nom_pais = pais AND id_cens_local <> id_cens_loc;
		  IF ( num_censos_pais = 0 ) THEN
			DELETE FROM votacionsPaisos WHERE id_pais = pais AND id_votacio = id_vota;
		  END IF;


		  -- Obtenim el nmero de ciutadans censats que tenia el cens associat en el moment de donar-lo d'alta a la taula censosAssociats
		  SELECT ciutadans_censats INTO censats FROM censosAssociats WHERE id_votacio = id_vota AND id_cens_local = id_cens_loc;

		  -- actualizem el nmero d'electors de la votaci
		  SELECT num_electors INTO electors_votacio FROM votacions WHERE id = id_vota;
		  UPDATE votacions SET num_electors = (electors_votacio - censats) WHERE id = id_vota;

		  -- eliminiem, de la taula ciutadansVotacions, tots els electors la votaci id_vota del cens que donem de baixa
		  -- Recorrem el cursor que hem declarat (que cont l'identificador dels ciutadans del cens local que es desassociar de la votaci).
		  FOR ciutada IN ciutadans_cens LOOP

			trobat := 0;
			SELECT COUNT(*) INTO trobat FROM ciutadansVotacions WHERE id_ciutada = ciutada.num_aleatori AND id_votacio = id_vota;
			IF ( trobat = 1 ) THEN
				DELETE FROM ciutadansVotacions WHERE id_ciutada = ciutada.num_aleatori AND id_votacio = id_vota;
			END IF;
			
		  END LOOP;



		  -- Donem de baixa, a la taula censosAssociats,  el cens associat a la votaci
		  DELETE FROM censosAssociats WHERE id_votacio = id_vota AND id_cens_local = id_cens_loc;



		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN llargada_camp THEN

		   RSP := 'ERROR: l''identificador del cens es massa llarg.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN numero_incorrecte THEN

		   RSP := 'ERROR: l''identificador de la votacio no pot ser <= 0';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_inexistent THEN

		   RSP := 'ERROR: l''identificador de votacio no correspon a cap votacio';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN cens_local_inexistent THEN

		   RSP := 'ERROR: l''identificador de cens local no correspon a cap cens local';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN cens_no_associat THEN

		   RSP := 'ERROR: el cens que es vol ''desassociar'' no esta associat a la votacio.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_no_editable THEN

		   RSP := 'ERROR: la votacio no es troba en estat d''edicio. Ja no s''hi pot associar cap nou cens.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;













	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_baixaOpcioVotacio
	-- 			
	-- Descripci:		esborra un registre a la taula opcionsVotacio i crida el procediment p_registrarLog del paquet pk_utils
	-- 			
	-- Parmetres entrada:	id_vota		NUMBER			NOT NULL	REFERENCES votacions(id)
	-- 			id_opcio_v	NUMBER			NOT NULL	REFERENCES opcions(id)
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada que no pot ser buit o NULL ho s
	-- 			valor_incorrecte	-> quan id_vota i id_opcio_v no sn naturals > 0
	-- 			votacio_inexistent	-> quan id_vota no referencia cap votaci del sistema
	-- 			opcio_inexistent	-> quan id_opcio_v no referencia cap opci del sistema
	-- 			opcio_de_baixa		-> quan id_opcio_v referencia una opcio que ha estat donada de baixa
	-- 			opcio_no_associada	-> quan id_opcio_v no estava associada realment a id_vota
	-- 			votacio_no_editable	-> quan id_vota referencia una votaci a la qual ja no es poden esborrar opcions
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_baixaOpcioVotacio (
	  id_vota	IN	opcionsVotacio.id_votacio%TYPE,
	  id_opcio_v	IN	opcionsVotacio.id_opcio%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_baixaOpcioVotacio';
	  	parametres_entrada 	VARCHAR2(60 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		id_estat_vota		NUMBER := 0;
		estat_vota		VARCHAR2(20 CHAR);
	  	camp_obligatori 	EXCEPTION;
		valor_incorrecte	EXCEPTION;
		votacio_inexistent	EXCEPTION;
		opcio_inexistent	EXCEPTION;
		opcio_no_associada	EXCEPTION;
		votacio_no_editable	EXCEPTION;



		BEGIN

		  parametres_entrada := id_vota||', '||id_opcio_v;

		  IF (  id_vota IS NULL OR
			id_opcio_v IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (	id_vota <= 0 OR
			id_opcio_v <= 0 ) THEN
			RAISE valor_incorrecte;
		  END IF;

		  -- comprovem que la votaci existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM votacions WHERE id = id_vota;
		  IF ( trobat = 0 ) THEN
			RAISE votacio_inexistent;
		  END IF;


		  -- comprovem que la opci existeixi
		  trobat := 0;
		  SELECT COUNT (*) INTO trobat FROM opcions WHERE id = id_opcio_v;
		  IF ( trobat = 0 ) THEN
			RAISE opcio_inexistent;
		  END IF;

		  -- comprovem que la opci de vot realment estigui associada a la votaci
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM opcionsVotacio WHERE id_votacio = id_vota AND id_opcio = id_opcio_v;
		  IF ( trobat = 0 ) THEN
			RAISE opcio_no_associada;
		  END IF;


		  -- comprovem que la votaci estigui en estat 'edicio'
		  SELECT id_estat INTO id_estat_vota FROM votacions WHERE id = id_vota;
		  SELECT nom_estat INTO estat_vota FROM estatsVotacio	WHERE id = id_estat_vota;
		  IF ( estat_vota <> 'edicio' ) THEN
			RAISE votacio_no_editable;
		  END IF;


		  -- si tot s correcte donem de baixa la opci de la votaci
		  DELETE FROM opcionsVotacio WHERE id_votacio = id_vota AND id_opcio = id_opcio_v;

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN valor_incorrecte THEN

		   RSP := 'ERROR: els camps id_opcio i id_votacio han de ser nombres naturals diferents a zero';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_inexistent THEN

		   RSP := 'ERROR: no es pot afegir una opcio de vot a una votacio que no existeix';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN opcio_inexistent THEN

		   RSP := 'ERROR: la opcio de vot no existeix al sistema.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN opcio_no_associada THEN

		   RSP := 'ERROR: la opcio de vot no esta associada a la votacio, per tant no es pot ''desassociar''.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_no_editable THEN

		   RSP := 'ERROR: no es pot ''desassociar'' cap opcio a una votacio que no estigui en estat d''edicio.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;















	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_modificaOpcio_descripcio
	-- 			
	-- Descripci:		canvia la descripci d'una opci i crida el procediment p_registrarLog del paquet pk_utils
	-- 			
	-- Parmetres entrada:	nom		VARCHAR2(15 CHAR)
	-- 			cod_pais	CHAR(2)			NOT NULL	REFERENCES codisISO(codi_pais)
	-- 			nova_desc	VARCHAR2(100 CHAR)
	-- 			NOTA: (nom, cod_pais) s clau alternativa de 'opcions'
	-- 
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada que no pot ser buit o NULL ho s
	-- 			llargada_camp		-> quan el parmetre nom t ms de 15 i/o quan el parmetre codi_pais no t 
	-- 						   exactament 2 carcters
	-- 			codi_pais_inexistent	-> quan cod_pais no referencia el codi ISO de cap pais (incls el pas neuter 'ZQ')
	-- 			nom_opcio_inexistent	-> quan nom no referencia el nom de cap opci del sistema
	-- 			opcio_codi_inexistent	-> quan la combinaci nom+cod_pais no referencien la clau alternativa de 'opcions'
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_modificaOpcio_descripcio (
	  nom		IN	opcions.nom_opcio%TYPE,
	  cod_pais	IN	opcions.codi_pais%TYPE,
	  nova_desc	IN	opcions.descripcio%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_modificaOpcio_descripcio';
	  	parametres_entrada 	VARCHAR2(140 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER := 0;
		id_opc			NUMBER := 0;
		id_estat_oberta		NUMBER := 0;
		id_estat_vota		NUMBER := 0;
	  	camp_obligatori 	EXCEPTION;
		llargada_camp		EXCEPTION;
		codi_pais_inexistent	EXCEPTION;
		nom_opcio_inexistent	EXCEPTION;
		opcio_codi_inexistent	EXCEPTION;


		BEGIN

		  parametres_entrada := nom||', '||cod_pais||', '||nova_desc;

		  IF (	nom IS NULL OR
			cod_pais IS NULL OR
			nova_desc IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (	length(nom) > 15 OR
			length(cod_pais) <> 2 OR
			length(nova_desc) > 100) tHEN
				RAISE llargada_camp;
		  END IF;


		  -- comprovem que codi_pais referenci un pas del sistema o al codi de pas neutre 'ZQ'
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM codisISO WHERE codi_pais = cod_pais;
		  IF ( trobat = 0 ) THEN
			RAISE codi_pais_inexistent;
		  END IF;

		 
		  -- comprovem que 'nom' referenci un nom d'alguna opci de la taula 'opcions'
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM opcions WHERE nom_opcio = nom;
		  IF ( trobat = 0 ) THEN
			RAISE nom_opcio_inexistent;
		  END IF;


		  -- comprovem que nom+cod_pais referencin la clau alternativa d'opcions
		  trobat := 0;
		  FOR opcions IN ( SELECT nom_opcio FROM opcions WHERE codi_pais = cod_pais ) LOOP

			trobat := trobat;
			IF ( trobat = 0 ) THEN

				IF ( opcions.nom_opcio = nom ) THEN
					trobat := 1;
				END IF;

			END IF;

		  END LOOP;


		  IF ( trobat = 0 ) THEN
			RAISE opcio_codi_inexistent;
		  END IF;




		  -- superades les verificacions anteriors, es pot canviar la decripcio de la opcio
		  UPDATE opcions SET descripcio = nova_desc WHERE nom_opcio = nom AND codi_pais = cod_pais;



		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN llargada_camp THEN

		   RSP := 'ERROR: nombre de caracters inapropiat. Revisi la llargada dels camps nom_opcio i/o codi_pais.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN codi_pais_inexistent THEN

		   RSP := 'ERROR: el codi del pais no referencia cap dels codis ISO assignats als paisos, ni al codi del pais neutre ''ZQ''.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN nom_opcio_inexistent THEN

		   RSP := 'ERROR: el nom de la opcio no existeix a la taula ''opcions''';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN opcio_codi_inexistent THEN

		   RSP := 'ERROR: el nom de la opcio i el codi del pais no referencien cap opcio valida.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		END;







	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_modificaVotacio_titol
	-- 			
	-- Descripci:		modifica el titol d'una votaci a dues taules (votacions i estadistiquesVotacions) i crida al procediment
	-- 			p_registrarLog de paquet pk_utils. No es permetr modificar el ttol de votacions que s'estiguin celebrant
	-- 			o que ja hagin finalitzat.
	-- 
	-- Parmetres entrada:	titol_vota_vell		VARCHAR2(200 CHAR) 	titol_vota_vell s clau alternativa de votacions
	-- 			titol_vota_nou		VARCHAR2(200 CHAR) 	titol_vota_vell ha de ser clau alternativa de votacions
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan titol_vota_vell i/o titol_vota_nou sn buits o NULL
	-- 			llargada_incorrecte	-> quan el ttol de la votaci (vell i/o nou) s massa llarg
	-- 			votacio_inexistent	-> quan titol_vota_vell no referencia cap votacio
	-- 			estat_vota_no_permet	-> quan s'intenta esborrar una votaci que est 'oberta' o 'finaltzada'
	-- 			titol_nou_invalid	-> quan titol_vota_nou s igual a un ttol ja existent a la taula votacions (ttol s UNIQUE)
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_modificaVotacio_titol (
	  titol_vota_vell	IN	votacions.titol%TYPE,
	  titol_vota_nou	IN	votacions.titol%TYPE,
	  RSP			OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_modificaVotacio_titol';
	  	parametres_entrada 	VARCHAR2(500 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		id_estat_vota		NUMBER := 0;
		estat_vota		VARCHAR2(20 CHAR);
	  	camp_obligatori 	EXCEPTION;
		llargada_incorrecte 	EXCEPTION;
		votacio_inexistent	EXCEPTION;
		estat_vota_no_permet	EXCEPTION;
		titol_nou_invalid	EXCEPTION;


		BEGIN

		  parametres_entrada := titol_vota_vell||', '||titol_vota_nou;


		  IF (  titol_vota_vell IS NULL OR
			titol_vota_nou IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF ( length(titol_vota_vell) > 200 OR
			length(titol_vota_nou) > 200 ) THEN
				RAISE llargada_incorrecte;
		  END IF;

		  -- comprovem que la votacio de la qual es vol modificar el titol existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM votacions WHERE titol = titol_vota_vell;
		  IF (trobat = 0) THEN
			RAISE votacio_inexistent;
		  END IF;

		  -- comprovem que l'estat de la votaci permeti modificar-ne el titol (no es permet si est en estat 'oberta' o 'finalitzada')
		  SELECT id_estat INTO id_estat_vota FROM votacions WHERE titol = titol_vota_vell;
		  SELECT nom_estat INTO estat_vota FROM estatsVotacio WHERE id = id_estat_vota;

		  IF ( estat_vota IN ('oberta', 'finalitzada') ) THEN
			RAISE estat_vota_no_permet;
		  END IF;
		  

		  -- comprovem que no hi hagi cap altra votaci que tingui el titol nou (el ttol d'una votaci s clau alternativa)
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM votacions WHERE titol = titol_vota_nou;
		  IF (trobat = 1) THEN
			RAISE titol_nou_invalid;
		  END IF;


		  -- superades les verificacions prvies es pot modificar el titol de la votaci titol_vota_vell a la taula votacions i 
		  -- a la taula estadistiquesVotacions.
		  UPDATE votacions SET titol = titol_vota_nou WHERE titol = titol_vota_vell;
		  UPDATE estadistiquesVotacions SET titol_votacio = titol_vota_nou WHERE titol_votacio = titol_vota_vell;

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: el titol (vell i/o nou) de la votacio es buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN llargada_incorrecte THEN

		   RSP := 'ERROR: el titol (vell i/o nou) de la votacio no pot tenir mes de 200 caracters.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_inexistent THEN

		   RSP := 'ERROR: no es pot canviar el titol d''una votacio que no existeix.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN estat_vota_no_permet THEN

		   RSP := 'ERROR: no es canviar el titol d''una votacio que s''esta celebrant o que ja ha finalitzat.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN titol_nou_invalid THEN

		   RSP := 'ERROR: hi ha una altra votacio que te el nou titol i un titol d''una votacio ha de ser unic.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;






	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_modificaVotacio_dataInici
	-- 			
	-- Descripci:		modifica la data d'inici d'una votaci i crida al procediment p_registrarLog de paquet pk_utils.
	-- 			No es permetr modificar la data de votacions que s'estiguin celebrant o que ja hagin finalitzat.
	-- 			No es permetr que data_inici < data_actual ni que data_inici > data_fi.
	-- 
	-- Parmetres entrada:	titol_vota		VARCHAR2(200 CHAR) 	titol_vota s clau alternativa de votacions
	-- 			data_in			DATE
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan titol_vota s buit o NULL
	-- 			llargada_incorrecte	-> quan el ttol de la votaci s massa llarg
	-- 			votacio_inexistent	-> quan titol_vota no referencia cap votacio
	-- 			estat_vota_no_permet	-> quan s'intenta esborrar una votaci que est 'oberta' o 'finaltzada'
	-- 			inici_menor_actual	-> quan data_in s menor que la data actual (en aquest cas ja s'hauria d'estar celebreant
	-- 						   la votaci o ja hauria d'haver acabat)
	-- 			inici_major_fi		-> quan data_in s major que la data de fi de la votaci amb ttol titol_vota
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_modificaVotacio_dataInici (
	  titol_vota	IN	votacions.titol%TYPE,
	  data_in	IN	votacions.data_inici%TYPE,
	  RSP			OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_modificaVotacio_dataInici';
	  	parametres_entrada 	VARCHAR2(300 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		id_estat_vota		NUMBER := 0;
		estat_vota		VARCHAR2(20 CHAR);
		data_fi_vota		DATE;
		data_actual		DATE;
	  	camp_obligatori 	EXCEPTION;
		llargada_incorrecte 	EXCEPTION;
		votacio_inexistent	EXCEPTION;
		estat_vota_no_permet	EXCEPTION;
		inici_menor_actual	EXCEPTION;
		inici_major_fi		EXCEPTION;



		BEGIN

		  parametres_entrada := titol_vota||', '||data_in;


		  IF (  titol_vota IS NULL OR
			data_in IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF ( length(titol_vota) > 200 ) THEN
				RAISE llargada_incorrecte;
		  END IF;

		  -- comprovem que la votacio (de la qual es vol modificar la data d'inici) existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM votacions WHERE titol = titol_vota;
		  IF (trobat = 0) THEN
			RAISE votacio_inexistent;
		  END IF;

		  -- comprovem que l'estat de la votaci permeti modificar-ne la data d'inici (no es permet si est en estat 'oberta' o
		  -- 'finalitzada')
		  SELECT id_estat INTO id_estat_vota FROM votacions WHERE titol = titol_vota;
		  SELECT nom_estat INTO estat_vota FROM estatsVotacio WHERE id = id_estat_vota;

		  IF ( estat_vota IN ('oberta', 'finalitzada') ) THEN
			RAISE estat_vota_no_permet;
		  END IF;
		  

		  -- comprovem que la nova data d'inici sigui major que la data actual i menor o igual que la data de fi de la votaci
		  SELECT data_fi INTO data_fi_vota FROM votacions WHERE titol = titol_vota;
		  SELECT SYSDATE INTO data_actual FROM DUAL;
		  IF ( data_in < data_actual ) THEN
			RAISE inici_menor_actual;
		  END IF;

		  IF ( data_in > data_fi_vota ) THEN
			RAISE inici_major_fi;
		  END IF;


		  -- superades les verificacions prvies, es pot modificar la data d'inici de la votaci de ttol titol_vota
		  UPDATE votacions SET data_inici = data_in WHERE titol = titol_vota;


		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: el titol de la votacio i/o la data d''inici son buits o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN llargada_incorrecte THEN

		   RSP := 'ERROR: el titol de la votacio no pot tenir mes de 200 caracters.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_inexistent THEN

		   RSP := 'ERROR: no es pot canviar la data d''inici d''una votacio que no existeix.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN estat_vota_no_permet THEN

		   RSP := 'ERROR: no es canviar la data d''inici d''una votacio que s''esta celebrant o que ja ha finalitzat.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN inici_menor_actual THEN

		   RSP := 'ERROR: la nova data d''inici d''una votacio no pot ser menor que la data actual.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN inici_major_fi THEN

		   RSP := 'ERROR: la nova data d''inici d''una votacio no pot ser major que la data de finalitzacio.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;








	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_modificaVotacio_dataFi
	-- 			
	-- Descripci:		modifica la data de finalitzaci d'una votaci i crida al procediment p_registrarLog de paquet pk_utils.
	-- 			No es permetr modificar la data de votacions que s'estiguin celebrant o que ja hagin finalitzat.
	-- 			No es permetr que data_fi_vota <= data_actual ni que data_fi_vota < data_inici.
	-- 			
	-- 
	-- Parmetres entrada:	titol_vota		VARCHAR2(200 CHAR) 	titol_vota s clau alternativa de votacions
	-- 			data_fi_vota		DATE
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan titol_vota s buit o NULL
	-- 			llargada_incorrecte	-> quan ttol_vota s massa llarg (ms de 200 carcters)
	-- 			votacio_inexistent	-> quan titol_vota no referencia cap votacio
	-- 			estat_vota_no_permet	-> quan s'intenta esborrar una votaci que est 'oberta' o 'finaltzada'
	-- 			fi_menor_actual		-> quan data_fi_vota s menor o igual que la data actual (la votacio ja hauria d'haver 
	-- 						-> acabat o s'estaria celebrant)
	-- 			fi_menor_inici		-> quan data_fi_vota s menor que la data d'inici de la votaci amb ttol titol_vota
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_modificaVotacio_dataFi (
	  titol_vota	IN	votacions.titol%TYPE,
	  data_fi_vota	IN	votacions.data_fi%TYPE,
	  RSP			OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_modificaVotacio_dataFi';
	  	parametres_entrada 	VARCHAR2(300 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		id_estat_vota		NUMBER := 0;
		estat_vota		VARCHAR2(20 CHAR);
		data_inici_vota		DATE;
		data_actual		DATE;
	  	camp_obligatori 	EXCEPTION;
		llargada_incorrecte 	EXCEPTION;
		votacio_inexistent	EXCEPTION;
		estat_vota_no_permet	EXCEPTION;
		fi_menor_actual		EXCEPTION;
		fi_menor_inici		EXCEPTION;



		BEGIN

		  parametres_entrada := titol_vota||', '||data_fi_vota;


		  IF (  titol_vota IS NULL OR
			data_fi_vota IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF ( length(titol_vota) > 200 ) THEN
				RAISE llargada_incorrecte;
		  END IF;

		  -- comprovem que la votacio (de la qual es vol modificar la data d'inici) existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM votacions WHERE titol = titol_vota;
		  IF (trobat = 0) THEN
			RAISE votacio_inexistent;
		  END IF;

		  -- comprovem que l'estat de la votaci permeti modificar-ne la data d'inici (no es permet si est en estat 'oberta' o
		  -- 'finalitzada')
		  SELECT id_estat INTO id_estat_vota FROM votacions WHERE titol = titol_vota;
		  SELECT nom_estat INTO estat_vota FROM estatsVotacio WHERE id = id_estat_vota;

		  IF ( estat_vota IN ('oberta', 'finalitzada') ) THEN
			RAISE estat_vota_no_permet;
		  END IF;
		  

		  -- comprovem que la nova data de fi sigui major que la data actual i major o igual que la data de fi de la votaci
		  SELECT data_inici INTO data_inici_vota FROM votacions WHERE titol = titol_vota;
		  SELECT SYSDATE INTO data_actual FROM DUAL;
		  IF ( data_fi_vota <= data_actual ) THEN
			RAISE fi_menor_actual;
		  END IF;

		  IF ( data_fi_vota < data_inici_vota ) THEN
			RAISE fi_menor_inici;
		  END IF;


		  -- superades les verificacions prvies, es pot modificar la data d'inici de la votaci de ttol titol_vota
		  UPDATE votacions SET data_fi = data_fi_vota WHERE titol = titol_vota;


		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: el titol de la votacio i/o la data de fi son buits o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN llargada_incorrecte THEN

		   RSP := 'ERROR: el titol de la votacio no pot tenir mes de 200 caracters.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_inexistent THEN

		   RSP := 'ERROR: no es pot canviar la data de fi d''una votacio que no existeix.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN estat_vota_no_permet THEN

		   RSP := 'ERROR: no es canviar la data de fi d''una votacio que s''esta celebrant o que ja ha finalitzat.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN fi_menor_actual THEN

		   RSP := 'ERROR: la nova data de fi d''una votacio no pot ser menor o igual que la data actual (ja s''hauria celebrat o s''estaria celebrant).';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN fi_menor_inici THEN

		   RSP := 'ERROR: la nova data de fi d''una votacio no pot ser menor que la data d''inici.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;






	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_modificaVotacio_web
	-- 			
	-- Descripci:		modifica la data de finalitzaci d'una votaci i crida al procediment p_registrarLog de paquet pk_utils.
	-- 			Es permetr modificar la web d'una votaci en qualsevol moment
	-- 
	-- Parmetres entrada:	titol_vota		VARCHAR2(200 CHAR) 	titol_vota s clau alternativa de votacions
	-- 			nova_web		VARCHAR2(200 CHAR)
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan titol_vota s buit o NULL
	-- 			llargada_incorrecte	-> quan titol_vota i/o l'URL de la nova web sn massa llargs (ms de 200 carcters)
	-- 			votacio_inexistent	-> quan titol_vota no referencia cap votacio
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_modificaVotacio_web (
	  titol_vota	IN	votacions.titol%TYPE,
	  nova_web	IN	votacions.adreca_web%TYPE,
	  RSP			OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_modificaVotacio_web';
	  	parametres_entrada 	VARCHAR2(500 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
	  	camp_obligatori 	EXCEPTION;
		llargada_incorrecte 	EXCEPTION;
		votacio_inexistent	EXCEPTION;



		BEGIN

		  parametres_entrada := titol_vota||', '||nova_web;


		  IF (  titol_vota IS NULL OR
			nova_web IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF ( 	length(titol_vota) > 200 OR
			length(nova_web) > 200 ) THEN
				RAISE llargada_incorrecte;
		  END IF;

		  -- comprovem que la votacio (de la qual es vol modificar la web) existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM votacions WHERE titol = titol_vota;
		  IF (trobat = 0) THEN
			RAISE votacio_inexistent;
		  END IF;


		  -- modifiquem l'URL de la votaci
		  UPDATE votacions SET adreca_web = nova_web WHERE titol = titol_vota;


		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: el titol de la votacio i/o la URL de la nova web son buits o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN llargada_incorrecte THEN

		   RSP := 'ERROR: el titol de la votacio i/o la URL de la nova web no poden tenir mes de 200 caracters.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_inexistent THEN

		   RSP := 'ERROR: no es pot canviar la web d''una votacio que no existeix.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;









	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_modificaVotacio_president
	-- 			
	-- Descripci:		modifica la data de finalitzaci d'una votaci i crida al procediment p_registrarLog de paquet pk_utils.
	-- 			Es permetr modificar el nom del president de la mesa d'una votaci en tot moment sempre que la votaci no
	-- 			estigui en estat 'finalitzada'
	-- 
	-- Parmetres entrada:	titol_vota		VARCHAR2(200 CHAR) 	titol_vota s clau alternativa de votacions
	-- 			nou_president		VARCHAR2(100 CHAR)
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan titol_vota s buit o NULL
	-- 			llargada_incorrecte	-> quan titol_vota i/o nou_president sn massa llargs (ms de 200 i 100 carcters respectivament)
	-- 			votacio_inexistent	-> quan titol_vota no referencia cap votacio
	-- 			estat_vota_no_permet	-> quan s'intenta canviar el nom del president d'una votaci finalitzada
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_modificaVotacio_president (
	  titol_vota	IN	votacions.titol%TYPE,
	  nou_president	IN	votacions.president_mesa%TYPE,
	  RSP			OUT	VARCHAR2
	  ) IS

		nom_procedure 		VARCHAR2(30 CHAR) := 'p_modificaVotacio_president';
	  	parametres_entrada 	VARCHAR2(350 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		id_estat_vota		NUMBER := 0;
		estat_vota		VARCHAR2(20 CHAR);
	  	camp_obligatori 	EXCEPTION;
		llargada_incorrecte 	EXCEPTION;
		votacio_inexistent	EXCEPTION;
		estat_vota_no_permet	EXCEPTION;



		BEGIN

		  parametres_entrada := titol_vota||', '||nou_president;


		  IF (  titol_vota IS NULL OR
			nou_president IS NULL) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF ( 	length(titol_vota) > 200 OR
			length(nou_president) > 100 ) THEN
				RAISE llargada_incorrecte;
		  END IF;

		  -- comprovem que la votacio (de la qual es vol modificar el nou president de mesa) existeixi
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM votacions WHERE titol = titol_vota;
		  IF (trobat = 0) THEN
			RAISE votacio_inexistent;
		  END IF;


		  -- comprovem que l'estat de la votaci permeti modificar-ne el president (no es permet si est en estat 'finalitzada')
		  SELECT id_estat INTO id_estat_vota FROM votacions WHERE titol = titol_vota;
		  SELECT nom_estat INTO estat_vota FROM estatsVotacio WHERE id = id_estat_vota;

		  IF ( estat_vota = 'finalitzada' ) THEN
			RAISE estat_vota_no_permet;
		  END IF;


		  -- modifiquem el nom del president de la mesa
		  UPDATE votacions SET president_mesa = nou_president WHERE titol = titol_vota;


		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: el titol de la votacio i/o el nom del nou president son buits o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN llargada_incorrecte THEN

		   RSP := 'ERROR: el titol de la votacio i/o el del nom del president no poden tenir mes de 200 i 100 caracters respectivament.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN votacio_inexistent THEN

		   RSP := 'ERROR: no es pot canviar el nom de president de mesa d''una votacio que no existeix.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN estat_vota_no_permet THEN

		   RSP := 'ERROR: no es canviar el president de mesa d''una votacio que ja ha finalitzat.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;








END; --fi implementaci paquet pk_votacions
/














-- ###################################################################   pk_consultes    ########################################################################







-- ******************************************************************************************************************************
-- ******************************************************************************************************************************
-- *																*
-- *						FASE D'ESPECIFICACI								*
-- *																*
-- ******************************************************************************************************************************
-- ******************************************************************************************************************************


-- ----------------------------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------------
-- Nom_paquet: 		pk_consultes
-- 
-- Descripci:		especificaci dels procediment que donen resposta a les consultes del requisit R6
-- 
-- Procediments:	p_consulta_votacionsPais, p_consulta_votacionsMesDif, p_consulta_votacionsMenysDif,
-- 			p_consulta_censosCiutada, p_consulta_votacionsCiutada
-- 
-- Funcions:		--
-- ----------------------------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE pk_consultes IS



	PROCEDURE p_consulta_votacionsPais (
	  pais		IN	paisos.nom_pais%TYPE,
	  RSP		OUT	VARCHAR2
	  );



	PROCEDURE p_consulta_votacionsMesDif (
	  RSP		OUT	VARCHAR2
	  );


	PROCEDURE p_consulta_votacionsMenysDif (
	  RSP		OUT	VARCHAR2
	  );


	PROCEDURE p_consulta_censosCiutada (
	  ciutada	IN	ciutadans.num_aleatori%TYPE,
	  RSP		OUT	VARCHAR2
	  );


	PROCEDURE p_consulta_votacionsCiutada (
	  ciutada	IN	ciutadans.num_aleatori%TYPE,
	  RSP		OUT	VARCHAR2
	  );


END; --fi especificaci paquet pk_consultes
/












-- ******************************************************************************************************************************
-- ******************************************************************************************************************************
-- *																*
-- *						FASE D'IMPLEMENTACI								*
-- *																*
-- ******************************************************************************************************************************
-- ******************************************************************************************************************************




-- ----------------------------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------------
-- Nom_paquet: 		pk_consultes
-- 
-- Descripci:		implementaci dels procediment que donen resposta a les consultes del requisit R6
-- 
-- Procediments:	p_consulta_votacionsPais, p_consulta_votacionsMesDif, p_consulta_votacionsMenysDif,
-- 			p_consulta_censosCiutada, p_consulta_votacionsCiutada
-- 
-- Funcions:		--
-- ----------------------------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE BODY pk_consultes IS






	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_consulta_votacionsPais
	-- 			
	-- Descripci:		procediment que mostra dades, pel canal de sortida, de les votacions a les que ha participat un pais rebut per 
	-- 			parmetre ordenat ascendentment per la data d'inici de les votacions. 
	-- 			En concret, mostra el ttol de la votaci, les dates d'inici i de fi de la mateixa, el nombre d'electors,
	-- 			l'estat de la votaci i, si la votaci ha finalitzat, el percentatge de participaci total, la opci (o opcions)
	-- 			guanyadora i el percentatge de vots d'aquesta/es.
	-- 			
	-- Parmetres entrada:	pais	VARCHAR2(30 CHAR)
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan un parmetre d'entrada que no pot ser buit o NULL ho s
	-- 			llargada_incorrecte	-> quan el parmetre pais t ms de 30 carcters
	-- 			pais_inexistent		-> quan el pas no existeix al sistema
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_consulta_votacionsPais (
	  pais		IN	paisos.nom_pais%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS



		nom_procedure 		VARCHAR2(30 CHAR) := 'p_consulta_votacionsPais';
	  	parametres_entrada 	VARCHAR2(30 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		num			NUMBER := 0;

		estat_vota		VARCHAR2(25 CHAR);
		opcio_guanyadora	VARCHAR2(25 CHAR);
		perc_part_total		VARCHAR2(5 CHAR);
		perc_opc_guanyadora	VARCHAR2(5 CHAR);
		titol			VARCHAR2(100 CHAR);
		electors		VARCHAR2(12 CHAR);
		particip		VARCHAR2(6 CHAR);
		estat			VARCHAR2(16 CHAR);
			
	  	camp_obligatori 	EXCEPTION;
		camp_incorrecte 	EXCEPTION;
		llargada_incorrecte	EXCEPTION;
		pais_inexistent		EXCEPTION;


		-- el cursor c_votacions t les dades sollicitades.

		CURSOR c_votacions IS SELECT vp.id_votacio, v.titol, v.data_inici, v.data_fi, v.num_electors, v.id_estat FROM votacionsPaisos vp, votacions v 
			WHERE vp.id_pais = pais AND vp.id_votacio = v.id ORDER BY v.data_inici;

		BEGIN

		  parametres_entrada := pais;

		  IF (  pais IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (	length(pais) > 30 ) THEN
			RAISE llargada_incorrecte;
		  END IF;

		  -- comprovem que el pas existeixi a la BD
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM paisos WHERE nom_pais = pais;
		  IF ( trobat = 0 ) THEN
			RAISE pais_inexistent;
		  END IF;

		  dbms_output.put_line(CHR(9)||CHR(9)||CHR(10)||CHR(13)||CHR(10)||CHR(13)||CHR(9)||CHR(9)||CHR(9)||'CONSULTA DE LES VOTACIONS QUE S''HAN PRODUIT A '||pais);
		  dbms_output.put_line(CHR(9)||CHR(9)||'---------------------------------------------------------------------------');


		  DBMS_OUTPUT.PUT_LINE(CHR(10)||CHR(13)||CHR(10)||CHR(13)||'TITOL DE LA VOTACIO'||CHR(9)||CHR(9)||CHR(9)||CHR(9)||CHR(9)||CHR(9)||'DATA INICI'||CHR(9)||'DATA FI'||CHR(9)||CHR(9)||'ELECTORS'||CHR(9)||'ESTAT'||CHR(9)||CHR(9)||CHR(9)||'PARTICIPACIO(%)'||' OPCIO GUANY.'||CHR(9)||'VOTS (%)');
		  DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------');



		  -- coprovem si el pais ha participat a alguna votaci
		  num := 0;
		  SELECT COUNT(*) INTO num FROM votacionsPaisos WHERE id_pais = pais;
		  -- si no ha participat, mostrem un missatge descriptiu
		  IF ( num = 0 ) THEN

			DBMS_OUTPUT.PUT_LINE(CHR(10)||CHR(13)||CHR(10)||CHR(13)||'SENSE RESULTATS. '||pais||' no ha participat a cap votacio');

		  -- Si el pas ha participat a alguna votaci, formategem els resultats i finalment, els mostrem
		  ELSIF ( num > 0 ) THEN

			-- recorrem el cursor que hem declarat abans
			FOR votacio IN c_votacions LOOP

				SELECT nom_estat INTO estat_vota FROM estatsVotacio WHERE id = votacio.id_estat ;
				IF ( estat_vota <> 'finalitzada' ) THEN

					opcio_guanyadora := '-';
					perc_part_total	:= '-';
 					perc_opc_guanyadora := '-';

				ELSE

					SELECT participacio, nom_opcio_mes_vots, percentatge_opcio_mes_vots 
						INTO perc_part_total, opcio_guanyadora, perc_opc_guanyadora FROM estadistiquesVotacions 
						WHERE id_votacio = votacio.id_votacio;

				END IF;

				-- alguns camps que sn numrics els transformarem a format carcter.
				titol := votacio.titol;
				electors := TO_CHAR(votacio.num_electors);
				particip := TO_CHAR(perc_part_total);

				-- formategem les dades de la fila actual (per tal de que totes les dades d'una mateixa columna tinguin una 
				-- llargada equivalent afegim espais en blanc al darrera fins arribar a una determinada longitud)

				WHILE (length(titol) < 60 ) LOOP

					titol := titol||' ';

				END LOOP;


				WHILE (length(electors) < 12 ) LOOP

					electors := electors||' ';

				END LOOP;

				WHILE (length(particip) < 6 ) LOOP

					particip := particip||' ';

				END LOOP;


				WHILE (length(opcio_guanyadora) < 18 ) LOOP

					opcio_guanyadora := opcio_guanyadora||' ';

				END LOOP;


				WHILE (length(estat_vota) < 16 ) LOOP

					estat_vota := estat_vota||' ';

				END LOOP;				

				DBMS_OUTPUT.PUT_LINE(titol||CHR(9)||votacio.data_inici||CHR(9)||votacio.data_fi||CHR(9)||electors||CHR(9)||estat_vota||CHR(9)||perc_part_total||CHR(9)||CHR(9)||opcio_guanyadora||perc_opc_guanyadora||CHR(10)||CHR(13));



			END LOOP;



		  END IF;

		  

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: Hi ha algun camp buit o amb valor NULL.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN llargada_incorrecte THEN

		   RSP := 'ERROR: el pais no pot tenir mes de 30 caracters.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN pais_inexistent THEN

		   RSP := 'ERROR: el pais no existeix a la BD. Revisi l''escriptura.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;














	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_consulta_votacionsMesDif
	-- 			
	-- Descripci:		procediment que mostra dades pel canal de sortida de les 10 votacions on hi ha hagut ms diferncia percentual
	-- 			entre la opci ms votada i la menys votada. En concret, mostra el ttol de la votaci, les dates d'inici i de fi
	-- 			de la mateixa, la opci (o opcions) amb major percentatge de vots i aquest percentatge, la opci (o opcions) amb 
	-- 			menor percentatge i aquest percentatge i, finalment, la diferncia percentual entre la opci ms votada i la menys 
	-- 			votada. Finalment crida al procediment p_registrarLog del paquet pk_utils.
	-- 			
	-- Parmetres entrada:	--
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument al procediment p_registrarLog)
	-- 			
	-- Excepcions:		OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_consulta_votacionsMesDif (
	  RSP		OUT	VARCHAR2
	  ) IS



		nom_procedure 		VARCHAR2(30 CHAR) := 'p_consulta_votacionsMesDif';
	  	parametres_entrada 	VARCHAR2(30 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;

		opcio_guanyadora	VARCHAR2(35 CHAR);
		opcio_perdedora		VARCHAR2(35 CHAR);
		perc_opc_guanyadora	VARCHAR2(6 CHAR);
		perc_opc_perdedora	VARCHAR2(6 CHAR);
		diferencia_perc		VARCHAR2(6 CHAR);
		titol			VARCHAR2(100 CHAR);
		compensar		NUMBER := 0;
			

		CURSOR c_votacions IS SELECT 	v.id, v.titol, v.data_inici, v.data_fi, ev.nom_opcio_mes_vots, ev.percentatge_opcio_mes_vots, 
						ev.nom_opcio_menys_vots, ev.percentatge_opcio_menys_vots, ev.dif_perc_opc_mes_i_menys_vots 
						FROM votacions v, estadistiquesVotacions ev 
						WHERE v.id = ev.id_votacio AND finalitzada = 1 ORDER BY ev.dif_perc_opc_mes_i_menys_vots DESC;

		-- 'votacio' ser del mateix tipus que un registre (fila) del cursor
		votacio c_votacions%ROWTYPE;

		BEGIN

		  parametres_entrada := ' ';

		  -- si el cursor no s obert, l'obrim
		  IF NOT c_votacions%ISOPEN THEN
			OPEN c_votacions;
		  END IF;

		  dbms_output.put_line(CHR(9)||CHR(9)||CHR(10)||CHR(13)||CHR(10)||CHR(13)||CHR(10)||CHR(13)||CHR(10)||CHR(13)||CHR(9)||CHR(9)||CHR(9)||'LLISTAT DE LES 10 VOTACIONS AMB MES DIFERENCIA PERCENTUAL ENTRE LA OPCIO GUANYADORA I LA PERDEDORA');
		  dbms_output.put_line(CHR(9)||CHR(9)||'----------------------------------------------------------------------------------------------------------------------');


		  DBMS_OUTPUT.PUT_LINE(CHR(10)||CHR(13)||CHR(10)||CHR(13)||'TITOL DE LA VOTACIO'||CHR(9)||CHR(9)||CHR(9)||CHR(9)||CHR(9)||CHR(9)||'DATA INICI'||CHR(9)||'DATA FI'||CHR(9)||CHR(9)||'OPCIO GUANYADORA'||CHR(9)||'VOTS(%)'||CHR(9)||'OPCIO PERDEDORA'||CHR(9)||CHR(9)||'VOTS(%)'||CHR(9)||'DIFERENCIA(%)');
		  DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------');


		  -- recorrem 10 files del cursor (si hi sn)
		  LOOP

			FETCH c_votacions INTO votacio;

			titol := votacio.titol;
			opcio_guanyadora := votacio.nom_opcio_mes_vots;
			opcio_perdedora := votacio.nom_opcio_menys_vots;
			perc_opc_guanyadora := TO_CHAR(votacio.percentatge_opcio_mes_vots);
			perc_opc_perdedora := TO_CHAR(votacio.percentatge_opcio_menys_vots);
			diferencia_perc := TO_CHAR(votacio.dif_perc_opc_mes_i_menys_vots);


			-- formategem les dades de la fila actual (per tal de que totes les dades d'una mateixa columna tinguin una 
			-- llargada equivalent afegim espais en blanc al darrera fins arribar a una determinada longitud)

			WHILE (length(titol) < 60 ) LOOP

				titol := titol||' ';

			END LOOP;

			compensar := 0;
			IF ( length(opcio_guanyadora) <= 4 ) THEN
				compensar := 1;
			END IF;

			WHILE (length(opcio_guanyadora) < 20 ) LOOP

				opcio_guanyadora := opcio_guanyadora||' ';

			END LOOP;


			IF ( compensar = 1 ) THEN
				opcio_guanyadora := opcio_guanyadora||'        ';
			END IF;


			WHILE (length(opcio_perdedora) < 20 ) LOOP

				opcio_perdedora := opcio_perdedora||' ';

			END LOOP;


			IF ( perc_opc_guanyadora > 0 AND perc_opc_guanyadora < 1 ) THEN 
				perc_opc_guanyadora := '0'||perc_opc_guanyadora; 
			END IF;

			WHILE (length(perc_opc_guanyadora) < 6 ) LOOP

				perc_opc_guanyadora := perc_opc_guanyadora||' ';

			END LOOP;



			IF ( perc_opc_perdedora > 0 AND perc_opc_perdedora < 1 ) THEN 
				perc_opc_perdedora := '0'||perc_opc_perdedora; 
			END IF;

			WHILE (length(perc_opc_perdedora) < 6 ) LOOP

				perc_opc_perdedora := perc_opc_perdedora||' ';

			END LOOP;



			IF ( diferencia_perc > 0 AND diferencia_perc < 1 ) THEN 
				diferencia_perc := '0'||diferencia_perc; 
			END IF;

			WHILE (length(diferencia_perc) < 6 ) LOOP

				diferencia_perc := diferencia_perc||' ';

			END LOOP;

			-- una vegada formatejades, mostrem les dades de la fila actual
			DBMS_OUTPUT.PUT_LINE(titol||CHR(9)||votacio.data_inici||CHR(9)||votacio.data_fi||CHR(9)||opcio_guanyadora||CHR(9)||perc_opc_guanyadora||CHR(9)||opcio_perdedora||CHR(9)||perc_opc_perdedora||CHR(9)||CHR(9)||diferencia_perc||CHR(10)||CHR(13));


			EXIT WHEN c_votacions%ROWCOUNT >= 10 OR c_votacions%NOTFOUND OR c_votacions%NOTFOUND IS NULL;
		  END LOOP;

		  IF ( c_votacions%ROWCOUNT < 10 ) THEN

			DBMS_OUTPUT.PUT_LINE(CHR(10)||CHR(13)||CHR(10)||CHR(13)||'No s''han pogut mostrar 10 votacions perque nomes n''hi ha: '||c_votacions%ROWCOUNT);

		  END IF;


		  -- si el cursor s obert, el tanquem
		  IF c_votacions%ISOPEN THEN
			CLOSE c_votacions;
		  END IF;


		  

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		EXCEPTION

	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;













	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_consulta_votacionsMenysDif
	-- 			
	-- Descripci:		procediment que mostra dades pel canal de sortida de les 10 votacions on hi ha hagut ms diferncia percentual
	-- 			entre la opci ms votada i la menys votada. En concret, mostra el ttol de la votaci, les dates d'inici i de fi
	-- 			de la mateixa, la opci (o opcions) amb major percentatge de vots i aquest percentatge, la opci (o opcions) amb 
	-- 			menor percentatge i aquest percentatge i, finalment, la diferncia percentual entre la opci ms votada i la menys 
	-- 			votada. Finalment crida al procediment p_registrarLog del paquet pk_utils.
	-- 			
	-- Parmetres entrada:	--
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument al procediment p_registrarLog)
	-- 			
	-- Excepcions:		OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_consulta_votacionsMenysDif (
	  RSP		OUT	VARCHAR2
	  ) IS



		nom_procedure 		VARCHAR2(30 CHAR) := 'p_consulta_votacionsMenysDif';
	  	parametres_entrada 	VARCHAR2(30 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;

		votacions_finalitzades	NUMBER := 0;
		votacions_a_llistar	NUMBER := 0;
		opcio_guanyadora	VARCHAR2(35 CHAR);
		opcio_perdedora		VARCHAR2(35 CHAR);
		perc_opc_guanyadora	VARCHAR2(6 CHAR);
		perc_opc_perdedora	VARCHAR2(6 CHAR);
		diferencia_perc		VARCHAR2(6 CHAR);
		titol			VARCHAR2(100 CHAR);
		compensar		NUMBER := 0;
			

		CURSOR c_votacions IS SELECT 	v.id, v.titol, v.data_inici, v.data_fi, ev.nom_opcio_mes_vots, ev.percentatge_opcio_mes_vots, 
						ev.nom_opcio_menys_vots, ev.percentatge_opcio_menys_vots, ev.dif_perc_opc_mes_i_menys_vots 
						FROM votacions v, estadistiquesVotacions ev 
						WHERE v.id = ev.id_votacio AND finalitzada = 1 ORDER BY ev.dif_perc_opc_mes_i_menys_vots DESC;

		-- 'votacio' ser del mateix tipus que un registre (fila) del cursor c_votacions
		votacio c_votacions%ROWTYPE;

		BEGIN

		  parametres_entrada := ' ';

		  -- comprovem quantes votacions finalitzades hi ha
		  SELECT COUNT(*) INTO votacions_finalitzades FROM estadistiquesVotacions WHERE finalitzada = 1;
		  votacions_a_llistar := votacions_finalitzades;

		  -- si el cursor no s obert, l'obrim
		  IF NOT c_votacions%ISOPEN THEN
			OPEN c_votacions;
		  END IF;

		  dbms_output.put_line(CHR(9)||CHR(9)||CHR(10)||CHR(13)||CHR(10)||CHR(13)||CHR(10)||CHR(13)||CHR(10)||CHR(13)||CHR(9)||CHR(9)||CHR(9)||'LLISTAT DE LES 10 VOTACIONS AMB MENYS DIFERENCIA PERCENTUAL ENTRE LA OPCIO GUANYADORA I LA PERDEDORA');
		  dbms_output.put_line(CHR(9)||CHR(9)||'----------------------------------------------------------------------------------------------------------------------');


		  DBMS_OUTPUT.PUT_LINE(CHR(10)||CHR(13)||CHR(10)||CHR(13)||'TITOL DE LA VOTACIO'||CHR(9)||CHR(9)||CHR(9)||CHR(9)||CHR(9)||CHR(9)||'DATA INICI'||CHR(9)||'DATA FI'||CHR(9)||CHR(9)||'OPCIO GUANYADORA'||CHR(9)||'VOTS(%)'||CHR(9)||'OPCIO PERDEDORA'||CHR(9)||CHR(9)||'VOTS(%)'||CHR(9)||'DIFERENCIA(%)');
		  DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------');


		  -- recorrem el cursor (ja t les dades ordenades descendentment)
		  LOOP

			-- si cal, descartem files del cursor fins que noms en quedin 10
			WHILE ( votacions_a_llistar > 10 ) LOOP

				FETCH c_votacions INTO votacio;
				votacions_a_llistar := votacions_a_llistar - 1;

			END LOOP;


			FETCH c_votacions INTO votacio;


			titol := votacio.titol;
			opcio_guanyadora := votacio.nom_opcio_mes_vots;
			opcio_perdedora := votacio.nom_opcio_menys_vots;
			perc_opc_guanyadora := TO_CHAR(votacio.percentatge_opcio_mes_vots);
			perc_opc_perdedora := TO_CHAR(votacio.percentatge_opcio_menys_vots);
			diferencia_perc := TO_CHAR(votacio.dif_perc_opc_mes_i_menys_vots);


			-- formategem les dades de la fila actual (per tal de que totes les dades d'una mateixa columna tinguin una 
			-- llargada equivalent afegim espais en blanc al darrera fins arribar a una determinada longitud)

			WHILE (length(titol) < 60 ) LOOP

				titol := titol||' ';

			END LOOP;

			compensar := 0;
			IF ( length(opcio_guanyadora) <= 4 ) THEN
				compensar := 1;
			END IF;

			WHILE (length(opcio_guanyadora) < 20 ) LOOP

				opcio_guanyadora := opcio_guanyadora||' ';

			END LOOP;


			IF ( compensar = 1 ) THEN
				opcio_guanyadora := opcio_guanyadora||'        ';
			END IF;


			WHILE (length(opcio_perdedora) < 20 ) LOOP

				opcio_perdedora := opcio_perdedora||' ';

			END LOOP;


			IF ( perc_opc_guanyadora > 0 AND perc_opc_guanyadora < 1 ) THEN 
				perc_opc_guanyadora := '0'||perc_opc_guanyadora; 
			END IF;

			WHILE (length(perc_opc_guanyadora) < 6 ) LOOP

				perc_opc_guanyadora := perc_opc_guanyadora||' ';

			END LOOP;



			IF ( perc_opc_perdedora > 0 AND perc_opc_perdedora < 1 ) THEN 
				perc_opc_perdedora := '0'||perc_opc_perdedora; 
			END IF;

			WHILE (length(perc_opc_perdedora) < 6 ) LOOP

				perc_opc_perdedora := perc_opc_perdedora||' ';

			END LOOP;


			IF ( diferencia_perc > 0 AND diferencia_perc < 1 ) THEN 
				diferencia_perc := '0'||diferencia_perc; 
			END IF;

			WHILE (length(diferencia_perc) < 6 ) LOOP

				diferencia_perc := diferencia_perc||' ';

			END LOOP;

			-- una vegada formatejades, mostrem les dades de la fila actual
			DBMS_OUTPUT.PUT_LINE(titol||CHR(9)||votacio.data_inici||CHR(9)||votacio.data_fi||CHR(9)||opcio_guanyadora||CHR(9)||perc_opc_guanyadora||CHR(9)||opcio_perdedora||CHR(9)||perc_opc_perdedora||CHR(9)||CHR(9)||diferencia_perc||CHR(10)||CHR(13));


			EXIT WHEN c_votacions%ROWCOUNT = votacions_finalitzades OR c_votacions%NOTFOUND OR c_votacions%NOTFOUND IS NULL;
		  END LOOP;

		  IF ( c_votacions%ROWCOUNT < 10 ) THEN

			DBMS_OUTPUT.PUT_LINE(CHR(10)||CHR(13)||CHR(10)||CHR(13)||'No s''han pogut mostrar 10 votacions perque nomes n''hi ha: '||c_votacions%ROWCOUNT);

		  END IF;


		  -- si el cursor s obert, el tanquem
		  IF c_votacions%ISOPEN THEN
			CLOSE c_votacions;
		  END IF;


		  

		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		EXCEPTION

	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;











	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_consulta_censosCiutada
	-- 			
	-- Descripci:		procediment que mostra, pel canal de sortida, els censos als quals pertany un ciutad del qual se'n rep el
	-- 			nmero aleatori que l'identifica per parmetre.
	-- 			
	-- Parmetres entrada:	ciutada		NUMBER(10)	PK
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan el parmetre d'entrada s buit o NULL
	-- 			numero_incorrecte	-> quan el parmetre no s un nombre natural > 0 o quan t ms de 9 dgits decimals
	-- 			ciutada_inexistent	-> quan el nmero aleatori (parmetre d'entrada) no identifica cap ciutad de la BD
	-- 						   o quan el ciutad que identifica ha estat donat de baixa lgica.
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_consulta_censosCiutada (
	  ciutada	IN	ciutadans.num_aleatori%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS



		nom_procedure 		VARCHAR2(30 CHAR) := 'p_consulta_censosCiutada';
	  	parametres_entrada 	VARCHAR2(30 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		cens_local		VARCHAR2(40 CHAR) := NULL;
		cens_regional		VARCHAR2(40 CHAR) := NULL;
		cens_estatal		VARCHAR2(30 CHAR) := NULL;
		cens_global		VARCHAR2(30 CHAR) := NULL;
	  	camp_obligatori 	EXCEPTION;
		numero_incorrecte 	EXCEPTION;
		ciutada_inexistent	EXCEPTION;



		BEGIN


		  parametres_entrada := ciutada;

		  IF ( ciutada IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (	ciutada <= 0 OR
			length(ciutada) > 9 ) THEN
			RAISE numero_incorrecte;
		  END IF;

		  -- comprovem que el ciutad existeixi a la BD i no estigui donat de baixa
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM ciutadans WHERE num_aleatori = ciutada AND baixa_logica = 0;
		  IF ( trobat = 0 ) THEN
			RAISE ciutada_inexistent;
		  END IF;


		  dbms_output.put_line(CHR(9)||CHR(9)||CHR(10)||CHR(13)||CHR(10)||CHR(13)||CHR(9)||CHR(9)||CHR(9)||'CONSULTA DELS CENSOS ALS QUALS PERTANY EL CIUTADA AMB NUMERO ALEATORI '||ciutada);
		  dbms_output.put_line(CHR(9)||CHR(9)||'---------------------------------------------------------------------------------------------------');


		  DBMS_OUTPUT.PUT_LINE(CHR(10)||CHR(13)||CHR(10)||CHR(13)||'CENS GLOBAL'||CHR(9)||CHR(9)||CHR(9)||CHR(9)||'CENS ESTATAL'||CHR(9)||CHR(9)||CHR(9)||'CENS REGIONAL'||CHR(9)||CHR(9)||CHR(9)||CHR(9)||CHR(9)||'CENS LOCAL');
		  DBMS_OUTPUT.PUT_LINE('------------------------------------------------------------------------------------------------------------------------------------------------------');




		  -- obtenim els censos als quals pertany
		  SELECT id_cens_local INTO cens_local FROM ciutadans WHERE num_aleatori = ciutada;
		  SELECT id_cens_regional INTO cens_regional FROM censosLocals WHERE nom_cens = cens_local;
		  SELECT id_cens_estatal INTO cens_estatal FROM censosRegionals WHERE nom_cens = cens_regional;
		  SELECT id_cens_global INTO cens_global FROM censosEstatals WHERE nom_cens = cens_estatal;



		  -- formategem les dades (per tal de que totes les dades d'una mateixa columna tinguin una llargada equivalent 
		  -- afegim espais en blanc al darrera fins arribar a una determinada longitud)

		  WHILE (length(cens_global) < 29 ) LOOP

			cens_global := cens_global||' ';

		  END LOOP;


		  WHILE (length(cens_estatal) < 29 ) LOOP

			cens_estatal := cens_estatal||' ';

		  END LOOP;


		  WHILE (length(cens_regional) < 39 ) LOOP

			cens_regional := cens_regional||' ';

		  END LOOP;


		  WHILE (length(cens_local) < 39 ) LOOP

			cens_local := cens_local||' ';

		  END LOOP;

	

		  DBMS_OUTPUT.PUT_LINE(cens_global||CHR(9)||CHR(9)||cens_estatal||CHR(9)||cens_regional||CHR(9)||cens_local);


		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: el numero aleatori es NULL o sense valor.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN numero_incorrecte THEN

		   RSP := 'ERROR: el numero aleatori ha de ser un nombre natural diferent de zero (i ha de tenir menys de 10 digits).';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN ciutada_inexistent THEN

		   RSP := 'ERROR: el ciutada no existeix o esta donat de baixa.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;














	-- ---------------------------------------------------------------------------------------------------------------------------------
	-- Nom procediment:	p_consulta_votacionsCiutada
	-- 			
	-- Descripci:		procediment que mostra, pel canal de sortida, els censos als quals pertany un ciutad del qual se'n rep el
	-- 			nmero aleatori que l'identifica per parmetre.
	-- 			
	-- Parmetres entrada:	ciutada		NUMBER(10)	PK
	--			
	-- Parmetres sortida:	RSP	VARCHAR2	guardar 'OK' o b un missatge d'error (es passar com a argument
	-- 						al procediment p_registrarLog)
	-- 			
	-- Excepcions:		camp_obligatori		-> quan el parmetre d'entrada s buit o NULL
	-- 			numero_incorrecte	-> quan el parmetre no s un nombre natural > 0 o quan t ms de 9 dgits decimals
	-- 			ciutada_inexistent	-> quan el nmero aleatori (parmetre d'entrada) no identifica cap ciutad de la BD
	--			OTHERS			-> qualsevol altre esdeveniment
	-- ---------------------------------------------------------------------------------------------------------------------------------
	PROCEDURE p_consulta_votacionsCiutada (
	  ciutada	IN	ciutadans.num_aleatori%TYPE,
	  RSP		OUT	VARCHAR2
	  ) IS



		nom_procedure 		VARCHAR2(30 CHAR) := 'p_consulta_votacionsCiutada';
	  	parametres_entrada 	VARCHAR2(30 CHAR) := NULL;
		esCorrecte 		NUMBER(1) := 1;
		confirmada 		NUMBER(1) := 1;
		trobat			NUMBER(1) := 0;
		baixa			NUMBER(1) := 0;
		cens_local		VARCHAR2(40 CHAR) := NULL;
		cens_regional		VARCHAR2(40 CHAR) := NULL;
		cens_estatal		VARCHAR2(30 CHAR) := NULL;
		pais			VARCHAR2(30 CHAR) := NULL;
		cod_pais		CHAR(2) := NULL;
		titol_SI		VARCHAR2(250 CHAR);
		titol_NO		VARCHAR2(100 CHAR);
		n_votacions_NO		NUMBER := 0;
		n_votacions_SI		NUMBER := 0;
	  	iteracions 		NUMBER := 0;
		n_cridat_urnes		NUMBER := 0;
		camp_obligatori		EXCEPTION;
		numero_incorrecte 	EXCEPTION;
		ciutada_inexistent	EXCEPTION;



		-- c_votacions_NO tindr els identificadors de les voacions en les quals no ha participat el ciutad
		-- c_votacions_SI tindr els identificadors de les voacions en les quals ha participat el ciutad

		CURSOR c_votacions_NO IS SELECT id_votacio FROM ciutadansVotacions WHERE id_ciutada = ciutada AND ha_participat = 0;
		CURSOR c_votacions_SI IS SELECT id_votacio FROM ciutadansVotacions WHERE id_ciutada = ciutada AND ha_participat = 1;


		-- 'votacio_NO' i 'votacio_SI' seran del mateix tipus que un registre (fila) del seu cursor corresponent
		votacio_NO c_votacions_NO%ROWTYPE;	
		votacio_SI c_votacions_SI%ROWTYPE;	

		BEGIN


		  parametres_entrada := ciutada;

		  IF ( ciutada IS NULL ) THEN
				RAISE camp_obligatori;
		  END IF;

		  IF (	ciutada <= 0 OR
			length(ciutada) > 9 ) THEN
			RAISE numero_incorrecte;
		  END IF;

		  -- comprovem que el ciutad existeixi a la BD
		  trobat := 0;
		  SELECT COUNT(*) INTO trobat FROM ciutadans WHERE num_aleatori = ciutada;
		  IF ( trobat = 0 ) THEN
			RAISE ciutada_inexistent;
		  END IF;

		  -- mirem si el ciutad est de baixa (per presentar unes o altres dades ms endavant)
		  SELECT baixa_logica INTO baixa FROM ciutadans WHERE num_aleatori = ciutada;
		  


		  dbms_output.put_line(CHR(9)||CHR(9)||CHR(10)||CHR(13)||CHR(10)||CHR(13)||CHR(9)||CHR(9)||CHR(9)||'CONSULTA DE L''HISTORIC DE PARTICIPACIONS A LES VOTACIONS DEL CIUTADA AMB NUMERO ALEATORI '||ciutada);
		  dbms_output.put_line(CHR(9)||CHR(9)||'---------------------------------------------------------------------------------------------------------------------'||CHR(10)||CHR(13)||CHR(10)||CHR(13));


		  -- calculem les iteracions que haurem de fer al bucle de ms endavant (ja que vull mostrar les votacions en dues columnes, 
		  -- en comptes d'una)
		  SELECT COUNT(*) INTO n_votacions_NO FROM ciutadansVotacions WHERE id_ciutada = ciutada AND ha_participat = 0;
		  SELECT COUNT(*) INTO n_votacions_SI FROM ciutadansVotacions WHERE id_ciutada = ciutada AND ha_participat = 1;
		  IF ( n_votacions_NO > n_votacions_SI ) THEN
			iteracions := n_votacions_NO;
		  ELSE
			iteracions := n_votacions_SI;			
		  END IF;

		  -- n de vegades que ha estat cridat a les urnes
		  n_cridat_urnes := n_votacions_SI + n_votacions_NO;


		  -- obtenim el pas al qual pertany el ciutad (per mostrar informaci complementria)
		  SELECT id_cens_local INTO cens_local FROM ciutadans WHERE num_aleatori = ciutada;
		  SELECT id_cens_regional INTO cens_regional FROM censosLocals WHERE nom_cens = cens_local;
		  SELECT id_cens_estatal INTO cens_estatal FROM censosRegionals WHERE nom_cens = cens_regional;
		  SELECT codi_pais INTO cod_pais FROM censosEstatals WHERE nom_cens = cens_estatal;
		  SELECT nom_pais INTO pais FROM paisos WHERE codi_pais = cod_pais;

		  IF ( baixa = 0 ) THEN

			  DBMS_OUTPUT.PUT_LINE('Pais del ciutada: '||pais||CHR(10)||CHR(13));
			  DBMS_OUTPUT.PUT_LINE('Cens local: '||cens_local||CHR(10)||CHR(13));

		  ELSIF (baixa = 1) THEN

			  DBMS_OUTPUT.PUT_LINE('Pais del ciutada: El ciutada ha estat donat de baixa.'||CHR(10)||CHR(13));
			  DBMS_OUTPUT.PUT_LINE('Cens local: El ciutada ha estat donat de baixa.'||CHR(10)||CHR(13));

		  END IF;

		  DBMS_OUTPUT.PUT_LINE('Numero de vegades que ha estat cridat a les urnes: '||n_cridat_urnes||CHR(10)||CHR(13));

 

		  DBMS_OUTPUT.PUT_LINE(CHR(10)||CHR(13)||CHR(10)||CHR(13)||'VOTACIONS EN LES QUALS HA PARTICIPAT'||CHR(9)||CHR(9)||CHR(9)||CHR(9)||CHR(9)||'VOTACIONS EN LES QUALS NO HA PARTICIPAT');
		  DBMS_OUTPUT.PUT_LINE('----------------------------------------------------------------------------------------------------------------------------------');


		  -- si els cursors no estan oberts, els obrim
		  IF NOT c_votacions_SI%ISOPEN THEN
			OPEN c_votacions_SI;
		  END IF;

		  IF NOT c_votacions_NO%ISOPEN THEN
			OPEN c_votacions_NO;
		  END IF;

		  -- recorrem les files dels cursors i anem formatejant les dades a mostrar (sn de longitud variable i hi afegirem espais en 
		  -- blanc fins a una determinada longitud)
		  LOOP

			-- obtenim una fila de cada cursor
			FETCH c_votacions_NO INTO votacio_NO;
			FETCH c_votacions_SI INTO votacio_SI;

			-- comprovem si la fila del cursor c_votacions_NO t dades o no, i actuem en conseqncia
			IF ( c_votacions_NO%NOTFOUND OR c_votacions_NO%NOTFOUND IS NULL ) THEN
				titol_NO := '-';

			ELSE
				SELECT titol INTO titol_NO FROM votacions WHERE id = votacio_NO.id_votacio;
	
			END IF;
 


			-- comprovem si la fila del cursor c_votacions_SI t dades o no, i actuem en conseqncia
			IF ( c_votacions_SI%NOTFOUND OR c_votacions_SI%NOTFOUND IS NULL ) THEN

				titol_SI := '-';

			ELSE

				SELECT titol INTO titol_SI FROM votacions WHERE id = votacio_SI.id_votacio;
	
			END IF;


			-- formategem les dades (per tal de que totes tinguin una llargada equivalent afegim espais en blanc al darrera
			-- fins arribar a una determinada longitud)

			WHILE ( length(titol_NO) < 60 ) LOOP

				titol_NO := titol_NO||' ';

		  	END LOOP;


			WHILE ( length(titol_SI) < 60 ) LOOP

				titol_SI := titol_SI||' ';

		  	END LOOP;


			DBMS_OUTPUT.PUT_LINE(titol_SI||CHR(9)||CHR(9)||titol_NO);
 


			IF ( n_votacions_NO > n_votacions_SI) THEN

				EXIT WHEN c_votacions_NO%ROWCOUNT = iteracions OR c_votacions_NO%NOTFOUND OR c_votacions_NO%NOTFOUND IS NULL;
			ELSE

				EXIT WHEN c_votacions_SI%ROWCOUNT = iteracions OR c_votacions_SI%NOTFOUND OR c_votacions_SI%NOTFOUND IS NULL;

			END IF;


		  END LOOP;


		  -- si els cursors estan oberts, els tanquem
		  IF c_votacions_SI%ISOPEN THEN
			CLOSE c_votacions_SI;
		  END IF;

		  IF c_votacions_NO%ISOPEN THEN
			CLOSE c_votacions_NO;
		  END IF;




		  RSP := 'OK';
		  pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		EXCEPTION

		WHEN camp_obligatori THEN

		   RSP := 'ERROR: el numero aleatori es NULL o sense valor.';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);


		WHEN numero_incorrecte THEN

		   RSP := 'ERROR: el numero aleatori ha de ser un nombre natural diferent de zero (i ha de tenir menys de 10 digits).';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

		WHEN ciutada_inexistent THEN

		   RSP := 'ERROR: el ciutada no existeix a la BD';
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);

	
		WHEN OTHERS THEN
	
		   RSP := 'Error SQL: '||SQLCODE||'-'||SQLERRM;
		   esCorrecte := 0;
		   confirmada := 0;
		   pk_utils.p_registrarLog(LOCALTIMESTAMP, nom_procedure, parametres_entrada, RSP, esCorrecte, confirmada);
	
		END;





END; --fi implementaci paquet pk_consultes
/













